// ===========================================================================
//	PP Basic Starter.cp 		1994-1997 Metrowerks Inc. All rights reserved.
// ===========================================================================
//
//	This file contains the starter code for a basic PowerPlant application

#include <LGrowZone.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <PP_Resources.h>
#include <PPobClasses.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <URegistrar.h>
#include <LEditField.h>
#include <UModalDialogs.h>

#include <string.h>

// #include "GCNewControlsPP.h"
#include "LOffscreenBufferPane.h"
#include "gorStructs.h"
#include "gor stuff.h"

#include "PP Basic Starter.h"

// put declarations for resource ids (ResIDTs) here

const ResIDT	window_Sample		= 1;	// EXAMPLE


// ===========================================================================
//		 Main Program
// ===========================================================================

void main(void)
{
									// Set Debugging options
	SetDebugThrow_(debugAction_Alert);
	SetDebugSignal_(debugAction_Alert);

	InitializeHeap(3);				// Initialize Memory Manager
									// Parameter is number of Master Pointer
									//   blocks to allocate
	
									// Initialize standard Toolbox managers
	UQDGlobals::InitializeToolbox(&qd);
	
	new LGrowZone(20000);			// Install a GrowZone function to catch
									//    low memory situations.

	ThrowIfOSErr_(InitGrayCouncil());
	gAGAExtraControls = true;

	CPPStarterApp	theApp;			// replace this with your App type
	theApp.Run();
}


// ---------------------------------------------------------------------------
//		 CPPStarterApp 			// replace this with your App type
// ---------------------------------------------------------------------------
//	Constructor

CPPStarterApp::CPPStarterApp()
{
	// Register functions to create core PowerPlant classes
	
//	RegisterAllPPClasses();
	RegisterClass_(LButton);
	RegisterClass_(LCaption);
	RegisterClass_(LDialogBox);
	RegisterClass_(LEditField);
	RegisterClass_(LPane);
	RegisterClass_(LStdControl);
	RegisterClass_(LStdButton);
	RegisterClass_(LView);
	RegisterClass_(LWindow);
	
	URegistrar::RegisterClass(LOffscreenBufferPane::class_ID, (ClassCreatorFunc) LOffscreenBufferPane::CreateStream);
//	URegistrar::RegisterClass(AGAProgressIndicatorPP::class_ID, (ClassCreatorFunc) AGAProgressIndicatorPP::CreateStream);
}


// ---------------------------------------------------------------------------
//		 ~CPPStarterApp			// replace this with your App type
// ---------------------------------------------------------------------------
//	Destructor
//

CPPStarterApp::~CPPStarterApp()
{
}

// ---------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------
//	This function lets you do something when the application starts up
//	without a document. For example, you could issue your own new command.

void
CPPStarterApp::StartUp()
{
	ObeyCommand(cmd_Open, nil);
}

// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
CPPStarterApp::ObeyCommand(
	CommandT	inCommand,
	void		*ioParam)
{
	Boolean		cmdHandled = true;

	switch (inCommand) {
	
		// Deal with command messages (defined in PP_Messages.h).
		// Any that you don't handle will be passed to LApplication
 			
		case cmd_Open:
			ChooseDocument();
			break;


		case cmd_New:
										// EXAMPLE, create a new window
			LWindow		*theWindow;
			theWindow = LWindow::CreateWindow(window_Sample, this);	
			theWindow->Show();
			break;



		default:
			cmdHandled = LApplication::ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}

// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	This function enables menu commands.
//

void
CPPStarterApp::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{

	switch (inCommand) {
	
		// Return menu item status according to command messages.
		// Any that you don't handle will be passed to LApplication

case cmd_Open:
//		case cmd_New:					// EXAMPLE
			outEnabled = true;			// enable the New command
			break;

		default:
			LApplication::FindCommandStatus(inCommand, outEnabled,
												outUsesMark, outMark, outName);
			break;
	}
}

// ---------------------------------------------------------------------------------
//		 ChooseDocument
// ---------------------------------------------------------------------------------

void
CPPStarterApp::ChooseDocument()
{
	// Deactivate the desktop.
	::UDesktop::Deactivate();

	// Browse for a document.
	SFTypeList			theTypeList;
	StandardFileReply	theReply;
	memset(&theReply, 0, sizeof(StandardFileReply));

	::StandardGetFile(nil, -1, theTypeList, &theReply);
	
	// Activate the desktop.
	::UDesktop::Activate();
	
	// Send an apple event to open the file.	
	// if ( theReply.sfGood ) SendAEOpenDoc( theReply.sfFile );
	
	// Read the file header
	if (theReply.sfGood) {
		LFileStream theDataFile(theReply.sfFile);
		LFileStream theNewFile(theReply.sfFile);
		
		dir_Header theFileHeader;
		memset(&theFileHeader, 0, sizeof(dir_Header));
		Int32 byteCount = sizeof(dir_Header);
		
		theDataFile.OpenDataFork(fsRdPerm);
		theDataFile.GetBytes(&theFileHeader, byteCount);
		
		collection_header theInfoHeader;
		memset(&theInfoHeader, 0, sizeof(collection_header));
		byteCount = sizeof(collection_header);
		
		theDataFile.GetBytes(&theInfoHeader, byteCount);
		Int32 endOfHeader = theDataFile.GetMarker();
		
		// Let's make a color table
		Assert_(theInfoHeader.color_table_count >= 1);
		Ptr theColorData = ::NewPtr(theInfoHeader.color_tables_size);
		ThrowIfNil_(theColorData);
		CTabHandle theColors = ::GetCTable(40);		// default 8-bit grey
		theDataFile.SetMarker(endOfHeader + theInfoHeader.color_tables_offset, streamFrom_Start);
		byteCount = theInfoHeader.color_tables_size;
		theDataFile.GetBytes(theColorData, byteCount);
		
		::HLock((Handle) theColors);
		Ptr colorSourcePtr = theColorData + 32;		// skip header
		for (Int16 colorCount = 0; colorCount <= (*theColors)->ctSize; colorCount++) {
			(*theColors)->ctTable[colorCount].rgb = *((RGBColor*) colorSourcePtr);
			colorSourcePtr += 8;
		}
		::HUnlock((Handle) theColors);
		
		::CTabChanged(theColors);
		::DisposePtr(theColorData);
		
		// Let's read some bitmaps
		Int32 bitmapRefMarker = endOfHeader + theInfoHeader.bitmap_references_offset;
		
		StDialogHandler theDialog(1000, this);
		MessageT theButton = msg_OK;	// will read first time thru loop
		Int32 bitmapCount = theInfoHeader.bitmap_count;
		
		Rect bitBounds = { 0, 0, 0, 0 };
		LGWorld *theOffscreenPtr = nil;

		bitmap_reference theBitmapRef;

		while (theButton != msg_Cancel) {
			if (theButton == msg_OK) {
				if (bitmapCount-- >= 1) {
					if (theOffscreenPtr)
						delete theOffscreenPtr;
					theDataFile.SetMarker(bitmapRefMarker, streamFrom_Start);
					memset(&theBitmapRef, 0, sizeof(bitmap_reference));
					byteCount = sizeof(bitmap_reference);
					theDataFile.GetBytes(&theBitmapRef, byteCount);
					bitmapRefMarker= theDataFile.GetMarker();
					
					theDataFile.SetMarker(endOfHeader + theBitmapRef.offset, streamFrom_Start);

					Ptr theBitmapData = ::NewPtr(theBitmapRef.size);
					ThrowIfNil_(theBitmapData);
					byteCount = theBitmapRef.size;
					theDataFile.GetBytes(theBitmapData, byteCount);
					
					bitmapData_Header *bitmapHeaderPtr = (bitmapData_Header*) theBitmapData;
					Ptr bitmapDataPtr = theBitmapData + sizeof(bitmapData_Header);
					if (theInfoHeader.user_data_flags & _collection_is_color_map_flag)
						bitmapDataPtr += 4;
					else
						bitmapDataPtr += bitmapHeaderPtr->height * sizeof(Ptr);	// jump over the row ptrs (?)
					
					bitBounds.bottom = bitmapHeaderPtr->height;
					bitBounds.right = bitmapHeaderPtr->width;
					
					if (bitmapHeaderPtr->bitDepth > 8)
						theOffscreenPtr = new LGWorld(bitBounds, bitmapHeaderPtr->bitDepth);
					else
						theOffscreenPtr = new LGWorld(bitBounds, 8, 0, theColors);
					UInt8 bytesPerColor = bitmapHeaderPtr->bitDepth / 8;
						
					if (theOffscreenPtr->BeginDrawing()) {
						PixMapHandle thePixH = ::GetGWorldPixMap(theOffscreenPtr->GetMacGWorld());
						UInt32 realPixRowBytes = (*thePixH)->rowBytes & 0x3FFF;
						Ptr realPixPtr = ::GetPixBaseAddr(thePixH);

						if (bitmapHeaderPtr->rowBytes == -1) {	// Encoded sprite
							::EraseRect(&bitBounds);

							for (UInt16 rowCount = 1; rowCount <= bitmapHeaderPtr->height; rowCount++) {
								UInt16 *encodingHeader = (UInt16 *) bitmapDataPtr;

								bitmapDataPtr += (2 * (encodingHeader[0] + 1) * sizeof(UInt16));
								for (UInt16 runCount = 1; runCount <= encodingHeader[0]; runCount++) {
									UInt16 columnStartLoc = encodingHeader[runCount * 2];
									UInt16 columnStopLoc = encodingHeader[runCount * 2 + 1];
									UInt16 byteCount = (columnStopLoc - columnStartLoc) * bytesPerColor;
									Ptr rowPixPtr = realPixPtr + (columnStartLoc * bytesPerColor);
									::BlockMoveData(bitmapDataPtr, rowPixPtr, byteCount);
									bitmapDataPtr += byteCount;
								}

								if ((Int32) bitmapDataPtr & 0x01)	// check for uneven row
									bitmapDataPtr++;
								realPixPtr += realPixRowBytes;
							}
						}
						else {
							for (UInt16 rowCount = 1; rowCount <= bitmapHeaderPtr->height; rowCount++) {
								::BlockMoveData(bitmapDataPtr, realPixPtr, bitmapHeaderPtr->rowBytes);
								bitmapDataPtr += bitmapHeaderPtr->rowBytes;
								realPixPtr += realPixRowBytes;
							}
						}
					}
					theOffscreenPtr->EndDrawing();
					::DisposePtr(theBitmapData);
				}
				else // no more bitmaps
					SysBeep(1);
			}
			
			theButton = theDialog.DoDialog();
			theOffscreenPtr->CopyImage(theDialog.GetDialog()->GetMacPort(), bitBounds);
		}		

		delete theOffscreenPtr;
		
/*
		for (UInt8 count = 0; count < theInfoHeader.bitmap_count; count++) {
			byteCount = sizeof(bitmap_reference);
			theDataFile.GetBytes(&theBitmapRef, byteCount);
		}
*/
				
		theDataFile.CloseDataFork();
	}
}
