//
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//

#include "MyConstants.h"

#include "GCDialogManager.h"
#include "GCNewControls.h"

// A quick cut & paste sorta merge of CApplicationBase and the
// MacOS CApplication from my experimental cross-platform framework...

class Application
	{
	public:
	
		Application();
		virtual ~Application();
		
		virtual void Run();

		virtual void MarkApplicationDone();

		static Application	*fgApplication;
	
	protected:
	
		virtual void DoInitialize();
		virtual void DoCleanup();

		virtual void InitializeMacOS();
		virtual void InitializeMacMenus();
		virtual void InitializeAppleEvents();

		virtual void DoEventLoop();
		virtual void EventLoopOnce();
		virtual void RetrieveAnEvent();
		virtual void ProcessAnEvent();
		virtual void ProcessIdle();
		virtual void PrepareToQuit();

		virtual void HandleMacOSMouseDown(EventRecord *event);
		virtual void HandleMacOSActivateEvent(EventRecord *event);
		virtual void HandleMacOSUpdateEvent(EventRecord *event);
		virtual void HandleMacOSKeyDownEvent(EventRecord *event);
		virtual void DoMacOSMenuEvent(SInt32 menuSelectResult);

		virtual void HandleMacOSWindowMouseDown(WindowPtr aWindow, EventRecord *event, Point globalMouse, SInt16 partCode);

		virtual void ActivateApplication(Boolean convertClipboard);
		virtual void DeactivateApplication(Boolean convertClipboard);
		
		virtual OSErr HandleAEOpenApplication(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon);
		virtual OSErr HandleAEOpenDocuments(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon);
		virtual OSErr HandleAEPrintDocuments(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon);
		virtual OSErr HandleAEQuitApplication(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon);

		//CWindow	*MacOSWindowPtrToWindow(WindowPtr aWindowPtr);
		Boolean IsFrontMacOSProcess();
		
		Boolean	fDone;
		UInt32	fEventLevel;
		UInt32	fModalLevel;
		Boolean	fGotAnEvent;

		SInt16		fEventMask;
		SInt32		fEventSleep;
		RgnHandle	fEventSleepRgn;
		EventRecord	fEventRecord;

		ProcessSerialNumber	fProcessNumber;
		
		static pascal OSErr fgAEOpenApplicationHandler(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon);
		static pascal OSErr fgAEOpenDocumentsHandler(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon);
		static pascal OSErr fgAEPrintDocumentsHandler(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon);
		static pascal OSErr fgAEQuitApplicationHandler(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon);

		void	DestroyMainWindow();
		void	CreateMainWindow();

		void	DoStdModalDialog();
		void	DoAGAModalDialog();
		void	DoAboutBox();
		void	DoBigDialog();
		
		void	DoMainWindowButtonHit(SInt16 itemHit);
		
		void	SwapAGAOnFlag();
		void	SwapAGADialogFlag();
		
		Boolean		mMainWindowIsAGA;
		DialogPtr	mMainWindowDialogPtr;
		AGADialog	mMainWindowAGADialog;
	};

Application* Application::fgApplication = NULL;

void main()
	{
	Application	myApplication;
	
	myApplication.Run();
	}

Application::Application()
	{
	fgApplication = this;

	fDone = FALSE;
	fEventLevel = 0;
	fModalLevel = 0;
	
	fEventMask = everyEvent;
	fEventSleep = 6;	// 0.1 sec
	fEventSleepRgn = NULL;
	
	mMainWindowIsAGA = true;
	mMainWindowDialogPtr = NULL;
	}

#define USE_AGA

Application::~Application()
	{
	this->DestroyMainWindow();
	}

void Application::Run()
	{
	this->DoInitialize();
	this->DoEventLoop();
	this->DoCleanup();
	}

void Application::DoInitialize()
	{
	this->InitializeMacOS();
	this->InitializeMacMenus();
	this->InitializeAppleEvents();
	
	if (InitGrayCouncil() != noErr)
		::ExitToShell();

	this->CreateMainWindow();
	}

void Application::DoEventLoop()
	{
	while (! fDone)
		{
		this->EventLoopOnce();

		if (fDone)
			this->PrepareToQuit();	// may set fDone FALSE
		}
	}

void Application::DoCleanup()
	{
	this->DeactivateApplication(TRUE);
	}

void Application::InitializeMacOS()
	{
	::InitGraf(&qd.thePort);
	::InitFonts();
	::InitWindows();
	::InitMenus();
	::TEInit();
	::InitDialogs(NULL);
	::InitCursor();
	
	::MaxApplZone();
	}

void Application::InitializeMacMenus()
	{
	Handle	menuBarResource = ::GetNewMBar(kStandardMenuBarID);
	
	::SetMenuBar(menuBarResource);
	
	::DisposeHandle(menuBarResource);
	
	::AppendResMenu(::GetMenuHandle(kAppleMenuID), 'DRVR');
	
	::DrawMenuBar();
	}

void Application::InitializeAppleEvents()
	{
	(void) AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, NewAEEventHandlerProc(Application::fgAEOpenApplicationHandler), 0, FALSE);
	(void) AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, NewAEEventHandlerProc(Application::fgAEOpenDocumentsHandler), 0, FALSE);
	(void) AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments, NewAEEventHandlerProc(Application::fgAEPrintDocumentsHandler), 0, FALSE);
	(void) AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, NewAEEventHandlerProc(Application::fgAEQuitApplicationHandler), 0, FALSE);
	}

void Application::EventLoopOnce()
	{
	fEventLevel++;
	
	//this->CommandSynchronize();
	
	this->RetrieveAnEvent();
	
	if (fGotAnEvent)
		this->ProcessAnEvent();
	else if (! fDone)
		this->ProcessIdle();
	
	fEventLevel--;
	}

void Application::RetrieveAnEvent()
	{
	fGotAnEvent = ::WaitNextEvent(fEventMask | osMask, &fEventRecord, fEventSleep, fEventSleepRgn);
	}

void Application::ProcessAnEvent()
	{
	//fNeedSynchronize = TRUE;

	if (::IsDialogEvent(&fEventRecord))
		{
		DialogPtr	theDialog;
		SInt16		itemHit;
		Boolean		wasItemHit = false;
		
		if (mMainWindowIsAGA)
			wasItemHit = mMainWindowAGADialog.DialogSelect(&fEventRecord, &itemHit);
		else
			wasItemHit = ::DialogSelect(&fEventRecord, &theDialog, &itemHit);

		if (wasItemHit)
			{
			this->DoMainWindowButtonHit(itemHit);
			return;
			}
		}

	//
	// Some Mac Toolbox messages can just be ignored.
	// A couple are little things we can handle right here.
	// The mouseDown, keyDown, and window activate/update
	// events are the important ones, and we handle those
	// in separate member functions.
	//

	switch (fEventRecord.what)
		{
		case mouseUp:
			// yawn
			break;

		case mouseDown:
			this->HandleMacOSMouseDown(&fEventRecord);
			break;

		case activateEvt:
			this->HandleMacOSActivateEvent(&fEventRecord);
			break;

		case updateEvt:
			this->HandleMacOSUpdateEvent(&fEventRecord);
			break;

		case keyDown:
		case autoKey:
			this->HandleMacOSKeyDownEvent(&fEventRecord);
			break;

		case keyUp:
			// yawn
			break;

		case diskEvt:
			if (HiWord(fEventRecord.message) != noErr)
				{
				Point	aPoint;
				
				::SetPt(&aPoint, 112, 80);
				(void) ::DIBadMount(aPoint, fEventRecord.message);
				}
			break;

		case osEvt:
			if (((unsigned long) (fEventRecord.message & osEvtMessageMask)) >> 24 == suspendResumeMessage)
				{
				Boolean switchingIn = ((fEventRecord.message) & resumeFlag) != 0;
				Boolean convertClipboard = ((fEventRecord.message) & convertClipboardFlag) != 0;
	
				if (switchingIn)
					this->ActivateApplication(convertClipboard);
				else
					this->DeactivateApplication(convertClipboard);
				}
			break;

		case nullEvent:
		case networkEvt:
		case driverEvt:
		case app1Evt:
		case app2Evt:
		case app3Evt:
			// yawn
			break;

		case kHighLevelEvent:
			(void) ::AEProcessAppleEvent(&fEventRecord);
			break;

		default:
			// yawn
			break;
		}
	}

void Application::ProcessIdle()
	{
	/*
	if (fgTarget != NULL)
		fgTarget->ReceiveIdleTime();

	ArrayIndex	numIdlers = fIdlerList->GetSize();
	
	for (ArrayIndex index = 0; index < numIdlers; index++)
		((MMessageListener *) fIdlerList->At(index))->ReceiveIdleTime();
	*/
	}

void Application::PrepareToQuit()
	{
	// This is where we need to close all open documents, allowing
	// the user to cancel the quit. If the quit is to be cancelled,
	// we simply set fDone to FALSE in here. This causes the event
	// loop to keep going.
	}

void Application::MarkApplicationDone()
	{
	//
	// Send ourself a 'QUIT' Apple Event.
	//
	
	Boolean				aeSent = FALSE;
	ProcessSerialNumber	ourPSN;
	AEAddressDesc		ourAddressDesc;
	AppleEvent			theQuitEvent;
	AppleEvent			theQuitReply;

	ourPSN.highLongOfPSN = 0;
	ourPSN.lowLongOfPSN = kCurrentProcess;

	if (AECreateDesc(typeProcessSerialNumber, (Ptr) &ourPSN, sizeof(ProcessSerialNumber), &ourAddressDesc) == noErr)
		if (AECreateAppleEvent(kCoreEventClass, kAEQuitApplication, &ourAddressDesc, kAutoGenerateReturnID, kAnyTransactionID, &theQuitEvent) == noErr)
			{
			aeSent = (noErr == AESend(&theQuitEvent, &theQuitReply, kAENoReply + kAECanInteract, kAENormalPriority, 3600, NULL, NULL));
			(void) AEDisposeDesc(&theQuitEvent);
			}

	//	
	// If for some reason we failed to send ourself the 'QUIT'
	// event, do here what we would have done on receipt of the
	// event.
	//

	if (! aeSent)
		fDone = TRUE;
	}

void Application::ActivateApplication(Boolean convertClipboard)
	{
	if (convertClipboard)
		(void) ::TEFromScrap();

	//Inherited::ActivateApplication(convertClipboard);
	}

void Application::DeactivateApplication(Boolean convertClipboard)
	{
	if (convertClipboard)
		{
		::ZeroScrap();
		(void) ::TEToScrap();
		}

	//Inherited::DeactivateApplication(convertClipboard);
	}

void Application::HandleMacOSMouseDown(EventRecord *event)
	{
	WindowPtr aWMgrWindow;

	SInt16 whereMouseDown = ::FindWindow(event->where, &aWMgrWindow);
//	event->fClickCount = this->CountClicks(event, whereMouseDown);

	//CWindow *aWindow = this->MacOSWindowPtrToWindow(aWMgrWindow);

	if ((whereMouseDown != inMenuBar) && (fModalLevel > 0))// && (aWindow != Application::fgActiveWindow))
		{
		::SysBeep(5);
		return;
		}

	switch (whereMouseDown)
		{
		case inMenuBar:
			//this->CommandSynchronize();		// gives application a chance to setup individual menu items
			::SetCursor(&(qd.arrow));
			this->DoMacOSMenuEvent(MenuSelect(event->where));
			::HiliteMenu(0);
			break;

		case inSysWindow:
			::SystemClick(event, aWMgrWindow);
			break;

		default:
			this->HandleMacOSWindowMouseDown(aWMgrWindow, event, event->where, whereMouseDown);

			/*
			if (aWindow != NULL)
				{
				if (aWindow->Focus())	// if we can't focus, we're in trouble 
					(void) aWindow->HandleMacOSWindowMouseDown(event->where, event);
				}
			*/
			break;

		}
	}

void Application::HandleMacOSActivateEvent(EventRecord */*event*/)
	{
	/*
	CWindow *aWindow = this->MacOSWindowPtrToWindow((WindowPtr) (event->message));

	if (aWindow != NULL)
		aWindow->Activate(((event->modifiers) & activeFlag) != 0);
	*/
	}

void Application::HandleMacOSUpdateEvent(EventRecord */*event*/)
	{
	/*
	CWindow *aWindow = this->MacOSWindowPtrToWindow((WindowPtr) (event->message));

	if (aWindow != NULL)
		aWindow->Update();
	*/
	}

void Application::HandleMacOSKeyDownEvent(EventRecord *event)
	{
	Boolean	passToTarget = TRUE;
	SInt32	menuKeyResult;

	if (((event->modifiers) & cmdKey) != 0)
		{
		if (event->what != autoKey)
			{
			menuKeyResult = ::MenuKey((UInt8) (((event->message) & charCodeMask)));
			passToTarget = (menuKeyResult == 0);

			if (! passToTarget)
				{
				//this->CommandSynchronize();
				this->DoMacOSMenuEvent(menuKeyResult);
				//fNeedSynchronize = TRUE;
				//this->CommandSynchronize();
				}

			::HiliteMenu(0);
			}
		}

	if (passToTarget)
		{
		//CKeystroke	*aKeystroke = new CKeystroke(event);
	
		//fgTarget->ReceiveKeystroke(aKeystroke, this);
		}
	}

void Application::HandleMacOSWindowMouseDown(WindowPtr aWindow, EventRecord */*event*/, Point globalMouse, SInt16 aPartCode)
	{
	Boolean		result = TRUE;
//	Point		localMouse = globalMouse;
	
//	::GlobalToLocal(&localMouse);

//	aPartCode = this->GetPartCode(localMouse);

/*
	if (fMacOSWindowPtr != ::FrontWindow())
		{
		if ((aPartCode != inDrag) || ((event->modifiers & cmdKey) == 0))
			this->Select();
		}
*/	
	switch (aPartCode)
		{
		case inContent:
			{
			// Convert from QD 16-bit to framework long coordinates.
//			CPoint	aPoint(localMouse.x, localMouse.y);
//			(void) this->HandleMouseDown(aPoint, aMessage);
			}
			break;

		case inDrag:
			{
			Rect		moveBounds = (**(GetGrayRgn())).rgnBBox;
			//Rect		windowFrame;
			
			::DragWindow(aWindow, globalMouse, &moveBounds);
			
			//this->GetMacOSWindowFrame(windowFrame);
			//this->SetFrame(windowFrame, kDontRedraw);	// let updateEvt cause redraw
			}
			break;
/*
		case inGrow:
			this->TrackResizeByUser(localMouse);
			break;

		case inGoAway:
			this->TrackGoAwayByUser(localMouse);
			break;

		case inZoomIn:
		case inZoomOut:
			this->TrackZoomByUser(localMouse, aPartCode);
			break;

		case inDesk:
			break;
*/
		}
	}

static void OldFlashButton(DialogPtr theDialog, SInt16 buttonItemNo)
	{
	SInt32	dontCare;
	SInt16	itemType;
	Handle	itemHandle;
	Rect	itemRect;

	::GetDialogItem(theDialog, buttonItemNo, &itemType, &itemHandle, &itemRect);

	::HiliteControl((ControlHandle) itemHandle, 1);
	::Delay(8, &dontCare);
	::HiliteControl((ControlHandle) itemHandle, 0);
	}

static void NewFlashButton(DialogPtr theDialog, SInt16 buttonItemNo)
	{
	AGADialog*	agaDialog = AGADialog::ObjectFromDialogPtr(theDialog);
	
	if (agaDialog != NULL)
		agaDialog->FlashButton(buttonItemNo);
	}

static pascal Boolean OldCustomKeyFilter(DialogPtr theDialog, EventRecord* theEvent, SInt16* itemHit)
	{
	UInt16	key = (theEvent->message) & charCodeMask;
	Boolean	isCommandKey = (((theEvent->modifiers) & cmdKey) != 0);

	if (isCommandKey && ((key == 'd') || (key == 'D')))
		{
		OldFlashButton(theDialog, 3);

		*itemHit = 3;	// "Don't Save"
		return true;
		}
	else if ((key == 0x1B) || (isCommandKey && (key == '.')))
		{
		OldFlashButton(theDialog, cancel);

		*itemHit = cancel;
		return true;
		}
	else if ((key == 0x03) || (key == 0x0D))
		{
		OldFlashButton(theDialog, ok);

		*itemHit = ok;
		return true;
		}
	else
		return false;
	}

static pascal Boolean NewCustomKeyFilter(DialogPtr theDialog, EventRecord* theEvent, SInt16* itemHit)
	{
	UInt16	key = (theEvent->message) & charCodeMask;
	Boolean	isCommandKey = (((theEvent->modifiers) & cmdKey) != 0);

	if (isCommandKey && ((key == 'd') || (key == 'D')))
		{
		NewFlashButton(theDialog, 3);

		*itemHit = 3;	// "Don't Save"
		return true;
		}
	else if ((key == 0x1B) || (isCommandKey && (key == '.')))
		{
		NewFlashButton(theDialog, cancel);

		*itemHit = cancel;
		return true;
		}
	else if ((key == 0x03) || (key == 0x0D))
		{
		NewFlashButton(theDialog, ok);

		*itemHit = ok;
		return true;
		}
	else
		return false;
	}

static ModalFilterUPP gOldCustomKeyFilterUPP = NewModalFilterProc(OldCustomKeyFilter);
static ModalFilterUPP gNewCustomKeyFilterUPP = NewModalFilterProc(NewCustomKeyFilter);

void Application::DoMacOSMenuEvent(SInt32 menuSelectResult)
	{
	SInt16	menuID = (SInt16) (menuSelectResult >> 16);
	SInt16	itemNo = (SInt16) (menuSelectResult & 0x0000FFFF);
	
	if ((menuID == kAppleMenuID) && (itemNo != 1))
		{
		//
		// It's a system item. Let the system handle it.
		//

		Str255	itemName;
		GrafPtr savedPort;

		::GetMenuItemText(::GetMHandle(menuID), itemNo, itemName);

		::GetPort(&savedPort);
		::OpenDeskAcc(itemName);
		::SetPort(savedPort);
		}
	else if ((menuID == kAppleMenuID) && (itemNo == 1))
		{
		this->DoAboutBox();
		}
	else if ((menuID == kFileMenuID) && (itemNo == 1))
		{
		fDone = true;
		}
	else if (menuID == kStandardMenuID)
		{
		switch (itemNo)
			{
			case 1:
				::Alert(kSampleAlertID, gOldCustomKeyFilterUPP);
				break;
			case 2:
				::NoteAlert(kSampleAlertID, gOldCustomKeyFilterUPP);
				break;
			case 3:
				::CautionAlert(kSampleAlertID, gOldCustomKeyFilterUPP);
				break;
			case 4:
				::StopAlert(kSampleAlertID, gOldCustomKeyFilterUPP);
				break;
			case 5:
				this->DoStdModalDialog();
				break;
			}
		}
	else if (menuID == kCustomMenuID)
		{
		switch (itemNo)
			{
			case 1:
				::AGAAlert(kSampleAlertID, gNewCustomKeyFilterUPP);
				break;
			case 2:
				::AGANoteAlert(kSampleAlertID, gNewCustomKeyFilterUPP);
				break;
			case 3:
				::AGACautionAlert(kSampleAlertID, gNewCustomKeyFilterUPP);
				break;
			case 4:
				::AGAStopAlert(kSampleAlertID, gNewCustomKeyFilterUPP);
				break;
			case 5:
				this->DoAGAModalDialog();
				break;
			}
		}
	else
		{
		/*
		CommandID	aCommandID =
			CCommandManager::fgCommandManager->GetCommandFromMenuInfo(menuID, itemNo);

		if (aCommandID != kNoCmd)
			{
			CMessage	*aMessage = new CMessage(kFrameworkMessageKind, aCommandID, aCommandID, NULL);
		
			fgTarget->ReceiveMessage(aMessage, this);
			}
		*/
		}
	}

/*
CWindow *Application::MacOSWindowPtrToWindow(WindowPtr aWindowPtr)
	{
	CWindow		*result = NULL;
	ArrayIndex	numWindows = fWindowList->GetSize();
	
	for (ArrayIndex index = 0; index < numWindows; index++)
		{
		CWindow	*aWindow = (CWindow *) fWindowList->At(index);
		if (aWindowPtr == aWindow->GetMacOSWindowPtr())
			{
			result = aWindow;
			break;
			}
		}
	
	return result;
	}
*/

Boolean Application::IsFrontMacOSProcess()
	{
	ProcessSerialNumber	aPSN;
	ProcessSerialNumber	applicationPSN;
	Boolean				result = TRUE;	// assume we're in front if things go haywire

	if ((fProcessNumber.highLongOfPSN != kNoProcess) &&
		(fProcessNumber.lowLongOfPSN != kNoProcess))
		if (::GetFrontProcess(&aPSN) == noErr)
			{
			applicationPSN = fProcessNumber;
			if (::SameProcess(&aPSN, &applicationPSN, &result) != noErr)
				result = FALSE;
			}

	return result;
	}

OSErr Application::HandleAEOpenApplication(AppleEvent */*anAppleEvent*/, AppleEvent */*reply*/, SInt32 /*refCon*/)
	{
	//this->CreateNewDocument(kNewCmd);

	return noErr;
	}

OSErr Application::HandleAEOpenDocuments(AppleEvent */*anAppleEvent*/, AppleEvent */*reply*/, SInt32 /*refCon*/)
	{
	return noErr;
	}

OSErr Application::HandleAEPrintDocuments(AppleEvent */*anAppleEvent*/, AppleEvent */*reply*/, SInt32 /*refCon*/)
	{
	return noErr;
	}

OSErr Application::HandleAEQuitApplication(AppleEvent */*anAppleEvent*/, AppleEvent */*reply*/, SInt32 /*refCon*/)
	{
	//
	// We get here if we or someone else sends us a 'QUIT' Apple Event.
	// We send ourself this event if the MarkApplicationDone function is called.
	//

	fDone = TRUE;
	return noErr;
	}

pascal OSErr Application::fgAEOpenApplicationHandler(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon)
	{
	return Application::fgApplication->HandleAEOpenApplication(anAppleEvent, reply, refCon);
	}

pascal OSErr Application::fgAEOpenDocumentsHandler(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon)
	{
	return Application::fgApplication->HandleAEOpenDocuments(anAppleEvent, reply, refCon);
	}

pascal OSErr Application::fgAEPrintDocumentsHandler(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon)
	{
	return Application::fgApplication->HandleAEPrintDocuments(anAppleEvent, reply, refCon);
	}

pascal OSErr Application::fgAEQuitApplicationHandler(AppleEvent *anAppleEvent, AppleEvent *reply, SInt32 refCon)
	{
	return Application::fgApplication->HandleAEQuitApplication(anAppleEvent, reply, refCon);
	}

// Sample dialog box item numbers
#define kIconItemNo			3
#define kStaticItemNo		4
#define kEdit1ItemNo		5
#define kEdit2ItemNo		6
#define kEdit3ItemNo		7
#define kRadio1ItemNo		8
#define kRadio2ItemNo		9
#define kRadio3ItemNo		10
#define kRadioGroupItemNo	11
#define kCheckBoxItemNo		12
#define kOutlineUserItemNo	13
#define kPopupUserItemNo	14

// Main window dialog item numbers
#define kStdAlertItemNo				1
#define kStdNoteAlertItemNo			2
#define kStdCautionAlertItemNo		3
#define kStdStopAlertItemNo			4
#define kStdModalDialogItemNo		5
#define kMainWindowGroupBox1ItemNo	6
#define kAGAAlertItemNo				7
#define kAGANoteAlertItemNo			8
#define kAGACautionAlertItemNo		9
#define kAGAStopAlertItemNo			10
#define kAGAModalDialogItemNo		11
#define kMainWindowGroupBox2ItemNo	12
#define kAGAOnCheckBoxItemNo		13
#define kStdDialogRadioItemNo		14
#define kAGADialogRadioItemNo		15
#define kMainWindowGroupBox3ItemNo	16
#define kBigDialogButtonItemNo		17

static ControlHandle GetDialogControlHandle(DialogPtr aDialog, SInt16 itemNo)
	{
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;

	::GetDialogItem(aDialog, itemNo, &itemType, &itemHandle, &itemRect);
	
	return (ControlHandle) itemHandle;
	}

static pascal void GroupBoxUserItem(WindowPtr theWindow, SInt16 itemNo)
	{
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;
	FontInfo	fontInfo;
	Str255		title;
	
	// This is a hacky way to use one group box user item for different titles.
	// It only works because our group boxes all use different item numbers.
	if (itemNo == kRadioGroupItemNo)
		::GetIndString(title, kMyStringsID, kSampleDialogGroupBoxTitle);
	else if (itemNo == kMainWindowGroupBox1ItemNo)
		::GetIndString(title, kMyStringsID, kMainWindowGroupBox1Title);
	else if (itemNo == kMainWindowGroupBox2ItemNo)
		::GetIndString(title, kMyStringsID, kMainWindowGroupBox2Title);
	else
		::GetIndString(title, kMyStringsID, kMainWindowGroupBox3Title);

	::GetDialogItem(theWindow, itemNo, &itemType, &itemHandle, &itemRect);
	
	::RGBForeColor(&gAGARamp[rB]);
	
	::GetFontInfo(&fontInfo);
	
	itemRect.top += fontInfo.ascent;
	::MoveTo(itemRect.left + 12, itemRect.top);
	::LineTo(itemRect.left, itemRect.top);
	::LineTo(itemRect.left, itemRect.bottom);
	::LineTo(itemRect.right, itemRect.bottom);
	::LineTo(itemRect.right, itemRect.top);
	::LineTo(itemRect.left + 18 + ::StringWidth(title), itemRect.top);
	
	MoveTo(itemRect.left + 15, itemRect.top + 1);
	::DrawString(title);
	}

static UserItemUPP gGroupBoxUserItemUPP = NewUserItemProc(GroupBoxUserItem);

static pascal void DefaultOutlineUserItem(WindowPtr theWindow, SInt16 itemNo)
	{
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;

	::GetDialogItem(theWindow, itemNo, &itemType, &itemHandle, &itemRect);
	
	::PenNormal();
	
	::PenSize(3, 3);
	::FrameRoundRect(&itemRect, 16, 16);
	
	::PenNormal();
	}

static UserItemUPP gDefaultOutlineUserItemUPP = NewUserItemProc(DefaultOutlineUserItem);

// This is used below to demonstrate that when using the call AGAObject::InstallAGAUserItem,
// the object bounds are *usually* irrelevant because InstallAGAUserItem sets its bounds to those of
// the userItem. The only exception are controls that use the constructed bounds to decide
// how the control works: for example, sliders use it to determine whether they're horizontal or
// vertical.
static Rect gDummyRect;

//
// These routines show the changes you need to make to convert your old
// dialogs to AGA dialogs. Read the comments in DoAGAModalDialog for details.
//

void Application::DoStdModalDialog()
	{
	DialogPtr	theDialog;
	SInt16		itemHit = -1;
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;
	
	if ((theDialog = ::GetNewDialog(kSampleDialogID, NULL, (WindowPtr) -1)) == NULL)
		return;
	
	::SetDialogDefaultItem(theDialog, ok);
	::SetDialogCancelItem(theDialog, cancel);

	::SelectDialogItemText(theDialog, kEdit1ItemNo, 0, 32767);
	::SetControlValue(GetDialogControlHandle(theDialog, kRadio1ItemNo), 1);

	::GetDialogItem(theDialog, kRadioGroupItemNo, &itemType, &itemHandle, &itemRect);
	::SetDialogItem(theDialog, kRadioGroupItemNo, itemType, (Handle) gGroupBoxUserItemUPP, &itemRect);

	::GetDialogItem(theDialog, kOutlineUserItemNo, &itemType, &itemHandle, &itemRect);
	::SetDialogItem(theDialog, kOutlineUserItemNo, itemType, (Handle) gDefaultOutlineUserItemUPP, &itemRect);

	::ShowWindow(theDialog);
	
	while ((itemHit != ok) && (itemHit != cancel))
		{
		::ModalDialog(NULL, &itemHit);
		
		switch (itemHit)
			{
			case kRadio1ItemNo:
			case kRadio2ItemNo:
			case kRadio3ItemNo:
				::SetControlValue(GetDialogControlHandle(theDialog, kRadio1ItemNo), (itemHit == kRadio1ItemNo) ? 1 : 0);
				::SetControlValue(GetDialogControlHandle(theDialog, kRadio2ItemNo), (itemHit == kRadio2ItemNo) ? 1 : 0);
				::SetControlValue(GetDialogControlHandle(theDialog, kRadio3ItemNo), (itemHit == kRadio3ItemNo) ? 1 : 0);
				break;

			case kCheckBoxItemNo:
				::SetControlValue(GetDialogControlHandle(theDialog, kCheckBoxItemNo), 1 - ::GetControlValue(GetDialogControlHandle(theDialog, kCheckBoxItemNo)));
				break;
			}
		}

	::DisposeDialog(theDialog);
	}

//
// Here are the notes on how this routine is different from the one above, and why.
// These numbered items are marked as comments in the code like this: // ! n !
//
// 1.
// You must use an AGADialog object. It can either be on the stack or on the heap.
// You will be calling its methods instead of calling a lot of Dialog Manager and
// Control Manager routines directly.
//
// 2.
// Instead of calling GetNewDialog() directly, you must call AGADialog::GetNewDialog().
// It adds userItems to the item list, each associated with an AGAObject.
//
// 3.
// Instead of calling SetDialogDefaultItem() directly, you must call AGADialog::SetDialogDefaultItem().
// It gives the AGA button the outline attribute, and sets up to handle enter and return.
//
// 4.
// Instead of calling SetDialogCancelItem() directly, you must call AGADialog::SetDialogCancelItem().
// It sets up to handle escape and command-period.
//
// 5.
// Instead of calling SetControlValue() directly, you must call AGADialog::SetControlValue().
// It set the value of both the original control and the userItem AGA object.
//
// 6.
// If you are installing a user item proc of your own, you can still use SetDialogItem().
// However, in this case we are using the AGAGroupBox class to replace the black-and-white
// roll-your-own group box user item that we used to have write ourself (and did up above).
// To do this, use AGADialog::InstallAGAUserItem(). This associates the AGA object with the
// extra userItem object in the item list, just like the replacement objects.
// Note that the bounds passed to the AGAObject constructor are irrelevant, because
// AGADialog::InstallAGAUserItem sets the object's bounds to those of the userItem.
//
// 7.
// It is not strictly necessary, but for consistency there is a method of AGADialog that
// you can use instead of SetDialogItem. Here we are installing the default button outline
// userItemProc. We could call ::SetDialogItem instead. But see the next note.
//
// 8.
// If you have a default button outline userItem proc, you should continue to install it,
// as we have done in the previous note. But you should then call AGADialog::MarkDefaultOutlineUserItem
// so that if and only if the dialog is using AGAPushButton to replace the standard
// pushbutton control, the AGADialog can remove your userItem proc, since the AGAPushButton
// draws its own AGA outline. The alternative would be for you to check the setting of
// the global AGA controls switch and decide whether to install your proc. This method
// lets you just call it and not worry about whether AGA is turned on or not.
//
// 9.
// Instead of calling ModalDialog() directly, you must call AGADialog::ModalDialog().
// This lets the AGADialog insert its own filter proc alongside yours, and adjust
// the itemHit to account for the custom userItems.
//
// 10.
// Instead of calling DisposeDialog() directly, you must call AGADialog::DisposeDialog().
// This lets the AGADialog clean up all of the custom AGAObjects.
//

void Application::DoAGAModalDialog()
	{
	AGADialog	agaDialog;	// ! 1 !
	DialogPtr	theDialog;
	SInt16		itemHit = -1;
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;
	
	if ((theDialog = agaDialog.GetNewDialog(kSampleDialogID, NULL, (WindowPtr) -1)) == NULL)	// ! 2 !
		return;

	agaDialog.SetDialogDefaultItem(ok);		// ! 3 !
	agaDialog.SetDialogCancelItem(cancel);	// ! 4 !

	::SelectDialogItemText(theDialog, kEdit1ItemNo, 0, 32767);
	agaDialog.SetControlValue(GetDialogControlHandle(theDialog, kRadio1ItemNo), 1);	// ! 5 !
	
//	::GetDialogItem(theDialog, kRadioGroupItemNo, &itemType, &itemHandle, &itemRect);	bounds will be irrelevant in next statement
	agaDialog.InstallAGAUserItem(new AGAGroupBox(&gDummyRect, gAGAStdSystemStyle, AGAGroupBox::kPrimaryGroupBox, kMyStringsID, kSampleDialogGroupBoxTitle), kRadioGroupItemNo, AGADialog::kUserItemDoesNotTrackMouse);	// ! 6 !

	::GetDialogItem(theDialog, kOutlineUserItemNo, &itemType, &itemHandle, &itemRect);
	agaDialog.SetDialogItem(kOutlineUserItemNo, itemType, (Handle) gDefaultOutlineUserItemUPP, &itemRect);	// ! 7 !
	agaDialog.MarkDefaultOutlineUserItem(kOutlineUserItemNo);	// ! 8 !

// New 2.0.1
agaDialog.HintPopupControl(kPopupUserItemNo, 500);

	::ShowWindow(theDialog);
	
	while ((itemHit != ok) && (itemHit != cancel))
		{
		agaDialog.ModalDialog(NULL, &itemHit);	// ! 9 !
		
		switch (itemHit)
			{
			case kRadio1ItemNo:
			case kRadio2ItemNo:
			case kRadio3ItemNo:
				agaDialog.SetControlValue(GetDialogControlHandle(theDialog, kRadio1ItemNo), (itemHit == kRadio1ItemNo) ? 1 : 0);	// ! 5 !
				agaDialog.SetControlValue(GetDialogControlHandle(theDialog, kRadio2ItemNo), (itemHit == kRadio2ItemNo) ? 1 : 0);	// ! 5 !
				agaDialog.SetControlValue(GetDialogControlHandle(theDialog, kRadio3ItemNo), (itemHit == kRadio3ItemNo) ? 1 : 0);	// ! 5 !
				break;

			case kCheckBoxItemNo:
				agaDialog.SetControlValue(GetDialogControlHandle(theDialog, kCheckBoxItemNo), 1 - ::GetControlValue(GetDialogControlHandle(theDialog, kCheckBoxItemNo)));	// ! 5 !
				break;
			}
		}

	agaDialog.DisposeDialog();	// ! 10 !
	}

#define kAboutTitleItemNo		2
#define kAboutCaptionAItemNo	3
#define kAboutCaptionBItemNo	4
#define kAboutSeparatorItemNo	5
#define kAboutCaptionCItemNo	6
#define kAboutCaptionDItemNo	7
#define kAboutOutlineItemNo		8

void Application::DoAboutBox()
	{
	AGADialog	agaDialog;
	DialogPtr	theDialog;
	SInt16		itemHit = -1;
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;
	AGAObject*	anAGAObject;
	
	if ((theDialog = agaDialog.GetNewDialog(kAboutBoxID, NULL, (WindowPtr) -1)) == NULL)
		return;

	agaDialog.SetDialogDefaultItem(ok);
	agaDialog.SetDialogCancelItem(ok);
	
	agaDialog.InstallAGAUserItem(new AGASeparator(&gDummyRect), kAboutSeparatorItemNo, AGADialog::kUserItemDoesNotTrackMouse);

	::GetDialogItem(theDialog, kAboutOutlineItemNo, &itemType, &itemHandle, &itemRect);
	agaDialog.SetDialogItem(kAboutOutlineItemNo, itemType, (Handle) gDefaultOutlineUserItemUPP, &itemRect);
	agaDialog.MarkDefaultOutlineUserItem(kAboutOutlineItemNo);

	if ((anAGAObject = agaDialog.GetAGAObjectPtr(kAboutTitleItemNo)) != NULL)
		{
		((AGAStaticText*) anAGAObject)->SetTextStyle(AGATextStyle("\pPalatino", (short) 24, (UInt8) (bold + underline)), AGAObject::kDontRedraw);
		((AGAStaticText*) anAGAObject)->SetJustification(teJustCenter, AGAObject::kDontRedraw);
		}

	if ((anAGAObject = agaDialog.GetAGAObjectPtr(kAboutCaptionAItemNo)) != NULL)
		{
		((AGAStaticText*) anAGAObject)->SetTextStyle(gAGAExtraSmallStyle, AGAObject::kDontRedraw);
		((AGAStaticText*) anAGAObject)->SetJustification(teJustCenter, AGAObject::kDontRedraw);
		}

	if ((anAGAObject = agaDialog.GetAGAObjectPtr(kAboutCaptionBItemNo)) != NULL)
		{
		((AGAStaticText*) anAGAObject)->SetTextStyle(gAGAExtraSmallStyle, AGAObject::kDontRedraw);
		((AGAStaticText*) anAGAObject)->SetJustification(teJustCenter, AGAObject::kDontRedraw);
		}

	if ((anAGAObject = agaDialog.GetAGAObjectPtr(kAboutCaptionCItemNo)) != NULL)
		{
		((AGAStaticText*) anAGAObject)->SetTextStyle(gAGAExtraSmallStyle, AGAObject::kDontRedraw);
		((AGAStaticText*) anAGAObject)->SetJustification(teJustCenter, AGAObject::kDontRedraw);
		}

	if ((anAGAObject = agaDialog.GetAGAObjectPtr(kAboutCaptionDItemNo)) != NULL)
		{
		((AGAStaticText*) anAGAObject)->SetTextStyle(gAGAExtraSmallStyle, AGAObject::kDontRedraw);
		((AGAStaticText*) anAGAObject)->SetJustification(teJustCenter, AGAObject::kDontRedraw);
		}

	::ShowWindow(theDialog);
	
	while ((itemHit != ok) && (itemHit != cancel))
		agaDialog.ModalDialog(NULL, &itemHit);

	agaDialog.DisposeDialog();
	}

void Application::DestroyMainWindow()
	{
	if (mMainWindowDialogPtr != NULL)
		{
		if (mMainWindowIsAGA)
			mMainWindowAGADialog.DisposeDialog();
		else
			::DisposeDialog(mMainWindowDialogPtr);
		}
	}

void Application::CreateMainWindow()
	{
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;

	if (mMainWindowIsAGA)
		{
		mMainWindowDialogPtr = mMainWindowAGADialog.GetNewDialog(kMainWindowID, NULL, (WindowPtr) -1);
		
		mMainWindowAGADialog.InstallAGAUserItem(new AGAGroupBox(&gDummyRect, gAGAStdSystemStyle, AGAGroupBox::kPrimaryGroupBox, kMyStringsID, kMainWindowGroupBox1Title), kMainWindowGroupBox1ItemNo, AGADialog::kUserItemDoesNotTrackMouse);
		
		mMainWindowAGADialog.InstallAGAUserItem(new AGAGroupBox(&gDummyRect, gAGAStdSystemStyle, AGAGroupBox::kPrimaryGroupBox, kMyStringsID, kMainWindowGroupBox2Title), kMainWindowGroupBox2ItemNo, AGADialog::kUserItemDoesNotTrackMouse);

		mMainWindowAGADialog.InstallAGAUserItem(new AGAGroupBox(&gDummyRect, gAGAStdSystemStyle, AGAGroupBox::kPrimaryGroupBox, kMyStringsID, kMainWindowGroupBox3Title), kMainWindowGroupBox3ItemNo, AGADialog::kUserItemDoesNotTrackMouse);

		::GetDialogItem(mMainWindowDialogPtr, kAGAOnCheckBoxItemNo, &itemType, &itemHandle, &itemRect);
		mMainWindowAGADialog.SetControlValue((ControlHandle) itemHandle, gAGAStdControls ? 1 : 0);

		::GetDialogItem(mMainWindowDialogPtr, kStdDialogRadioItemNo, &itemType, &itemHandle, &itemRect);
		mMainWindowAGADialog.SetControlValue((ControlHandle) itemHandle, mMainWindowIsAGA ? 0 : 1);

		::GetDialogItem(mMainWindowDialogPtr, kAGADialogRadioItemNo, &itemType, &itemHandle, &itemRect);
		mMainWindowAGADialog.SetControlValue((ControlHandle) itemHandle, mMainWindowIsAGA ? 1 : 0);
		}
	else
		{
		mMainWindowDialogPtr = ::GetNewDialog(kMainWindowID, NULL, (WindowPtr) -1);

		::GetDialogItem(mMainWindowDialogPtr, kMainWindowGroupBox1ItemNo, &itemType, &itemHandle, &itemRect);
		::SetDialogItem(mMainWindowDialogPtr, kMainWindowGroupBox1ItemNo, itemType, (Handle) gGroupBoxUserItemUPP, &itemRect);

		::GetDialogItem(mMainWindowDialogPtr, kMainWindowGroupBox2ItemNo, &itemType, &itemHandle, &itemRect);
		::SetDialogItem(mMainWindowDialogPtr, kMainWindowGroupBox2ItemNo, itemType, (Handle) gGroupBoxUserItemUPP, &itemRect);

		::GetDialogItem(mMainWindowDialogPtr, kMainWindowGroupBox3ItemNo, &itemType, &itemHandle, &itemRect);
		::SetDialogItem(mMainWindowDialogPtr, kMainWindowGroupBox3ItemNo, itemType, (Handle) gGroupBoxUserItemUPP, &itemRect);

		::GetDialogItem(mMainWindowDialogPtr, kAGAOnCheckBoxItemNo, &itemType, &itemHandle, &itemRect);
		::SetControlValue((ControlHandle) itemHandle, gAGAStdControls ? 1 : 0);

		::GetDialogItem(mMainWindowDialogPtr, kStdDialogRadioItemNo, &itemType, &itemHandle, &itemRect);
		::SetControlValue((ControlHandle) itemHandle, mMainWindowIsAGA ? 0 : 1);

		::GetDialogItem(mMainWindowDialogPtr, kAGADialogRadioItemNo, &itemType, &itemHandle, &itemRect);
		::SetControlValue((ControlHandle) itemHandle, mMainWindowIsAGA ? 1 : 0);
		}

	::ShowWindow(mMainWindowDialogPtr);
	}

void Application::DoMainWindowButtonHit(SInt16 itemHit)
	{
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;

	switch (itemHit)
		{
		case kStdAlertItemNo:
			::Alert(kSampleAlertID, gOldCustomKeyFilterUPP);
			break;
		case kStdNoteAlertItemNo:
			::NoteAlert(kSampleAlertID, gOldCustomKeyFilterUPP);
			break;
		case kStdCautionAlertItemNo:
			::CautionAlert(kSampleAlertID, gOldCustomKeyFilterUPP);
			break;
		case kStdStopAlertItemNo:
			::StopAlert(kSampleAlertID, gOldCustomKeyFilterUPP);
			break;
		case kStdModalDialogItemNo:
			this->DoStdModalDialog();
			break;

		case kAGAAlertItemNo:
			::AGAAlert(kSampleAlertID, gNewCustomKeyFilterUPP);
			break;
		case kAGANoteAlertItemNo:
			::AGANoteAlert(kSampleAlertID, gNewCustomKeyFilterUPP);
			break;
		case kAGACautionAlertItemNo:
			::AGACautionAlert(kSampleAlertID, gNewCustomKeyFilterUPP);
			break;
		case kAGAStopAlertItemNo:
			::AGAStopAlert(kSampleAlertID, gNewCustomKeyFilterUPP);
			break;
		case kAGAModalDialogItemNo:
			this->DoAGAModalDialog();
			break;

		case kAGAOnCheckBoxItemNo:
			this->SwapAGAOnFlag();
			break;

		case kBigDialogButtonItemNo:
			this->DoBigDialog();
			break;

		case kStdDialogRadioItemNo:
		case kAGADialogRadioItemNo:
			// If this button is currently off, it means we're toggling
			// the setting

			::GetDialogItem(mMainWindowDialogPtr, itemHit, &itemType, &itemHandle, &itemRect);
			
			if (::GetControlValue((ControlHandle) itemHandle) == 0)
				this->SwapAGADialogFlag();

			break;
		}
	}

void Application::SwapAGAOnFlag()
	{
	this->DestroyMainWindow();
	gAGAStdControls = ! gAGAStdControls;
	gAGAExtraControls = ! gAGAExtraControls;
	this->CreateMainWindow();
	}

void Application::SwapAGADialogFlag()
	{
	this->DestroyMainWindow();
	mMainWindowIsAGA = ! mMainWindowIsAGA;
	this->CreateMainWindow();
	}

#define kBigDialogDefaultOutlineUserItem	2
#define kBigDialogIconP1					4
#define kBigDialogIconP2					5
#define kBigDialogIconP3					6
#define kBigDialogIconP4					7
#define kBigDialogIconP5					8
#define kBigDialogIconC1					10
#define kBigDialogIconC2					11
#define kBigDialogIconC3					12
#define kBigDialogIconC4					13
#define kBigDialogIconC5					14
#define kBigDialogIconR1					16
#define kBigDialogIconR2					17
#define kBigDialogIconR3					18
#define kBigDialogIconR4					19
#define kBigDialogIconR5					20
#define kBigDialogSeparator1				21
#define kBigDialogSlider1					22
#define kBigDialogSlider2					23
#define kBigDialogSeparator2				24
#define kBigDialogLittleArrows				25
#define kBigDialogEditText					26
#define kBigDialogDisclosureTriangle		27
#define kBigDialogDisclosureCheckBox		28
#define kBigDialogProgress1					30
#define kBigDialogProgress2					31
#define kBigDialogTabPanel					32

AGAProgressIndicator*	gDeterminateIndicator;
AGAProgressIndicator*	gIndeterminateIndicator;

static pascal Boolean BigDialogFilter(DialogPtr theDialog, EventRecord* /*theEvent*/, SInt16* /*itemHit*/)
	{
	::SetPort(theDialog);

	gDeterminateIndicator->Increment(1, AGAObject::kRedraw);
	gIndeterminateIndicator->QuantizedAnimate();
	
	if (gDeterminateIndicator->GetValue() == 100)
		gDeterminateIndicator->SetValue(0, AGAObject::kRedraw);

	return false;
	}

static ModalFilterUPP gBigDialogFilterUPP = NewModalFilterProc(BigDialogFilter);

static AGADialog*	gTheBigDialog;

static void BigDialogLittleArrowsNotifier(AGALittleArrows* /*theAGAObject*/, SInt32 deltaValue, void* userData)
	{
	// The userData parameter is the item handle for the edit text item that
	// we supplied when installing this notifier.
	// For edit text items, we don't need to involve the AGADialog object in
	// setting the text, since it doesn't process or display the text.
	// This is different from controls.
	
	Handle	textHandle = (Handle) userData;
	Str255	theText;
	SInt32	theNumber;

	::GetDialogItemText(textHandle, theText);
	
	::StringToNum(theText, &theNumber);
	theNumber += deltaValue;
	::NumToString(theNumber, theText);

	::SetDialogItemText(textHandle, theText);
	
	gTheBigDialog->SelectDialogItemText(kBigDialogEditText, 0, 32767);
	}

void Application::DoBigDialog()
	{
	AGADialog	agaDialog;
	DialogPtr	theDialog;
	SInt16		itemHit = -1;
	SInt16		itemType;
	Handle		itemHandle;
	Rect		itemRect;
	
	gTheBigDialog = &agaDialog;
	
	if ((theDialog = agaDialog.GetNewDialog(kBigDialogID, NULL, (WindowPtr) -1)) == NULL)
		return;

	::SetPort(theDialog);

	agaDialog.SetDialogDefaultItem(ok);
	agaDialog.SetDialogCancelItem(ok);
	
	::GetDialogItem(theDialog, kBigDialogDefaultOutlineUserItem, &itemType, &itemHandle, &itemRect);
	agaDialog.SetDialogItem(kBigDialogDefaultOutlineUserItem, itemType, (Handle) gDefaultOutlineUserItemUPP, &itemRect);
	agaDialog.MarkDefaultOutlineUserItem(kBigDialogDefaultOutlineUserItem);
	
	agaDialog.InstallAGAUserItem(new AGAIconPushButton(&gDummyRect, -16482, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconP1, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconPushButton(&gDummyRect, -3999, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconP2, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconPushButton(&gDummyRect, -3996, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconP3, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconPushButton(&gDummyRect, -3989, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconP4, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconPushButton(&gDummyRect, -16494, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconP5, AGADialog::kUserItemTracksMouse);

	agaDialog.InstallAGAUserItem(new AGAIconCheckBox(&gDummyRect, AGAObject::kAutomaticState, -16482, -16482, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconC1, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconCheckBox(&gDummyRect, AGAObject::kAutomaticState, -3999, -3999, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconC2, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconCheckBox(&gDummyRect, AGAObject::kAutomaticState, -3996, -3996, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconC3, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconCheckBox(&gDummyRect, AGAObject::kAutomaticState, -3989, -3989, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconC4, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconCheckBox(&gDummyRect, AGAObject::kAutomaticState, -16494, -16494, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconC5, AGADialog::kUserItemTracksMouse);

	agaDialog.InstallAGAUserItem(new AGAIconRadioButton(&gDummyRect, (UInt32) theDialog, 0, AGAObject::kAutomaticState, -16482, -16482, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconR1, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconRadioButton(&gDummyRect, (UInt32) theDialog, 0, AGAObject::kAutomaticState, -3999, -3999, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconR2, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconRadioButton(&gDummyRect, (UInt32) theDialog, 0, AGAObject::kAutomaticState, -3996, -3996, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconR3, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconRadioButton(&gDummyRect, (UInt32) theDialog, 0, AGAObject::kAutomaticState, -3989, -3989, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconR4, AGADialog::kUserItemTracksMouse);
	agaDialog.InstallAGAUserItem(new AGAIconRadioButton(&gDummyRect, (UInt32) theDialog, 0, AGAObject::kAutomaticState, -16494, -16494, kAutoFrame, MIconButtonObject::kAutoIconFamily), kBigDialogIconR5, AGADialog::kUserItemTracksMouse);

	agaDialog.InstallAGAUserItem(new AGASeparator(&gDummyRect), kBigDialogSeparator1, AGADialog::kUserItemDoesNotTrackMouse);
	agaDialog.InstallAGAUserItem(new AGASeparator(&gDummyRect), kBigDialogSeparator2, AGADialog::kUserItemDoesNotTrackMouse);

	::GetDialogItem(theDialog, kBigDialogSlider1, &itemType, &itemHandle, &itemRect);
	agaDialog.InstallAGAUserItem(new AGASlider(&itemRect, 0, 0, 0, gAGAStdSmallStyle, kBigDialogSlider1Labels), kBigDialogSlider1, AGADialog::kUserItemTracksMouse);
	
	::GetDialogItem(theDialog, kBigDialogSlider2, &itemType, &itemHandle, &itemRect);
	AGASlider*	aSlider = new AGASlider(&itemRect, 0, 0, 0, gAGAStdSmallStyle, kBigDialogSlider2Labels);
	aSlider->SetJustification(teFlushRight);
	agaDialog.InstallAGAUserItem(aSlider, kBigDialogSlider2, AGADialog::kUserItemTracksMouse);

	::GetDialogItem(theDialog, kBigDialogEditText, &itemType, &itemHandle, &itemRect);
	Handle	bigDialogTextHandle = itemHandle;

	AGALittleArrows*	littleArrows = new AGALittleArrows(&gDummyRect);
	agaDialog.InstallAGAUserItem(littleArrows, kBigDialogLittleArrows, AGADialog::kUserItemTracksMouse);
	littleArrows->InstallNotificationRoutine(BigDialogLittleArrowsNotifier, bigDialogTextHandle);

	AGADisclosureTriangle*	disclosureTriangle = new AGADisclosureTriangle(&gDummyRect, AGAObject::kAutomaticState);	
	agaDialog.InstallAGAUserItem(disclosureTriangle, kBigDialogDisclosureTriangle, AGADialog::kUserItemTracksMouse);

	::GetDialogItem(theDialog, kBigDialogDisclosureCheckBox, &itemType, &itemHandle, &itemRect);
	ControlHandle	disclosureCheckBox = (ControlHandle) itemHandle;

	gDeterminateIndicator = new AGAProgressIndicator(&gDummyRect, 0, 100);
	agaDialog.InstallAGAUserItem(gDeterminateIndicator, kBigDialogProgress1, AGADialog::kUserItemDoesNotTrackMouse);

	gIndeterminateIndicator = new AGAProgressIndicator(&gDummyRect, 0, 0);
	agaDialog.InstallAGAUserItem(gIndeterminateIndicator, kBigDialogProgress2, AGADialog::kUserItemDoesNotTrackMouse);

	AGATabPanel*	tabPanel = new AGATabPanel(&gDummyRect, 0, gAGAStdBoldSmallStyle, kBigDialogTabLabels);
	agaDialog.InstallAGAUserItem(tabPanel, kBigDialogTabPanel, AGADialog::kUserItemTracksMouse);

	::ShowWindow(theDialog);
	
	while ((itemHit != ok) && (itemHit != cancel))
		{
		::SetPort(theDialog);

		agaDialog.ModalDialog(gBigDialogFilterUPP, &itemHit);
		
		::SetPort(theDialog);

		switch (itemHit)
			{
			case kBigDialogTabPanel:
				tabPanel->SetCurrentTab(tabPanel->GetLastTabClicked(), AGAObject::kRedraw);
				break;
			case kBigDialogDisclosureTriangle:
				agaDialog.SetControlValue(disclosureCheckBox, disclosureTriangle->GetState() ? 1 : 0);
				break;
			case kBigDialogDisclosureCheckBox:
				agaDialog.SetControlValue(disclosureCheckBox, 1 - ::GetControlValue(disclosureCheckBox));
				disclosureTriangle->SetStateAnimate(::GetControlValue(disclosureCheckBox) != 0);
				break;
			}
		}

	agaDialog.DisposeDialog();
	}

