//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// See header file "GCCoreMA.h" for usage.
//

#include "GCCoreMA.h"

void RegisterGCCoreMA()
	{
	//
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.
	//

	MA_REGISTER_CLASS(AGABackgroundAdornerMA);
	MA_REGISTER_CLASS(AGAModalBackgroundAdornerMA);
	MA_REGISTER_CLASS(AGAFlatBackgroundBehaviorMA);
	MA_REGISTER_CLASS(AGAWhiteBackgroundAdornerMA);
	MA_REGISTER_CLASS(AGANotchedWhiteBackgroundAdornerMA);
	MA_REGISTER_CLASS(AGABorderFrameAdornerMA);
	MA_REGISTER_CLASS(AGANotchedBorderFrameAdornerMA);
	MA_REGISTER_CLASS(AGATargetBorderViewMA);
	MA_REGISTER_CLASS(AGANotchedTargetBorderViewMA);
	MA_REGISTER_CLASS(AGATargetBorderFrameViewMA);
	MA_REGISTER_CLASS(AGANotchedTargetBorderFrameViewMA);
	}

void SetAGAObjectFrame(TView* itsView, AGAObject* theAGAObject, const VRect& /*newViewFrame*/, Boolean /*redraw*/)
	{
	if (theAGAObject != NULL)
		{
		VRect	viewRect(0, 0, itsView->fSize.h, itsView->fSize.v);
		CRect	qdRect;
		
		// TControl views have control insets that must be honored, rather just grabbing the total size.
		if (MA_MEMBER(itsView, TControl))
			((TControl*) itsView)->ControlArea(viewRect);

		itsView->ViewToQDRect(viewRect, qdRect);

		theAGAObject->SetObjectBounds(qdRect, AGAObject::kDontRedraw);
		}
	}

//
// AGABackgroundAdornerMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS_M1(AGABackgroundAdornerMA, Inherited);

AGABackgroundAdornerMA::AGABackgroundAdornerMA()
	{
	mBackgroundKind = kRaisedModelessBackground;
	}

void AGABackgroundAdornerMA::DoHighlightSelection(TView* itsView, const VRect& area, HLState /*fromHL*/, HLState toHL)
	{
	// Draw in the specified active/inactive state.

	this->DrawBackground(itsView, area, false, (toHL == hlOn));
	}

void AGABackgroundAdornerMA::Draw(TView* itsView, const VRect& area)
	{
	// Draw in active state.

	this->DrawBackground(itsView, area, true, itsView->IsActive());
	}

void AGABackgroundAdornerMA::ViewChangedFrame(TView* itsView, const VRect& oldFrame, const VRect& newFrame, Boolean	/*invalidate*/)
	{
	// Make sure we invalidate the edges that will need to be
	// redrawn with the background edge pixel color.
	// Note that oldFrame & newFrame are in global coordinates.

	VRect	rectToInvalidate;
	VPoint	sizeDelta(newFrame.GetLength(hSel) - oldFrame.GetLength(hSel),
					newFrame.GetLength(vSel) - oldFrame.GetLength(vSel));
	
	if (sizeDelta.h != 0)
		{
		if (sizeDelta.h > 0)
			rectToInvalidate = oldFrame;
		else	// sizeDelta.h < 0
			rectToInvalidate = newFrame;

		rectToInvalidate.left = rectToInvalidate.right - 1;
		itsView->SuperToLocalVRect(rectToInvalidate);
		itsView->InvalidateVRect(rectToInvalidate);
		}
	
	if (sizeDelta.v != 0)
		{
		if (sizeDelta.v > 0)
			rectToInvalidate = oldFrame;
		else	// sizeDelta.v < 0
			rectToInvalidate = newFrame;

		rectToInvalidate.top = rectToInvalidate.bottom - 1;
		itsView->SuperToLocalVRect(rectToInvalidate);
		itsView->InvalidateVRect(rectToInvalidate);
		}
	
	if (this->HasGrowBox(itsView))
		{
		rectToInvalidate = oldFrame;
		rectToInvalidate.left = rectToInvalidate.right - 16;
		rectToInvalidate.top = rectToInvalidate.bottom - 16;
		itsView->SuperToLocalVRect(rectToInvalidate);
		itsView->InvalidateVRect(rectToInvalidate);

		rectToInvalidate = newFrame;
		rectToInvalidate.left = rectToInvalidate.right - 16;
		rectToInvalidate.top = rectToInvalidate.bottom - 16;
		itsView->SuperToLocalVRect(rectToInvalidate);
		itsView->InvalidateVRect(rectToInvalidate);
		}
	}

void AGABackgroundAdornerMA::AddedToView(TView* itsView)
	{
	// Install the background color if we're in a window.

	TWindow*	itsWindow = itsView->GetWindow();

	if (itsWindow != NULL)
		FailOSErr(AGAInstallGrayWCTB(itsWindow->fWMgrWindow));

	// Set the current bg color of the view's port so that any of our
	// AGA adapter views that get instantiated will pick it up.

	if (itsView->Focus())
		::RGBBackColor(&gAGARamp[r2]);
	}

void AGABackgroundAdornerMA::DrawBackground(TView* itsView, const VRect& /*area*/, Boolean fill, Boolean active)
	{
	CRect	qdArea;
	
	itsView->GetQDExtent(qdArea);
	
	AGABackgroundPaint(qdArea, fill, mBackgroundKind, active, this->HasGrowBox(itsView));
	}

Boolean AGABackgroundAdornerMA::HasGrowBox(TView* itsView)
	{
	if (MA_MEMBER(itsView, TWindow))
		return ((TWindow*) itsView)->fIsResizable;
	else
		return false;
	}


//
// AGAModalBackgroundAdornerMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGABackgroundAdornerMA

MA_DEFINE_CLASS_M1(AGAModalBackgroundAdornerMA, Inherited);

AGAModalBackgroundAdornerMA::AGAModalBackgroundAdornerMA()
	{
	mBackgroundKind = kRaisedModalBackground;
	}

//
// AGAFlatBackgroundBehaviorMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TBehavior

MA_DEFINE_CLASS_M1(AGAFlatBackgroundBehaviorMA, Inherited);

AGAFlatBackgroundBehaviorMA::AGAFlatBackgroundBehaviorMA()
	{
	}

AGAFlatBackgroundBehaviorMA::~AGAFlatBackgroundBehaviorMA()
	{
	}

void AGAFlatBackgroundBehaviorMA::DoPostCreate(TDocument* itsDocument)
	{
	// Install the background color if we're in a window.

	Inherited::DoPostCreate(itsDocument);
	
	TWindow*	itsWindow = ((TView*) fOwner)->GetWindow();

	if (itsWindow != NULL)
		{
		FailOSErr(AGAInstallGrayWCTB(itsWindow->fWMgrWindow));

		// Set the current bg color of the view's port so that any of our
		// AGA adapter views that get instantiated will pick it up.

		if (itsWindow->Focus())
			::RGBBackColor(&gAGARamp[r2]);
		}
	}

//
// AGAWhiteBackgroundAdornerMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS_M1(AGAWhiteBackgroundAdornerMA, Inherited);

void AGAWhiteBackgroundAdornerMA::Draw(TView* itsView, const VRect& /*area*/)
	{
	// Draw the background in white, inset to avoid the border
	// frame pixels.

	CRect	qdArea;
	
	itsView->GetQDExtent(qdArea);
	
	::RGBForeColor(&gAGARamp[rW]);
	::RGBBackColor(&gAGARamp[rW]);
	::InsetRect(qdArea, 2, 2);
	::PaintRect(qdArea);

	::RGBForeColor(&gAGARamp[rB]);
	}

//
// AGANotchedWhiteBackgroundAdornerMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS_M1(AGANotchedWhiteBackgroundAdornerMA, Inherited);

void AGANotchedWhiteBackgroundAdornerMA::Draw(TView* itsView, const VRect& /*area*/)
	{
	// Draw the background in white, inset to avoid the border
	// frame pixels.

	CRect	qdArea;
	
	itsView->GetQDExtent(qdArea);
	
	::RGBForeColor(&gAGARamp[rW]);
	::InsetRect(qdArea, 2, 2);

	qdArea.right -= 15;
	::PaintRect(qdArea);
	qdArea.right += 15;
	
	qdArea.bottom -= 15;
	::PaintRect(qdArea);

	::RGBForeColor(&gAGARamp[rB]);
	}

//
// AGABorderFrameAdornerMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS_M1(AGABorderFrameAdornerMA, Inherited);

void AGABorderFrameAdornerMA::Draw(TView* itsView, const VRect& /*area*/)
	{
	// Draw the frame with shading outside.
	
	CRect	qdArea;
	
	itsView->GetQDExtent(qdArea);
	
	AGADrawBorderFrame(qdArea, itsView->IsEnabled(), false /*isNotched*/, true /*drawFrame*/, &gAGARamp[r2]);
	
	::RGBForeColor(&gAGARamp[rB]);
	::RGBBackColor(&gAGARamp[rW]);
	}

//
// AGANotchedBorderFrameAdornerMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TAdorner

MA_DEFINE_CLASS_M1(AGANotchedBorderFrameAdornerMA, Inherited);

void AGANotchedBorderFrameAdornerMA::Draw(TView* itsView, const VRect& /*area*/)
	{
	// Draw the frame with shading outside.
	
	CRect	qdArea;
	
	itsView->GetQDExtent(qdArea);
	
	AGADrawBorderFrame(qdArea, itsView->IsEnabled(), true /*isNotched*/, true /*drawFrame*/, &gAGARamp[r2]);
	
	::RGBForeColor(&gAGARamp[rB]);
	::RGBBackColor(&gAGARamp[rW]);
	}

//
// AGATargetBorderViewMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TTargetBorderView

MA_DEFINE_CLASS_M1(AGATargetBorderViewMA, Inherited);

void AGATargetBorderViewMA::IAGATargetBorderViewMA(TDocument* itsDocument, TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, IDType itsTargetView)
	{
	this->ITargetBorderView(itsDocument, itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTargetView);
	}

void AGATargetBorderViewMA::DoPostCreate(TDocument* itsDocument)
	{
	// Save the background colors we will need to erase.

	Inherited::DoPostCreate(itsDocument);
	
	this->Focus();
	if (! DetermineAGABackgroundColors(&mEnabledBackgroundColor, &mDisabledBackgroundColor, kInstallOnlyIfCustom))
		{
		mEnabledBackgroundColor = gAGARamp[r2];
		mDisabledBackgroundColor = gAGARamp[r2];
		}
	}

void AGATargetBorderViewMA::Draw(const VRect& /*area*/)
	{
	// Set the color, always draw. TTargetBorderView leaves
	// background unerased in 1-bit.

	AGADrawingEnvironment	env;

	GDIterator	iter;
	Boolean		deep;
	Boolean		isTarget = this->ContainsTarget(gApplication->GetTarget());
	Boolean		enabled = this->IsEnabled();

	CTemporaryRegion borderRegion;
	
	while (iter.NextDevice(deep))
		{
		if (deep)
			{
			if (isTarget && enabled)
				::RGBForeColor(&gAGARamp[r8]);
			else if (enabled)
				::RGBForeColor(&mEnabledBackgroundColor);
			else
				::RGBForeColor(&mDisabledBackgroundColor);
			}
		else
			{
			if (isTarget && enabled)
				::RGBForeColor(&gAGARamp[rB]);
			else
				::RGBForeColor(&gAGARamp[rW]);
			}

		this->ComputeBorderRegion(borderRegion);
		PaintRgn(borderRegion);
		}
	}

void AGATargetBorderViewMA::ComputeBorderRegion(RgnHandle borderRegion)
	{
	// Return the border region to be painted or invalidated.

	CRect	qdArea;

	this->GetQDExtent(qdArea);

	AGAComputeTargetBorderRegion(borderRegion, qdArea, false /*isNotched*/);
	}

//
// AGANotchedTargetBorderViewMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGATargetBorderViewMA

MA_DEFINE_CLASS_M1(AGANotchedTargetBorderViewMA, Inherited);

void AGANotchedTargetBorderViewMA::IAGANotchedTargetBorderViewMA(TDocument* itsDocument, TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, IDType itsTargetView)
	{
	this->IAGATargetBorderViewMA(itsDocument, itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTargetView);
	}

void AGANotchedTargetBorderViewMA::ComputeBorderRegion(RgnHandle borderRegion)
	{
	// Return the border region to be painted or invalidated.

	CRect	qdArea;

	this->GetQDExtent(qdArea);

	AGAComputeTargetBorderRegion(borderRegion, qdArea, true /*isNotched*/);
	}

//
// AGATargetBorderFrameViewMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGATargetBorderViewMA

MA_DEFINE_CLASS_M1(AGATargetBorderFrameViewMA, Inherited);

void AGATargetBorderFrameViewMA::IAGATargetBorderFrameViewMA(TDocument* itsDocument, TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, IDType itsTargetView)
	{
	this->IAGATargetBorderViewMA(itsDocument, itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTargetView);
	}

void AGATargetBorderFrameViewMA::Draw(const VRect& area)
	{
	// Let the AGATargetBorderViewMA draw the target border,
	// then draw the "3D" sunken frame portion..

	Inherited::Draw(area);
	
	CRect	qdArea;

	this->GetQDExtent(qdArea);
	
	::InsetRect(qdArea, 3, 3);

	AGADrawBorderFrame(qdArea, this->IsEnabled(), false /*isNotched*/, true /*drawFrame*/, &gAGARamp[r2]);

	::RGBForeColor(&gAGARamp[rB]);
	::RGBBackColor(&gAGARamp[rW]);
	}

void AGATargetBorderFrameViewMA::Dim()
	{
	// Suppress patBic painting over our frame.
	}

void AGATargetBorderFrameViewMA::SetEnable(Boolean state)
	{
	// Redraw now; unlike TControl, TView does not automatically
	// redraw when you disable/dim it.
	
	Inherited::SetEnable(state);

	if (this->Focus() && this->IsVisible())
		{
		VRect	extent;
		
		this->GetExtent(extent);
		this->Draw(extent);
		}
	}

//
// AGANotchedTargetBorderFrameViewMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGATargetBorderViewMA

MA_DEFINE_CLASS_M1(AGANotchedTargetBorderFrameViewMA, Inherited);

void AGANotchedTargetBorderFrameViewMA::IAGANotchedTargetBorderFrameViewMA(TDocument* itsDocument, TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, IDType itsTargetView)
	{
	this->IAGATargetBorderViewMA(itsDocument, itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTargetView);
	}

void AGANotchedTargetBorderFrameViewMA::Draw(const VRect& area)
	{
	// Let the AGATargetBorderViewMA draw the target border,
	// then draw the "3D" sunken frame portion..

	Inherited::Draw(area);
	
	CRect	qdArea;

	this->GetQDExtent(qdArea);
	
	::InsetRect(qdArea, 3, 3);

	AGADrawBorderFrame(qdArea, this->IsEnabled(), true /*isNotched*/, true /*drawFrame*/, &gAGARamp[r2]);

	::RGBForeColor(&gAGARamp[rB]);
	::RGBBackColor(&gAGARamp[rW]);
	}

void AGANotchedTargetBorderFrameViewMA::Dim()
	{
	// Suppress patBic painting over our frame.
	}

void AGANotchedTargetBorderFrameViewMA::ComputeBorderRegion(RgnHandle borderRegion)
	{
	// Return the border region to be painted or invalidated.

	CRect	qdArea;

	this->GetQDExtent(qdArea);

	AGAComputeTargetBorderRegion(borderRegion, qdArea, true /*isNotched*/);
	}

