//
// This is an OPTIONAL file for The Gray Council MacApp users.
//
// It contains tweaked copies of the global MacApp functions
// MacAppAlert and DoAlertKeyDown. If you use this file in
// your project ahead of the MacApp libraries that contain the
// original versions, the linker will discard the original
// MacApp versions (with a duplicate function warning).
//
// These replacement functions will cause the alerts displayed
// by the MacApp framework code to use the GCDialogManager
// routines for grayscale Dialog Manager alerts.
//
// There are separate functions here for MacApp 3.3.x and r11+,
// because of changes in MacApp.
//
// The modifications below are marked with "!!!". 
//

#include "UErrorMgr.h"
#include "GCDialogManager.h"

#if defined (qPart)
// MACAPP R11 VERSION HERE!

//----------------------------------------------------------------------------------------
// MacAppAlert: 
//----------------------------------------------------------------------------------------
// Don't require a segment load for this 
#pragma segment MAGlobalsRes

short MacAppAlert(ResNumber alertID,
				  ProcPtr filterProc)
{
	short alertReturn = cancel;

	gRsrcCheck = 0;								// force immediate check. 

	// don't provide an idle proc, since we don't want events right now
	MAInteractWithUserNoIdleProc();

	CTempUseResFile tempUseResFile(gApplicationRefNum);

	SetCursor(&(qd.arrow));

	// preflight the ALRT 
	AlertTHndl alrtTemplate = (AlertTHndl)GetResource('ALRT', alertID);
	if (alrtTemplate)
	{
		// preflight the DITL 
		if (GetResource('DITL', (*alrtTemplate)->itemsID))
		{
			// Success at last!
			CTempHandleLock tempHandleLock((Handle)alrtTemplate, eMoveHi);

			// Assume that the bit is set in the ALRT template in order to do the centering.

			PullApplicationToFront();

			ModalFilterUPP theFilter = NewModalFilterProc(filterProc ? filterProc : gMacAppAlertFilter);
			CTempRoutineDescriptor tempRoutineDescriptor((UniversalProcPtr)theFilter);

// !!! Modification for The Gray Council !!!
//			alertReturn = Alert(alertID, theFilter);
			alertReturn = AGAAlert(alertID, theFilter);
		}
		else
		{
#if qDebug
			CStr255 theString;
			ConcatNumber("Unable to find or load 'DITL' resource ", alertID, theString);
			::ProgramBreak(theString);
#endif
			SysBeep(2);								// At least give some indication 
		}
	}
	else
	{
#if qDebug
		CStr255 theString;
		ConcatNumber("Unable to find or load 'ALRT' resource ", alertID, theString);
		::ProgramBreak(theString);
#endif
		SysBeep(2);								// At least give some indication 
	}

	return alertReturn;
}

//----------------------------------------------------------------------------------------
// DoAlertKeyDown: Handle a keypress that has been mapped to one of the button controls.
//----------------------------------------------------------------------------------------
#pragma segment MAGlobalsRes

void DoAlertKeyDown(DialogRef theDialog,
					short itemNo)
{
// !!! Modification for The Gray Council !!!

	AGADialog*	agaDialog = AGADialog::ObjectFromDialogPtr(theDialog);
	
	if (agaDialog != NULL)
		agaDialog->FlashButton(itemNo);
	else
		{
// !!! Original MacApp code follows !!!

			short itemType;
			Handle item;
			CRect box;

			GetDialogItem(theDialog, itemNo, &itemType, &item, &box);
			if (itemType == (ctrlItem + btnCtrl))
			{
				// this code gives visual feedback 
				HiliteControl((ControlRef)item, kControlUpButtonPart);// hilite the button 

				long finalTicks;
				Delay(8, &finalTicks);					// delay for 8 ticks 

				HiliteControl((ControlRef)item, kControlNoPart);// stop hiliting the button 
			}
		}
}

#else	// qPart
// MACAPP 3.3.x VERSION HERE!

//----------------------------------------------------------------------------------------
// MacAppAlert: 
//----------------------------------------------------------------------------------------
// Don't require a segment load for this 
#pragma segment MAGlobalsRes

short MacAppAlert(ResNumber alertID,
				  ProcPtr filterProc)
{
	AlertTHndl alrtTemplate;
	SignedByte savedState;
	short alertReturn;

	gRsrcCheck = 0;								// force immediate check. 

	// don't provide an idle proc, since we don't want events right now
	gApplication->InteractWithUser(gNotificationPtr, NULL);

	short oldResFile = MAUseResFile(gApplicationRefNum);

	SetCursor(&(qd.arrow));

	// preflight the ALRT 
	alrtTemplate = (AlertTHndl)GetResource('ALRT', alertID);
	if (!alrtTemplate)
	{
#if qDebug
		CStr255 theString;
		ConcatNumber("Unable to find or load ALRT resource ", alertID, theString);
		ProgramBreak(theString);
#endif

		SysBeep(2);								// At least give some indication 
		MAUseResFile(oldResFile);
		return cancel;							// safe result 
	}

	// preflight the DITL 
	if (!GetResource('DITL', (*alrtTemplate)->itemsID))
	{
#if qDebug
		CStr255 theString;
		ConcatNumber("Unable to find or load DITL resource ", alertID, theString);
		ProgramBreak(theString);
#endif

		SysBeep(2);								// At least give some indication 
		MAUseResFile(oldResFile);
		return cancel;							// safe result 
	}

	// Success at last!
	savedState = LockHandleHigh((Handle)alrtTemplate);

	// Assume that the bit is set in the ALRT template in order to do the centering.

	PullApplicationToFront();

	ProcPtr aProcPtr = (filterProc == NULL) ? gMacAppAlertFilter : filterProc;
	ModalFilterUPP theFilter = NewModalFilterProc(aProcPtr);

// !!! Modification for The Gray Council !!!
//	alertReturn = Alert(alertID, theFilter);
	alertReturn = AGAAlert(alertID, theFilter);

	theFilter = (ModalFilterUPP)DisposeIfRoutineDescriptor((UniversalProcPtr)theFilter);
	
	// restore the state of the DITL's handle
	HSetState((Handle)alrtTemplate, savedState);
	MAUseResFile(oldResFile);
	return alertReturn;
} // MacAppAlert 

//----------------------------------------------------------------------------------------
// DoAlertKeyDown: Handle a keypress that has been mapped to one of the button controls.
//----------------------------------------------------------------------------------------
#pragma segment MAGlobalsRes

void DoAlertKeyDown(DialogRef theDialog, short itemNo)
{
// !!! Modification for The Gray Council !!!

	AGADialog*	agaDialog = AGADialog::ObjectFromDialogPtr(theDialog);
	
	if (agaDialog != NULL)
		agaDialog->FlashButton(itemNo);
	else
		{
// !!! Original MacApp code follows !!!

			short itemType;
			Handle item;
			Rect box;
			long finalTicks;

			GetDialogItem(theDialog, itemNo, &itemType, &item, &box);
			if (itemType == (ctrlItem + btnCtrl))
			{
				// this code gives visual feedback 
				HiliteControl((ControlRef)item, kControlUpButtonPart);	// hilite the button 
				Delay(8, &finalTicks);										// delay for 8 ticks 
				HiliteControl((ControlRef)item, kControlNoPart);		// stop hiliting the button 
			}

		}
} // DoAlertKeyDown 

#endif	// qPart
