//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// See header file "GCNewControlsMA.h" for usage.
//

#include "GCCoreMA.h"
#include "GCNewControlsMA.h"

void RegisterGCNewControlsMA()
	{
	//
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.
	//

	MA_REGISTER_CLASS(AGAIconPushButtonMA);
	MA_REGISTER_CLASS(AGAIconCheckBoxMA);
	MA_REGISTER_CLASS(AGAIconRadioButtonMA);
	MA_REGISTER_CLASS(AGASliderMA);
	MA_REGISTER_CLASS(AGALittleArrowsMA);
	MA_REGISTER_CLASS(AGADisclosureTriangleMA);
	MA_REGISTER_CLASS(AGAProgressIndicatorMA);
	MA_REGISTER_CLASS(AGASeparatorMA);
	MA_REGISTER_CLASS(AGAGroupBoxMA);
	MA_REGISTER_CLASS(AGASecondaryGroupBoxMA);
	MA_REGISTER_CLASS(AGATabPanelMA);
	MA_REGISTER_CLASS(AGASmallTabPanelMA);
	}

//
// AGAIconPushButtonMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAPushButtonMA

MA_DEFINE_CLASS_M1(AGAIconPushButtonMA, Inherited);

AGAIconPushButtonMA::AGAIconPushButtonMA()
	{
	mAGASuppressable = false;
	}

AGAIconPushButtonMA::~AGAIconPushButtonMA()
	{
	}

void AGAIconPushButtonMA::IAGAIconPushButtonMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel)
	{
	this->IAGAPushButtonMA(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel);
	}

void AGAIconPushButtonMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	(void) this->SetCMgrVisibility(false);

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	ResNumber	iconID = fUserArea;

	FailNIL(mAGAObject = new AGAIconPushButton(bounds, iconID, kAutoFrame, MIconButtonObject::kAutoIconFamily));

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

//
// AGAIconCheckBoxMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGACheckBoxMA

MA_DEFINE_CLASS_M1(AGAIconCheckBoxMA, Inherited);

AGAIconCheckBoxMA::AGAIconCheckBoxMA()
	{
	mAGASuppressable = false;
	}

AGAIconCheckBoxMA::~AGAIconCheckBoxMA()
	{
	}

void AGAIconCheckBoxMA::IAGAIconCheckBoxMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel, Boolean isTurnedOn)
	{
	this->IAGAIconCheckBoxMA(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel, isTurnedOn);
	}

void AGAIconCheckBoxMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	(void) this->SetCMgrVisibility(false);

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	ResNumber	iconID = fUserArea;

	FailNIL(mAGAObject = new AGAIconCheckBox(bounds, AGAObject::kNoAutomaticState, iconID, iconID, kAutoFrame, MIconButtonObject::kAutoIconFamily));

	if (this->GetLongVal() != 0)
		mAGAObject->SetValue(AGACheckBox::kCheckBoxOn, AGAObject::kDontRedraw);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

//
// AGAIconRadioButtonMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGARadioButtonMA

MA_DEFINE_CLASS_M1(AGAIconRadioButtonMA, Inherited);

AGAIconRadioButtonMA::AGAIconRadioButtonMA()
	{
	mAGASuppressable = false;
	}

AGAIconRadioButtonMA::~AGAIconRadioButtonMA()
	{
	}

void AGAIconRadioButtonMA::IAGAIconRadioButtonMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel, Boolean isTurnedOn)
	{
	this->IAGARadioButtonMA(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel, isTurnedOn);
	}

void AGAIconRadioButtonMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	(void) this->SetCMgrVisibility(false);

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	ResNumber	iconID = fUserArea;

	FailNIL(mAGAObject = new AGAIconRadioButton(bounds, kNoGroupID, kNoGroupID, AGAObject::kNoAutomaticState, iconID, iconID, kAutoFrame, MIconButtonObject::kAutoIconFamily));

	if (this->GetLongVal() != 0)
		mAGAObject->SetValue(AGARadioButton::kRadioButtonOn, AGAObject::kDontRedraw);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

//
// AGASliderMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS_M1(AGASliderMA, Inherited);

AGASliderMA::AGASliderMA()
	{
	mNotificationRoutine = NULL;
	mUserData = 0;
	mAGAObject = NULL;
	}

AGASliderMA::~AGASliderMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGASliderMA::IAGASliderMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const TextStyle& itsTextStyle)
	{
	this->IControl(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle);

	this->CreateAGAObject();
	}

void AGASliderMA::InstallNotificationRoutine(AGANotifyMAPtr notificationRoutine, void* userData)
	{
	// Save the notification routine fn pointer and user data
	// for notification handling.

	mNotificationRoutine = notificationRoutine;
	mUserData = userData;
	}

void AGASliderMA::HandleNotification(SInt32 dataValue)
	{
	// Update the control value, call the installed notification
	// function, and propagate our event number.

	//this->SetLongVal(dataValue, kDontRedraw);	TControl has no implicit value

	if (mNotificationRoutine != NULL)
		(*mNotificationRoutine)(this, dataValue, mUserData);
	
	this->HandleEvent(fEventNumber, this, NULL);

	(void) this->Focus();	// HandleEvent may have caused another view to draw
	}

void AGASliderMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGASliderMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	AGATextStyle::NormalizeTextStyle(fTextStyle, gSystemStyle.tsFont, gApplicationStyle.tsFont);

	FailNIL(mAGAObject = new AGASlider(bounds, 0, 1, 0, AGATextStyle(fTextStyle), fUserArea));
	
	mAGAObject->InstallNotificationRoutine(AGASliderMA::RealAGANotifier, this);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	// Slider has to erase when drawing, so make sure that
	// it doesn't assume we're on a gray background.
	
	this->Focus();
	InstallAGABackgroundColors(mAGAObject, kInstallOnlyIfCustom);
	}

void AGASliderMA::Draw(const VRect& /*area*/)
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGASliderMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGASliderMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* /*event*/, CPoint /*hysteresis*/)
	{
	// Let the AGAObject track the mouse; if tracking succeeds
	// handle our event number.

	if (mAGAObject->TrackMouse(this->ViewToQDPt(theMouse)))
		this->HandleEvent(fEventNumber, this, NULL);
	}

void AGASliderMA::DimState(Boolean state, Boolean redraw)
	{
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGASliderMA::Dim()
	{
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGASliderMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

void AGASliderMA::RealAGANotifier(AGATrackingIndicator* /*theIndicator*/, SInt32 dataValue, void* userData)
	{
	// This is a static function. Cast the user data to get
	// the AGASliderMA object, have it handle the notification.

	((AGASliderMA*) userData)->HandleNotification(dataValue);
	}

//
// AGALittleArrowsMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS_M1(AGALittleArrowsMA, Inherited);

AGALittleArrowsMA::AGALittleArrowsMA()
	{
	mNotificationRoutine = NULL;
	mUserData = 0;
	mAGAObject = NULL;
	mLinkedNumberText = NULL;
	}

AGALittleArrowsMA::~AGALittleArrowsMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGALittleArrowsMA::IAGALittleArrowsMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const TextStyle& itsTextStyle)
	{
	this->IControl(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle);

	this->CreateAGAObject();
	}

void AGALittleArrowsMA::InstallNotificationRoutine(AGANotifyMAPtr notificationRoutine, void* userData)
	{
	// Save the notification routine fn pointer and user data
	// for notification handling.

	mNotificationRoutine = notificationRoutine;
	mUserData = userData;
	}

void AGALittleArrowsMA::HandleNotification(SInt32 deltaValue)
	{
	// Update the linked TNumberText, if any, call the
	// installed notification function, and propagate our
	// event number.

	if (mLinkedNumberText != NULL)
		mLinkedNumberText->SetValue(mLinkedNumberText->GetValue() + deltaValue, kRedraw);

	if (mNotificationRoutine != NULL)
		(*mNotificationRoutine)(this, deltaValue, mUserData);
	
	this->HandleEvent(fEventNumber, this, NULL);

	(void) this->Focus();
	}

void AGALittleArrowsMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGALittleArrowsMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	FailNIL(mAGAObject = new AGALittleArrows(bounds));
	
	mAGAObject->InstallNotificationRoutine(AGALittleArrowsMA::RealAGANotifier, this);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	if (fUserArea != 0)
		{
		mLinkedNumberText = (TNumberText*) this->GetRootView()->FindSubView(fUserArea);
		if (mLinkedNumberText != NULL)
			if (! MA_MEMBER(mLinkedNumberText, TNumberText))
				mLinkedNumberText = NULL;
		}
	}

void AGALittleArrowsMA::Draw(const VRect& /*area*/)
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGALittleArrowsMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGALittleArrowsMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* /*event*/, CPoint /*hysteresis*/)
	{
	// Let the AGAObject track the mouse; if tracking succeeds
	// handle our event number.

	if (mAGAObject->TrackMouse(this->ViewToQDPt(theMouse)))
		this->HandleEvent(fEventNumber, this, NULL);
	}

void AGALittleArrowsMA::DimState(Boolean state, Boolean redraw)
	{
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGALittleArrowsMA::Dim()
	{
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGALittleArrowsMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

void AGALittleArrowsMA::RealAGANotifier(AGALittleArrows* /*theAGAObject*/, SInt32 deltaValue, void* userData)
	{
	// This is a static function. Cast the user data to get
	// the AGAScrollerScrollBarMA object, have it handle the notification.

	((AGALittleArrowsMA*) userData)->HandleNotification(deltaValue);
	}

//
// AGADisclosureTriangleMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS_M1(AGADisclosureTriangleMA, Inherited);

AGADisclosureTriangleMA::AGADisclosureTriangleMA()
	{
	mAGAObject = NULL;
	}

AGADisclosureTriangleMA::~AGADisclosureTriangleMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGADisclosureTriangleMA::IAGADisclosureTriangleMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const TextStyle& itsTextStyle)
	{
	this->IControl(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle);

	this->CreateAGAObject();
	}

void AGADisclosureTriangleMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGADisclosureTriangleMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	FailNIL(mAGAObject = new AGADisclosureTriangle(bounds, AGAObject::kAutomaticState));

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	// Disclosure triangle has to erase when animating, so make sure that
	// it doesn't assume we're on a gray background.

	this->Focus();
	InstallAGABackgroundColors(mAGAObject, kInstallOnlyIfCustom);
	}

void AGADisclosureTriangleMA::Draw(const VRect& /*area*/)
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGADisclosureTriangleMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGADisclosureTriangleMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* /*event*/, CPoint /*hysteresis*/)
	{
	// Let the AGAObject track the mouse; if tracking succeeds
	// handle our event number.

	if (mAGAObject->TrackMouse(this->ViewToQDPt(theMouse)))
		this->HandleEvent(fEventNumber, this, NULL);
	}

void AGADisclosureTriangleMA::DimState(Boolean state, Boolean redraw)
	{
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGADisclosureTriangleMA::Dim()
	{
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGADisclosureTriangleMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

//
// AGAProgressIndicatorMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS_M1(AGAProgressIndicatorMA, Inherited);

AGAProgressIndicatorMA::AGAProgressIndicatorMA()
	{
	mAGAObject = NULL;
	}

AGAProgressIndicatorMA::~AGAProgressIndicatorMA()
	{
	// Delete AGA object if we successfully allocated it.

	this->StopAutoAnimate();

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAProgressIndicatorMA::IAGAProgressIndicatorMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const TextStyle& itsTextStyle)
	{
	this->IControl(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle);

	this->CreateAGAObject();
	}

void AGAProgressIndicatorMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGAProgressIndicatorMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	FailNIL(mAGAObject = new AGAProgressIndicator(bounds, 0, 100));

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	if (fUserArea != 0)
		this->StartAutoAnimate();
	}

void AGAProgressIndicatorMA::Draw(const VRect& /*area*/)
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAProgressIndicatorMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

Boolean AGAProgressIndicatorMA::DoIdle(IdlePhase phase)
	{
	// Let the AGAObject do one animation step.

	if ((phase == idleContinue) && (mAGAObject != NULL) && this->Focus())
		mAGAObject->QuantizedAnimate();

	return false;	// did not free self
	}

void AGAProgressIndicatorMA::StartAutoAnimate()
	{
	// Place ourself on the cohandler chain so we'll get idle time.
	
	fNextHandler = NULL;	// if we still point our superview, the two chains will be confused
	this->SetIdleFreq(0);	// idle often
	this->SetEnable(true);	// we won't get idle time if we're disabled!

	gApplication->InstallCohandler(this, true);
	}

void AGAProgressIndicatorMA::StopAutoAnimate()
	{
	// Remove ourself on the cohandler chain so we don't get idle time.
	
	gApplication->InstallCohandler(this, false);
	}

//
// AGASeparatorMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TView

MA_DEFINE_CLASS_M1(AGASeparatorMA, Inherited);

AGASeparatorMA::AGASeparatorMA()
	{
	mAGAObject = NULL;
	}

AGASeparatorMA::~AGASeparatorMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGASeparatorMA::IAGASeparatorMA(TDocument* itsDocument, TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet)
	{
	this->IView(itsDocument, itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet);

	this->CreateAGAObject();
	}

void AGASeparatorMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGASeparatorMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	VRect	viewRect(0, 0, fSize.h, fSize.v);
	CRect	bounds;
	
	this->ViewToQDRect(viewRect, bounds);

	FailNIL(mAGAObject = new AGASeparator(bounds));
	}

void AGASeparatorMA::Draw(const VRect& /*area*/)
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGASeparatorMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

//
// AGAGroupBoxMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TCluster

MA_DEFINE_CLASS_M1(AGAGroupBoxMA, Inherited);

AGAGroupBoxMA::AGAGroupBoxMA()
	{
	mAGAObject = NULL;
	mIsPrimaryType = true;
	}

AGAGroupBoxMA::~AGAGroupBoxMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAGroupBoxMA::IAGAGroupBoxMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, ResNumber itsRsrcID, short itsIndex)
	{
	this->ICluster(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsRsrcID, itsIndex);

	this->CreateAGAObject();
	}

void AGAGroupBoxMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGAGroupBoxMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	CStr255	label;
	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);
	this->GetLabel(label);

	AGATextStyle::NormalizeTextStyle(fTextStyle, gSystemStyle.tsFont, gApplicationStyle.tsFont);

	FailNIL(mAGAObject = new AGAGroupBox(bounds, AGATextStyle(fTextStyle), mIsPrimaryType, label));

	if ((fUserArea != 0) && (fSuperView != NULL))
		{
		TView*	gapView = fSuperView->FindSubView(fUserArea);

		if (gapView != NULL)
			{
			CRect	gapExtent;

			gapView->GetQDExtent(gapExtent);
			mAGAObject->SetTitleGap(5 + gapExtent.GetLength(hSel));
			}
		}

	// Note that unlike other controls, for group boxes, we allow
	// "disabled" text to draw normal. This is because group box
	// views typically are "disabled" but not "dimmed".
	if (fDimmed)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

void AGAGroupBoxMA::Draw(const VRect& /*area*/)
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAGroupBoxMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGAGroupBoxMA::DoMouseCommand(VPoint& /*theMouse*/, TToolboxEvent* /*event*/, CPoint /*hysteresis*/)
	{
	// Group box drawing is based on enable state, but we don't
	// want clicking on the group box itself to cause highlighting.
	}

void AGAGroupBoxMA::DimState(Boolean state, Boolean redraw)
	{
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGAGroupBoxMA::Dim()
	{
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGAGroupBoxMA::SetLabel(const CStr255& theLabel, Boolean redraw)
	{
	// Change the current text in TCluster and in
	// the AGA object. Let TCluster use the redraw
	// to invalidate as usual.

	Inherited::SetLabel(theLabel, redraw);

	if (mAGAObject != NULL)
		{
		Str255	textString;

		AGA_PLstrcpy(textString, theLabel);
		mAGAObject->SetTitle(textString, AGAObject::kDontRedraw);
		}
	}

//
// AGASecondaryGroupBoxMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAGroupBoxMA

MA_DEFINE_CLASS_M1(AGASecondaryGroupBoxMA, Inherited);

AGASecondaryGroupBoxMA::AGASecondaryGroupBoxMA()
	{
	mIsPrimaryType = false;
	}

AGASecondaryGroupBoxMA::~AGASecondaryGroupBoxMA()
	{
	}

void AGASecondaryGroupBoxMA::IAGASecondaryGroupBoxMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, ResNumber itsRsrcID, short itsIndex)
	{
	this->IAGAGroupBoxMA(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsRsrcID, itsIndex);
	}

//
// AGATabPanelMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS_M1(AGATabPanelMA, Inherited);

AGATabPanelMA::AGATabPanelMA()
	{
	mAGAObject = NULL;
	mPanelContainerView = NULL;
	mActivePanelView = NULL;
	}

AGATabPanelMA::~AGATabPanelMA()
	{
	// Delete AGA object if we successfully allocated it.
	// We also need to make sure that all panel views not currently
	// installed get destructed.

	if (mAGAObject != NULL)
		{
		SInt32	numTabs;

		numTabs = mAGAObject->GetNumTabs();
		
		for (SInt32 tabIndex = 0; tabIndex < numTabs; tabIndex++)
			{
			if (tabIndex != mAGAObject->GetCurrentTab())
				{
				TView*	aPanelView = (TView*) mAGAObject->GetTabUserData(tabIndex);
				(void) FreeIfObject(aPanelView);
				}
			}

		delete mAGAObject;
		}
	}

void AGATabPanelMA::IAGATabPanelMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const TextStyle& itsTextStyle)
	{
	this->IControl(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle);

	this->CreateAGAObject();
	
	// Create the container view that we'll load panels in and out of.
	
	this->CreateContainerView();
	}

void AGATabPanelMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	
	// Create the container view that we'll load panels in and out of.
	
	this->CreateContainerView();
	}

void AGATabPanelMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	AGATextStyle::NormalizeTextStyle(fTextStyle, gSystemStyle.tsFont, gApplicationStyle.tsFont);

	FailNIL(mAGAObject = new AGATabPanel(bounds, 0, AGATextStyle(fTextStyle), fUserArea));

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	// Tabs have to erase when drawing, so make sure that
	// it doesn't assume we're on a gray background.

	this->Focus();
	InstallAGABackgroundColors(mAGAObject, kInstallOnlyIfCustom);
	}

void AGATabPanelMA::CreateContainerView()
	{
	if (mAGAObject->HasFrame())	// if no frame, we just run the tabs, not the panels
		{
		// Create a subview to contain the panels that we load and unload.
		// We locate it 3 pixels inset from our the frame content area.
		
		VCoordinate	tabHeight = mAGAObject->GetTabHeight();
		VPoint		viewLocation(3, 3 + tabHeight);
		VPoint		viewSize(fSize.h - 6, fSize.v - tabHeight - 6);
		
		mPanelContainerView = new TView;
		mPanelContainerView->IView(fDocument, this, viewLocation, viewSize,
									fSizeDeterminer[hSel], fSizeDeterminer[vSel]);
		}
	}

void AGATabPanelMA::Draw(const VRect& /*area*/)
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();

	// Set the background color so that any non-AGA subviews
	// that erase their background without assuming a color
	// will use our current tab panel color.

	if (this->IsEnabled())
		::RGBBackColor(&gAGARamp[r1]);
	else
		::RGBBackColor(&gAGARamp[r2]);
	}

void AGATabPanelMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.
	
	(void) this->Focus();
	CRect	frame;
	this->GetQDExtent(frame);
	::InvalRect(frame);
	
	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGATabPanelMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* /*event*/, CPoint /*hysteresis*/)
	{
	// Let the AGAObject track the mouse; if tracking succeeds
	// handle our event number.

	if (mAGAObject->TrackMouse(this->ViewToQDPt(theMouse)))
		this->HandleTabSwitch(mAGAObject->GetLastTabClicked());
	}

void AGATabPanelMA::DimState(Boolean state, Boolean redraw)
	{
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}

	this->SetSubviewsEnable(! state);
	}

void AGATabPanelMA::Dim()
	{
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGATabPanelMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);

	this->SetSubviewsEnable(state);
	}

void AGATabPanelMA::SwitchPanels(SInt32 newTabIndex)
	{
	// Switch to the specified tab index.
	
	if (mAGAObject != NULL)
		mAGAObject->SetCurrentTab(newTabIndex, this->Focus());

	this->SwitchPanelIn(newTabIndex, this->SwitchPanelOut());
	}

void AGATabPanelMA::HandleTabSwitch(SInt32 newTabIndex)
	{
	// If it's OK to switch tabs (may need to validate current
	// panel before switching), do the switch.

	SInt32	oldTabIndex = mAGAObject->GetCurrentTab();

	if ((newTabIndex != oldTabIndex) && this->ValidatePanel(oldTabIndex))
		{
		this->SwitchPanels(newTabIndex);
		this->HandleEvent(fEventNumber, this, NULL);
		}
	}

void AGATabPanelMA::AddPanel(SInt16 panelResourceID, StringPtr tabLabel)
	{
	// Create another tab and then install the specified View resource
	// onto that tab.

	if (mAGAObject != NULL)
		{
		Str255	actualTabLabel;
		SInt32	numTabs;

		if (tabLabel != NULL)
			AGA_PLstrcpy(actualTabLabel, tabLabel);
		else
			{
			Handle	viewHandle;
			SInt16	viewID;
			OSType	viewType;
			
			FailNIL(viewHandle = ::GetResource('View', panelResourceID));
			
			::GetResInfo(viewHandle, &viewID, &viewType, actualTabLabel);
			
			// Don't release it -- we're about to cause it to be read anyway.
			}

		numTabs = mAGAObject->GetNumTabs();
		
		mAGAObject->SetNumTabs(numTabs + 1, AGATabPanel::kDontShrink);
		
		this->InstallPanel(numTabs, panelResourceID, actualTabLabel);
		}
	}

void AGATabPanelMA::InstallPanel(SInt32 tabIndex, SInt16 panelResourceID, StringPtr tabLabel)
	{
	// Install the specified View resource onto the specified existing tab.

	if (mAGAObject != NULL)
		{
		TView*	newPanelView;
		SInt32	numTabs;

		numTabs = mAGAObject->GetNumTabs();
		
		if (tabIndex < numTabs)
			{
			// Wrap the DoCreateViews with custom color values so that the
			// AGA objects in the panel hierarchy will get the right background
			// colors.

			TurnOnCustomAGABackgroundColors(&gAGARamp[r1], &gAGARamp[r2]);
//			FailNIL(newPanelView = gViewServer->DoCreateViews(fDocument, NULL, panelResourceID, gZeroVPt));
			FailNIL(newPanelView = gViewServer->DoCreateViews(fDocument, mPanelContainerView, panelResourceID, gZeroVPt));
			TurnOffCustomAGABackgroundColors();

			newPanelView->Show(false, kRedraw);

			mAGAObject->SetTabUserData(tabIndex, newPanelView);
			
			if (tabLabel != NULL)
				{
				mAGAObject->SetTabLabel(tabIndex, tabLabel);
				mAGAObject->CalculateTabWidths();
				}

			// If this panel corresponds to the initial value, switch it in.

			if (mAGAObject->GetCurrentTab() == tabIndex)
				this->SwitchPanels(tabIndex);
			}
		}
	}

void AGATabPanelMA::RemoveTab(SInt32 tabIndex, Boolean redraw)
	{
	// Remove the specified tab from the AGA object.

	mAGAObject->RemoveTab(tabIndex);
	
	if (redraw && this->Focus())
		mAGAObject->DrawObject();
	}

TView* AGATabPanelMA::GetPanelView(SInt32 tabIndex)
	{
	// Return the TView* of the specified tab's panel view hierarchy.

	if (mAGAObject == NULL)
		return NULL;
	else
		return (TView*) mAGAObject->GetTabUserData(tabIndex);
	}

void AGATabPanelMA::SetSubviewsEnable(Boolean state)
	{
	// Do a recursive walk of our panel subviews, setting the
	// enable state of all of its subviews.

	if (mPanelContainerView != NULL)
		{
		this->DoSubviewsEnable(mPanelContainerView, state);
		
		mPanelContainerView->ForceRedraw();
		}
	}

void AGATabPanelMA::DoSubviewsEnable(TView* aSuperView, Boolean state)
	{
	// Do a recursive walk of our panel subviews, setting the
	// enable state of all of its subviews.

	CSubViewIterator	iter(aSuperView);
	
#if defined (qPart)
	for (TView* aSubView = iter.FirstSubView(); iter.Current(); aSubView = iter.NextSubView())
#else
	for (TView* aSubView = iter.FirstSubView(); iter.More(); aSubView = iter.NextSubView())
#endif
		{
		aSubView->SetEnable(state);

		if (MA_MEMBER(aSubView, TControl))
			((TControl*) aSubView)->DimState(! state, kDontRedraw);
		
		this->DoSubviewsEnable(aSubView, state);
		}
	}

Boolean AGATabPanelMA::ValidatePanel(SInt32 /*currentTabIndex*/)
	{
	// (Override for app-specific validation.)
	//
	// Return true if the current (specified) panel is valid and
	// it's OK to switch to a different panel. Return false otherwise
	// and the tab switch will not occur; it's OK to alert the user
	// of the invalid data before returning.
	//

	return true;
	}

Boolean AGATabPanelMA::SwitchPanelOut()
	{
	// Yank the current panel view hierarchy out of the container view.
	
	Boolean	panelContainedTarget = false;

	if (mActivePanelView != NULL)
		{
		TEventHandler*	currentTarget = gApplication->GetTarget();
		
		if (currentTarget != NULL)
			if (MA_MEMBER(currentTarget, TView))
				if (mActivePanelView->ContainsSubView((TView*) currentTarget))
					{
					panelContainedTarget = true;
					(void) currentTarget->ResignTarget();
					}

		mActivePanelView->Show(false, kRedraw);
		//mPanelContainerView->RemoveSubView(mActivePanelView);
		mActivePanelView = NULL;
		}
	
	return panelContainedTarget;
	}

void AGATabPanelMA::SwitchPanelIn(SInt32 newTabIndex, Boolean targetInPanel)
	{
	// Switch the specified panel view hierarchy in buy placing it
	// inside the container view.

	if (mAGAObject != NULL)
		{
		TView*	newPanelView = (TView*) mAGAObject->GetTabUserData(newTabIndex);
		
		if (newPanelView != NULL)
			{
			mActivePanelView = newPanelView;

			//
			// Before redrawing, center the new panel inside the container.
			//
/*			
			VRect	containerFrame;
			VPoint	containerSize;
			VRect	panelFrame;
			VPoint	panelSize;
			VPoint	leftoverSize;
			
			mPanelContainerView->GetFrame(containerFrame);
			containerSize = containerFrame.GetSize();
			
			newPanelView->GetFrame(panelFrame);
			panelSize = panelFrame.GetSize();
			
			leftoverSize.h = containerSize.h - panelSize.h;
			leftoverSize.v = containerSize.v - panelSize.v;
			
			panelFrame.left = leftoverSize.h / 2;
			panelFrame.top = leftoverSize.v / 2;
			panelFrame.right = panelFrame.left + panelSize.h;
			panelFrame.bottom = panelFrame.top + panelSize.v;
			
			//
			// Install the panel as a subpane of the container,
			// then make it visible.
			//

			mPanelContainerView->AddSubView(newPanelView);
			
			newPanelView->SetFrame(panelFrame, kInvalidate);
*/
			newPanelView->Show(true, kRedraw);
			
			if (targetInPanel)
				{
				TView*	wannabeTarget = newPanelView->FindTarget();
				
				if (wannabeTarget != NULL)
					{
					(void) wannabeTarget->BecomeTarget();
					
					if (MA_MEMBER(wannabeTarget, TEditText))
						((TEditText*) wannabeTarget)->SetTargetSelection(kRedraw);
					}
				}
			}
		}
	}

//
// AGASmallTabPanelMA ----------------------------------------------------
//

#undef Inherited
#define Inherited AGATabPanelMA

MA_DEFINE_CLASS_M1(AGASmallTabPanelMA, Inherited);

AGASmallTabPanelMA::AGASmallTabPanelMA()
	{
	}

AGASmallTabPanelMA::~AGASmallTabPanelMA()
	{
	}

void AGASmallTabPanelMA::IAGASmallTabPanelMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const TextStyle& itsTextStyle)
	{
	this->IAGATabPanelMA(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle);
	}

void AGASmallTabPanelMA::CreateAGAObject()
	{
	Inherited::CreateAGAObject();
	
	// Set the tabs to use the standard small appearance.

	mAGAObject->SetTabSize(AGATabPanel::kSmallTabs);
	mAGAObject->SetLabelsStyle(gAGAStdBoldSmallStyle);
	}

