//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// See header file "GCStdControlsMA.h" for usage.
//

#include "GCCoreMA.h"
#include "GCStdControlsMA.h"

void RegisterGCStdControlsMA()
	{
	//
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.
	//

	MA_REGISTER_CLASS(AGAPushButtonMA);
	MA_REGISTER_CLASS(AGACheckBoxMA);
	MA_REGISTER_CLASS(AGARadioButtonMA);
	MA_REGISTER_CLASS(AGAScrollBarMA);
	MA_REGISTER_CLASS(AGAScrollerScrollBarMA);
	MA_REGISTER_CLASS(AGAPopupMA);
	MA_REGISTER_CLASS(AGAStaticTextMA);
	}

//
// AGAPushButtonMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TButton

MA_DEFINE_CLASS_M1(AGAPushButtonMA, Inherited);

AGAPushButtonMA::AGAPushButtonMA()
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	}

AGAPushButtonMA::~AGAPushButtonMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAPushButtonMA::IAGAPushButtonMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel)
	{
	this->IButton(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel);

	this->CreateAGAObject();
	}

void AGAPushButtonMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGAPushButtonMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

if (gAGAStdControls || !mAGASuppressable)
	(void) this->SetCMgrVisibility(false);

	VRect	viewRect;
	CRect	bounds;
	CStr255	viewTitle;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);
	this->GetText(viewTitle);

	AGATextStyle::NormalizeTextStyle(fTextStyle, gSystemStyle.tsFont, gApplicationStyle.tsFont);

	FailNIL(mAGAObject = new AGAPushButton(bounds, AGATextStyle(fTextStyle), viewTitle));

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	if (! gAGAStdControls && mAGASuppressable) { return; }

	//
	// We implement the default button outline by checking for the
	// presence of the TRRectAdorner by std ID, tossing it, and
	// telling the AGA button object that it is a default button.
	// The AGA object uses bounds that respect the control inset,
	// so when we give it the 3x3 AGA default ring, we need to
	// outset its bounds by 3x3. Note that typically this is 1x1
	// smaller than the view bounds, which have room for the 4x4
	// control inset used for the non-AGA default ring.
	//

	TAdorner*	rrectAdorner = this->FindRRectAdorner();
	if (rrectAdorner != NULL)
		{
		this->DeleteAdorner(rrectAdorner, kDontInvalidate);
		mAGAObject->SetDefault(AGAObject::kIsDefault, AGAObject::kFrameInside);
		
		mAGAObject->GetObjectBounds(bounds);
		::InsetRect(bounds, -3, -3);
		mAGAObject->SetObjectBounds(bounds, AGAObject::kDontRedraw);
		}
	}

void AGAPushButtonMA::Draw(const VRect& area)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Draw(area); return; }
	
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAPushButtonMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);

	if ((mAGAObject != NULL) && mAGAObject->IsDefault())
		{
		// As noted above in AGAPushButtonMA::CreateObject, we need to
		// outset the AGA object frame by 3x3 if it has a default outline.
		
		CRect	bounds;
		
		mAGAObject->GetObjectBounds(bounds);
		::InsetRect(bounds, -3, -3);
		mAGAObject->SetObjectBounds(bounds, AGAObject::kDontRedraw);
		}
	}

void AGAPushButtonMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DoMouseCommand(theMouse, event, hysteresis); return; }
	
	// Let the AGAObject track the mouse; if tracking succeeds
	// handle our event number.

	if (mAGAObject->TrackMouse(this->ViewToQDPt(theMouse)))
		this->HandleEvent(fEventNumber, this, NULL);
	}

void AGAPushButtonMA::HiliteState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::HiliteState(state, redraw); return; }
	
	// Let the AGAPushButton draw in the specified state.

	if ((mAGAObject != NULL) && this->Focus())
		mAGAObject->DrawButton(state);
	}

void AGAPushButtonMA::Hilite()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Hilite(); return; }
	
	// Let the AGAPushButton draw in pressed state.

	if ((mAGAObject != NULL) && this->Focus())
		mAGAObject->DrawButton(AGAObject::kPressed);
	}

void AGAPushButtonMA::DimState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DimState(state, redraw); return; }
	
	// Set the AGAObject's enable state.

	fDimmed = state;

	if ((mAGAObject != NULL) && this->Focus())
		mAGAObject->SetEnable(!state, redraw && this->IsVisible());
	}

void AGAPushButtonMA::Dim()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Dim(); return; }
	
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGAPushButtonMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

void AGAPushButtonMA::SetText(const CStr255& itsText, Boolean redraw)
	{
	// Change the current text in TCtlMgr and in
	// the AGA object. Do NOT let control mgr draw.

	Inherited::SetText(itsText, kDontRedraw);

	if (mAGAObject != NULL)
		{
		Str255	textString;
		Boolean	wantRedraw = (gAGAStdControls || !mAGASuppressable) && redraw && this->Focus() && this->IsVisible();

		AGA_PLstrcpy(textString, itsText);
		mAGAObject->SetTitle(textString, wantRedraw);
		}
	}

TAdorner* AGAPushButtonMA::FindRRectAdorner()
	{
	// Return the TRRectAdorner, if any, that is attached to
	// this push button.

	TAdorner*	foundAdorner = NULL;

	if (fAdorners != NULL)
		{
		CAdornerIterator	iter(this);

#if defined (qPart)
		for (TAdorner* theAdorner = iter.FirstAdorner(); iter.Current(); theAdorner = iter.NextAdorner())
#else
		for (TAdorner* theAdorner = iter.FirstAdorner(); iter.More(); theAdorner = iter.NextAdorner())
#endif
			{
			if (MA_MEMBER(theAdorner, TRRectAdorner))
				foundAdorner = theAdorner;
			}
		}

	return foundAdorner;
	}

//
// AGACheckBoxMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TCheckBox

MA_DEFINE_CLASS_M1(AGACheckBoxMA, Inherited);

AGACheckBoxMA::AGACheckBoxMA()
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	}

AGACheckBoxMA::~AGACheckBoxMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGACheckBoxMA::IAGACheckBoxMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel, Boolean isTurnedOn)
	{
	this->ICheckBox(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel, isTurnedOn);

	this->CreateAGAObject();
	}

void AGACheckBoxMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGACheckBoxMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

if (gAGAStdControls || !mAGASuppressable)
	(void) this->SetCMgrVisibility(false);

	VRect	viewRect;
	CRect	bounds;
	CStr255	viewTitle;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);
	this->GetText(viewTitle);

	AGATextStyle::NormalizeTextStyle(fTextStyle, gSystemStyle.tsFont, gApplicationStyle.tsFont);

	FailNIL(mAGAObject = new AGACheckBox(bounds, AGATextStyle(fTextStyle), viewTitle, AGAObject::kNoAutomaticState));

	if (this->GetLongVal() != 0)
		mAGAObject->SetValue(AGACheckBox::kCheckBoxOn, AGAObject::kDontRedraw);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

void AGACheckBoxMA::Draw(const VRect& area)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Draw(area); return; }
	
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGACheckBoxMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGACheckBoxMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DoMouseCommand(theMouse, event, hysteresis); return; }
	
	// Let the AGAObject track the mouse; if tracking succeeds
	// handle our event number.

	if (mAGAObject->TrackMouse(this->ViewToQDPt(theMouse)))
		this->HandleEvent(fEventNumber, this, NULL);
	}

void AGACheckBoxMA::HiliteState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::HiliteState(state, redraw); return; }
	
	// Let the AGACheckBox draw in the specified state.

	if ((mAGAObject != NULL) && this->Focus())
		mAGAObject->DrawButton(state);
	}

void AGACheckBoxMA::Hilite()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Hilite(); return; }
	
	// Let the AGACheckBox draw in pressed state.

	if ((mAGAObject != NULL) && this->Focus())
		mAGAObject->DrawButton(AGAObject::kPressed);
	}

void AGACheckBoxMA::DimState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DimState(state, redraw); return; }
	
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGACheckBoxMA::Dim()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Dim(); return; }
	
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGACheckBoxMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

void AGACheckBoxMA::SetText(const CStr255& itsText, Boolean redraw)
	{
	// Change the current text in TCtlMgr and in
	// the AGA object. Do NOT let control mgr draw.

	Inherited::SetText(itsText, kDontRedraw);

	if (mAGAObject != NULL)
		{
		Str255	textString;
		Boolean	wantRedraw = (gAGAStdControls || !mAGASuppressable) && redraw && this->Focus() && this->IsVisible();

		AGA_PLstrcpy(textString, itsText);
		mAGAObject->SetTitle(textString, wantRedraw);
		}
	}

Boolean AGACheckBoxMA::IsOn()
	{
	if (! gAGAStdControls && mAGASuppressable) { return Inherited::IsOn(); }

	// Return true if it's on.

	if (mAGAObject == NULL)
		return Inherited::IsOn();
	else
		return mAGAObject->GetValue() == AGACheckBox::kCheckBoxOn;
	}

void AGACheckBoxMA::SetState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::SetState(state, redraw); return; }

	// Set the state to on or off.

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetValue(state, wantRedraw);
		}
	}

void AGACheckBoxMA::Toggle(Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Toggle(redraw); return; }

	// Toggle the state between on and off.

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		this->SetState(! this->IsOn(), wantRedraw);
		}
	}

void AGACheckBoxMA::ToggleIf(Boolean matchState, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ToggleIf(matchState, redraw); return; }

	// Toggle the state if it's currently in the specified state.

	if (mAGAObject != NULL)
		if (this->IsOn() == matchState)
			{
			Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

			this->SetState(! this->IsOn(), wantRedraw);
			}
	}

//
// AGARadioButtonMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TRadio

MA_DEFINE_CLASS_M1(AGARadioButtonMA, Inherited);

AGARadioButtonMA::AGARadioButtonMA()
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	}

AGARadioButtonMA::~AGARadioButtonMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGARadioButtonMA::IAGARadioButtonMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel, Boolean isTurnedOn)
	{
	this->IRadio(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel, isTurnedOn);

	this->CreateAGAObject();
	}

void AGARadioButtonMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGARadioButtonMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

if (gAGAStdControls || !mAGASuppressable)
	(void) this->SetCMgrVisibility(false);

	VRect	viewRect;
	CRect	bounds;
	CStr255	viewTitle;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);
	this->GetText(viewTitle);

	AGATextStyle::NormalizeTextStyle(fTextStyle, gSystemStyle.tsFont, gApplicationStyle.tsFont);

	FailNIL(mAGAObject = new AGARadioButton(bounds, AGATextStyle(fTextStyle), viewTitle, kNoGroupID, kNoGroupID, AGAObject::kNoAutomaticState));

	if (this->GetLongVal() != 0)
		mAGAObject->SetValue(AGARadioButton::kRadioButtonOn, AGAObject::kDontRedraw);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

void AGARadioButtonMA::Draw(const VRect& area)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Draw(area); return; }
	
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGARadioButtonMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGARadioButtonMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DoMouseCommand(theMouse, event, hysteresis); return; }
	
	// Let the AGAObject track the mouse; if tracking succeeds
	// handle our event number.

	if (mAGAObject->TrackMouse(this->ViewToQDPt(theMouse)))
		this->HandleEvent(fEventNumber, this, NULL);
	}

void AGARadioButtonMA::HiliteState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::HiliteState(state, redraw); return; }
	
	// Let the AGARadioButton draw in the specified state.

	if ((mAGAObject != NULL) && this->Focus() && this->IsVisible())
		mAGAObject->DrawButton(state);
	}

void AGARadioButtonMA::Hilite()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Hilite(); return; }
	
	// Let the AGARadioButton draw in pressed state.

	if ((mAGAObject != NULL) && this->Focus() && this->IsVisible())
		mAGAObject->DrawButton(AGAObject::kPressed);
	}

void AGARadioButtonMA::DimState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DimState(state, redraw); return; }
	
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGARadioButtonMA::Dim()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Dim(); return; }
	
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGARadioButtonMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

void AGARadioButtonMA::SetText(const CStr255& itsText, Boolean redraw)
	{
	// Change the current text in TCtlMgr and in
	// the AGA object. Do NOT let control mgr draw.

	Inherited::SetText(itsText, kDontRedraw);

	if (mAGAObject != NULL)
		{
		Str255	textString;
		Boolean	wantRedraw = (gAGAStdControls || !mAGASuppressable) && redraw && this->Focus() && this->IsVisible();

		AGA_PLstrcpy(textString, itsText);
		mAGAObject->SetTitle(textString, wantRedraw);
		}
	}

Boolean AGARadioButtonMA::IsOn()
	{
	if (! gAGAStdControls && mAGASuppressable) { return Inherited::IsOn(); }

	// Return true if it's on.

	if (mAGAObject == NULL)
		return Inherited::IsOn();
	else
		return mAGAObject->GetValue() == AGARadioButton::kRadioButtonOn;
	}

void AGARadioButtonMA::SetState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::SetState(state, redraw); return; }

	// Set the state to on or off.

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetValue(state, wantRedraw);
		}
	}

void AGARadioButtonMA::Toggle(Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::Toggle(redraw); return; }

	// Toggle the state between on and off.

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		this->SetState(! this->IsOn(), wantRedraw);
		}
	}

void AGARadioButtonMA::ToggleIf(Boolean matchState, Boolean redraw)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ToggleIf(matchState, redraw); return; }

	// Toggle the state if it's currently in the specified state.

	if (mAGAObject != NULL)
		if (this->IsOn() == matchState)
			{
			Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

			this->SetState(! this->IsOn(), wantRedraw);
			}
	}

//
// AGAScrollBarMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TScrollBar

MA_DEFINE_CLASS_M1(AGAScrollBarMA, Inherited);

AGAScrollBarMA::AGAScrollBarMA()
	{
	mNotificationRoutine = NULL;
	mUserData = 0;
	mAGAObject = NULL;
	}

AGAScrollBarMA::~AGAScrollBarMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAScrollBarMA::IAGAScrollBarMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, VHSelect itsDirection, long itsVal, long itsMin, long itsMax)
	{
	this->IScrollBar(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsDirection, itsVal, itsMin, itsMax);

	this->CreateAGAObject();
	}

void AGAScrollBarMA::InstallNotificationRoutine(AGANotifyMAPtr notificationRoutine, void* userData)
	{
	// Save the notification routine fn pointer and user data
	// for notification handling.

	mNotificationRoutine = notificationRoutine;
	mUserData = userData;
	}

void AGAScrollBarMA::HandleNotification(SInt32 dataValue)
	{
	// Update the scrollbar value, call the installed notification
	// function, and propagate our event number.

	this->SetLongVal(dataValue, kDontRedraw);

	if (mNotificationRoutine != NULL)
		(*mNotificationRoutine)(this, dataValue, mUserData);
	
	this->HandleEvent(fEventNumber, this, NULL);

	(void) this->Focus();	// HandleEvent may have caused another view to draw
	}

void AGAScrollBarMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGAScrollBarMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

if (gAGAStdControls)
	(void) this->SetCMgrVisibility(false);

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	FailNIL(mAGAObject = new AGAScrollBar(bounds, this->GetLongMin(), this->GetLongMax(), this->GetLongVal()));
	
	mAGAObject->InstallNotificationRoutine(AGAScrollBarMA::RealAGANotifier, this);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

void AGAScrollBarMA::Draw(const VRect& area)
	{
	if (! gAGAStdControls) { Inherited::Draw(area); return; }
	
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAScrollBarMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGAScrollBarMA::SetLongVal(VCoordinate itsVal, Boolean redraw)
	{
	// Catch all value changes here and synchronize the
	// AGAScrollBar accordingly.

	Inherited::SetLongVal(itsVal, redraw && !gAGAStdControls);
	
	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = gAGAStdControls && redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetValue(itsVal, wantRedraw);
		}
	}

void AGAScrollBarMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis)
	{
	if (! gAGAStdControls) { Inherited::DoMouseCommand(theMouse, event, hysteresis); return; }
	
	// Let the AGAObject track the mouse. Don't call HandleEvent;
	// it has already been handled through HandleNotification.

	(void) mAGAObject->TrackMouse(this->ViewToQDPt(theMouse));
	}

void AGAScrollBarMA::HiliteState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls) { Inherited::HiliteState(state, redraw); return; }

	// Suppress Control Manager scroll bar drawing.
	}

void AGAScrollBarMA::Hilite()
	{
	if (! gAGAStdControls) { Inherited::Hilite(); return; }
	
	// Suppress Control Manager scroll bar drawing.
	}

void AGAScrollBarMA::DimState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls) { Inherited::DimState(state, redraw); return; }
	
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGAScrollBarMA::Dim()
	{
	if (! gAGAStdControls) { Inherited::Dim(); return; }
	
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGAScrollBarMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

void AGAScrollBarMA::Activate(Boolean entering)
	{
	if (! gAGAStdControls) { Inherited::Activate(entering); return; }
	
	// Set the AGAScrollBar's activated state.

	if (mAGAObject != NULL)
		mAGAObject->SetActive(entering, this->Focus() && this->IsVisible());
	}

void AGAScrollBarMA::SetLongMax(VCoordinate itsMax, Boolean redraw)
	{
	// Set the AGAScrollBar's limit.
	
	Inherited::SetLongMax(itsMax, redraw && ! gAGAStdControls);

	if (mAGAObject != NULL)
		{
		SInt32	itsMin;
		SInt32	itsMax;
		Boolean	wantRedraw = gAGAStdControls && redraw && this->Focus() && this->IsVisible();

		mAGAObject->GetRange(&itsMin, &itsMax);
		mAGAObject->SetRange(itsMin, this->GetLongMax(), wantRedraw);
		}
	}

void AGAScrollBarMA::RealAGANotifier(AGATrackingIndicator* /*theIndicator*/, SInt32 dataValue, void* userData)
	{
	// This is a static function. Cast the user data to get
	// the AGAScrollBarMA object, have it handle the notification.

	((AGAScrollBarMA*) userData)->HandleNotification(dataValue);
	}

//
// AGAScrollerScrollBarMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TScrollerScrollBar

MA_DEFINE_CLASS_M1(AGAScrollerScrollBarMA, Inherited);

AGAScrollerScrollBarMA::AGAScrollerScrollBarMA()
	{
	mNotificationRoutine = NULL;
	mUserData = 0;
	mAGAObject = NULL;
	}

AGAScrollerScrollBarMA::~AGAScrollerScrollBarMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAScrollerScrollBarMA::IAGAScrollerScrollBarMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, VHSelect itsDirection, long itsMax, TScroller* itsScroller)
	{
	this->IScrollerScrollBar(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsDirection, itsMax, itsScroller);

	this->CreateAGAObject();
	}

void AGAScrollerScrollBarMA::InstallNotificationRoutine(AGANotifyMAPtr notificationRoutine, void* userData)
	{
	// Save the notification routine fn pointer and user data
	// for notification handling.

	mNotificationRoutine = notificationRoutine;
	mUserData = userData;
	}

void AGAScrollerScrollBarMA::HandleNotification(SInt32 dataValue)
	{
	// Scroll our scrollers and call the installed notification
	// function.

	CObjectIterator iter(fScrollers);

#if defined (qPart)
	for (TScroller* aScroller = (TScroller*) iter.FirstObject(); iter.Current(); aScroller = (TScroller*) iter.NextObject())
#else
	for (TScroller* aScroller = (TScroller*) iter.FirstObject(); iter.More(); aScroller = (TScroller*) iter.NextObject())
#endif
		(void) aScroller->ScrollRelative(fDirection, dataValue);

	if (mNotificationRoutine != NULL)
		(*mNotificationRoutine)(this, dataValue, mUserData);

	(void) this->Focus();
	}

void AGAScrollerScrollBarMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGAScrollerScrollBarMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

if (gAGAStdControls)
	(void) this->SetCMgrVisibility(false);

	VRect	viewRect;
	CRect	bounds;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);

	FailNIL(mAGAObject = new AGAScrollBar(bounds, this->GetLongMin(), this->GetLongMax(), this->GetLongVal()));
	
	mAGAObject->InstallNotificationRoutine(AGAScrollerScrollBarMA::RealAGANotifier, this);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	this->UpdateScrollAmounts();
	}

void AGAScrollerScrollBarMA::Draw(const VRect& area)
	{
	if (! gAGAStdControls) { Inherited::Draw(area); return; }
	
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAScrollerScrollBarMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp stuff relocate and recalculate,
	// then resynchronize the AGAScrollBar to the scrollers.

	Inherited::SetFrame(newFrame, invalidate);
	
	this->UpdateScrollAmounts();
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGAScrollerScrollBarMA::AttachScroller(TScroller* itsScroller)
	{
	// Synchronize the AGAScrollBar to the supplied scroller.

	Inherited::AttachScroller(itsScroller);
	
	this->UpdateScrollAmounts();
	}

void AGAScrollerScrollBarMA::SetLongVal(VCoordinate itsVal, Boolean redraw)
	{
	// Catch all value changes here and synchronize the
	// AGAScrollBar accordingly.

	Inherited::SetLongVal(itsVal, redraw && !gAGAStdControls);
	
	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = gAGAStdControls && redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetValue(itsVal, wantRedraw);
		}
	}

void AGAScrollerScrollBarMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis)
	{
	if (! gAGAStdControls) { Inherited::DoMouseCommand(theMouse, event, hysteresis); return; }
	
	// Let the AGAObject track the mouse. Don't call HandleEvent;
	// it has already been handled through HandleNotification.

	(void) mAGAObject->TrackMouse(this->ViewToQDPt(theMouse));
	}

void AGAScrollerScrollBarMA::HiliteState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls) { Inherited::HiliteState(state, redraw); return; }

	// Suppress Control Manager scroll bar drawing.
	}

void AGAScrollerScrollBarMA::Hilite()
	{
	if (! gAGAStdControls) { Inherited::Hilite(); return; }
	
	// Suppress Control Manager scroll bar drawing.
	}

void AGAScrollerScrollBarMA::DimState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls) { Inherited::DimState(state, redraw); return; }
	
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGAScrollerScrollBarMA::Dim()
	{
	if (! gAGAStdControls) { Inherited::Dim(); return; }
	
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGAScrollerScrollBarMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

void AGAScrollerScrollBarMA::Activate(Boolean entering)
	{
	if (! gAGAStdControls) { Inherited::Activate(entering); return; }
	
	// Set the AGAScrollBar's activated state.

	if (mAGAObject != NULL)
		mAGAObject->SetActive(entering, this->Focus() && this->IsVisible());
	}

void AGAScrollerScrollBarMA::SetLongMax(VCoordinate itsMax, Boolean redraw)
	{
	// Set the AGAScrollBar's limit.
	
	Inherited::SetLongMax(itsMax, redraw && ! gAGAStdControls);

	if (mAGAObject != NULL)
		{
		SInt32	itsMin;
		SInt32	itsMax;
		Boolean	wantRedraw = gAGAStdControls && redraw && this->Focus() && this->IsVisible();

		mAGAObject->GetRange(&itsMin, &itsMax);
		mAGAObject->SetRange(itsMin, this->GetLongMax(), wantRedraw);

		this->UpdateScrollAmounts();
		}
	}

void AGAScrollerScrollBarMA::UpdateScrollAmounts()
	{
	// Synchronize the AGAScrollBar to reflect the scroller state.

	if ((mAGAObject != NULL) && (fScrollers->GetSize() > 0))
		{
		TScroller*	aScroller = (TScroller*) fScrollers->At(1);
		VCoordinate	maxTranslation = aScroller->fMaxTranslation[fDirection];
		VCoordinate	viewSize = aScroller->fSize[fDirection];
		VCoordinate	singleStepSize = aScroller->fScrollUnit[fDirection];
		VCoordinate	pageStepSize = Max(0, viewSize - singleStepSize);
		VCoordinate	pageSize = maxTranslation * (((float) (Max(0, viewSize - singleStepSize))) / ((float) (viewSize + maxTranslation)));

		(void) this->Focus();

		mAGAObject->SetRange(0, maxTranslation, kDontRedraw);
		mAGAObject->SetPageSize(pageSize, gAGAStdControls);
		mAGAObject->SetStepSizes(singleStepSize, pageStepSize);
		}
	}

void AGAScrollerScrollBarMA::RealAGANotifier(AGATrackingIndicator* /*theIndicator*/, SInt32 dataValue, void* userData)
	{
	// This is a static function. Cast the user data to get
	// the AGAScrollerScrollBarMA object, have it handle the notification.

	((AGAScrollerScrollBarMA*) userData)->HandleNotification(dataValue);
	}

//
// AGAPopupMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TPopup

MA_DEFINE_CLASS_M1(AGAPopupMA, Inherited);

AGAPopupMA::AGAPopupMA()
	{
	mAGAObject = NULL;
	}

AGAPopupMA::~AGAPopupMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAPopupMA::IAGAPopupMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, short itsMenuID, short itsCurrentItem, short itsItemOffset, short itsStrListID, short itsIndex, short itsStyle, short itsJust, Boolean useAddResMenu, ResType useAddResMenuResType, const TextStyle& itsTextStyle)
	{
	this->IPopup(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsMenuID, itsCurrentItem, itsItemOffset, itsStrListID, itsIndex, itsStyle, itsJust, useAddResMenu, useAddResMenuResType, itsTextStyle);

	this->CreateAGAObject();
	}

void AGAPopupMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();
	}

void AGAPopupMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

if (gAGAStdControls)
	(void) this->SetCMgrVisibility(false);

	AGAPopupMenu::WidthAdjust	widthAdjust;
	
	AGATextStyle::NormalizeTextStyle(fTextStyle, gSystemStyle.tsFont, gApplicationStyle.tsFont);

	VRect			viewRect(0, 0, fSize.h, fSize.v);
	CRect			bounds;
	CStr255			viewTitle;
	AGATextStyle	titleStyle(fTextStyle);
	AGATextStyle	textStyle(fTextStyle);
	
	titleStyle.mFontStyle = fTitleStyle;
	textStyle.mFontStyle = normal;	// the actual popup can't have a style, even though AdLib allows it to be set
	
	this->ViewToQDRect(viewRect, bounds);
	this->GetText(viewTitle);

	if (fSizeDeterminer[hSel] == sizeFixed)
		widthAdjust = AGAPopupMenu::kFixedWidth;
	else
		widthAdjust = AGAPopupMenu::kSystemMDEFAdjustment;

	FailNIL(mAGAObject = new AGAPopupMenu(bounds, fItemOffset, viewTitle, fTitleJust, titleStyle, textStyle, widthAdjust, this->GetMenuRef(), AGAPopupMenu::kDontDispose));

	mAGAObject->SetCurrentItemNo((SInt16) this->GetLongVal(), AGAObject::kDontRedraw);

	if (fDimmed || ! fEnabled)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

void AGAPopupMA::Draw(const VRect& area)
	{
	if (! gAGAStdControls) { Inherited::Draw(area); return; }
	
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAPopupMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

void AGAPopupMA::DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis)
	{
	if (! gAGAStdControls) { Inherited::DoMouseCommand(theMouse, event, hysteresis); return; }
	
	// Let the AGAObject track the mouse; if tracking succeeds
	// handle our event number.

	if (mAGAObject->TrackMouse(this->ViewToQDPt(theMouse)))
		this->HandleEvent(fEventNumber, this, NULL);
	}

void AGAPopupMA::DimState(Boolean state, Boolean redraw)
	{
	if (! gAGAStdControls) { Inherited::DimState(state, redraw); return; }
	
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGAPopupMA::Dim()
	{
	if (! gAGAStdControls) { Inherited::Dim(); return; }
	
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGAPopupMA::SetEnable(Boolean state)
	{
	Inherited::SetEnable(state);

	// Set the AGAObject's enable state, without redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(state, AGAObject::kDontRedraw);
	}

void AGAPopupMA::AttachMenuRef(MenuRef itsMenuRef)
	{
	//
	// Since TPopup owns the menu, it will do the disposal.
	//
	
	Inherited::AttachMenuRef(itsMenuRef);

	if (mAGAObject != NULL)
		mAGAObject->SetMenuRef(itsMenuRef, AGAPopupMenu::kDontDispose);
	}

void AGAPopupMA::SetText(const CStr255& title, Boolean redraw)
	{
	// Install the new popup title (label).

	Inherited::SetText(title, redraw);

	if (mAGAObject != NULL)
		{
		CStr255	newTitle = title;

		mAGAObject->SetTitle(newTitle, gAGAStdControls && redraw && this->Focus() && this->IsVisible());
		}
	}

void AGAPopupMA::SetLongVal(VCoordinate itsVal, Boolean redraw)
	{
	// Catch all value changes here and synchronize the
	// AGAPopupMenu accordingly.

	Inherited::SetLongVal(itsVal, redraw && !gAGAStdControls);
	
	if (mAGAObject != NULL)
		mAGAObject->SetCurrentItemNo((SInt16) itsVal, gAGAStdControls && redraw && this->Focus() && this->IsVisible());
	}

SInt16 AGAPopupMA::GetVal()
	{
	// Return the currently selected item number.

	if (mAGAObject != NULL)
		return mAGAObject->GetCurrentItemNo();
	else
		return Inherited::GetVal();
	}

//
// AGAStaticTextMA ----------------------------------------------------
//

#undef Inherited
#define Inherited TStaticText

MA_DEFINE_CLASS_M1(AGAStaticTextMA, Inherited);

AGAStaticTextMA::AGAStaticTextMA()
	{
	mAGAObject = NULL;
	}

AGAStaticTextMA::~AGAStaticTextMA()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAStaticTextMA::IAGAStaticTextMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, ResNumber itsRsrcID, short itsIndex)
	{
	this->IStaticText(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsRsrcID, itsIndex);

	this->CreateAGAObject();
	}

void AGAStaticTextMA::DoPostCreate(TDocument* itsDocument)
	{
	// Create and set up the AGAObject.

	Inherited::DoPostCreate(itsDocument);
	
	this->CreateAGAObject();

	// A static text paints its background, so make sure that
	// it doesn't assume we're on a gray background.

	this->Focus();
	InstallAGABackgroundColors(mAGAObject, kInstallOnlyIfCustom);
	}

void AGAStaticTextMA::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	VRect	viewRect;
	CRect	bounds;
	CStr255	viewTitle;
	
	this->ControlArea(viewRect);
	this->ViewToQDRect(viewRect, bounds);
	this->GetText(viewTitle);

	AGATextStyle::NormalizeTextStyle(fTextStyle, gSystemStyle.tsFont, gApplicationStyle.tsFont);

	FailNIL(mAGAObject = new AGAStaticText(bounds, AGATextStyle(fTextStyle), fJust, viewTitle));

	// Note that unlike other controls, for static text, we allow
	// "disabled" text to draw normal. This is because static text
	// views typically are "disabled" but not "dimmed".
	if (fDimmed)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

void AGAStaticTextMA::DimState(Boolean state, Boolean redraw)
	{
	// Set the AGAObject's enable state.

	fDimmed = state;

	if (mAGAObject != NULL)
		{
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		mAGAObject->SetEnable(!state, wantRedraw);
		}
	}

void AGAStaticTextMA::Dim()
	{
	// Set the AGAObject's enable state, with immediate redraw.

	if (mAGAObject != NULL)
		mAGAObject->SetEnable(AGAObject::kDisabled, this->Focus() && this->IsVisible());
	}

void AGAStaticTextMA::SetText(const CStr255& theText, Boolean redraw)
	{
	// Change the current text in TStaticText and in
	// the AGAStaticText. Do NOT let TStaticText redraw,
	// because it erases to white.

	Inherited::SetText(theText, kDontRedraw);

	if (mAGAObject != NULL)
		{
		Str255	textString;
		Boolean	wantRedraw = redraw && this->Focus() && this->IsVisible();

		AGA_PLstrcpy(textString, theText);
		mAGAObject->SetTitle(textString, wantRedraw);
		}
	}

#if defined (qPart)
void AGAStaticTextMA::InstallTextStyle(const CTextStyle& theTextStyle, Boolean redraw)
#else
void AGAStaticTextMA::InstallTextStyle(const TextStyle& theTextStyle, Boolean redraw)
#endif
	{
	// Change the text style of the AGA object.
	
	if (mAGAObject != NULL)
		mAGAObject->SetTextStyle(AGATextStyle(theTextStyle), kDontRedraw);

	Inherited::InstallTextStyle(theTextStyle, redraw);
	}

void AGAStaticTextMA::Draw(const VRect& /*area*/)
	{
	// Let the AGAObject draw itself.
	// Actually, to properly mimic TStaticText, we need to do text substitution
	// and use the potentially modified text.
	
	if (mAGAObject != NULL)
		{
		CStr255	theText;

		this->GetText(theText);
		this->DoSubstitution(theText);

		mAGAObject->SetTitle(theText, kDontRedraw);
		mAGAObject->DrawObject();
		}
	}

void AGAStaticTextMA::SetFrame(const VRect& newFrame, Boolean invalidate)
	{
	// Let the MacApp and AGA objects update their locations.

	Inherited::SetFrame(newFrame, invalidate);
	
	SetAGAObjectFrame(this, mAGAObject, newFrame, invalidate);
	}

