//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// The Gray Council provides a set of standard C++ classes that implement
// the standard Apple Grayscale Appearance. The core classes do not
// require any other code such as a particular class framework.
// There are separate files provided that plug The Gray Council core
// into class frameworks.
//
// This file defines MacApp adapter classes for the standard Control
// Manager controls.
//
// Classes defined below:
//	AGAPushButtonMA -- TButton subclass for AGAPushButton
//	AGACheckBoxMA -- TCheckBox subclass for AGACheckBox
//	AGARadioButtonMA -- TRadio subclass for AGARadioButton
//	AGAScrollBarMA -- TScrollBar subclass for AGAScrollBar
//	AGAScrollerScrollBarMA -- TScrollerScrollBar subclass for AGAScrollBar
//		Remember the difference between TScrollBar and TScrollerScrollBar!
//	AGAPopupMA -- TPopup subclass for AGAPopupMenu
//	AGAStaticTextMA -- TStaticText subclass for AGAStaticText 
//

#pragma once

#ifndef __GCSTDCONTROLSMA__
#define __GCSTDCONTROLSMA__

#include "GCCore.h"
#include "GCStdControls.h"

extern void RegisterGCStdControlsMA();

#pragma mark AGAPushButtonMA

//
// AGAPushButtonMA ----------------------------------------------------
//
// TButton replacement subclass. Automatically detects presence of a
// TRRectAdorner to make the AGAPushButton take on the default button
// appearance. You could also set kIsDefault state manually.
//

class AGAPushButtonMA : public TButton
	{
	MA_DECLARE_CLASS;

	public:

		AGAPushButtonMA();
		virtual ~AGAPushButtonMA();

		// TButton-compatible I method
		void IAGAPushButtonMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel);

		// MacApp overrides
		virtual void DoPostCreate(TDocument* itsDocument);
		virtual void Draw(const VRect& area);
		virtual void SetFrame(const VRect& newFrame, Boolean invalidate); 
		virtual void DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis);
		virtual void HiliteState(Boolean state, Boolean redraw);
		virtual void Hilite();
		virtual void DimState(Boolean state, Boolean redraw);
		virtual void Dim();
		virtual void SetEnable(Boolean state);
		virtual void SetText(const CStr255& itsText, Boolean redraw);

		AGAPushButton*	mAGAObject;
		Boolean			mAGASuppressable;	// false for icon buttons, which can't be drawn by Control Manager

	protected:

		virtual void CreateAGAObject();

		TAdorner*	FindRRectAdorner();
	};

#pragma mark AGACheckBoxMA

//
// AGACheckBoxMA ----------------------------------------------------
//
// TCheckBox replacement subclass. To use mixed-state capability,
// set/get the AGACheckBox state directly.
//

class AGACheckBoxMA : public TCheckBox
	{
	MA_DECLARE_CLASS;

	public:

		AGACheckBoxMA();
		virtual ~AGACheckBoxMA();

		// TCheckBox-compatible I method
		void IAGACheckBoxMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel, Boolean isTurnedOn);

		// MacApp overrides
		virtual void DoPostCreate(TDocument* itsDocument);
		virtual void Draw(const VRect& area);
		virtual void SetFrame(const VRect& newFrame, Boolean invalidate); 
		virtual void DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis);
		virtual void HiliteState(Boolean state, Boolean redraw);
		virtual void Hilite();
		virtual void DimState(Boolean state, Boolean redraw);
		virtual void Dim();
		virtual void SetEnable(Boolean state);
		virtual void SetText(const CStr255& itsText, Boolean redraw);
		virtual Boolean IsOn();
		virtual void SetState(Boolean state, Boolean redraw);
		virtual void Toggle(Boolean redraw);
		virtual void ToggleIf(Boolean matchState, Boolean redraw);

		AGACheckBox*	mAGAObject;
		Boolean			mAGASuppressable;	// false for icon buttons, which can't be drawn by Control Manager

	protected:

		virtual void CreateAGAObject();
	};

#pragma mark AGARadioButtonMA

//
// AGARadioButtonMA ----------------------------------------------------
//
// TRadio replacement subclass. To use mixed-state capability,
// set/get the AGARadioButton state directly.
//

class AGARadioButtonMA : public TRadio
	{
	MA_DECLARE_CLASS;

	public:

		AGARadioButtonMA();
		virtual ~AGARadioButtonMA();

		// TRadio-compatible I method
		void IAGARadioButtonMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, const CStr255& itsLabel, Boolean isTurnedOn);

		// MacApp overrides
		virtual void DoPostCreate(TDocument* itsDocument);
		virtual void Draw(const VRect& area);
		virtual void SetFrame(const VRect& newFrame, Boolean invalidate); 
		virtual void DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis);
		virtual void HiliteState(Boolean state, Boolean redraw);
		virtual void Hilite();
		virtual void DimState(Boolean state, Boolean redraw);
		virtual void Dim();
		virtual void SetEnable(Boolean state);
		virtual void SetText(const CStr255& itsText, Boolean redraw);
		virtual Boolean IsOn();
		virtual void SetState(Boolean state, Boolean redraw);
		virtual void Toggle(Boolean redraw);
		virtual void ToggleIf(Boolean matchState, Boolean redraw);

		AGARadioButton*	mAGAObject;
		Boolean			mAGASuppressable;	// false for icon buttons, which can't be drawn by Control Manager

	protected:

		virtual void CreateAGAObject();
	};

#pragma mark AGAScrollBarMA

//
// AGAScrollBarMA ----------------------------------------------------
//
// TScrollBar replacement subclass. To use live scrolling, set the
// AGAScrollBar state directly and either install a notification routine
// or handle the scroll bar's fEventNumber by checking the current value.
// To use proportional indicator, set the AGAScrollBar state directly.
//
// Please note that TScrollBar and TScrollerScrollBar are different!
//

typedef void (*AGANotifyMAPtr)(TView* theIndicator, SInt32 dataValue, void* userData);

class AGAScrollBarMA : public TScrollBar
	{
	MA_DECLARE_CLASS;

	public:

		AGAScrollBarMA();
		virtual ~AGAScrollBarMA();

		// TScrollBar-compatible I method
		void IAGAScrollBarMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, VHSelect itsDirection, long itsVal, long itsMin, long itsMax);

		// MacApp overrides
		virtual void DoPostCreate(TDocument* itsDocument);
		virtual void Draw(const VRect& area);
		virtual void SetFrame(const VRect& newFrame, Boolean invalidate); 
		virtual void SetLongVal(VCoordinate itsVal, Boolean redraw);
		virtual void DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis);
		virtual void HiliteState(Boolean state, Boolean redraw);
		virtual void Hilite();
		virtual void DimState(Boolean state, Boolean redraw);
		virtual void Dim();
		virtual void SetEnable(Boolean state);
		virtual void Activate(Boolean entering);
		virtual void SetLongMax(VCoordinate itsMax, Boolean redraw);
		
		virtual void	InstallNotificationRoutine(AGANotifyMAPtr notificationRoutine, void* userData);
		virtual void	HandleNotification(SInt32 dataValue);
		
		AGANotifyMAPtr	mNotificationRoutine;
		void*			mUserData;

		AGAScrollBar*	mAGAObject;

	protected:

		virtual void CreateAGAObject();
		
		static void RealAGANotifier(AGATrackingIndicator* theIndicator, SInt32 dataValue, void* userData);
	};

#pragma mark AGAScrollerScrollBarMA

//
// AGAScrollerScrollBarMA ----------------------------------------------------
//
// TScrollerScrollBar replacement subclass. To use live scrolling, just set
// the AGAScrollBar state directly; the scroller will get the live scroll
// nofication messages automatically. To use proportional indicator,
// set the AGAScrollBar state directly.
//
// Please note that TScrollBar and TScrollerScrollBar are different!
//

class AGAScrollerScrollBarMA : public TScrollerScrollBar
	{
	MA_DECLARE_CLASS;

	public:

		AGAScrollerScrollBarMA();
		virtual ~AGAScrollerScrollBarMA();

		// TScrollerScrollBar-compatible I method
		void IAGAScrollerScrollBarMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, VHSelect itsDirection, long itsMax, TScroller* itsScroller);

		// MacApp overrides
		virtual void DoPostCreate(TDocument* itsDocument);
		virtual void Draw(const VRect& area);
		virtual void SetFrame(const VRect& newFrame, Boolean invalidate); 
		virtual void AttachScroller(TScroller* itsScroller);
		virtual void SetLongVal(VCoordinate itsVal, Boolean redraw);
		virtual void DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis);
		virtual void HiliteState(Boolean state, Boolean redraw);
		virtual void Hilite();
		virtual void DimState(Boolean state, Boolean redraw);
		virtual void Dim();
		virtual void SetEnable(Boolean state);
		virtual void Activate(Boolean entering);
		virtual void SetLongMax(VCoordinate itsMax, Boolean redraw);
		
		virtual void	InstallNotificationRoutine(AGANotifyMAPtr notificationRoutine, void* userData);
		virtual void	HandleNotification(SInt32 dataValue);
		virtual void	UpdateScrollAmounts();
		
		AGANotifyMAPtr	mNotificationRoutine;
		void*			mUserData;

		AGAScrollBar*	mAGAObject;

	protected:

		virtual void CreateAGAObject();
		
		static void RealAGANotifier(AGATrackingIndicator* theIndicator, SInt32 dataValue, void* userData);
	};

#pragma mark AGAPopupMA

//
// AGAPopupMA ----------------------------------------------------
//
// TPopup replacement subclass.
//

class AGAPopupMA : public TPopup
	{
	MA_DECLARE_CLASS;

	public:

		AGAPopupMA();
		virtual ~AGAPopupMA();

		// TPopup-compatible I method
		void IAGAPopupMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, short itsMenuID, short itsCurrentItem, short itsItemOffset, short itsStrListID, short itsIndex, short itsStyle, short itsJust, Boolean useAddResMenu, ResType useAddResMenuResType, const TextStyle& itsTextStyle = gSystemStyle);

		// MacApp overrides
		virtual void DoPostCreate(TDocument* itsDocument);
		virtual void Draw(const VRect& area);
		virtual void SetFrame(const VRect& newFrame, Boolean invalidate); 
		virtual void DoMouseCommand(VPoint& theMouse, TToolboxEvent* event, CPoint hysteresis);
		virtual void DimState(Boolean state, Boolean redraw);
		virtual void Dim();
		virtual void SetEnable(Boolean state);
		
		// TPopup and TControl overrides
		virtual void AttachMenuRef(MenuRef itsMenuRef);				// installs the menu
		virtual void SetText(const CStr255& title, Boolean redraw);	// sets the popup title
		virtual void SetLongVal(VCoordinate itsVal, Boolean redraw);// sets cur item no
		virtual SInt16 GetVal();									// gets cur item no

		AGAPopupMenu*	mAGAObject;

	protected:

		virtual void CreateAGAObject();
	};

#pragma mark AGAStaticTextMA

//
// AGAStaticTextMA ----------------------------------------------------
//
// TStaticText replacement subclass. Dims with gray text instead of
// patBic gray pattern erasing; suppresses erasing background to white
// when text is changed.
//

class AGAStaticTextMA : public TStaticText
	{
	MA_DECLARE_CLASS;

	public:

		AGAStaticTextMA();
		virtual ~AGAStaticTextMA();

		// TStaticText-compatible I method
		void IAGAStaticTextMA(TView* itsSuperView, const VPoint& itsLocation, const VPoint& itsSize, SizeDeterminer itsHSizeDet, SizeDeterminer itsVSizeDet, ResNumber itsRsrcID, short itsIndex);

		// MacApp overrides
		virtual void DoPostCreate(TDocument* itsDocument);
		virtual void Draw(const VRect& area);
		virtual void SetFrame(const VRect& newFrame, Boolean invalidate); 
		virtual void DimState(Boolean state, Boolean redraw);
		virtual void Dim();
		virtual void SetText(const CStr255& theText, Boolean redraw);

#if defined (qPart)
		virtual void InstallTextStyle(const CTextStyle& theTextStyle, Boolean redraw);
#else
		virtual void InstallTextStyle(const TextStyle& theTextStyle, Boolean redraw);
#endif

		AGAStaticText*	mAGAObject;

	protected:

		virtual void CreateAGAObject();
	};

#endif // __GCSTDCONTROLSMA__
