//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// See header file "GCCorePP.h" for usage.
//

#include "GCCorePP.h"
#include "GCStdControlsPP.h"

#ifndef NO_RTTI
#include <typeinfo.h>
#endif // NO_RTTI

// The URegistrar::RegisterClass calls are making my eyes hurt.
// Define a macro to do it instead. We take advantage of the fact that
// we use the names class_ID and CreateStream in all of our classes.
#define GCPP_REGISTER_CLASS(classname) URegistrar::RegisterClass(classname::class_ID, (ClassCreatorFunc) classname::CreateStream)

void RegisterGCCorePP()
	{
	//
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.
	//

	GCPP_REGISTER_CLASS(AGAWindowPP);
	GCPP_REGISTER_CLASS(AGAWindowPPX);
	GCPP_REGISTER_CLASS(AGADialogBoxPP);
	GCPP_REGISTER_CLASS(AGADialogBoxPPX);
	GCPP_REGISTER_CLASS(AGAWhiteBackgroundAttachmentPP);
	GCPP_REGISTER_CLASS(AGANotchedWhiteBackgroundAttachmentPP);
	GCPP_REGISTER_CLASS(AGABorderFrameAttachmentPP);
	GCPP_REGISTER_CLASS(AGANotchedBorderFrameAttachmentPP);
	GCPP_REGISTER_CLASS(AGAFocusBoxPP);
	GCPP_REGISTER_CLASS(AGANotchedFocusBoxPP);
	}

SInt32 GetTextTraitsJustification(ResIDT inTextTraitsID)
	{
	TextTraitsH	aTTHandle = UTextTraits::LoadTextTraits(inTextTraitsID);
	
	if (aTTHandle == NULL)
		return teFlushDefault;
	else
		return (**aTTHandle).justification;
	}

void ResetAGAObjectFrame(LPane* itsPane, AGAObject* theAGAObject)
	{
	// Change the AGA object's bounds to match the pane's new
	// bounds. Check for the AGA not yet having been created,
	// and tell it not to redraw (it will get a draw message
	// with the update event).

	if (theAGAObject != NULL)
		{
		Rect	bounds;
		
		(void) itsPane->CalcLocalFrameRect(bounds);

		theAGAObject->SetObjectBounds(&bounds, AGAObject::kDontRedraw);
		}
	}

UInt8 ReadStreamEnum(LStream* inStream)
	{
	// There's a compiler flag for "enums always ints", which interferes
	// with reading an 8-bit value from the PPob stream into an enum, such
	// as the icon button frame and image types. Normally sizeof(myEnum)
	// is one byte, so inStream->ReadData(&mMyEnum, sizeof(mMyEnum)) works,
	// but when this flag is set, the sizeof is wrong. So when reading
	// our one byte CPPb extra data fields, which is what we use for enums,
	// we will explicitly read one byte and return it.
	
	UInt8	streamByte;
	
	inStream->ReadData(&streamByte, sizeof(streamByte));
	
	return streamByte;
	}

//
// This is the AGATextStyle constructor that is only available
// when compiling with __PowerPlant__ defined. It is conditionally
// declared in GCCore.h, but implemented here since the symbol
// won't be defined when GCCore.cpp is compiled unless precompiled
// headers are used, which is a bad assumption.
//

AGATextStyle::AGATextStyle(ResIDT inTextTraitsID)
	{
	// Construct from a PowerPlant Text Traits resource.

	TextTraitsH aTTHandle = UTextTraits::LoadTextTraits(inTextTraitsID);
	
	if (aTTHandle == NULL)	// likely, cos there is no "0" Txtr resource
		{
		mFontNum = systemFont;	// typically Chicago
		mFontSize = 0;			// application size, typically 12-point
		mFontStyle = normal;
		}
	else
		{
		mFontNum = (**aTTHandle).fontNumber;
		mFontSize = (**aTTHandle).size;
		mFontStyle = (**aTTHandle).style;
		
		if (mFontNum == UTextTraits::fontNumber_Unknown) // means use supplied font name
			{
			Str255	fontName;
			
			AGA_PLstrcpy(fontName, (**aTTHandle).fontName);
			::GetFNum(fontName, &mFontNum);
			}
		}
	}

//
// AGAWindowPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LWindow

AGAWindowPP* AGAWindowPP::CreateStream(LStream* inStream)
	{
	return new AGAWindowPP(inStream);
	}

AGAWindowPP::AGAWindowPP(LStream* inStream)
: Inherited(inStream)
	{
	if (this->HasAttribute(windAttr_Modal))
		mBackgroundKind = kRaisedModalBackground;
	else
		mBackgroundKind = kRaisedModelessBackground;
	}

AGAWindowPP::~AGAWindowPP()
	{
	}

void AGAWindowPP::FinishCreate()
	{
	// Make sure that the gray background color is installed
	// prior to subpanes having their FinishCreateSelf method
	// called.

	this->SetForeAndBackColors(NULL, &gAGARamp[r2]);
	this->ApplyForeAndBackColors();
	
	Inherited::FinishCreate();
	}

void AGAWindowPP::FinishCreateSelf()
	{
	// Let LWindow do its normal stuff first.
	
	Inherited::FinishCreateSelf();
	
	// Unless we've been told to leave the background alone,
	// we need to both modify the WCTB background color (so
	// that non-AGA std controls respect it) and set the QD
	// background color.

	if (mBackgroundKind != kNoBackgroundPaint)
		{
		ThrowIfOSErr_(AGAInstallGrayWCTB(mMacWindowP));
		this->SetForeAndBackColors(NULL, &gAGARamp[r2]);
		}
	}

void AGAWindowPP::DrawSelf()
	{
	if (mBackgroundKind == kNoBackgroundPaint)
		Inherited::DrawSelf();
	else
		{
		// Change normal DrawSelf by drawing our background.

		if (this->HasAttribute(windAttr_EraseOnUpdate))
			this->DrawBackground(&mMacWindowP->portRect, true, mActive == triState_On);
		
		this->DrawSizeBox();
		}
	}

void AGAWindowPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// For raised backgrounds,
	// if the window size is changing in either direction, we need
	// to invalidate the right/bottom edge pixels that will
	// need to turn gray when the window is expanded, either before
	// or after the resize.
	
	this->InvalidateEdges(true, inWidthDelta, inHeightDelta);

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	this->InvalidateEdges(false, inWidthDelta, inHeightDelta);
	}

void AGAWindowPP::ActivateSelf()
	{
	// Redraw just the edge pixels that change based on active state.

	this->FocusDraw();
	this->DrawBackground(&mMacWindowP->portRect, false, true);
	
	Inherited::ActivateSelf();
	}

void AGAWindowPP::DeactivateSelf()
	{
	// Redraw just the edge pixels that change based on active state.

	this->FocusDraw();
	this->DrawBackground(&mMacWindowP->portRect, false, false);
	
	Inherited::DeactivateSelf();
	}

void AGAWindowPP::DrawBackground(const Rect* area, Boolean fill, Boolean active)
	{
	if (mBackgroundKind != kNoBackgroundPaint)
		AGABackgroundPaint(area, fill, mBackgroundKind, active, this->HasAttribute(windAttr_SizeBox));
	}

void AGAWindowPP::InvalidateEdges(Boolean before, Int16 inWidthDelta, Int16 inHeightDelta)
	{
	if ((mBackgroundKind == kRaisedModelessBackground) ||
		(mBackgroundKind == kRaisedModalBackground))
		{
		// Invalidate the extra "edge" pixels that need to be redrawn
		// when the window is resized.

		Rect	bounds;
		Rect	rectToInvalidate;
		
		this->CalcPortFrameRect(bounds);

		if (((inWidthDelta > 0) && before) ||
			((inWidthDelta < 0) && ! before))
			{
			rectToInvalidate = bounds;
			rectToInvalidate.left = rectToInvalidate.right - 1;
			this->InvalPortRect(&rectToInvalidate);
			}
		
		if (((inHeightDelta > 0) && before) ||
			((inHeightDelta < 0) && ! before))
			{
			rectToInvalidate = bounds;
			rectToInvalidate.top = rectToInvalidate.bottom - 1;
			this->InvalPortRect(&rectToInvalidate);
			}
		
		if (this->HasAttribute(windAttr_EraseOnUpdate))
			{
			rectToInvalidate = bounds;
			rectToInvalidate.left = rectToInvalidate.right - 16;
			rectToInvalidate.top = rectToInvalidate.bottom - 16;
			this->InvalPortRect(&rectToInvalidate);
			}
		}
	}

//
// AGAWindowPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAWindowPP

AGAWindowPPX* AGAWindowPPX::CreateStream(LStream* inStream)
	{
	return new AGAWindowPPX(inStream);
	}

AGAWindowPPX::AGAWindowPPX(LStream* inStream)
: Inherited(inStream)
	{
	mBackgroundKind = (AGABackgroundKind) ReadStreamEnum(inStream);
	}

AGAWindowPPX::~AGAWindowPPX()
	{
	}

//
// AGADialogBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LDialogBox

AGADialogBoxPP* AGADialogBoxPP::CreateStream(LStream* inStream)
	{
	return new AGADialogBoxPP(inStream);
	}

AGADialogBoxPP::AGADialogBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	mDeleteDefaultOutline = false;
	mBackgroundKind = kFlatWindowBackground;
	
	//
	// We need to determine the window proc ID in order to really
	// decide what kind of background to do. So, let's find out
	// the same way that PowerPlant does. The SWindowInfo back
	// there in the stream contains a WINDid field, which is the
	// id of a 'WIND' resource. The 'WIND' resource has the proc ID.
	//
	
	SInt32	originalMarker = inStream->GetMarker();
	
	// Undo the LDialogBox stream read.
	inStream->SetMarker(-sizeof(PaneIDT), streamFrom_Marker);
	inStream->SetMarker(-sizeof(PaneIDT), streamFrom_Marker);
	
	// Undo the LWindow stream read.
	inStream->SetMarker(-sizeof(SWindowInfo), streamFrom_Marker);
	
	// Re-read the SWindowInfo.
	SWindowInfo	windowInfo;
	inStream->ReadData(&windowInfo, sizeof(SWindowInfo));
	
	// Peek into the 'WIND' resource, as done by LWindow::MakeMacWindow.
	SWINDResourceH	theWIND = (SWINDResourceH) ::GetResource('WIND', windowInfo.WINDid);
	SInt16			procID = (**theWIND).procID;
	::ReleaseResource((Handle) theWIND);
	
	// Restore the stream marker.
	inStream->SetMarker(originalMarker, streamFrom_Start);
	
	// Now determine the background kind based on the procID.
	
	if ((procID == dBoxProc) || (procID == movableDBoxProc))
		mBackgroundKind = kFlatWindowBackground;
	else if (this->HasAttribute(windAttr_Modal))
		mBackgroundKind = kRaisedModalBackground;
	else
		mBackgroundKind = kRaisedModelessBackground;
	}

AGADialogBoxPP::~AGADialogBoxPP()
	{
	}

void AGADialogBoxPP::FinishCreate()
	{
	// Make sure that the gray background color is installed
	// prior to subpanes having their FinishCreateSelf method
	// called.

	this->SetForeAndBackColors(NULL, &gAGARamp[r2]);
	this->ApplyForeAndBackColors();
	
	Inherited::FinishCreate();
	}

void AGADialogBoxPP::FinishCreateSelf()
	{
	// Let LDialogBox do its normal stuff first.
	
	Inherited::FinishCreateSelf();
	
	// Unless we've been told to leave the background alone,
	// we need to both modify the WCTB background color (so
	// that non-AGA std controls respect it) and set the QD
	// background color.

	if (mBackgroundKind != kNoBackgroundPaint)
		{
		ThrowIfOSErr_(AGAInstallGrayWCTB(mMacWindowP));
		this->SetForeAndBackColors(NULL, &gAGARamp[r2]);
		}
	
	if (! gAGAStdControls) { return; }

	//
	// Now we remove its LDefaultOutline and set the
	// default our way.
	//
	
	if (mDefaultOutline != NULL)
		{
		delete mDefaultOutline;
		mDefaultOutline = NULL;
		}

	this->SetDefaultButton(mDefaultButtonID);
	}

void AGADialogBoxPP::SetDefaultButton(PaneIDT inButtonID)
	{
	if (! gAGAStdControls) { Inherited::SetDefaultButton(inButtonID); return; }

	// If you bypass the RTTI here, you must ensure that the
	// panes represented by the mDefaultButtonID and the
	// inButtonID are a subclass of AGAPushButtonPP.

	AGAPushButtonPP*	oldDefaultButton = NULL;
	AGAPushButtonPP*	newDefaultButton = NULL;
	LPane*				oldDefaultPane = this->FindPaneByID(mDefaultButtonID);
	LPane*				newDefaultPane = this->FindPaneByID(inButtonID);

#ifndef NO_RTTI

	if (oldDefaultPane != NULL)
		oldDefaultButton = dynamic_cast<AGAPushButtonPP*>(oldDefaultPane);

	if (newDefaultPane != NULL)
		newDefaultButton = dynamic_cast<AGAPushButtonPP*>(newDefaultPane);

#else // NO_RTTI

	oldDefaultButton = (AGAPushButtonPP*) oldDefaultPane;
	newDefaultButton = (AGAPushButtonPP*) newDefaultPane;

#endif // NO_RTTI

	mDefaultButtonID = inButtonID;
	
	if (oldDefaultButton != NULL)
		{
		if (oldDefaultButton->mAGAObject->IsDefault())
			{
			// Tell the old default button that it is no longer
			// the default. This means insetting its frame.

			oldDefaultButton->MoveBy(3, 3, true);
			oldDefaultButton->ResizeFrameBy(-6, -6, true);
			oldDefaultButton->mAGAObject->SetDefault(AGAObject::kIsNotDefault, AGAObject::kFrameInside);
			oldDefaultButton->Refresh();
			}
		}
	else if (mDefaultOutline != nil)
		{
		// Not an AGA object. Do it the LDialogBox way...

		mDefaultOutline->Refresh();
		delete mDefaultOutline;
		mDefaultOutline = NULL;
		}
	
	if (newDefaultButton != NULL)
		{
		if (! newDefaultButton->mAGAObject->IsDefault())
			{
			// Tell the new default button that it has become
			// the default. This means outsetting its frame.

			newDefaultButton->MoveBy(-3, -3, false);
			newDefaultButton->ResizeFrameBy(6, 6, false);
			newDefaultButton->mAGAObject->SetDefault(AGAObject::kIsDefault, AGAObject::kFrameInside);
			newDefaultButton->Refresh();
			}
		}
	else if (newDefaultPane != NULL)
		{
		// Not an AGA object. Do it the LDialogBox way...

		((LControl*) newDefaultPane)->AddListener(this);
		mDefaultOutline = new LDefaultOutline(newDefaultPane);
		mDefaultOutline->Refresh();
		}
	}

void AGADialogBoxPP::DrawSelf()
	{
	if (mBackgroundKind == kNoBackgroundPaint)
		Inherited::DrawSelf();
	else
		{
		// Change normal DrawSelf by drawing our background.

		if (this->HasAttribute(windAttr_EraseOnUpdate))
			this->DrawBackground(&mMacWindowP->portRect, true, mActive == triState_On);
		
		this->DrawSizeBox();
		}
	}

void AGADialogBoxPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// For raised backgrounds,
	// if the window size is changing in either direction, we need
	// to invalidate the right/bottom edge pixels that will
	// need to turn gray when the window is expanded, either before
	// or after the resize.
	
	this->InvalidateEdges(true, inWidthDelta, inHeightDelta);

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	this->InvalidateEdges(false, inWidthDelta, inHeightDelta);
	}

void AGADialogBoxPP::ActivateSelf()
	{
	// Redraw just the edge pixels that change based on active state.

	this->FocusDraw();
	this->DrawBackground(&mMacWindowP->portRect, false, true);
	
	Inherited::ActivateSelf();
	}

void AGADialogBoxPP::DeactivateSelf()
	{
	// Redraw just the edge pixels that change based on active state.

	this->FocusDraw();
	this->DrawBackground(&mMacWindowP->portRect, false, false);
	
	Inherited::DeactivateSelf();
	}

void AGADialogBoxPP::DrawBackground(const Rect* area, Boolean fill, Boolean active)
	{
	if (mBackgroundKind != kNoBackgroundPaint)
		AGABackgroundPaint(area, fill, mBackgroundKind, active, this->HasAttribute(windAttr_SizeBox));
	}

void AGADialogBoxPP::InvalidateEdges(Boolean before, Int16 inWidthDelta, Int16 inHeightDelta)
	{
	if ((mBackgroundKind == kRaisedModelessBackground) ||
		(mBackgroundKind == kRaisedModalBackground))
		{
		// Invalidate the extra "edge" pixels that need to be redrawn
		// when the window is resized.

		Rect	bounds;
		Rect	rectToInvalidate;
		
		this->CalcPortFrameRect(bounds);

		if (((inWidthDelta > 0) && before) ||
			((inWidthDelta < 0) && ! before))
			{
			rectToInvalidate = bounds;
			rectToInvalidate.left = rectToInvalidate.right - 1;
			this->InvalPortRect(&rectToInvalidate);
			}
		
		if (((inHeightDelta > 0) && before) ||
			((inHeightDelta < 0) && ! before))
			{
			rectToInvalidate = bounds;
			rectToInvalidate.top = rectToInvalidate.bottom - 1;
			this->InvalPortRect(&rectToInvalidate);
			}
		
		if (this->HasAttribute(windAttr_EraseOnUpdate))
			{
			rectToInvalidate = bounds;
			rectToInvalidate.left = rectToInvalidate.right - 16;
			rectToInvalidate.top = rectToInvalidate.bottom - 16;
			this->InvalPortRect(&rectToInvalidate);
			}
		}
	}

//
// AGADialogBoxPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGADialogBoxPP

AGADialogBoxPPX* AGADialogBoxPPX::CreateStream(LStream* inStream)
	{
	return new AGADialogBoxPPX(inStream);
	}

AGADialogBoxPPX::AGADialogBoxPPX(LStream* inStream)
: Inherited(inStream)
	{
	mBackgroundKind = (AGABackgroundKind) ReadStreamEnum(inStream);
	}

AGADialogBoxPPX::~AGADialogBoxPPX()
	{
	}

//
// AGAWhiteBackgroundAttachmentPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LAttachment

AGAWhiteBackgroundAttachmentPP* AGAWhiteBackgroundAttachmentPP::CreateStream(LStream* inStream)
	{
	return new AGAWhiteBackgroundAttachmentPP(inStream);
	}

AGAWhiteBackgroundAttachmentPP::AGAWhiteBackgroundAttachmentPP(MessageT inMessage, Boolean inExecuteHost)
: Inherited(inMessage, inExecuteHost)
	{
	mIsNotched = false;
	}

AGAWhiteBackgroundAttachmentPP::AGAWhiteBackgroundAttachmentPP(LStream *inStream)
: Inherited(inStream)
	{
	mIsNotched = false;
	}

Boolean AGAWhiteBackgroundAttachmentPP::Execute(MessageT inMessage, void *ioParam)
	{
	if (inMessage == msg_DrawOrPrint)
		{
		AGADrawingEnvironment	env;

		// Draw the background in white, inset to avoid the border
		// frame pixels.

		Rect	paneFrame = *((Rect*) ioParam);
		
		::RGBForeColor(&gAGARamp[rW]);
		::InsetRect(&paneFrame, 1, 1);
		
		if (mIsNotched)
			{
			Rect	r;
			
			r = paneFrame;
			r.right -= 15;
			::PaintRect(&r);
			
			r = paneFrame;
			r.bottom -= 15;
			::PaintRect(&r);
			}
		else
			{
			::PaintRect(&paneFrame);
			}
		}
	
	return true;	// owner should also do its work
	}

//
// AGANotchedWhiteBackgroundAttachmentPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAWhiteBackgroundAttachmentPP

AGANotchedWhiteBackgroundAttachmentPP* AGANotchedWhiteBackgroundAttachmentPP::CreateStream(LStream* inStream)
	{
	return new AGANotchedWhiteBackgroundAttachmentPP(inStream);
	}

AGANotchedWhiteBackgroundAttachmentPP::AGANotchedWhiteBackgroundAttachmentPP(MessageT inMessage, Boolean inExecuteHost)
: Inherited(inMessage, inExecuteHost)
	{
	mIsNotched = true;
	}

AGANotchedWhiteBackgroundAttachmentPP::AGANotchedWhiteBackgroundAttachmentPP(LStream *inStream)
: Inherited(inStream)
	{
	mIsNotched = true;
	}

//
// AGABorderFrameAttachmentPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LAttachment

AGABorderFrameAttachmentPP* AGABorderFrameAttachmentPP::CreateStream(LStream* inStream)
	{
	return new AGABorderFrameAttachmentPP(inStream);
	}

AGABorderFrameAttachmentPP::AGABorderFrameAttachmentPP(MessageT inMessage, Boolean inExecuteHost)
: Inherited(inMessage, inExecuteHost)
	{
	mIsNotched = false;
	}

AGABorderFrameAttachmentPP::AGABorderFrameAttachmentPP(LStream *inStream)
: Inherited(inStream)
	{
	mIsNotched = false;
	}

Boolean AGABorderFrameAttachmentPP::Execute(MessageT inMessage, void *ioParam)
	{
	if (inMessage == msg_DrawOrPrint)
		{
		AGADrawingEnvironment	env;

		// Draw the frame with shading outside.

		// If you bypass the RTTI here, the frame will always
		// draw enabled. With the 7/96 AGA spec revision, this
		// is now visually obvious.
		
		Boolean	ownerEnabled = true;
		
#ifndef NO_RTTI
		LPane*	ownerPane = dynamic_cast<LPane*>(mOwnerHost);
		if (ownerPane != NULL)
			ownerEnabled = ownerPane->IsEnabled();
#endif // NO_RTTI

		// The sunken border is drawn in the pixel outside the
		// pane frame. We need to make sure clipping includes
		// this.

		Rect	paneFrame = *((Rect*) ioParam);
		
		::InsetRect(&paneFrame, -1, -1);
		
		AGADrawBorderFrame(&paneFrame, ownerEnabled, mIsNotched, true /*drawFrame*/, &gAGARamp[r2]);
		}

	return true;	// owner should also do its work
	}

//
// AGANotchedBorderFrameAttachmentPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGABorderFrameAttachmentPP

AGANotchedBorderFrameAttachmentPP* AGANotchedBorderFrameAttachmentPP::CreateStream(LStream* inStream)
	{
	return new AGANotchedBorderFrameAttachmentPP(inStream);
	}

AGANotchedBorderFrameAttachmentPP::AGANotchedBorderFrameAttachmentPP(MessageT inMessage, Boolean inExecuteHost)
: Inherited(inMessage, inExecuteHost)
	{
	mIsNotched = true;
	}

AGANotchedBorderFrameAttachmentPP::AGANotchedBorderFrameAttachmentPP(LStream *inStream)
: Inherited(inStream)
	{
	mIsNotched = true;
	}

//
// AGAFocusBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LFocusBox

AGAFocusBoxPP* AGAFocusBoxPP::CreateStream(LStream* inStream)
	{
	return new AGAFocusBoxPP(inStream);
	}

AGAFocusBoxPP::AGAFocusBoxPP()
	{
	mIsNotched = false;
	}

AGAFocusBoxPP::AGAFocusBoxPP(const LFocusBox &inOriginal)
: Inherited(inOriginal)
	{
	mIsNotched = false;
	}

AGAFocusBoxPP::AGAFocusBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	mIsNotched = false;
	}

AGAFocusBoxPP::~AGAFocusBoxPP()
	{
	}

void AGAFocusBoxPP::DrawSelf()
	{
	// Draw the AGA-style focus frame.

	GDIterator	iter;
	Boolean		deep;
	Rect		bounds;

	RgnHandle borderRegion = ::NewRgn();
	(void) this->CalcLocalFrameRect(bounds);
	
	AGAComputeTargetBorderRegion(borderRegion, &bounds, mIsNotched);
		
	while (iter.NextDevice(deep))
		{
		if (deep)
			::RGBForeColor(&gAGARamp[r8]);
		else
			::RGBForeColor(&gAGARamp[rB]);

		PaintRgn(borderRegion);
		}
	
	::DisposeRgn(borderRegion);
	}

RgnHandle AGAFocusBoxPP::GetBoxRegion(const Rect &inFrame, const Rect &inRevealed) const
	{
	RgnHandle	boxRgn = ::NewRgn();

	AGAComputeTargetBorderRegion(boxRgn, &inFrame, mIsNotched);
	
	// Clip to revealed rect
	RgnHandle	revealedRgn = ::NewRgn();
	::RectRgn(revealedRgn, &inRevealed);
	::SectRgn(boxRgn, revealedRgn, boxRgn);
	::DisposeRgn(revealedRgn);
	
	return boxRgn;
	}

//
// AGANotchedFocusBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAFocusBoxPP

AGANotchedFocusBoxPP* AGANotchedFocusBoxPP::CreateStream(LStream* inStream)
	{
	return new AGANotchedFocusBoxPP(inStream);
	}

AGANotchedFocusBoxPP::AGANotchedFocusBoxPP()
	{
	mIsNotched = true;
	}

AGANotchedFocusBoxPP::AGANotchedFocusBoxPP(const LFocusBox &inOriginal)
: Inherited(inOriginal)
	{
	mIsNotched = true;
	}

AGANotchedFocusBoxPP::AGANotchedFocusBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	mIsNotched = true;
	}

AGANotchedFocusBoxPP::~AGANotchedFocusBoxPP()
	{
	}

