//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// See header file "GCNewControlsPP.h" for usage.
//

#include "GCNewControlsPP.h"
#include "GCCorePP.h"

#include <UReanimator.h>

#ifndef NO_RTTI
#include <typeinfo.h>
#endif // NO_RTTI

// The URegistrar::RegisterClass calls are making my eyes hurt.
// Define a macro to do it instead. We take advantage of the fact that
// we use the names class_ID and CreateStream in all of our classes.
#define GCPP_REGISTER_CLASS(classname) URegistrar::RegisterClass(classname::class_ID, (ClassCreatorFunc) classname::CreateStream)

void RegisterGCNewControlsPP()
	{
	//
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.
	//

	GCPP_REGISTER_CLASS(AGAIconPushButtonPP);
	GCPP_REGISTER_CLASS(AGAIconPushButtonPPX);
	GCPP_REGISTER_CLASS(AGAIconCheckBoxPP);
	GCPP_REGISTER_CLASS(AGAIconCheckBoxPPX);
	GCPP_REGISTER_CLASS(AGAIconRadioButtonPP);
	GCPP_REGISTER_CLASS(AGAIconRadioButtonPPX);
	GCPP_REGISTER_CLASS(AGAGroupBoxPP);
	GCPP_REGISTER_CLASS(AGAGroupBoxPPX);
	GCPP_REGISTER_CLASS(AGASecondaryGroupBoxPP);
	GCPP_REGISTER_CLASS(AGASliderPP);
	GCPP_REGISTER_CLASS(AGASliderPPX);
	GCPP_REGISTER_CLASS(AGALittleArrowsPP);
	GCPP_REGISTER_CLASS(AGALittleArrowsPPX);
	GCPP_REGISTER_CLASS(AGADisclosureTrianglePP);
	GCPP_REGISTER_CLASS(AGAProgressIndicatorPP);
	GCPP_REGISTER_CLASS(AGASeparatorPP);
	GCPP_REGISTER_CLASS(AGAPanelEnclosurePP);
	GCPP_REGISTER_CLASS(AGATabPanelPP);
	GCPP_REGISTER_CLASS(AGATabPanelPPX);
	GCPP_REGISTER_CLASS(AGASmallTabPanelPP);
	}

//
// AGAIconPushButtonPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAPushButtonPP

AGAIconPushButtonPP* AGAIconPushButtonPP::CreateStream(LStream* inStream)
	{
	return new AGAIconPushButtonPP(inStream);
	}

AGAIconPushButtonPP::AGAIconPushButtonPP()
	{
	mAGASuppressable = false;
	mIconID = MIconButtonObject::kNoIconID;
	mFrameType = kAutoFrame;
	mImageType = MIconButtonObject::kAutoIconFamily;
	}

AGAIconPushButtonPP::AGAIconPushButtonPP(LStream* inStream)
: Inherited(inStream)
	{
	// Use the mUserCon for the icon ID.

	mAGASuppressable = false;
	mIconID = mUserCon;
	mFrameType = kAutoFrame;
	mImageType = MIconButtonObject::kAutoIconFamily;
	}

AGAIconPushButtonPP::~AGAIconPushButtonPP()
	{
	}

void AGAIconPushButtonPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGAIconPushButton(&bounds, mIconID, mFrameType, mImageType));

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

//
// AGAIconPushButtonPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAIconPushButtonPP

AGAIconPushButtonPPX* AGAIconPushButtonPPX::CreateStream(LStream* inStream)
	{
	return new AGAIconPushButtonPPX(inStream);
	}

AGAIconPushButtonPPX::AGAIconPushButtonPPX(LStream* inStream)
: Inherited(inStream)
	{
	// Read the icon ID, frame type, and image type from the stream.

	inStream->ReadData(&mIconID, sizeof(mIconID));
	mFrameType = (ButtonFrameType) ReadStreamEnum(inStream);
	mImageType = (MIconButtonObject::ButtonImageType) ReadStreamEnum(inStream);
	}

AGAIconPushButtonPPX::~AGAIconPushButtonPPX()
	{
	}

//
// AGAIconCheckBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGACheckBoxPP

AGAIconCheckBoxPP* AGAIconCheckBoxPP::CreateStream(LStream* inStream)
	{
	return new AGAIconCheckBoxPP(inStream);
	}

AGAIconCheckBoxPP::AGAIconCheckBoxPP()
	{
	mAGASuppressable = false;
	mOffIconID = MIconButtonObject::kNoIconID;
	mOnIconID = MIconButtonObject::kNoIconID;
	mFrameType = kAutoFrame;
	mImageType = MIconButtonObject::kAutoIconFamily;
	}

AGAIconCheckBoxPP::AGAIconCheckBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	// Use the mUserCon for both icon IDs.

	mAGASuppressable = false;
	mOffIconID = mUserCon;
	mOnIconID = mUserCon;
	mFrameType = kAutoFrame;
	mImageType = MIconButtonObject::kAutoIconFamily;
	}

AGAIconCheckBoxPP::~AGAIconCheckBoxPP()
	{
	}

void AGAIconCheckBoxPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.
	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGAIconCheckBox(&bounds, AGAObject::kNoAutomaticState, mOffIconID, mOnIconID, mFrameType, mImageType));

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	mAGAObject->SetValue(this->GetValue(), AGAObject::kDontRedraw);
	}

//
// AGAIconCheckBoxPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAIconCheckBoxPP

AGAIconCheckBoxPPX* AGAIconCheckBoxPPX::CreateStream(LStream* inStream)
	{
	return new AGAIconCheckBoxPPX(inStream);
	}

AGAIconCheckBoxPPX::AGAIconCheckBoxPPX(LStream* inStream)
: Inherited(inStream)
	{
	// Read the icon IDs, frame type, and image type from the stream.

	inStream->ReadData(&mOffIconID, sizeof(mOffIconID));
	inStream->ReadData(&mOnIconID, sizeof(mOnIconID));
	mFrameType = (ButtonFrameType) ReadStreamEnum(inStream);
	mImageType = (MIconButtonObject::ButtonImageType) ReadStreamEnum(inStream);
	}

AGAIconCheckBoxPPX::~AGAIconCheckBoxPPX()
	{
	}

//
// AGAIconRadioButtonPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGARadioButtonPP

AGAIconRadioButtonPP* AGAIconRadioButtonPP::CreateStream(LStream* inStream)
	{
	return new AGAIconRadioButtonPP(inStream);
	}

AGAIconRadioButtonPP::AGAIconRadioButtonPP()
	{
	mAGASuppressable = false;
	mOffIconID = MIconButtonObject::kNoIconID;
	mOnIconID = MIconButtonObject::kNoIconID;
	mFrameType = kAutoFrame;
	mImageType = MIconButtonObject::kAutoIconFamily;
	}

AGAIconRadioButtonPP::AGAIconRadioButtonPP(LStream* inStream)
: Inherited(inStream)
	{
	// Use the mUserCon for both icon IDs.

	mAGASuppressable = false;
	mOffIconID = mUserCon;
	mOnIconID = mUserCon;
	mFrameType = kAutoFrame;
	mImageType = MIconButtonObject::kAutoIconFamily;
	}

AGAIconRadioButtonPP::~AGAIconRadioButtonPP()
	{
	}

void AGAIconRadioButtonPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.
	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGAIconRadioButton(&bounds, kNoGroupID, kNoGroupID, AGAObject::kNoAutomaticState, mOffIconID, mOnIconID, mFrameType, mImageType));

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	mAGAObject->SetValue(this->GetValue(), AGAObject::kDontRedraw);
	}

//
// AGAIconRadioButtonPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAIconRadioButtonPP

AGAIconRadioButtonPPX* AGAIconRadioButtonPPX::CreateStream(LStream* inStream)
	{
	return new AGAIconRadioButtonPPX(inStream);
	}

AGAIconRadioButtonPPX::AGAIconRadioButtonPPX(LStream* inStream)
: Inherited(inStream)
	{
	// Read the icon IDs, frame type, and image type from the stream.

	inStream->ReadData(&mOffIconID, sizeof(mOffIconID));
	inStream->ReadData(&mOnIconID, sizeof(mOnIconID));
	inStream->ReadData(&mFrameType, sizeof(mFrameType));
	inStream->ReadData(&mImageType, sizeof(mImageType));
	}

AGAIconRadioButtonPPX::~AGAIconRadioButtonPPX()
	{
	}

//
// AGAGroupBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LGroupBox

AGAGroupBoxPP* AGAGroupBoxPP::CreateStream(LStream* inStream)
	{
	return new AGAGroupBoxPP(inStream);
	}

AGAGroupBoxPP::AGAGroupBoxPP()
	{
	// Assume primary group type, no gap pane.

	mAGAObject = NULL;
	mIsPrimaryGroup = true;
	mGapPaneID = 0;
	}

AGAGroupBoxPP::AGAGroupBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	// Assume primary group type, and use the mUserCon as the
	// gap pane's ID.

	mAGAObject = NULL;
	mIsPrimaryGroup = true;
	mGapPaneID = mUserCon;
	}

AGAGroupBoxPP::~AGAGroupBoxPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAGroupBoxPP::SetDescriptor(ConstStringPtr inDescriptor)
	{
	Inherited::SetDescriptor(inDescriptor);
	
	// Install the new text in the AGA object. Don't tell it
	// to redraw; the redraw comes from the caller.

	Str255	newText;
	this->GetDescriptor(newText);
	mAGAObject->SetTitle(newText, AGAObject::kDontRedraw);
	}

void AGAGroupBoxPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAGroupBoxPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAGroupBoxPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

	this->CreateAGAObject();
	}

void AGAGroupBoxPP::DrawSelf()
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAGroupBoxPP::EnableSelf()
	{
	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGAGroupBoxPP::DisableSelf()
	{
	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGAGroupBoxPP::ActivateSelf()
	{
	if (! gAGAStdControls) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGAGroupBoxPP::DeactivateSelf()
	{
	if (! gAGAStdControls) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGAGroupBoxPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGAGroupBox(&bounds, AGATextStyle(mTxtrID), mIsPrimaryGroup, mText));

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	if ((mGapPaneID != 0) && (this->GetSuperView() != NULL))
		{
		LPane*	gapPane = this->GetSuperView()->FindPaneByID(mGapPaneID);

		if (gapPane != NULL)
			{
			Rect	bounds;
			
			(void) gapPane->CalcLocalFrameRect(bounds);

			mAGAObject->SetTitleGap(5 + bounds.right - bounds.left);
			}
		}
	}

//
// AGAGroupBoxPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAGroupBoxPP

AGAGroupBoxPPX* AGAGroupBoxPPX::CreateStream(LStream* inStream)
	{
	return new AGAGroupBoxPPX(inStream);
	}

AGAGroupBoxPPX::AGAGroupBoxPPX(LStream* inStream)
: Inherited(inStream)
	{
	// Read the group type and gap pane ID from the stream.

	inStream->ReadData(&mIsPrimaryGroup, sizeof(mIsPrimaryGroup));
	inStream->ReadData(&mGapPaneID, sizeof(mGapPaneID));
	}

AGAGroupBoxPPX::~AGAGroupBoxPPX()
	{
	}

//
// AGASecondaryGroupBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAGroupBoxPP

AGASecondaryGroupBoxPP* AGASecondaryGroupBoxPP::CreateStream(LStream* inStream)
	{
	return new AGASecondaryGroupBoxPP(inStream);
	}

AGASecondaryGroupBoxPP::AGASecondaryGroupBoxPP()
	{
	// Set to secondary group box type. Will be used when AGA object
	// is created.

	mIsPrimaryGroup = false;
	}

AGASecondaryGroupBoxPP::AGASecondaryGroupBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	// Set to secondary group box type. Will be used when AGA object
	// is created.

	mIsPrimaryGroup = false;
	}

AGASecondaryGroupBoxPP::~AGASecondaryGroupBoxPP()
	{
	}

//
// AGASliderPP ----------------------------------------------------
//

#pragma segment GrayCouncilPP2

#undef Inherited
#define Inherited LControl

AGASliderPP* AGASliderPP::CreateStream(LStream* inStream)
	{
	return new AGASliderPP(inStream);
	}

AGASliderPP::AGASliderPP()
	{
	mAGAObject = NULL;
	mNotificationRoutine = NULL;
	mUserData = 0;
	mLabelsResourceID = 0;			// STR# resource ID of the slider labels (0 for none)
	mLabelsTextTraitsID = -1;		// -1 means standard labels style
	mSliderJustification = teFlushDefault;
	}

AGASliderPP::AGASliderPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	mNotificationRoutine = NULL;
	mUserData = 0;
	mLabelsResourceID = mUserCon;	// STR# resource ID of the slider labels (0 for none)
	mLabelsTextTraitsID = -1;		// -1 means standard labels style
	mSliderJustification = teFlushDefault;
	}

AGASliderPP::~AGASliderPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGASliderPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGASliderPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGASliderPP::InstallNotificationRoutine(AGANotifyPPPtr notificationRoutine, void* userData)
	{
	// Save the notification routine fn pointer and user data
	// for notification handling.

	mNotificationRoutine = notificationRoutine;
	mUserData = userData;
	}

void AGASliderPP::HandleNotification(SInt32 dataValue)
	{
	// Update the pane value, call the installed notification
	// function, and propagate our event number.

	this->SetValue(dataValue);

	if (mNotificationRoutine != NULL)
		(*mNotificationRoutine)(this, dataValue, mUserData);

//	this->BroadcastValueMessage();	not necessary -- SetValue already did it

	(void) this->FocusDraw();	// HandleEvent may have caused another view to draw
	}

void AGASliderPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

	this->CreateAGAObject();
	}

void AGASliderPP::DrawSelf()
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGASliderPP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		if (mAGAObject->TrackMouse(inMouseDown.whereLocal))
			this->HotSpotResult(this->FindHotSpot(inMouseDown.whereLocal));
		}
	}

void AGASliderPP::HotSpotResult(SInt16 /*inHotSpot*/)
	{
	this->SetValue(mAGAObject->GetValue());
	}

void AGASliderPP::EnableSelf()
	{
	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGASliderPP::DisableSelf()
	{
	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGASliderPP::ActivateSelf()
	{
	if (! gAGAStdControls) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGASliderPP::DeactivateSelf()
	{
	if (! gAGAStdControls) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGASliderPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	if (mLabelsTextTraitsID == -1)
		ThrowIfNil_(mAGAObject = new AGASlider(&bounds, mMinValue, mMaxValue, mValue, gAGAStdSmallStyle, mLabelsResourceID));
	else
		ThrowIfNil_(mAGAObject = new AGASlider(&bounds, mMinValue, mMaxValue, mValue, AGATextStyle(mLabelsTextTraitsID), mLabelsResourceID));

	mAGAObject->InstallNotificationRoutine(AGASliderPP::RealAGANotifier, this);

	if (mSliderJustification != teFlushDefault)
		mAGAObject->SetJustification(mSliderJustification);

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	
	// The auto-labeling may have increased the range. We need to
	// make sure the LControl min/max respect that, or SetValue
	// won't work.
	
	SInt32	newMin;
	SInt32	newMax;

	mAGAObject->GetRange(&newMin, &newMax);
	this->SetMinValue(newMin);
	this->SetMaxValue(newMax);

	// Slider has to erase when drawing, so make sure that
	// it doesn't assume we're on a gray background.
	
	this->FocusDraw();
	this->ApplyForeAndBackColors();
	InstallAGABackgroundColors(mAGAObject, kInstallAlways);
	}

void AGASliderPP::SetValue(SInt32 inValue)
	{
	// Set the slider value.

	Inherited::SetValue(inValue);

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		mAGAObject->SetValue(mValue, this->IsVisible());
		}
	}

void AGASliderPP::SetMinValue(SInt32 inMinValue)
	{
	// Set the slider min.

	Inherited::SetMinValue(inMinValue);

	if (mAGAObject != NULL)
		{
		SInt32	oldMin;
		SInt32	oldMax;

		this->FocusDraw();
		
		mAGAObject->GetRange(&oldMin, &oldMax);
		mAGAObject->SetRange(inMinValue, oldMax, this->IsVisible());
		}
	}

void AGASliderPP::SetMaxValue(SInt32 inMaxValue)
	{
	// Set the slider max.

	Inherited::SetMaxValue(inMaxValue);

	if (mAGAObject != NULL)
		{
		SInt32	oldMin;
		SInt32	oldMax;

		this->FocusDraw();
		
		mAGAObject->GetRange(&oldMin, &oldMax);
		mAGAObject->SetRange(oldMin, inMaxValue, this->IsVisible());
		}
	}

void AGASliderPP::RealAGANotifier(AGATrackingIndicator* /*theIndicator*/, SInt32 dataValue, void* userData)
	{
	// This is a static function. Cast the user data to get
	// the AGASliderPP object, have it handle the notification.

	((AGASliderPP*) userData)->HandleNotification(dataValue);
	}

//
// AGASliderPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGASliderPP

AGASliderPPX* AGASliderPPX::CreateStream(LStream* inStream)
	{
	return new AGASliderPPX(inStream);
	}

AGASliderPPX::AGASliderPPX(LStream* inStream)
: Inherited(inStream)
	{
	// Read the labels STR# ID, labels text traits ID, and
	// slider justification from the stream.

	inStream->ReadData(&mLabelsResourceID, sizeof(mLabelsResourceID));
	inStream->ReadData(&mLabelsTextTraitsID, sizeof(mLabelsTextTraitsID));
	inStream->ReadData(&mSliderJustification, sizeof(mSliderJustification));
	}

AGASliderPPX::~AGASliderPPX()
	{
	}

//
// AGALittleArrowsPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LControl

AGALittleArrowsPP* AGALittleArrowsPP::CreateStream(LStream* inStream)
	{
	return new AGALittleArrowsPP(inStream);
	}

AGALittleArrowsPP::AGALittleArrowsPP()
	{
	mNotificationRoutine = NULL;
	mAGAObject = NULL;
	mLinkedNumberText = NULL;
	mLinkedPaneID = 0;
	}

AGALittleArrowsPP::AGALittleArrowsPP(LStream* inStream)
: Inherited(inStream)
	{
	mNotificationRoutine = NULL;
	mAGAObject = NULL;
	mLinkedNumberText = NULL;
	mLinkedPaneID = mUserCon;
	}

AGALittleArrowsPP::~AGALittleArrowsPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGALittleArrowsPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGALittleArrowsPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGALittleArrowsPP::InstallNotificationRoutine(AGANotifyPPPtr notificationRoutine, void* userData)
	{
	// Save the notification routine fn pointer and user data
	// for notification handling.

	mNotificationRoutine = notificationRoutine;
	mUserData = userData;
	}

void AGALittleArrowsPP::HandleNotification(SInt32 deltaValue)
	{
	// Update the linked LEditField, if any, call the
	// installed notification function, and propagate our
	// hot spot result.
	
	SInt32	oldValue;
	SInt32	newValue;

	if (mLinkedNumberText == NULL)
		oldValue = this->GetValue();
	else
		oldValue = mLinkedNumberText->GetValue();
	
	// Set the LControl value, which will constrain between min and max.
	this->SetValue(oldValue + deltaValue);
	
	// Get the new value, which may be been constrained.
	newValue = this->GetValue();
	
	// Only set new linked text value if changed.
	if ((mLinkedNumberText != NULL) && (newValue != oldValue))
		{
		mLinkedNumberText->SetValue(newValue);
		mLinkedNumberText->Draw(NULL);
		mLinkedNumberText->DontRefresh();	// otherwise attachments will flicker after mouse tracking completes
		}

	if (mNotificationRoutine != NULL)
		(*mNotificationRoutine)(this, deltaValue, mUserData);
	
	this->FocusDraw();	// linked edit field prob'ly grabbed focus
	}

void AGALittleArrowsPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

	this->CreateAGAObject();
	}

void AGALittleArrowsPP::DrawSelf()
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGALittleArrowsPP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		// Note that we don't do anything after tracking; the
		// notification routine updates the value continuously
		// while tracking occurs.
		(void) mAGAObject->TrackMouse(inMouseDown.whereLocal);
		}
	}

void AGALittleArrowsPP::EnableSelf()
	{
	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGALittleArrowsPP::DisableSelf()
	{
	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGALittleArrowsPP::ActivateSelf()
	{
	if (! gAGAStdControls) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGALittleArrowsPP::DeactivateSelf()
	{
	if (! gAGAStdControls) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGALittleArrowsPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGALittleArrows(&bounds));
	
	mAGAObject->InstallNotificationRoutine(AGALittleArrowsPP::RealAGANotifier, this);

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	if (mLinkedPaneID != 0)
		{
		// If you bypass the RTTI here, you must ensure that the
		// pane ID specifies a pane that is a subclass of LEditField.
		
		LPane*	linkedPane = mSuperView->FindPaneByID(mLinkedPaneID);
		
		if (linkedPane != NULL)
			{
#ifndef NO_RTTI
			mLinkedNumberText = dynamic_cast<LEditField*>(linkedPane);
#else // NO_RTTI
			mLinkedNumberText = (LEditField*) linkedPane;
#endif // NO_RTTI

			if (mLinkedNumberText != NULL)
				mLinkedNumberText->SetValue(mValue);
			}
		}
	}

void AGALittleArrowsPP::RealAGANotifier(AGALittleArrows* /*theAGAObject*/, SInt32 deltaValue, void* userData)
	{
	// This is a static function. Cast the user data to get
	// the AGAScrollerScrollBarMA object, have it handle the notification.

	((AGALittleArrowsPP*) userData)->HandleNotification(deltaValue);
	}

//
// AGALittleArrowsPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGALittleArrowsPP

AGALittleArrowsPPX* AGALittleArrowsPPX::CreateStream(LStream* inStream)
	{
	return new AGALittleArrowsPPX(inStream);
	}

AGALittleArrowsPPX::AGALittleArrowsPPX(LStream* inStream)
: Inherited(inStream)
	{
	// Read the linked LEditField pane ID from the stream.

	inStream->ReadData(&mLinkedPaneID, sizeof(mLinkedPaneID));
	}

AGALittleArrowsPPX::~AGALittleArrowsPPX()
	{
	}

//
// AGADisclosureTrianglePP ----------------------------------------------------
//

#undef Inherited
#define Inherited LControl

AGADisclosureTrianglePP* AGADisclosureTrianglePP::CreateStream(LStream* inStream)
	{
	return new AGADisclosureTrianglePP(inStream);
	}

AGADisclosureTrianglePP::AGADisclosureTrianglePP()
	{
	mAGAObject = NULL;
	mMaxValue = 1;	// make sure SetValue doesn't get stuck at value = 0
	}

AGADisclosureTrianglePP::AGADisclosureTrianglePP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	mMaxValue = 1;	// make sure SetValue doesn't get stuck at value = 0
	}

AGADisclosureTrianglePP::~AGADisclosureTrianglePP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGADisclosureTrianglePP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGADisclosureTrianglePP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGADisclosureTrianglePP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

	this->CreateAGAObject();
	}

void AGADisclosureTrianglePP::DrawSelf()
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGADisclosureTrianglePP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		if (mAGAObject->TrackMouse(inMouseDown.whereLocal))
			this->HotSpotResult(this->FindHotSpot(inMouseDown.whereLocal));
		}
	}

void AGADisclosureTrianglePP::HotSpotResult(SInt16 /*inHotSpot*/)
	{
	this->SetValue(mAGAObject->GetState() ? 1 : 0);
	}

void AGADisclosureTrianglePP::EnableSelf()
	{
	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGADisclosureTrianglePP::DisableSelf()
	{
	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGADisclosureTrianglePP::ActivateSelf()
	{
	if (! gAGAStdControls) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGADisclosureTrianglePP::DeactivateSelf()
	{
	if (! gAGAStdControls) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGADisclosureTrianglePP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.
	
	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGADisclosureTriangle(&bounds, AGAObject::kAutomaticState));
	
	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	if (mValue != 0)	// allow initial state to be set to kDisclosed
		mAGAObject->SetState(AGADisclosureTriangle::kDisclosedState, AGAObject::kDontRedraw);

	// Disclosure triangle has to erase when animating, so make sure that
	// it doesn't assume we're on a gray background.

	this->FocusDraw();
	this->ApplyForeAndBackColors();
	InstallAGABackgroundColors(mAGAObject, kInstallAlways);
	}

void AGADisclosureTrianglePP::SetValue(SInt32 inValue)
	{
	// Set the actual LControl value and the AGA object state.

	Inherited::SetValue(inValue);

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetState(mValue != 0, this->IsVisible());
		}
	}

//
// AGAProgressIndicatorPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LControl

AGAProgressIndicatorPP* AGAProgressIndicatorPP::CreateStream(LStream* inStream)
	{
	return new AGAProgressIndicatorPP(inStream);
	}

AGAProgressIndicatorPP::AGAProgressIndicatorPP()
	{
	mAGAObject = NULL;
	}

AGAProgressIndicatorPP::AGAProgressIndicatorPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	}

AGAProgressIndicatorPP::~AGAProgressIndicatorPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAProgressIndicatorPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAProgressIndicatorPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAProgressIndicatorPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

	this->CreateAGAObject();
	}

void AGAProgressIndicatorPP::DrawSelf()
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAProgressIndicatorPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.
	
	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGAProgressIndicator(&bounds, mMinValue, mMaxValue));
	
	mAGAObject->SetValue(mValue, AGAObject::kDontRedraw);
	
	if (mUserCon != 0)
		this->StartAutoAnimate();
	}

void AGAProgressIndicatorPP::SetValue(SInt32 inValue)
	{
	// Set the value of the LControl and the AGA object.

	Inherited::SetValue(inValue);

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		mAGAObject->SetValue(mValue, this->IsVisible());
		}
	}

void AGAProgressIndicatorPP::SetMinValue(SInt32 inMinValue)
	{
	// Set the min of the LControl and the AGA object.

	Inherited::SetMinValue(inMinValue);

	if (mAGAObject != NULL)
		{
		SInt32	oldMin;
		SInt32	oldMax;

		this->FocusDraw();
		
		mAGAObject->GetRange(&oldMin, &oldMax);
		mAGAObject->SetRange(inMinValue, oldMax, this->IsVisible());
		}
	}

void AGAProgressIndicatorPP::SetMaxValue(SInt32 inMaxValue)
	{
	// Set the max of the LControl and the AGA object.

	Inherited::SetMaxValue(inMaxValue);

	if (mAGAObject != NULL)
		{
		SInt32	oldMin;
		SInt32	oldMax;

		this->FocusDraw();
		
		mAGAObject->GetRange(&oldMin, &oldMax);
		mAGAObject->SetRange(oldMin, inMaxValue, this->IsVisible());
		}
	}

void AGAProgressIndicatorPP::SpendTime(const EventRecord& /*inMacEvent*/)
	{
	// Let the AGAObject do one animation step.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->QuantizedAnimate();
		}
	}

void AGAProgressIndicatorPP::StartAutoAnimate()
	{
	// Let LPeriodical remove us from its idle queue.
	
	this->StartIdling();
	}

void AGAProgressIndicatorPP::StopAutoAnimate()
	{
	// Tell LPeriodical to add us to its idle queue.
	
	this->StopIdling();
	}

//
// AGASeparatorPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LPane

AGASeparatorPP* AGASeparatorPP::CreateStream(LStream* inStream)
	{
	return new AGASeparatorPP(inStream);
	}

AGASeparatorPP::AGASeparatorPP()
	{
	mAGAObject = NULL;
	}

AGASeparatorPP::AGASeparatorPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	}

AGASeparatorPP::~AGASeparatorPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGASeparatorPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGASeparatorPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGASeparatorPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

	this->CreateAGAObject();
	}

void AGASeparatorPP::DrawSelf()
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGASeparatorPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.
	
	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGASeparator(&bounds));
	}

//
// AGAPanelEnclosurePP ----------------------------------------------------
//

#undef Inherited

AGAPanelEnclosurePP* AGAPanelEnclosurePP::CreateStream(LStream* inStream)
	{
	return new AGAPanelEnclosurePP(inStream);
	}

AGAPanelEnclosurePP::AGAPanelEnclosurePP(LStream* inStream)
: LView(inStream)
	{
	mSavedTarget = NULL;
	}

AGAPanelEnclosurePP::AGAPanelEnclosurePP()
	{
	mSavedTarget = NULL;
	}

AGAPanelEnclosurePP::~AGAPanelEnclosurePP()
	{
	}

void AGAPanelEnclosurePP::SwitchPanelOut()
	{
	if (this->IsOnDuty())
		{
		mSavedTarget = LCommander::GetTarget();

		this->SwitchTarget(this);
		}
	}

void AGAPanelEnclosurePP::SwitchPanelIn()
	{
	if (mSavedTarget != NULL)
		this->SwitchTarget(mSavedTarget);

	mSavedTarget = NULL;

	if (this->GetLatentSub() != NULL)
		this->RestoreTarget();
	}

//
// AGATabPanelPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LView

AGATabPanelPP* AGATabPanelPP::CreateStream(LStream* inStream)
	{
	return new AGATabPanelPP(inStream);
	}

AGATabPanelPP::AGATabPanelPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	mLabelsTextTraitsID = 0;		// System font is std for large; Geneva 10 bold is std for small
	mLabelsResourceID = mUserCon;	// STR# resource ID of the slider labels (0 for none)
	mPanelContainerView = NULL;
	mActivePanelView = NULL;
	mTempSupressDisable = false;
	}

AGATabPanelPP::AGATabPanelPP()
	{
	mAGAObject = NULL;
	mLabelsTextTraitsID = 0;		// System font is std for large; Geneva 10 bold is std for small
	mLabelsResourceID = 0;			// STR# resource ID of the slider labels (0 for none)
	mPanelContainerView = NULL;
	mActivePanelView = NULL;
	mTempSupressDisable = false;
	}

AGATabPanelPP::~AGATabPanelPP()
	{
	// Delete AGA object if we successfully allocated it.
	// We also need to make sure that all panel views not currently
	// installed get destructed. Unfortunately, it seems that
	// the LStdControl-based classes try to Focus during destruction,
	// and don't check to see if they're in a port; this causes
	// FocusDraw to throw an exception. So it's easiest if we just
	// suck all the panels into our container and let them get deleted
	// by LView::DeleteAllSubPanes, which is called by our parent class
	// destructor.

	if (mAGAObject != NULL)
		{
/*
		SInt32	numTabs;

		numTabs = mAGAObject->GetNumTabs();
		
		for (SInt32 tabIndex = 0; tabIndex < numTabs; tabIndex++)
			{
			if (tabIndex != mAGAObject->GetCurrentTab())
				this->SwitchPanelIn(tabIndex, false);	// make it part of the hierarchy for deletion
			}
*/
		delete mAGAObject;
		}
	}

void AGATabPanelPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGATabPanelPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGATabPanelPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

	this->CreateAGAObject();
	
	// Create the container view that we'll load panels in and out of.
	
	this->CreateContainerView();
	}

void AGATabPanelPP::DrawSelf()
	{
	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGATabPanelPP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		if (mAGAObject->TrackMouse(inMouseDown.whereLocal))
			this->HandleTabSwitch(mAGAObject->GetLastTabClicked());
		}
	}

void AGATabPanelPP::EnableSelf()
	{
	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGATabPanelPP::DisableSelf()
	{
	// Let the AGAObject disable itself.

	if ((mAGAObject != NULL) && ! mTempSupressDisable)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGATabPanelPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGATabPanel(&bounds, 0, AGATextStyle(mLabelsTextTraitsID), mLabelsResourceID));

	// Tab panel has to erase when drawing, so make sure that
	// it doesn't assume we're on a gray background.

	this->FocusDraw();
	this->ApplyForeAndBackColors();
	InstallAGABackgroundColors(mAGAObject, kInstallAlways);
	}

void AGATabPanelPP::CreateContainerView()
	{
	if (mAGAObject->HasFrame())	// if no frame, we just run the tabs, not the panels
		{
		// Create a subview to contain the panels that we load and unload.
		
		LPane::SetDefaultView(this);

		SInt16		tabHeight = mAGAObject->GetTabHeight();
		SPaneInfo	paneInfo;
		SViewInfo	viewInfo;
		
		// Init the paneInfo with our own coordinates & bindings.
		paneInfo.paneID = 0;
		paneInfo.width = mFrameSize.width;
		paneInfo.height = mFrameSize.height;
		paneInfo.visible = true;
		paneInfo.enabled = true;
		paneInfo.bindings = mFrameBinding;
		paneInfo.left = mFrameLocation.h;
		paneInfo.top = mFrameLocation.v;
		paneInfo.userCon = 0;
		paneInfo.superView = mSuperView;
		
		// Inset the bounds by 4 pixels, plus the tab height at the top.
		paneInfo.top += (tabHeight + 4);
		paneInfo.left += 4;
		paneInfo.height -= (tabHeight + 8);
		paneInfo.width -= 8;
		
		// Init the viewInfo with the pane's dimensions.
		viewInfo.imageSize.width = paneInfo.width;
		viewInfo.imageSize.height = paneInfo.height;
		viewInfo.scrollPos.h = 0;
		viewInfo.scrollPos.v = 0;
		viewInfo.scrollUnit.h = 0;
		viewInfo.scrollUnit.v = 0;
		viewInfo.reconcileOverhang = 0;

		// Create the view and make it a subview of ourself.
		mPanelContainerView = new LView(paneInfo, viewInfo);
		mPanelContainerView->FinishCreate();
		mPanelContainerView->PutInside(this, true);
		}
	}

void AGATabPanelPP::SetValue(SInt32 inValue)
	{
	// Switch to the specified tab index.
	
	this->SwitchPanels(inValue);
	}

SInt32 AGATabPanelPP::GetValue() const
	{
	// Return the index of the currently selected tab.

	if (mAGAObject == NULL)
		return 0;
	else
		return mAGAObject->GetCurrentTab();
	}
	
void AGATabPanelPP::Disable()
	{
	// Need to change the way disable works because of the sequence of
	// drawing that normally occurs for a view with subpanes. The normal
	// sequence draws the parent view (us) after drawing the subpanes,
	// which is wrong because our disable drawing has to paint the whole
	// pane gray, which covers all subpanes. So instead we get ourself
	// to draw disabled first, then let the normal stuff occur while
	// setting a flag that tells us to skip the draw disabled message
	// that we'll get during that processing. Kinda ugly, but it works.
	
	if (this->IsEnabled())
		this->DisableSelf();

	mTempSupressDisable = true;
	Inherited::Disable();
	mTempSupressDisable = false;
	}

void AGATabPanelPP::SwitchPanels(SInt32 newTabIndex)
	{
	// Switch to the specified tab index.
	
	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		mAGAObject->SetCurrentTab(newTabIndex, this->IsVisible());
		}

	this->SwitchPanelOut();
	this->SwitchPanelIn(newTabIndex);
	}

void AGATabPanelPP::HandleTabSwitch(SInt32 newTabIndex)
	{
	// If it's OK to switch tabs (may need to validate current
	// panel before switching), do the switch.

	if (newTabIndex != mAGAObject->GetCurrentTab())
		if (this->ValidatePanel(mAGAObject->GetCurrentTab()))
			this->SetValue(newTabIndex);
	}

void AGATabPanelPP::AddPanel(ResIDT panelResourceID, StringPtr tabLabel)
	{
	// Create a new tab and assign the specified panel to it.

	if (mAGAObject != NULL)
		{
		Str255	actualTabLabel;
		SInt32	numTabs;

		if (tabLabel != NULL)
			AGA_PLstrcpy(actualTabLabel, tabLabel);
		else
			{
			Handle	ppobHandle;
			SInt16	ppobID;
			OSType	ppobType;
			
			ThrowIfNil_(ppobHandle = ::GetResource('PPob', panelResourceID));
			
			::GetResInfo(ppobHandle, &ppobID, &ppobType, actualTabLabel);
			
			// Don't release it -- we're about to cause it to be read anyway.
			}

		numTabs = mAGAObject->GetNumTabs();
		
		mAGAObject->SetNumTabs(numTabs + 1, AGATabPanel::kDontShrink);
		
		this->InstallPanel(numTabs, panelResourceID, actualTabLabel);
		}
	}

void AGATabPanelPP::InstallPanel(SInt32 tabIndex, ResIDT panelResourceID, StringPtr tabLabel)
	{
	// Assign the specified panel to the specified tab.

	if (mAGAObject != NULL)
		{
		LView*	newPanelView;
		SInt32	numTabs;

		numTabs = mAGAObject->GetNumTabs();
		
		if (tabIndex < numTabs)
			{
			// To keep the command chain in an orderly state, we need to create the
			// panel view as a subcommander of another commander. In the MacApp world,
			// this would just be its superview. But in PowerPlant, LPane is not a
			// commander; only certain pane classes mix in LCommander. So we have to
			// walk up the superview chain until we find an LCommander subclass object,
			// via dynamic casting.
			//
			// If you bypass the RTTI here, you must ensure that the tab panel view
			// lives in a view hierarchy with an LCommander (such as LWindow or LDialogBox)
			// at the root.
			
			LCommander*	nearestSuperCommander = NULL;
			LView*		aSuperView = mPanelContainerView;
			
			while ((nearestSuperCommander == NULL) && (aSuperView != NULL))
				{
#ifndef NO_RTTI

				// Found if this superview is an LCommander.
				nearestSuperCommander = dynamic_cast<LCommander*>(aSuperView);

#else // NO_RTTI

				// Found if this superview is the root view (window or dialog).
				if (aSuperView->GetSuperView() == NULL)
					nearestSuperCommander = (LCommander*) aSuperView;

#endif // NO_RTTI

				aSuperView = aSuperView->GetSuperView();
				}
			
			TurnOnCustomAGABackgroundColors(&gAGARamp[r1], &gAGARamp[r2]);
			ThrowIfNil_(newPanelView = UReanimator::CreateView(panelResourceID, mPanelContainerView, nearestSuperCommander));
			TurnOffCustomAGABackgroundColors();
			
			newPanelView->Hide();
			
			mAGAObject->SetTabUserData(tabIndex, newPanelView);
			
			if (tabLabel != NULL)
				{
				mAGAObject->SetTabLabel(tabIndex, tabLabel);
				mAGAObject->CalculateTabWidths();
				}

			// If this panel corresponds to the initial value, switch it in.

			if (this->GetValue() == tabIndex)
				this->SwitchPanels(tabIndex);
			}
		}
	}

void AGATabPanelPP::RemoveTab(SInt32 tabIndex, Boolean redraw)
	{
	// Remove the specified tab from the AGA object.

	mAGAObject->RemoveTab(tabIndex);
	
	if (redraw)
		{
		this->FocusDraw();
		mAGAObject->DrawObject();
		}
	}

LView* AGATabPanelPP::GetPanelView(SInt32 tabIndex)
	{
	// Return the top LView* of the specified tab's panel view hierarchy.

	if (mAGAObject == NULL)
		return NULL;
	else
		return (LView*) mAGAObject->GetTabUserData(tabIndex);
	}

Boolean AGATabPanelPP::ValidatePanel(SInt32 /*currentTabIndex*/)
	{
	// (Override for app-specific validation.)
	//
	// Return true if the current (specified) panel is valid and
	// it's OK to switch to a different panel. Return false otherwise
	// and the tab switch will not occur; it's OK to alert the user
	// of the invalid data before returning false.
	//

	return true;
	}

void AGATabPanelPP::SwitchPanelOut()
	{
	// Make the current panel invisible and disabled.

	if (mActivePanelView != NULL)
		{
#ifndef NO_RTTI

		// If the current panel root is an AGAPanelEnclosurePP,
		// we can give it a chance to clean up its target.
		// If you bypass the RTTI here, the current panel may not
		// give up the target.
		
		AGAPanelEnclosurePP*	enclosure = dynamic_cast<AGAPanelEnclosurePP*>(mActivePanelView);

		if (enclosure != NULL)
			enclosure->SwitchPanelOut();

#endif // NO_RTTI

		mActivePanelView->Hide();
		mActivePanelView->Disable();
		mActivePanelView = NULL;
		}
	}

void AGATabPanelPP::SwitchPanelIn(SInt32 newTabIndex)
	{
	// Make the specified tab's panel visible and enabled.

	if (mAGAObject != NULL)
		{
		LView*	newPanelView = (LView*) mAGAObject->GetTabUserData(newTabIndex);
		
		if (newPanelView != NULL)
			{
			mActivePanelView = newPanelView;

			newPanelView->Enable();
			newPanelView->Show();
			
			// If the current panel root is an AGAPanelEnclosurePP,
			// we can give it a chance to restore its target.
			// If you bypass the RTTI here, the new panel may not
			// grab the target.
			
#ifndef NO_RTTI

			AGAPanelEnclosurePP*	enclosure = dynamic_cast<AGAPanelEnclosurePP*>(newPanelView);

			if (enclosure != NULL)
				enclosure->SwitchPanelIn();

#endif // NO_RTTI
			}
		}
	}

//
// AGATabPanelPPX ----------------------------------------------------
//

#undef Inherited
#define Inherited AGATabPanelPP

AGATabPanelPPX* AGATabPanelPPX::CreateStream(LStream* inStream)
	{
	return new AGATabPanelPPX(inStream);
	}

AGATabPanelPPX::AGATabPanelPPX(LStream* inStream)
: Inherited(inStream)
	{
	// Read the labels STR# ID and labels text traits ID from the stream.

	inStream->ReadData(&mLabelsResourceID, sizeof(mLabelsResourceID));
	inStream->ReadData(&mLabelsTextTraitsID, sizeof(mLabelsTextTraitsID));
	}

AGATabPanelPPX::~AGATabPanelPPX()
	{
	}

//
// AGASmallTabPanelPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGATabPanelPP

AGASmallTabPanelPP* AGASmallTabPanelPP::CreateStream(LStream* inStream)
	{
	return new AGASmallTabPanelPP(inStream);
	}

AGASmallTabPanelPP::AGASmallTabPanelPP(LStream* inStream)
: Inherited(inStream)
	{
	}

AGASmallTabPanelPP::~AGASmallTabPanelPP()
	{
	}

void AGASmallTabPanelPP::CreateAGAObject()
	{
	Inherited::CreateAGAObject();
	
	// Set the tabs to use the standard small appearance.

	mAGAObject->SetTabSize(AGATabPanel::kSmallTabs);
	mAGAObject->SetLabelsStyle(gAGAStdBoldSmallStyle);
	}

