//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// The Gray Council provides a set of standard C++ classes that implement
// the standard Apple Grayscale Appearance. The core classes do not
// require any other code such as a particular class framework.
// There are separate files provided that plug The Gray Council core
// into class frameworks.
//
// This file defines PowerPlant adapter classes for the new controls defined
// in the AGA spec that do not exist in the Control Manager.
//
// Classes defined below:
//	AGAIconPushButtonPP -- AGAPushButtonPP (LStdButton) subclass for
//		AGAIconPushButton.
//	AGAIconPushButtonPPX -- AGAIconPushButtonPP (LStdButton) subclass for
//		AGAIconPushButton, that uses extended CPPb data for the icon ID.
//	AGAIconCheckBoxPP -- AGACheckBoxPP (LStdCheckBox) subclass for
//		AGAIconCheckBox.
//	AGAIconCheckBoxPPX -- AGAIconCheckBoxPP (LStdCheckBox) subclass for
//		AGAIconCheckBox, that uses extended CPPb data for the icon IDs.
//	AGAIconRadioButtonPP -- AGARadioButtonPP (LStdRadioButton) subclass
//		for AGAIconRadioButton.
//	AGAIconRadioButtonPPX -- AGAIconRadioButtonPP (LStdRadioButton) subclass
//		for AGAIconRadioButton, that uses extended CPPb data for the icon IDs.
//	AGAGroupBoxPP -- LGroupBox subclass for AGAGroupBox, with primary group
//		box appearance.
//	AGAGroupBoxPPX -- AGAGroupBoxPP (LGroupBox) subclass for AGAGroupBox,
//		that uses extended CPPb data for the box type and gap pane ID.
//	AGASecondaryGroupBoxPP -- AGAGroupBoxPP (LGroupBox) subclass for
//		AGAGroupBox, with secondary group box appearance.
//	AGASliderPP -- LControl subclass for AGASlider.
//	AGASliderPPX -- AGASliderPP (LControl) subclass for AGASlider, that
//		uses extended CPPb data for the label strings ID, style, and
//		slider justification.
//	AGALittleArrowsPP -- LControl subclass for AGALittleArrows.
//	AGALittleArrowsPPX -- AGALittleArrowsPP (LControl) subclass for
//		AGALittleArrows, that uses extended CPPb data for the linked
//		numeric LEditField pane ID.
//	AGADisclosureTrianglePP -- LControl subclass for AGADisclosureTriangle.
//	AGAProgressIndicatorPP -- LControl subclass for AGAProgressIndicator.
//	AGASeparatorPP -- LPane subclass for AGASeparator.
//	AGAPanelEnclosurePP -- LView+LCommander subclass, used by AGATabPanelPP to
//		handle target activation/deactivation between targets inside panels.
//	AGATabPanelPP -- LView subclass for AGATabPanel, set for standard large
//		tab format using the system font.
//	AGATabPanelPPX -- AGATabPanelPP (LView) subclass for AGATabPanel, that
//		uses extended CPPb data for the label strings ID, and text traits ID.
//		The font size of the text traits resource determines whether the tabs
//		are small or large format.
//	AGASmallTabPanelPP -- AGATabPanelPP (LView) subclass for AGATabPanel, set
//		for standard small tab format using Geneva 10 bold.
//

#pragma once

#ifndef __GCNEWCONTROLSPP__
#define __GCNEWCONTROLSPP__

#include <LAttachment.h>
#include <LCaption.h>
#include <LControl.h>
#include <LDialogBox.h>
#include <LEditField.h>
#include <LFocusBox.h>
#include <LGroupBox.h>
#include <LListBox.h>
#include <LScroller.h>
#include <LStdControl.h>
#include <LStream.h>
#include <LTextEdit.h>
#include <LWindow.h>
#include <UDrawingState.h>
#include <URegistrar.h>
#include <UTextTraits.h>

#include "GCNewControls.h"
#include "GCStdControlsPP.h"

extern void RegisterGCNewControlsPP();

#pragma mark AGAIconPushButtonPP

//
// AGAIconPushButtonPP ----------------------------------------------------
//
// LStdButton replacement subclass that implements an icon button that behaves
// like an LStdButton -- you click it does its HotSpotResult. The mUserCon is
// used to obtain the button's icon ID. You can also set the icon ID of the
// AGAIconPushButton directly (cast the mAGAObject to AGAIconPushButton*).
//

class AGAIconPushButtonPP : public AGAPushButtonPP
	{
	public:

		AGAIconPushButtonPP();
		AGAIconPushButtonPP(LStream* inStream);
		virtual ~AGAIconPushButtonPP();
		
		enum { class_ID = 'GCip' };
		static AGAIconPushButtonPP*	CreateStream(LStream* inStream);

	protected:

		virtual void	CreateAGAObject();
		
		// Extra AGA object initialization values
		SInt16			mIconID;

		ButtonFrameType	mFrameType;
		MIconButtonObject::ButtonImageType	mImageType;
	};

#pragma mark AGAIconPushButtonPPX

//
// AGAIconPushButtonPPX ----------------------------------------------------
//
// Extended AGAIconPushButtonPP class for use with custom pane template.
// The icon ID is read from the stream.
//

class AGAIconPushButtonPPX : public AGAIconPushButtonPP
	{
	public:

		AGAIconPushButtonPPX(LStream* inStream);
		virtual ~AGAIconPushButtonPPX();
		
		enum { class_ID = 'GXip' };
		static AGAIconPushButtonPPX*	CreateStream(LStream* inStream);
	};

#pragma mark AGAIconCheckBoxPP

//
// AGAIconCheckBoxPP ----------------------------------------------------
//
// LStdCheckBox replacement subclass that implements an icon button that behaves
// like a check box -- you click it and it toggles its on/off state. The mUserCon
// is used to obtain the button's icon ID. You can also set the icon ID of the
// AGAIconCheckBox directly (cast the mAGAObject to AGAIconCheckBox*). If you want
// the on/off states to have different icons, you can specify different icon IDs.
//

class AGAIconCheckBoxPP : public AGACheckBoxPP
	{
	public:

		AGAIconCheckBoxPP();
		AGAIconCheckBoxPP(LStream* inStream);
		virtual ~AGAIconCheckBoxPP();
		
		enum { class_ID = 'GCic' };
		static AGAIconCheckBoxPP*	CreateStream(LStream* inStream);

	protected:

		virtual void	CreateAGAObject();
		
		// Extra AGA object initialization values
		SInt16			mOffIconID;
		SInt16			mOnIconID;

		ButtonFrameType	mFrameType;
		MIconButtonObject::ButtonImageType	mImageType;
	};

#pragma mark AGAIconCheckBoxPPX

//
// AGAIconCheckBoxPPX ----------------------------------------------------
//
// Extended AGAIconCheckBoxPP class for use with custom pane template.
// The off and on icon IDs are read from the stream.
//

class AGAIconCheckBoxPPX : public AGAIconCheckBoxPP
	{
	public:

		AGAIconCheckBoxPPX(LStream* inStream);
		virtual ~AGAIconCheckBoxPPX();
		
		enum { class_ID = 'GXic' };
		static AGAIconCheckBoxPPX*	CreateStream(LStream* inStream);
	};

#pragma mark AGAIconRadioButtonPP

//
// AGAIconRadioButtonPP ----------------------------------------------------
//
// LStdRadioButton replacement subclass that implements an icon button that behaves
// like a radio button -- you click it and it turns on, and the others in its
// tab group are turned off. The mUserCon is used to obtain the
// button's icon ID. You can also set the icon ID of the AGAIconRadioButton
// directly (cast the mAGAObject to AGAIconRadioButton*). If you want the on/off
// states to have different icons, you can specify different icon IDs.
//

class AGAIconRadioButtonPP : public AGARadioButtonPP
	{
	public:

		AGAIconRadioButtonPP();
		AGAIconRadioButtonPP(LStream* inStream);
		virtual ~AGAIconRadioButtonPP();
		
		enum { class_ID = 'GCir' };
		static AGAIconRadioButtonPP*	CreateStream(LStream* inStream);

	protected:

		virtual void	CreateAGAObject();
		
		// Extra AGA object initialization values
		SInt16			mOffIconID;
		SInt16			mOnIconID;

		ButtonFrameType	mFrameType;
		MIconButtonObject::ButtonImageType	mImageType;
	};

#pragma mark AGAIconRadioButtonPPX

//
// AGAIconRadioButtonPPX ----------------------------------------------------
//
// Extended AGAIconRadioButtonPP class for use with custom pane template.
// The off and on icon IDs are read from the stream.
//

class AGAIconRadioButtonPPX : public AGAIconRadioButtonPP
	{
	public:

		AGAIconRadioButtonPPX(LStream* inStream);
		virtual ~AGAIconRadioButtonPPX();
		
		enum { class_ID = 'GXir' };
		static AGAIconRadioButtonPPX*	CreateStream(LStream* inStream);
	};

#pragma mark AGAGroupBoxPP

//
// AGAGroupBoxPP ----------------------------------------------------
//
// LGroupBox replacement subclass. Defaults to primary group box type.
//

class AGAGroupBoxPP : public LGroupBox
	{
	public:

		AGAGroupBoxPP();
		AGAGroupBoxPP(LStream* inStream);
		virtual ~AGAGroupBoxPP();
		
		AGAGroupBox*	mAGAObject;

		enum { class_ID = 'GCgb' };
		static AGAGroupBoxPP*	CreateStream(LStream* inStream);

		// PowerPlant overrides
		virtual void	SetDescriptor(ConstStringPtr inDescriptor);
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);

	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();

		virtual void	CreateAGAObject();
		
		// Extra AGA object initialization values
		Boolean	mIsPrimaryGroup;
		OSType	mGapPaneID;
	};

#pragma mark AGAGroupBoxPPX

//
// AGAGroupBoxPPX ----------------------------------------------------
//
// Extended AGAGroupBoxPP class for use with custom pane template.
// The group box type and the gap pane ID are read from the stream.
//

class AGAGroupBoxPPX : public AGAGroupBoxPP
	{
	public:

		AGAGroupBoxPPX(LStream* inStream);
		virtual ~AGAGroupBoxPPX();
		
		enum { class_ID = 'GXgb' };
		static AGAGroupBoxPPX*	CreateStream(LStream* inStream);
	};

#pragma mark AGASecondaryGroupBoxPP

//
// AGASecondaryGroupBoxPP ----------------------------------------------------
//
// LGroupBox replacement subclass with secondary group box type.
//

class AGASecondaryGroupBoxPP : public AGAGroupBoxPP
	{
	public:

		AGASecondaryGroupBoxPP();
		AGASecondaryGroupBoxPP(LStream* inStream);
		virtual ~AGASecondaryGroupBoxPP();
		
		enum { class_ID = 'GCgs' };
		static AGASecondaryGroupBoxPP*	CreateStream(LStream* inStream);
	};

#pragma mark AGASliderPP

//
// AGASliderPP -------------------------------------------------------
//
// LControl subclass. To set/get the value, use the LControl value
// methods. The mUserCon is used as the resource ID of an 'STR#' resource
// that contains the slider labels; in this case, the range is
// automatically set to match the number of labels, and the slider
// will have a pointy indicator. If the ID is zero or invalid, the
// slider will be rectangular.
//

class AGASliderPP : public LControl
	{
	public:

		AGASliderPP();
		AGASliderPP(LStream* inStream);
		virtual ~AGASliderPP();
		
		AGASlider*	mAGAObject;

		enum { class_ID = 'GCsl' };
		static AGASliderPP*	CreateStream(LStream* inStream);
		
		// PowerPlant overrides
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		virtual void	SetValue(SInt32 inValue);
		virtual void	SetMinValue(SInt32 inMinValue);
		virtual void	SetMaxValue(SInt32 inMaxValue);

		virtual void	InstallNotificationRoutine(AGANotifyPPPtr notificationRoutine, void* userData);
		virtual void	HandleNotification(SInt32 dataValue);
		
		AGANotifyPPPtr	mNotificationRoutine;
		void*			mUserData;
		
	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();
		virtual void	HotSpotResult(SInt16 inHotSpot);

		virtual void	CreateAGAObject();
		
		static void RealAGANotifier(AGATrackingIndicator* theIndicator, SInt32 dataValue, void* userData);
		
		// Extra AGA object initialization values
		ResIDT	mLabelsResourceID;
		ResIDT	mLabelsTextTraitsID;	// -1 for standard slider labels style
		SInt32	mSliderJustification;	// not used for unlabled sliders
	};

#pragma mark AGASliderPPX

//
// AGASliderPPX -------------------------------------------------------
//
// Extended AGASliderPP class for use with custom pane template.
// The labels STR# resource ID, text traits ID, and justification
// are read from the stream.
//

class AGASliderPPX : public AGASliderPP
	{
	public:

		AGASliderPPX(LStream* inStream);
		virtual ~AGASliderPPX();
		
		enum { class_ID = 'GXsl' };
		static AGASliderPPX*	CreateStream(LStream* inStream);
	};

#pragma mark AGALittleArrowsPP

//
// AGALittleArrowsPP -------------------------------------------------------
//
// LControl subclass. You will need to either install a notification
// routine to respond to arrow tracking, or subclass and override
// the HandleNotification member function. However:
//
// If you simply want to link the little arrows to a number LEditField
// pane, just put the LEditField's pane ID the little arrows pane's
// mUserCon, and the arrows will automatically increment/decrement
// the LEditField view's value, within the value range of the little
// arrows LControl settings.
//

class AGALittleArrowsPP : public LControl
	{
	public:

		AGALittleArrowsPP();
		AGALittleArrowsPP(LStream* inStream);
		virtual ~AGALittleArrowsPP();
		
		virtual void	InstallNotificationRoutine(AGANotifyPPPtr notificationRoutine, void* userData);
		virtual void	HandleNotification(SInt32 deltaValue);
		
		AGANotifyPPPtr	mNotificationRoutine;
		void*			mUserData;

		AGALittleArrows*	mAGAObject;

		enum { class_ID = 'GCla' };
		static AGALittleArrowsPP*	CreateStream(LStream* inStream);

		// PowerPlant overrides
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		
	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();

		virtual void	CreateAGAObject();
		
		static void RealAGANotifier(AGALittleArrows* theAGAObject, SInt32 deltaValue, void* userData);

		LEditField*	mLinkedNumberText;
		
		// Extra AGA object initialization values
		OSType	mLinkedPaneID;
	};

#pragma mark AGALittleArrowsPPX

//
// AGALittleArrowsPPX -------------------------------------------------------
//
// Extended AGALittleArrowsPP class for use with custom pane template.
// The linked LEditField pane ID is read from the stream.
//

class AGALittleArrowsPPX : public AGALittleArrowsPP
	{
	public:

		AGALittleArrowsPPX(LStream* inStream);
		virtual ~AGALittleArrowsPPX();
		
		enum { class_ID = 'GXla' };
		static AGALittleArrowsPPX*	CreateStream(LStream* inStream);
	};

#pragma mark AGADisclosureTrianglePP

//
// AGADisclosureTrianglePP -------------------------------------------------------
//
// LControl subclass Set/get the AGADisclosureTriangle state
// directly. Think of it as a check box -- it's either on or off.
// The initial on/off value is taken from the LControl value.
//

class AGADisclosureTrianglePP : public LControl
	{
	public:

		AGADisclosureTrianglePP();
		AGADisclosureTrianglePP(LStream* inStream);
		virtual ~AGADisclosureTrianglePP();
		
		AGADisclosureTriangle*	mAGAObject;

		enum { class_ID = 'GCdt' };
		static AGADisclosureTrianglePP*	CreateStream(LStream* inStream);

		// PowerPlant overrides
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		virtual void	SetValue(SInt32 inValue);
		
	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();
		virtual void	HotSpotResult(SInt16 inHotSpot);

		virtual void	CreateAGAObject();
	};

#pragma mark AGAProgressIndicatorPP

//
// AGAProgressIndicatorPP -------------------------------------------------------
//
// LControl subclass. Use the standard LControl value methods to set the
// progress indicator range and value. Access the AGAProgressIndicator
// directly to set the origin value if desired. If you set the min equal
// to the max, it will be an indeterminate progress indicator; if the
// mUserCon is non-zero, it will call this->StartAutoAnimate automatically
// when it is created so that you don't have to start the animation running. 
//

class AGAProgressIndicatorPP : public LControl, public LPeriodical
	{
	public:

		AGAProgressIndicatorPP();
		AGAProgressIndicatorPP(LStream* inStream);
		virtual ~AGAProgressIndicatorPP();
		
		AGAProgressIndicator*	mAGAObject;

		enum { class_ID = 'GCpr' };
		static AGAProgressIndicatorPP*	CreateStream(LStream* inStream);
		
		// PowerPlant overrides
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		virtual void	SetValue(SInt32 inValue);
		virtual void	SetMinValue(SInt32 inMinValue);
		virtual void	SetMaxValue(SInt32 inMaxValue);
		virtual void	SpendTime(const EventRecord &inMacEvent);

		// For an indeterminate progress indicator, you can use these functions
		// to set it running or stop it. It will start the LPeriodical repeating
		// and do a quantized animate at idle time.
		void	StartAutoAnimate();
		void	StopAutoAnimate();

	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();

		virtual void	CreateAGAObject();
	};

#pragma mark AGASeparatorPP

//
// AGASeparatorPP -------------------------------------------------------
//
// LPane subclass that implements an AGA separator.
//

class AGASeparatorPP : public LPane
	{
	public:

		AGASeparatorPP();
		AGASeparatorPP(LStream* inStream);
		virtual ~AGASeparatorPP();
		
		AGASeparator*	mAGAObject;

		enum { class_ID = 'GCse' };
		static AGASeparatorPP*	CreateStream(LStream* inStream);

		// PowerPlant overrides
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		
	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();

		virtual void	CreateAGAObject();
	};

#pragma mark AGAPanelEnclosurePP

//
// AGAPanelEnclosurePP ----------------------------------------------------
//
// LView + LCommander mixin class that should be used as the root view
// of your individual tab panel PPob resources. It's OK to just use a
// plain LView, but in that case commander targeting cannot be handled
// by AGATabPanelPP and may result in less-than-ideal activation of
// things like edit text fields as you switch between panels.
//

class AGAPanelEnclosurePP : public LView, public LCommander
	{
	public:

		AGAPanelEnclosurePP();
		AGAPanelEnclosurePP(LStream* inStream);
		virtual ~AGAPanelEnclosurePP();
		
		enum { class_ID = 'GCpe' };
		static AGAPanelEnclosurePP*	CreateStream(LStream* inStream);
	
		virtual void	SwitchPanelOut();
		virtual void	SwitchPanelIn();
		
		LCommander*	mSavedTarget;
	};

#pragma mark AGATabPanelPP

//
// AGATabPanelPP ----------------------------------------------------
//
// LView subclass for AGATabPanel set for large tabs. When instantiated
// from a view resource, the mUserCon, if non-zero, is used as the resource
// ID of an 'STR#' resource that contains the tab labels; the number of
// tabs will be the number of strings in the 'STR#' resource. If you have
// existing tabs, such as via the 'STR#', use InstallPanel to assign a
// panel to a tab. If you have no tabs yet, use AddPanel to create a new
// tab for a panel. Note that the panel views must be non-window 'PPob'
// resources, i.e. view hierarchies with an LView at the root, not an LWindow.
//

class AGATabPanelPP : public LView
	{
	public:

		AGATabPanelPP();
		AGATabPanelPP(LStream* inStream);
		virtual ~AGATabPanelPP();
		
		AGATabPanel*	mAGAObject;

		enum { class_ID = 'GCtp' };
		static AGATabPanelPP*	CreateStream(LStream* inStream);
		
		// PowerPlant overrides
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		virtual void	SetValue(SInt32 inValue);
		virtual SInt32	GetValue() const;
		virtual void	Disable();
		
		// HandleTabSwitch can be used to switch to the specified panel,
		// but it will call ValidatePanel before allowing the switch.
		// This is what ClickSelf does to handle a click in a tab.
		// To switch directly without validation, just call SetValue.
		virtual void	HandleTabSwitch(SInt32 newTabIndex);
		
		// AddPanel adds another tab, and assigns it the specified 'PPob' view.
		// If the specified label StringPtr is NULL, the 'PPob' rsrc name is used.
		virtual void	AddPanel(ResIDT panelResourceID,
								StringPtr tabLabel);

		// InstallPanel assigns the specified 'PPob' view to the specified
		// existing tab. If the specified label StringPtr is NULL, the existing
		// tab label is left untouched.
		virtual void	InstallPanel(SInt32 tabIndex,
								ResIDT panelResourceID,
								StringPtr tabLabel);

		// RemoveTab removes the specified tab. It does *not* destroy the
		// panel view associated with the tab. If you want to destroy the
		// panel view, call GetPanelView first to access it.
		// You are responsible for switching tabs *beforehand* if you are
		// removing the currently selected tab.
		virtual void	RemoveTab(SInt32 tabIndex, Boolean redraw);

		// GetPanelView returns the LView pointer to the root of the
		// pane hierarchy of the specified tab.
		virtual LView*	GetPanelView(SInt32 tabIndex);

	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	EnableSelf();
		virtual void	DisableSelf();

		virtual void	CreateAGAObject();
		virtual void	CreateContainerView();

		// Override ValidatePanel if you need to perform validation
		// before allowing the user to switch panels.
		virtual Boolean	ValidatePanel(SInt32 currentTabIndex);

		virtual void	SwitchPanels(SInt32 newTabIndex);
		virtual void	SwitchPanelOut();
		virtual void	SwitchPanelIn(SInt32 newTabIndex);
		
		SInt16	mLabelsResourceID;
		ResIDT	mLabelsTextTraitsID;
		
		LView*	mPanelContainerView;
		LView*	mActivePanelView;
		
		Boolean	mTempSupressDisable;	// workaround -- see Disable() method
	};

#pragma mark AGATabPanelPPX

//
// AGATabPanelPPX ----------------------------------------------------
//
// Extended AGATabPanelPP class for use with custom pane template.
// The labels STR# resource ID and text traits ID are read from the stream.
// The AGA standard for small tabs is Geneva 10pt bold.
// If you use a font size greater than 10, the tabs will be "large".
// If you use a font size of 10 or less, the tabs will be "small".
//

class AGATabPanelPPX : public AGATabPanelPP
	{
	public:

		AGATabPanelPPX(LStream* inStream);
		virtual ~AGATabPanelPPX();

		enum { class_ID = 'GXtp' };
		static AGATabPanelPPX*	CreateStream(LStream* inStream);
	};

#pragma mark AGASmallTabPanelPP

//
// AGASmallTabPanelPP ----------------------------------------------------
//
// AGATabPanelPP subclass that implements folder tabs, set for "small"
// tab appearance.
//

class AGASmallTabPanelPP : public AGATabPanelPP
	{
	public:

		AGASmallTabPanelPP(LStream* inStream);
		virtual ~AGASmallTabPanelPP();

		enum { class_ID = 'GCts' };
		static AGASmallTabPanelPP*	CreateStream(LStream* inStream);
	
	protected:

		// After creating the object we set the tab size and text style
		// to the standard small appearance.
		virtual void	CreateAGAObject();

	};

#endif // __GCNEWCONTROLSPP__
