//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// See header file "GCStdControlsPP.h" for usage.
//

#include "GCStdControlsPP.h"
#include "GCCorePP.h"

#include <UDrawingUtils.h>

// The URegistrar::RegisterClass calls are making my eyes hurt.
// Define a macro to do it instead. We take advantage of the fact that
// we use the names class_ID and CreateStream in all of our classes.
#define GCPP_REGISTER_CLASS(classname) URegistrar::RegisterClass(classname::class_ID, (ClassCreatorFunc) classname::CreateStream)

void RegisterGCStdControlsPP()
	{
	//
	// Register all of the classes defined here. If you prefer, you can
	// selectively register just the classes you will use.
	//

	GCPP_REGISTER_CLASS(AGAPushButtonPP);
	GCPP_REGISTER_CLASS(AGACheckBoxPP);
	GCPP_REGISTER_CLASS(AGARadioButtonPP);
	GCPP_REGISTER_CLASS(AGAScrollBarPP);
	GCPP_REGISTER_CLASS(AGAScrollerPP);
	GCPP_REGISTER_CLASS(AGAActiveScrollerPP);
	GCPP_REGISTER_CLASS(AGAPopupMenuPP);
	GCPP_REGISTER_CLASS(AGAEditFieldPP);
	GCPP_REGISTER_CLASS(AGATextEditPP);
	GCPP_REGISTER_CLASS(AGACaptionPP);
	GCPP_REGISTER_CLASS(AGAListBoxPP);
	GCPP_REGISTER_CLASS(AGANotchedListBoxPP);
	GCPP_REGISTER_CLASS(AGANotchedFocusListBoxPP);
	}

//
// AGAPushButtonPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LStdButton

AGAPushButtonPP* AGAPushButtonPP::CreateStream(LStream* inStream)
	{
	return new AGAPushButtonPP(inStream);
	}

AGAPushButtonPP::AGAPushButtonPP()
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	}

AGAPushButtonPP::AGAPushButtonPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	}

AGAPushButtonPP::~AGAPushButtonPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAPushButtonPP::SetDescriptor(ConstStringPtr inDescriptor)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::SetDescriptor(inDescriptor); return; }

	Inherited::SetDescriptor(inDescriptor);
	
	// Install the new text in the AGA object. Don't tell it
	// to redraw; the redraw comes from the caller.

	Str255	newText;
	this->GetDescriptor(newText);
	mAGAObject->SetTitle(newText, AGAObject::kDontRedraw);
	}

void AGAPushButtonPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAPushButtonPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAPushButtonPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

if (gAGAStdControls && mAGASuppressable)
	this->HideSelf();	// make sure the Control Manager object is not drawn
	
	this->CreateAGAObject();
	}

void AGAPushButtonPP::DrawSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DrawSelf(); return; }

	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAPushButtonPP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ClickSelf(inMouseDown); return; }

	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		if (mAGAObject->TrackMouse(inMouseDown.whereLocal))
			this->HotSpotResult(this->FindHotSpot(inMouseDown.whereLocal));
		}
	}

void AGAPushButtonPP::HotSpotAction(SInt16 inHotSpot, Boolean inCurrInside, Boolean inPrevInside)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::HotSpotAction(inHotSpot, inCurrInside, inPrevInside); return; }

	// Draw the button based on the new press state.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->DrawButton(inCurrInside);
		}
	}

void AGAPushButtonPP::EnableSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::EnableSelf(); return; }

	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGAPushButtonPP::DisableSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DisableSelf(); return; }

	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGAPushButtonPP::ShowSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ShowSelf(); return; }

	// Suppress showing the Control Manager control.
	}

void AGAPushButtonPP::ActivateSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGAPushButtonPP::DeactivateSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGAPushButtonPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	Str255	title;
	
	(void) this->CalcLocalFrameRect(bounds);
	::GetControlTitle(mMacControlH, title);

	ThrowIfNil_(mAGAObject = new AGAPushButton(&bounds, AGATextStyle(mTextTraitsID), title));

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

//
// AGACheckBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LStdCheckBox

AGACheckBoxPP* AGACheckBoxPP::CreateStream(LStream* inStream)
	{
	return new AGACheckBoxPP(inStream);
	}

AGACheckBoxPP::AGACheckBoxPP()
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	mMaxValue = 2;	// otherwise mixed-state SetValue messages will constrain to 1
	}

AGACheckBoxPP::AGACheckBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	mMaxValue = 2;	// otherwise mixed-state SetValue messages will constrain to 1
	}

AGACheckBoxPP::~AGACheckBoxPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGACheckBoxPP::SetDescriptor(ConstStringPtr inDescriptor)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::SetDescriptor(inDescriptor); return; }

	Inherited::SetDescriptor(inDescriptor);
	
	// Install the new text in the AGA object. Don't tell it
	// to redraw; the redraw comes from the caller.

	Str255	newText;
	this->GetDescriptor(newText);
	mAGAObject->SetTitle(newText, AGAObject::kDontRedraw);
	}

void AGACheckBoxPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGACheckBoxPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGACheckBoxPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

if (gAGAStdControls && mAGASuppressable)
	this->HideSelf();	// make sure the Control Manager object is not drawn
	
	this->CreateAGAObject();
	}

void AGACheckBoxPP::DrawSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DrawSelf(); return; }

	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGACheckBoxPP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ClickSelf(inMouseDown); return; }

	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		if (mAGAObject->TrackMouse(inMouseDown.whereLocal))
			this->HotSpotResult(this->FindHotSpot(inMouseDown.whereLocal));
		}
	}

void AGACheckBoxPP::HotSpotAction(SInt16 inHotSpot, Boolean inCurrInside, Boolean inPrevInside)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::HotSpotAction(inHotSpot, inCurrInside, inPrevInside); return; }

	// Draw the button based on the new press state.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->DrawButton(inCurrInside);
		}
	}

void AGACheckBoxPP::EnableSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::EnableSelf(); return; }

	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGACheckBoxPP::DisableSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DisableSelf(); return; }

	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGACheckBoxPP::ShowSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ShowSelf(); return; }

	// Suppress showing the Control Manager control.
	}

void AGACheckBoxPP::SetValue(SInt32 inValue)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::SetValue(inValue); return; }

	// Set the actual LControl value and the AGA object value.

	Inherited::SetValue(inValue);

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetValue(mValue, this->IsVisible());
		}
	}

void AGACheckBoxPP::ActivateSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGACheckBoxPP::DeactivateSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGACheckBoxPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	Str255	title;
	
	(void) this->CalcLocalFrameRect(bounds);
	::GetControlTitle(mMacControlH, title);

	ThrowIfNil_(mAGAObject = new AGACheckBox(&bounds, AGATextStyle(mTextTraitsID), title, AGAObject::kNoAutomaticState));

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	mAGAObject->SetValue(this->GetValue(), AGAObject::kDontRedraw);
	}

//
// AGARadioButtonPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LStdRadioButton

AGARadioButtonPP* AGARadioButtonPP::CreateStream(LStream* inStream)
	{
	return new AGARadioButtonPP(inStream);
	}

AGARadioButtonPP::AGARadioButtonPP()
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	mMaxValue = 2;	// otherwise mixed-state SetValue messages will constrain to 1
	}

AGARadioButtonPP::AGARadioButtonPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	mAGASuppressable = true;
	mMaxValue = 2;	// otherwise mixed-state SetValue messages will constrain to 1
	}

AGARadioButtonPP::~AGARadioButtonPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGARadioButtonPP::SetDescriptor(ConstStringPtr inDescriptor)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::SetDescriptor(inDescriptor); return; }

	Inherited::SetDescriptor(inDescriptor);
	
	// Install the new text in the AGA object. Don't tell it
	// to redraw; the redraw comes from the caller.

	Str255	newText;
	this->GetDescriptor(newText);
	mAGAObject->SetTitle(newText, AGAObject::kDontRedraw);
	}

void AGARadioButtonPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGARadioButtonPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGARadioButtonPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

if (gAGAStdControls && mAGASuppressable)
	this->HideSelf();	// make sure the Control Manager object is not drawn
	
	this->CreateAGAObject();
	}

void AGARadioButtonPP::DrawSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DrawSelf(); return; }

	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGARadioButtonPP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ClickSelf(inMouseDown); return; }

	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		if (mAGAObject->TrackMouse(inMouseDown.whereLocal))
			this->HotSpotResult(this->FindHotSpot(inMouseDown.whereLocal));
		}
	}

void AGARadioButtonPP::HotSpotAction(SInt16 inHotSpot, Boolean inCurrInside, Boolean inPrevInside)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::HotSpotAction(inHotSpot, inCurrInside, inPrevInside); return; }

	// Draw the button based on the new press state.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->DrawButton(inCurrInside);
		}
	}

void AGARadioButtonPP::EnableSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::EnableSelf(); return; }

	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGARadioButtonPP::DisableSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DisableSelf(); return; }

	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGARadioButtonPP::ShowSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ShowSelf(); return; }

	// Suppress showing the Control Manager control.
	}

void AGARadioButtonPP::SetValue(SInt32 inValue)
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::SetValue(inValue); return; }

	// Set the actual LControl value and the AGA object value.

	Inherited::SetValue(inValue);

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetValue(mValue, this->IsVisible());
		}
	}

void AGARadioButtonPP::ActivateSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGARadioButtonPP::DeactivateSelf()
	{
	if (! gAGAStdControls && mAGASuppressable) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGARadioButtonPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	Str255	title;
	
	(void) this->CalcLocalFrameRect(bounds);
	::GetControlTitle(mMacControlH, title);

	ThrowIfNil_(mAGAObject = new AGARadioButton(&bounds, AGATextStyle(mTextTraitsID), title, kNoGroupID, kNoGroupID, AGAObject::kNoAutomaticState));

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);

	mAGAObject->SetValue(this->GetValue(), AGAObject::kDontRedraw);
	}

//
// AGAScrollBarPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LStdControl

AGAScrollBarPP* AGAScrollBarPP::CreateStream(LStream* inStream)
	{
	return new AGAScrollBarPP(inStream);
	}

AGAScrollBarPP::AGAScrollBarPP()
	{
	mAGAObject = NULL;
	mNotificationRoutine = NULL;
	mUserData = 0;
	mInitWithLiveScrolling = AGAObject::kNoLiveTracking;
	}

AGAScrollBarPP::AGAScrollBarPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	mNotificationRoutine = NULL;
	mUserData = 0;
	mInitWithLiveScrolling = AGAObject::kNoLiveTracking;
	}

AGAScrollBarPP::AGAScrollBarPP(const SPaneInfo	&inPaneInfo,
						MessageT		inValueMessage,
						Int32			inValue,
						Int32			inMinValue,
						Int32			inMaxValue,
						Int16			inControlKind,
						ResIDT			inTextTraitsID,
						Str255			inTitle,
						Int32			inMacRefCon,
						Boolean			liveScrolling)
: Inherited(inPaneInfo, inValueMessage, inValue, inMinValue, inMaxValue, inControlKind, inTextTraitsID, inTitle, inMacRefCon)
	{
	mAGAObject = NULL;
	mNotificationRoutine = NULL;
	mUserData = 0;
	mInitWithLiveScrolling = liveScrolling;
	}

AGAScrollBarPP::~AGAScrollBarPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAScrollBarPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls) { Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAScrollBarPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls) { Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAScrollBarPP::InstallNotificationRoutine(AGANotifyPPPtr notificationRoutine, void* userData)
	{
	// Save the notification routine fn pointer and user data
	// for notification handling.

	mNotificationRoutine = notificationRoutine;
	mUserData = userData;
	}

void AGAScrollBarPP::HandleNotification(SInt32 dataValue)
	{
	// Update the scrollbar value, call the installed notification
	// function, and propagate our event number.

	this->SetValue(dataValue);

	if (mNotificationRoutine != NULL)
		(*mNotificationRoutine)(this, dataValue, mUserData);

//	this->BroadcastValueMessage();	not necessary -- SetValue already did it

	(void) this->FocusDraw();	// HandleEvent may have caused another view to draw
	}

void AGAScrollBarPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

if (gAGAStdControls)
	this->HideSelf();	// make sure the Control Manager object is not drawn

	this->CreateAGAObject();
	}

void AGAScrollBarPP::DrawSelf()
	{
	if (! gAGAStdControls) { Inherited::DrawSelf(); return; }

	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAScrollBarPP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	if (! gAGAStdControls) { Inherited::ClickSelf(inMouseDown); return; }

	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		if (mAGAObject->TrackMouse(inMouseDown.whereLocal))
			this->HotSpotResult(this->FindHotSpot(inMouseDown.whereLocal));
		}
	}

void AGAScrollBarPP::EnableSelf()
	{
	if (! gAGAStdControls) { Inherited::EnableSelf(); return; }

	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGAScrollBarPP::DisableSelf()
	{
	if (! gAGAStdControls) { Inherited::DisableSelf(); return; }

	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGAScrollBarPP::ShowSelf()
	{
	if (! gAGAStdControls) { Inherited::ShowSelf(); return; }

	// Suppress showing the Control Manager control.
	}

void AGAScrollBarPP::ActivateSelf()
	{
	if (! gAGAStdControls) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGAScrollBarPP::DeactivateSelf()
	{
	if (! gAGAStdControls) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGAScrollBarPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	Rect	bounds;
	
	(void) this->CalcLocalFrameRect(bounds);

	ThrowIfNil_(mAGAObject = new AGAScrollBar(&bounds, mMinValue, mMaxValue, mValue));
	
	mAGAObject->SetLiveTracking(mInitWithLiveScrolling);

	mAGAObject->InstallNotificationRoutine(AGAScrollBarPP::RealAGANotifier, this);

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

void AGAScrollBarPP::SetValue(SInt32 inValue)
	{
	if (! gAGAStdControls) { Inherited::SetValue(inValue); return; }

	// Set the LStdControl value and the AGA object value.

	Inherited::SetValue(inValue);

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		mAGAObject->SetValue(mValue, this->IsVisible());
		}
	}

void AGAScrollBarPP::SetMinValue(SInt32 inMinValue)
	{
	if (! gAGAStdControls) { Inherited::SetMinValue(inMinValue); return; }

	// Set the LStdControl min and the AGA object min.

	Inherited::SetMinValue(inMinValue);

	if (mAGAObject != NULL)
		{
		SInt32	oldMin;
		SInt32	oldMax;

		this->FocusDraw();
		
		mAGAObject->GetRange(&oldMin, &oldMax);
		mAGAObject->SetRange(inMinValue, oldMax, this->IsVisible());
		}
	}

void AGAScrollBarPP::SetMaxValue(SInt32 inMaxValue)
	{
	if (! gAGAStdControls) { Inherited::SetMaxValue(inMaxValue); return; }

	// Set the LStdControl max and the AGA object max.

	Inherited::SetMaxValue(inMaxValue);

	if (mAGAObject != NULL)
		{
		SInt32	oldMin;
		SInt32	oldMax;

		this->FocusDraw();
		
		mAGAObject->GetRange(&oldMin, &oldMax);
		mAGAObject->SetRange(oldMin, inMaxValue, this->IsVisible());
		}
	}

void AGAScrollBarPP::RealAGANotifier(AGATrackingIndicator* /*theIndicator*/, SInt32 dataValue, void* userData)
	{
	// This is a static function. Cast the user data to get
	// the AGAScrollBarPP object, have it handle the notification.

	((AGAScrollBarPP*) userData)->HandleNotification(dataValue);
	}

//
// AGAScrollerPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LScroller

AGAScrollerPP* AGAScrollerPP::CreateStream(LStream* inStream)
	{
	return new AGAScrollerPP(inStream);
	}

AGAScrollerPP::AGAScrollerPP()
	{
	mOldHorizontalBar = NULL;
	mOldVerticalBar = NULL;
	}

AGAScrollerPP::AGAScrollerPP(LStream* inStream)
: Inherited(inStream)
	{
	mOldHorizontalBar = NULL;
	mOldVerticalBar = NULL;

	if (! gAGAStdControls) { return; }

	//
	// Back up in the stream so we can find out what the
	// scroll bar indents are.
	//

	inStream->SetMarker(-sizeof(SScrollerInfo), streamFrom_Marker);

	SScrollerInfo	scrollerInfo;
	inStream->ReadData(&scrollerInfo, sizeof(SScrollerInfo));
	
	//
	// Replace the standard scroll bars with AGA scroll bars.
	//
	
	this->MakeCustomScrollBars(scrollerInfo.horizBarLeftIndent,
							   scrollerInfo.horizBarRightIndent,
							   scrollerInfo.vertBarTopIndent,
							   scrollerInfo.vertBarBottomIndent,
							   AGAObject::kNoLiveTracking);
	}

AGAScrollerPP::AGAScrollerPP(LStream* inStream, Boolean liveTracking)
: Inherited(inStream)
	{
	mOldHorizontalBar = NULL;
	mOldVerticalBar = NULL;

	if (! gAGAStdControls) { return; }

	//
	// Back up in the stream so we can find out what the
	// scroll bar indents are.
	//

	inStream->SetMarker(-sizeof(SScrollerInfo), streamFrom_Marker);

	SScrollerInfo	scrollerInfo;
	inStream->ReadData(&scrollerInfo, sizeof(SScrollerInfo));
	
	//
	// Replace the standard scroll bars with AGA scroll bars.
	//
	
	this->MakeCustomScrollBars(scrollerInfo.horizBarLeftIndent,
							   scrollerInfo.horizBarRightIndent,
							   scrollerInfo.vertBarTopIndent,
							   scrollerInfo.vertBarBottomIndent,
							   liveTracking);
	}

AGAScrollerPP::~AGAScrollerPP()
	{
	}

void AGAScrollerPP::FinishCreateSelf()
	{
	Inherited::FinishCreateSelf();
	
	if (! gAGAStdControls) { return; }

	//
	// We can now safely delete the standard scroll bar objects
	// that we have replaced with AGA scroll bars.
	//

	if (mOldHorizontalBar != NULL)
		{
		mOldHorizontalBar->Hide();
		delete mOldHorizontalBar;
		mOldHorizontalBar = NULL;
		}

	if (mOldVerticalBar != NULL)
		{
		mOldVerticalBar->Hide();
		delete mOldVerticalBar;
		mOldVerticalBar = NULL;
		}
	}

void AGAScrollerPP::MakeCustomScrollBars(
	Int16			inHorizBarLeftIndent,
	Int16			inHorizBarRightIndent,
	Int16			inVertBarTopIndent,
	Int16			inVertBarBottomIndent,
	Boolean			liveTracking)
{
	if (! gAGAStdControls) { return; }

//
// NOTE: This code is lifted from LScroller::MakeScrollBars,
// modified to create our scroll bars of our AGAScrollBarPP class
// and use our form of notification that supports live scrolling.
//

	SPaneInfo	barInfo;				// Common information for ScrollBars
	barInfo.visible = false;			// ScrollBars aren't visible until
	barInfo.enabled = true;				//    Scroller is activated
	barInfo.userCon = 0;
	barInfo.superView = this;
	
	mOldHorizontalBar = mHorizontalBar;
	
	mHorizontalBar = nil;
	if (inHorizBarLeftIndent >= 0) {
										// Create Horizontal ScrollBar
		barInfo.paneID = PaneIDT_HorizontalScrollBar;	
		barInfo.width = mFrameSize.width - inHorizBarLeftIndent -
										   inHorizBarRightIndent;
		barInfo.height = 16;
		barInfo.bindings.left = true;
		barInfo.bindings.right = true;
		barInfo.bindings.top = false;
		barInfo.bindings.bottom = true;
		barInfo.left = inHorizBarLeftIndent;
		barInfo.top = mFrameSize.height - 16;

		mHorizontalBar = this->MakeCustomScrollBar(barInfo, liveTracking);

		((AGAScrollBarPP*) mHorizontalBar)->InstallNotificationRoutine(AGAScrollerPP::ScrollBarNotifier, this);
	}

	mOldVerticalBar = mVerticalBar;
	
	mVerticalBar = nil;
	if (inVertBarTopIndent >= 0) {
										// Create Vertical ScrollBar
		barInfo.paneID = PaneIDT_VerticalScrollBar;	
		barInfo.width = 16;
		barInfo.height = mFrameSize.height - inVertBarTopIndent -
											 inVertBarBottomIndent;
		barInfo.bindings.left = false;
		barInfo.bindings.right = true;
		barInfo.bindings.top = true;
		barInfo.bindings.bottom = true;
		barInfo.left = mFrameSize.width - 16;
		barInfo.top = inVertBarTopIndent;

		mVerticalBar = this->MakeCustomScrollBar(barInfo, liveTracking);

		((AGAScrollBarPP*) mVerticalBar)->InstallNotificationRoutine(AGAScrollerPP::ScrollBarNotifier, this);
	}
}

AGAScrollBarPP*	AGAScrollerPP::MakeCustomScrollBar(const SPaneInfo& barInfo, Boolean liveTracking)
	{
	return new AGAScrollBarPP(barInfo, msg_Nothing, 0, 0, 0,
								scrollBarProc, 0, "\p", (Int32) this,
								liveTracking);
	}

void AGAScrollerPP::HandleNotification(LPane* theScrollBar, SInt32 dataValue)
	{
	if (mScrollingView == NULL) return;

	// Scroll the image to the new location specified.
	// This is similar to what LScroller does when it listens
	// to a thumb drag message.

	SPoint32	newScrollPosition;
	
	mScrollingView->GetScrollPosition(newScrollPosition);

	if (theScrollBar == mHorizontalBar)
		newScrollPosition.h = dataValue;
	else if (theScrollBar == mVerticalBar)
		newScrollPosition.v = dataValue;

	mScrollingView->ScrollImageTo(newScrollPosition.h, newScrollPosition.v, true);
	this->AdjustScrollBars();

	(void) this->FocusDraw();	// scrolling view may have grabbed focus
	}

void AGAScrollerPP::AdjustScrollBars()
	{
	if (! gAGAStdControls) { Inherited::AdjustScrollBars(); return; }
	
	if (mScrollingView == NULL) return;

	// Check the scrolling view's size compared to ours, so that
	// the scroll bars have to correct page and step sizes.

	AGAScrollBarPP*	horizontalBar = (AGAScrollBarPP*) mHorizontalBar;
	AGAScrollBarPP*	verticalBar = (AGAScrollBarPP*) mVerticalBar;

	SPoint32		scrollUnit;
	SDimension16	scrollFrameSize;
	SDimension32	scrollImageSize;
	SPoint32		scrollPosition;
	SPoint32		maxTranslation;
	SPoint32		pageStepSize;
	SPoint32		pageSize;

	mScrollingView->GetScrollUnit(scrollUnit);
	mScrollingView->GetFrameSize(scrollFrameSize);
	mScrollingView->GetImageSize(scrollImageSize);
	mScrollingView->GetScrollPosition(scrollPosition);
	
	// The maxTranslation is the max (pinned) scroll value.
	maxTranslation.h = scrollImageSize.width - scrollFrameSize.width;
	maxTranslation.v = scrollImageSize.height - scrollFrameSize.height;
	
	// The page step size is how far to scroll page scrolling
	pageStepSize.h = MaxSInt32(0, scrollFrameSize.width - scrollUnit.h);
	pageStepSize.v = MaxSInt32(0, scrollFrameSize.height - scrollUnit.v);

	// The page size is the range that is visible at once.
	// It's not the same as the page step size.
	if (scrollImageSize.width < 1)	// avoid divide by zero or weird negatives
		pageSize.h = 1;
	else
		pageSize.h = maxTranslation.h *
			(((float) scrollFrameSize.width) / ((float) scrollImageSize.width));

	if (scrollImageSize.height < 1)	// avoid divide by zero or weird negatives
		pageSize.v = 1;
	else
		pageSize.v = maxTranslation.v *
			(((float) scrollFrameSize.height) / ((float) scrollImageSize.height));

	// Update the scroll bar objects with the calculated values.

	if (horizontalBar != NULL)
		{
		horizontalBar->FocusDraw();
		horizontalBar->SetMaxValue(maxTranslation.h);
		horizontalBar->mAGAObject->SetPageSize(pageSize.h, horizontalBar->IsVisible());
		horizontalBar->mAGAObject->SetStepSizes(scrollUnit.h, pageStepSize.h);
		horizontalBar->SetValue(scrollPosition.h);
		}

	if (verticalBar != NULL)
		{
		verticalBar->FocusDraw();
		verticalBar->SetMaxValue(maxTranslation.v);
		verticalBar->mAGAObject->SetPageSize(pageSize.v, verticalBar->IsVisible());
		verticalBar->mAGAObject->SetStepSizes(scrollUnit.v, pageStepSize.v);
		verticalBar->SetValue(scrollPosition.v);
		}
	}

void AGAScrollerPP::ScrollBarNotifier(LPane* theScrollBar, SInt32 dataValue, void* userData)
	{
	// This is a static function. Cast the user data to get
	// the AGAScrollBarPP object, have it handle the notification.

	((AGAScrollerPP*) userData)->HandleNotification(theScrollBar, dataValue);
	}

//
// AGAActiveScrollerPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAScrollerPP

AGAActiveScrollerPP* AGAActiveScrollerPP::CreateStream(LStream* inStream)
	{
	return new AGAActiveScrollerPP(inStream);
	}

AGAActiveScrollerPP::AGAActiveScrollerPP()
	{
	}

AGAActiveScrollerPP::AGAActiveScrollerPP(LStream* inStream)
: Inherited(inStream, AGAObject::kLiveTracking)
	{
	}

AGAActiveScrollerPP::~AGAActiveScrollerPP()
	{
	}

//
// AGAPopupMenuPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LStdPopupMenu

AGAPopupMenuPP* AGAPopupMenuPP::CreateStream(LStream* inStream)
	{
	return new AGAPopupMenuPP(inStream);
	}

AGAPopupMenuPP::AGAPopupMenuPP(LStream* inStream)
: Inherited(inStream)
	{
	mAGAObject = NULL;
	
	// When creating the core AGA object, we will need to know
	// the title width and title variation of the popup menu.
	// Unfortunately, due to the hack nature of the CDEF and the
	// design peculiarities of LStdPopupMenu, we have no way to
	// access those values. The only solution is to back up in
	// the PPob stream and stash them aside until needed.
	
	SInt32	originalMarker = inStream->GetMarker();
	
	// Undo the LStdPopupMenu stream read.
	inStream->SetMarker(-sizeof(Int16), streamFrom_Marker);
	
	// Undo the LStdControl stream read.
	inStream->SetMarker(-sizeof(Int16), streamFrom_Marker);
	inStream->SetMarker(-sizeof(ResIDT), streamFrom_Marker);
	inStream->SetMarker(-sizeof(Int32), streamFrom_Marker);
	Str255	title;
	::GetControlTitle(mMacControlH, title);
	inStream->SetMarker(-1 -title[0], streamFrom_Marker);
	
	// Undo the LControl stream read.
	inStream->SetMarker(-sizeof(SControlInfo), streamFrom_Marker);
	
	// Finally, re-read the LControl pane data.
	SControlInfo	controlInfo;
	inStream->ReadData(&controlInfo, sizeof(SControlInfo));

	mStashedTitleVariation = controlInfo.value;	// popup CDEF uses ctl value for other stuff
	mStashedTitleWidth = controlInfo.maxValue;	// popup CDEF uses ctl max for width
	
	// And skip forward in the stream so it's back where it
	// was before we touched it, in case a subclass is reading it.
	inStream->SetMarker(originalMarker, streamFrom_Start);
	}

AGAPopupMenuPP::~AGAPopupMenuPP()
	{
	// Delete AGA object if we successfully allocated it.

	if (mAGAObject != NULL)
		delete mAGAObject;
	}

void AGAPopupMenuPP::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls) { Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::ResizeFrameBy(inWidthDelta, inHeightDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAPopupMenuPP::MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh)
	{
	if (! gAGAStdControls) { Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh); return; }

	// Let the PowerPlant and AGA objects update their bounds.

	Inherited::MoveBy(inHorizDelta, inVertDelta, inRefresh);
	
	ResetAGAObjectFrame(this, mAGAObject);
	}

void AGAPopupMenuPP::FinishCreateSelf()
	{
	// Create and set up the AGAObject.

if (gAGAStdControls)
	this->HideSelf();	// make sure the Control Manager object is not drawn
	
	this->CreateAGAObject();
	}

void AGAPopupMenuPP::DrawSelf()
	{
	if (! gAGAStdControls) { Inherited::DrawSelf(); return; }

	// Let the AGAObject draw itself.

	if (mAGAObject != NULL)
		mAGAObject->DrawObject();
	}

void AGAPopupMenuPP::ClickSelf(const SMouseDownEvent &inMouseDown)
	{
	if (! gAGAStdControls) { Inherited::ClickSelf(inMouseDown); return; }

	// Let the AGAObject track the mouse.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		
		if (mAGAObject->TrackMouse(inMouseDown.whereLocal))
			this->SetValue(mAGAObject->GetCurrentItemNo());
		}
	}

void AGAPopupMenuPP::EnableSelf()
	{
	if (! gAGAStdControls) { Inherited::EnableSelf(); return; }

	// Let the AGAObject enable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kEnabled, this->IsVisible());
		}
	}

void AGAPopupMenuPP::DisableSelf()
	{
	if (! gAGAStdControls) { Inherited::DisableSelf(); return; }

	// Let the AGAObject disable itself.

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetEnable(AGAObject::kDisabled, this->IsVisible());
		}
	}

void AGAPopupMenuPP::ShowSelf()
	{
	if (! gAGAStdControls) { Inherited::ShowSelf(); return; }

	// Suppress showing the Control Manager control.
	}

void AGAPopupMenuPP::ActivateSelf()
	{
	if (! gAGAStdControls) { Inherited::ActivateSelf(); return; }

	// Let the AGAObject un-dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kActive, this->IsVisible());
		}
	}

void AGAPopupMenuPP::DeactivateSelf()
	{
	if (! gAGAStdControls) { Inherited::DeactivateSelf(); return; }

	// Let the AGAObject dim itself.

	if (TestGrayCouncilDefault(kAGADimDeactivation) && mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetActive(AGAObject::kInactive, this->IsVisible());
		}
	}

void AGAPopupMenuPP::CreateAGAObject()
	{
	// Instantiate the particular AGAObject subclass.

	AGAPopupMenu::WidthAdjust	widthAdjust;

	Rect			bounds;
	Str255			title;
	MenuRef			theMenuRef = this->GetMacMenuH();
	SInt8			titleJustification = mStashedTitleVariation & 0xFF;
	AGATextStyle	titleStyle(mTextTraitsID);
	
	if (((mControlKind - popupMenuProc) & popupFixedWidth) != 0)
		widthAdjust = AGAPopupMenu::kFixedWidth;
	else
		widthAdjust = AGAPopupMenu::kSystemMDEFAdjustment;
	
	(void) this->CalcLocalFrameRect(bounds);
	::GetControlTitle(mMacControlH, title);
	
	// Look at the wacky popup CDEF title style bits.
	titleStyle.mFontStyle = 0;	// init to plain text
	if ((mStashedTitleVariation & popupTitleBold) != 0)
		titleStyle.mFontStyle |= bold;
	if ((mStashedTitleVariation & popupTitleItalic) != 0)
		titleStyle.mFontStyle |= italic;
	if ((mStashedTitleVariation & popupTitleUnderline) != 0)
		titleStyle.mFontStyle |= underline;
	if ((mStashedTitleVariation & popupTitleOutline) != 0)
		titleStyle.mFontStyle |= outline;
	if ((mStashedTitleVariation & popupTitleShadow) != 0)
		titleStyle.mFontStyle |= shadow;
	if ((mStashedTitleVariation & popupTitleCondense) != 0)
		titleStyle.mFontStyle |= condense;
	if ((mStashedTitleVariation & popupTitleExtend) != 0)
		titleStyle.mFontStyle |= extend;

	ThrowIfNil_(mAGAObject = new AGAPopupMenu(&bounds, mStashedTitleWidth, title, titleJustification, titleStyle, AGATextStyle(mTextTraitsID), widthAdjust, this->GetMacMenuH(), AGAPopupMenu::kDontDispose));

	mAGAObject->SetCurrentItemNo(mValue, AGAObject::kDontRedraw);

	if (mEnabled == triState_Off)
		mAGAObject->SetEnable(AGAObject::kDisabled, AGAObject::kDontRedraw);
	}

void AGAPopupMenuPP::SetValue(SInt32 inValue)
	{
	if (! gAGAStdControls) { Inherited::SetValue(inValue); return; }

	// Set the actual LStdControl value and the AGA object value.

	Inherited::SetValue(inValue);

	if (mAGAObject != NULL)
		{
		this->FocusDraw();
		mAGAObject->SetCurrentItemNo(mValue, this->IsVisible());
		}
	}

//
// AGAEditFieldPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LEditField

AGAEditFieldPP* AGAEditFieldPP::CreateStream(LStream* inStream)
	{
	return new AGAEditFieldPP(inStream);
	}

AGAEditFieldPP::AGAEditFieldPP()
	{
	}

AGAEditFieldPP::AGAEditFieldPP(LStream* inStream)
: Inherited(inStream)
	{
	}

AGAEditFieldPP::~AGAEditFieldPP()
	{
	}

void AGAEditFieldPP::DrawSelf()
	{
	this->DrawEverything();
	}

void AGAEditFieldPP::DrawBox()
	{
	this->DrawEverything();
	}

void AGAEditFieldPP::DrawEverything()
	{
	// LEditField just isn't set up to handle our situations where dimming
	// and background color changes affect the whole view. It tries to
	// optimize by only drawing the box upon enable/disable messages. But
	// our background may change or get painted when that happens. So we catch
	// the DrawSelf and separate DrawBox messages, and just draw it all.
	
	Rect		r;
	RGBColor	textForeColor;
	RGBColor	textBackColor;
	Boolean		enabled = (mEnabled == triState_On);
	
	(void) this->CalcLocalFrameRect(r);
	
	AGADrawingEnvironment	env;
	
	::GetForeColor(&textForeColor);
	::GetBackColor(&textBackColor);
	::RGBForeColor(&textBackColor);

	::InsetRect(&r, 1, 1);
	::PaintRect(&r);
	::InsetRect(&r, -1, -1);
	
	if (mHasBox)
		{
		// Note that we just use r2 with AGADrawBorderFrame, because even
		// with tab panels, r2 is the correct disabled background color.

		::InsetRect(&r, -1, -1);
		AGADrawBorderFrame(&r, enabled, false /*isNotched*/, true /*drawFrame*/, &gAGARamp[r2]);
		::InsetRect(&r, 3, 3);
		}
	
	// Get the text to draw, just as in LEditField::DrawSelf.
	// Except we draw gray text if dimmed.
	
	if (enabled)
		::RGBForeColor(&textForeColor);
	else
		::RGBForeColor(&gAGARamp[r7]);

	GrafPtr	savePort = (**mTextEditH).inPort;
	(**mTextEditH).inPort = UQDGlobals::GetCurrentPort();

	::TEUpdate(&r, mTextEditH);
	
	(**mTextEditH).inPort = savePort;
	
	// For any non-deep screens, we paint a gray pattern on
	// to the text if it's dim.
	
	if (! enabled)
		{
		GDIterator	iter;
		Boolean		deep;

		PenPatGray();
		::PenMode(patBic);

		while (iter.NextDevice(deep))
			if (! deep)
				::PaintRect(&r);
		}
	}

//
// AGATextEditPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LTextEdit

AGATextEditPP* AGATextEditPP::CreateStream(LStream* inStream)
	{
	return new AGATextEditPP(inStream);
	}

AGATextEditPP::AGATextEditPP()
	{
	}

AGATextEditPP::AGATextEditPP(LStream* inStream)
: Inherited(inStream)
	{
	}

AGATextEditPP::~AGATextEditPP()
	{
	}

void AGATextEditPP::DrawSelf()
	{
	// LTextEdit clipping prevents simple attachments from doing the job.
	// So we draw the border frame ourself and make sure there's an extra
	// pixel of white around the raw TE rect.
	
	Rect		r;
	RGBColor	textForeColor;
	RGBColor	textBackColor;
	Boolean		enabled = (mEnabled == triState_On);
	
	(void) this->CalcLocalFrameRect(r);
	
	{	// wrap drawing environment in its own scope
	AGADrawingEnvironment	env;
	
	RgnHandle	savedClipping = ::NewRgn();
	
	::GetClip(savedClipping);
	::InsetRect(&r, -3, -3);
	::ClipRect(&r);

	::GetForeColor(&textForeColor);
	::GetBackColor(&textBackColor);
	::RGBForeColor(&textBackColor);

	AGADrawBorderFrame(&r, enabled, false /*isNotched*/, true /*drawFrame*/, &gAGARamp[r2]);

	::InsetRect(&r, 2, 2);
	::PaintRect(&r);

	::SetClip(savedClipping);
	}

	Inherited::DrawSelf();
	}

//
// AGACaptionPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LCaption

AGACaptionPP* AGACaptionPP::CreateStream(LStream* inStream)
	{
	return new AGACaptionPP(inStream);
	}

AGACaptionPP::AGACaptionPP()
	{
	}

AGACaptionPP::AGACaptionPP(LStream* inStream)
: Inherited(inStream)
	{
	}

AGACaptionPP::~AGACaptionPP()
	{
	}

void AGACaptionPP::FinishCreateSelf()
	{
	// Need to erase when drawing, so make sure that
	// we doesn't assume we're on a gray background.
	
	(void) DetermineAGABackgroundColors(&mEnabledBackgroundColor, &mDisabledBackgroundColor, kInstallAlways);
	}

void AGACaptionPP::DrawSelf()
	{
	// Erase to gray before letting LCaption do its standard drawing.
	Rect	bounds;
	Boolean	enabled = this->IsEnabled();
	
	(void) this->CalcLocalFrameRect(bounds);
	
	AGADrawingEnvironment	env;
	
	{
	GDIterator	iter;
	Boolean		deep;
	
	while (iter.NextDevice(deep))
		{
		if (deep)
			{
			if (enabled)
				::RGBForeColor(&mEnabledBackgroundColor);
			else
				::RGBForeColor(&mDisabledBackgroundColor);
			}
		else
			::RGBForeColor(&gAGARamp[rW]);
		
		::PaintRect(&bounds);
		}
	}	// destruct the GDIterator and its clipping changes
	
	// The remaining is lifted from LCaption::DrawSelf(), except that
	// we set the text color to dimmed text color if we are not enabled.

	Rect	frame;
	CalcLocalFrameRect(frame);
	
	Int16	just = UTextTraits::SetPortTextTraits(mTxtrID);
	
	RGBColor	textColor;
	::GetForeColor(&textColor);
	
	ApplyForeAndBackColors();

	::TextMode(srcOr);

	::RGBForeColor(&textColor);
	
	if (! enabled)
		::RGBForeColor(&gAGARamp[r7]);
	
	UTextDrawing::DrawWithJustification((Ptr)&mText[1], mText[0], frame, just);

	// For any non-deep screens, we paint a gray pattern on
	// to the text if it's dim.
	
	if (! enabled)
		{
		GDIterator	iter;
		Boolean		deep;

		PenPatGray();
		::PenMode(patBic);

		while (iter.NextDevice(deep))
			if (! deep)
				::PaintRect(&bounds);
		}
	}

void AGACaptionPP::EnableSelf()
	{
	// We need to redraw when we are enabled.

	this->Draw(NULL);
	}

void AGACaptionPP::DisableSelf()
	{
	// We need to redraw when we are disabled.

	this->Draw(NULL);
	}

//
// AGAListBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited LListBox

AGAListBoxPP* AGAListBoxPP::CreateStream(LStream* inStream)
	{
	return new AGAListBoxPP(inStream);
	}

AGAListBoxPP::AGAListBoxPP()
	{
	mNotchedFrame = false;
	mNotchedFocusBox = false;
	}

AGAListBoxPP::AGAListBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	mNotchedFrame = false;
	mNotchedFocusBox = false;
	}

AGAListBoxPP::~AGAListBoxPP()
	{
	}

void AGAListBoxPP::FinishCreateSelf()
	{
	// Replace the standard LFocusBox with an AGAFocusBoxPP
	// initialized from the existing one.
	
	if (mFocusBox != NULL)
		{
		AGAFocusBoxPP*	newFocusBox;
		
		if (mNotchedFocusBox)
			newFocusBox = new AGANotchedFocusBoxPP(*mFocusBox);
		else
			newFocusBox = new AGAFocusBoxPP(*mFocusBox);
		
		delete mFocusBox;
		
		mFocusBox = newFocusBox;
		mFocusBox->AttachPane(this, false);
		
		// The standard LFocusBox is smaller by 1 pixel on each
		// side than the AGA focus box, so expand ours from where
		// LFocusBox::AttachPane located and sized it.
		
		mFocusBox->MoveBy(-1, -1, true);
		mFocusBox->ResizeFrameBy(2, 2, true);
		}

	// Attach the white background and border frame attachments.
	
	if (mNotchedFrame)
		{
		this->AddAttachment(new AGANotchedWhiteBackgroundAttachmentPP);
		this->AddAttachment(new AGANotchedBorderFrameAttachmentPP);
		}
	else
		{
		this->AddAttachment(new AGAWhiteBackgroundAttachmentPP);
		this->AddAttachment(new AGABorderFrameAttachmentPP);
		}
	}

void AGAListBoxPP::DrawSelf()
	{
	this->DrawEverything(kUseUpdateRgn);
	}

void AGAListBoxPP::ActivateSelf()
	{
	Inherited::ActivateSelf();
/*	
	Rect	frame;
	(void) this->CalcLocalFrameRect(frame);

	if (this->IsVisible() && this->ExecuteAttachments(msg_DrawOrPrint, &frame))
		this->DrawEverything(kDontUseUpdateRgn);
*/	}

void AGAListBoxPP::DeactivateSelf()
	{
	Inherited::DeactivateSelf();
/*
	Rect	frame;
	(void) this->CalcLocalFrameRect(frame);

	if (this->IsVisible() && this->ExecuteAttachments(msg_DrawOrPrint, &frame))
		this->DrawEverything(kDontUseUpdateRgn);
*/	}

void AGAListBoxPP::EnableSelf()
	{
	Rect	frame;
	(void) this->CalcLocalFrameRect(frame);

	if (this->IsVisible())
		if (this->ExecuteAttachments(msg_DrawOrPrint, &frame))
			this->DrawEverything(kDontUseUpdateRgn);
	}

void AGAListBoxPP::DisableSelf()
	{
	Rect	frame;
	(void) this->CalcLocalFrameRect(frame);

	if (this->IsVisible())
		if (this->ExecuteAttachments(msg_DrawOrPrint, &frame))
			this->DrawEverything(kDontUseUpdateRgn);
	}

void AGAListBoxPP::DrawEverything(Boolean useUpdateRgn)
	{
	// AGAListBoxPP just isn't set up to handle our situations where dimming
	// and background color changes affect the whole view. It tries to
	// optimize by not calling LUpdate upon enable/disable messages. But
	// our background may change or get painted when that happens. So we catch
	// the DrawSelf and other appropriate messages, and just draw it all.
	
	this->FocusDraw();

	Rect		r;
	Boolean		enabled = (mEnabled == triState_On);
	
	(void) this->CalcLocalFrameRect(r);
	
	::PenNormal();					// pen size 1,1, solid black pattern, srcOr
	::RGBBackColor(&gAGARamp[rW]);
	
	if (enabled)
		::RGBForeColor(&gAGARamp[rB]);
	else
		::RGBForeColor(&gAGARamp[r7]);

	GrafPtr	savePort = (**mMacListH).port;
	(**mMacListH).port = UQDGlobals::GetCurrentPort();
	
	RgnHandle	updateRgn;
	
	if (useUpdateRgn)
		updateRgn = GetLocalUpdateRgn();
	else
		{
		updateRgn = ::NewRgn();
		::RectRgn(updateRgn, &r);
		}

	::LUpdate(updateRgn, mMacListH);
	::DisposeRgn(updateRgn);
	
	(**mMacListH).port = savePort;
	
	// For any non-deep screens, we paint a gray pattern on
	// to the text if it's dim. Check the existence of scroll
	// bars to determine exactly where we should paint.

	if (! enabled)
		{
		GDIterator	iter;
		Boolean		deep;
		ListHandle	ourListHandle = this->GetMacListH();

		::InsetRect(&r, 1, 1);
		
		if ((**ourListHandle).hScroll != NULL)
			r.bottom -= 15;	// room for h scroll bar

		if ((**ourListHandle).vScroll != NULL)
			r.right -= 15;	// room for v scroll bar

		PenPatGray();
		::PenMode(patBic);

		while (iter.NextDevice(deep))
			if (! deep)
				::PaintRect(&r);
		}
	}

//
// AGANotchedListBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGAListBoxPP

AGANotchedListBoxPP* AGANotchedListBoxPP::CreateStream(LStream* inStream)
	{
	return new AGANotchedListBoxPP(inStream);
	}

AGANotchedListBoxPP::AGANotchedListBoxPP()
	{
	mNotchedFrame = true;
	}

AGANotchedListBoxPP::AGANotchedListBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	mNotchedFrame = true;
	}

AGANotchedListBoxPP::~AGANotchedListBoxPP()
	{
	}

//
// AGANotchedFocusListBoxPP ----------------------------------------------------
//

#undef Inherited
#define Inherited AGANotchedListBoxPP

AGANotchedFocusListBoxPP* AGANotchedFocusListBoxPP::CreateStream(LStream* inStream)
	{
	return new AGANotchedFocusListBoxPP(inStream);
	}

AGANotchedFocusListBoxPP::AGANotchedFocusListBoxPP()
	{
	mNotchedFocusBox = true;
	}

AGANotchedFocusListBoxPP::AGANotchedFocusListBoxPP(LStream* inStream)
: Inherited(inStream)
	{
	mNotchedFocusBox = true;
	}

AGANotchedFocusListBoxPP::~AGANotchedFocusListBoxPP()
	{
	}

