//
// The Gray Council
// Copyright 1996-1997 by Trygve Isaacson. All Rights Reserved.
//
// Before using any of the Gray Council source code, read and
// follow the licensing info in the accompanying documentation
// or contact:
//   <trygve@bombaydigital.com>
//   <http://www.bombaydigital.com>
//
// Also check the web site above to make sure you have the latest version!
//
// The Gray Council provides a set of standard C++ classes that implement
// the standard Apple Grayscale Appearance. The core classes do not
// require any other code such as a particular class framework.
// There are separate files provided that plug The Gray Council core
// into class frameworks.
//
// This file defines PowerPlant adapter classes for the standard Control
// Manager controls.
//
// Classes defined below:
//	AGAPushButtonPP -- LStdButton subclass for AGAPushButton.
//	AGACheckBoxPP -- LStdCheckBox subclass for AGACheckBox.
//	AGARadioButtonPP -- LStdRadioButton subclass for AGARadioButton.
//	AGAScrollBarPP -- LStdControl subclass for AGAScrollBar.
//	AGAScrollerPP -- LScroller subclass that replaces the normal LStdControl
//		scroll bars with ones of class AGAScrollBarPP, and correctly
//		maintain them during scrolling.
//	AGAActiveScrollerPP -- LActiveScroller replacement; actually an AGAScrollerPP
//		subclass that sets the AGA scroll bars for live scrolling.
//	AGAPopupMenuPP -- LStdPopupMenu subclass for AGAPopupMenu.
//	AGAEditFieldPP -- LEditField subclass that automatically attaches an
//		AGAWhiteBackgroundAttachmentPP and an AGABorderFrameAttachmentPP,
//		and also knows how to draw the frame in disabled gray when disabled.
//	AGATextEditPP -- LTextEdit subclass that automatically attaches an
//		AGAWhiteBackgroundAttachmentPP and an AGABorderFrameAttachmentPP.
//	AGACaptionPP -- LCaption subclass for AGAStaticText.
//	AGAListBoxPP -- LListBox subclass that automatically attaches an
//		AGAWhiteBackgroundAttachmentPP and an AGABorderFrameAttachmentPP,
//		and also replaces the normal LFocusBox with an AGAFocusBoxPP so
//		that the focus frame appears correct.
//	AGANotchedListBoxPP -- AGAListBoxPP (LListBox) subclass that uses an
//		AGANotchedWhiteBackgroundAttachmentPP instead of an AGAWhiteBackgroundAttachmentPP.
//	AGANotchedFocusListBoxPP -- AGANotchedListBoxPP (LListBox) subclass that
//		uses both an AGANotchedWhiteBackgroundAttachmentPP instead of an
//		AGAWhiteBackgroundAttachmentPP, and an AGANotchedFocusBoxPP instead of
//		an AGAFocusBoxPP.
//

#pragma once

#ifndef __GCSTDCONTROLSPP__
#define __GCSTDCONTROLSPP__

#include <LAttachment.h>
#include <LCaption.h>
#include <LControl.h>
#include <LDialogBox.h>
#include <LEditField.h>
#include <LFocusBox.h>
#include <LGroupBox.h>
#include <LListBox.h>
#include <LScroller.h>
#include <LStdControl.h>
#include <LStream.h>
#include <LTextEdit.h>
#include <LWindow.h>
#include <UDrawingState.h>
#include <URegistrar.h>
#include <UTextTraits.h>

#include "GCStdControls.h"

extern void RegisterGCStdControlsPP();

#pragma mark AGAPushButtonPP

//
// AGAPushButtonPP ----------------------------------------------------
//
// LStdButton replacement subclass.
//

class AGAPushButtonPP : public LStdButton
	{
	public:

		AGAPushButtonPP();
		AGAPushButtonPP(LStream* inStream);
		virtual ~AGAPushButtonPP();
		
		AGAPushButton*	mAGAObject;
		Boolean			mAGASuppressable;	// false for icon buttons, which can't be drawn by Control Manager

		enum { class_ID = 'GCpb' };
		static AGAPushButtonPP*	CreateStream(LStream* inStream);

		// PowerPlant overrides
		virtual void	SetDescriptor(ConstStringPtr inDescriptor);
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);

	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	HotSpotAction(SInt16 inHotSpot, Boolean inCurrInside, Boolean inPrevInside);
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ShowSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();

		virtual void	CreateAGAObject();
	};

#pragma mark AGACheckBoxPP

//
// AGACheckBoxPP ----------------------------------------------------
//
// LStdCheckBox replacement subclass. To use mixed-state capability,
// set/get the AGACheckBox state directly.
//

class AGACheckBoxPP : public LStdCheckBox
	{
	public:

		AGACheckBoxPP();
		AGACheckBoxPP(LStream* inStream);
		virtual ~AGACheckBoxPP();
		
		AGACheckBox*	mAGAObject;
		Boolean			mAGASuppressable;	// false for icon buttons, which can't be drawn by Control Manager

		enum { class_ID = 'GCcb' };
		static AGACheckBoxPP*	CreateStream(LStream* inStream);

		// PowerPlant overrides
		virtual void	SetDescriptor(ConstStringPtr inDescriptor);
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		virtual void	SetValue(SInt32 inValue);

	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	HotSpotAction(SInt16 inHotSpot, Boolean inCurrInside, Boolean inPrevInside);
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ShowSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();

		virtual void	CreateAGAObject();
	};

#pragma mark AGARadioButtonPP

//
// AGARadioButtonPP ----------------------------------------------------
//
// LStdRadioButton replacement subclass. To use mixed-state capability,
// set/get the AGARadioButton state directly.
//

class AGARadioButtonPP : public LStdRadioButton
	{
	public:

		AGARadioButtonPP();
		AGARadioButtonPP(LStream* inStream);
		virtual ~AGARadioButtonPP();
		
		AGARadioButton*	mAGAObject;
		Boolean			mAGASuppressable;	// false for icon buttons, which can't be drawn by Control Manager

		enum { class_ID = 'GCrb' };
		static AGARadioButtonPP*	CreateStream(LStream* inStream);

		// PowerPlant overrides
		virtual void	SetDescriptor(ConstStringPtr inDescriptor);
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		virtual void	SetValue(SInt32 inValue);

	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	HotSpotAction(SInt16 inHotSpot, Boolean inCurrInside, Boolean inPrevInside);
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ShowSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();

		virtual void	CreateAGAObject();
	};

#pragma mark AGAScrollBarPP

//
// AGAScrollBarPP -------------------------------------------------------
//
// LStdControl/scrollBarProc replacement subclass. To use live scrolling, set the
// AGAScrollBar state directly and either install a notification routine
// or handle the scroll bar's fEventNumber by checking the current value.
// To use proportional indicator, set the AGAScrollBar state directly.
//
// Note that LScroller instantiates its scroll bars for you, so you
// don't need to know about this class when it's being used as part of
// an AGAScrollerPP (LScroller) or AGAActiveScroller (LActiveScroller).
//

typedef void (*AGANotifyPPPtr)(LPane* theIndicator, SInt32 dataValue, void* userData);

class AGAScrollBarPP : public LStdControl
	{
	public:

		AGAScrollBarPP();
		AGAScrollBarPP(LStream* inStream);
		AGAScrollBarPP(const SPaneInfo	&inPaneInfo,
						MessageT		inValueMessage,
						Int32			inValue,
						Int32			inMinValue,
						Int32			inMaxValue,
						Int16			inControlKind,
						ResIDT			inTextTraitsID,
						Str255			inTitle,
						Int32			inMacRefCon,
						Boolean			liveScrolling);
		virtual ~AGAScrollBarPP();
		
		AGAScrollBar*	mAGAObject;

		enum { class_ID = 'GCsb' };
		static AGAScrollBarPP*	CreateStream(LStream* inStream);
		
		// PowerPlant overrides
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		virtual void	SetValue(SInt32 inValue);
		virtual void	SetMinValue(SInt32 inMinValue);
		virtual void	SetMaxValue(SInt32 inMaxValue);
		
		virtual void	InstallNotificationRoutine(AGANotifyPPPtr notificationRoutine, void* userData);
		virtual void	HandleNotification(SInt32 dataValue);
		
		AGANotifyPPPtr	mNotificationRoutine;
		void*			mUserData;
		
	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ShowSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();

		virtual void	CreateAGAObject();
		
		static void RealAGANotifier(AGATrackingIndicator* theIndicator, SInt32 dataValue, void* userData);
		
		Boolean	mInitWithLiveScrolling;
	};

#pragma mark AGAScrollerPP

//
// AGAScrollerPP ----------------------------------------------------
//
// LScroller replacement subclass that creates AGAScrollBarPP objects
// rather than LStdControl/scrollBarProc objects. The scroll bars are
// initialized with live scrolling turned off.
//
// The ugly part is that there's no single bottleneck for creating
// the scroll bars. MakeScrollBars is declared private, so we cannot
// override that. Basically we have to postprocess each constructor
// and make new scroll bars to replace the old ones.
//

class AGAScrollerPP : public LScroller
	{
	public:

		AGAScrollerPP();
		AGAScrollerPP(LStream* inStream);
		AGAScrollerPP(LStream* inStream, Boolean liveTracking);
		virtual ~AGAScrollerPP();
		
		enum { class_ID = 'GCsc' };
		static AGAScrollerPP*	CreateStream(LStream* inStream);

		virtual void	HandleNotification(LPane* theScrollBar, SInt32 dataValue);

	protected:
	
		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	AdjustScrollBars();

		virtual void	MakeCustomScrollBars(Int16 inHorizBarLeftIndent, Int16 inHorizBarRightIndent, Int16 inVertBarTopIndent, Int16 inVertBarBottomIndent, Boolean liveTracking);

		virtual AGAScrollBarPP*	MakeCustomScrollBar(const SPaneInfo& barInfo, Boolean liveTracking);

		LStdControl		*mOldVerticalBar;
		LStdControl		*mOldHorizontalBar;
		
		static void ScrollBarNotifier(LPane* theScrollBar, SInt32 dataValue, void* userData);
	};

#pragma mark AGAActiveScrollerPP

//
// AGAActiveScrollerPP ----------------------------------------------------
//
// LActiveScroller replacement subclass that creates AGAScrollBarPP objects
// rather than LStdControl/scrollBarProc objects. The scroll bars are
// initialized with live scrolling turned on.
//

class AGAActiveScrollerPP : public AGAScrollerPP
	{
	public:

		AGAActiveScrollerPP();
		AGAActiveScrollerPP(LStream* inStream);
		virtual ~AGAActiveScrollerPP();
		
		enum { class_ID = 'GCas' };
		static AGAActiveScrollerPP*	CreateStream(LStream* inStream);
	};

#pragma mark AGAPopupMenuPP

//
// AGAPopupMenuPP -------------------------------------------------------
//
// LStdPopupMenu replacement subclass that implements an AGA popup menu.
//

class AGAPopupMenuPP : public LStdPopupMenu
	{
	public:

		AGAPopupMenuPP(LStream* inStream);
		virtual ~AGAPopupMenuPP();
		
		AGAPopupMenu*	mAGAObject;

		enum { class_ID = 'GCpo' };
		static AGAPopupMenuPP*	CreateStream(LStream* inStream);

		// PowerPlant overrides
		virtual void	ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
		virtual void	MoveBy(Int32 inHorizDelta, Int32 inVertDelta, Boolean inRefresh);
		virtual void	SetValue(SInt32 inValue);
		
	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ClickSelf(const SMouseDownEvent	&inMouseDown);
		virtual void	EnableSelf();
		virtual void	DisableSelf();
		virtual void	ShowSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();

		virtual void	CreateAGAObject();
		
		SInt32	mStashedTitleVariation;
		SInt16	mStashedTitleWidth;
	};

#pragma mark AGAEditFieldPP

//
// AGAEditFieldPP -------------------------------------------------------
//
// LEditField subclass that attaches an AGAWhiteBackgroundAttachmentPP
// and an AGABorderFrameAttachmentPP to itself.
// It also knows how to draw the disabled text frame in gray color
// rather than dithered black pattern.
//

class AGAEditFieldPP : public LEditField
	{
	public:

		AGAEditFieldPP();
		AGAEditFieldPP(LStream* inStream);
		virtual ~AGAEditFieldPP();
		
		enum { class_ID = 'GCed' };
		static AGAEditFieldPP*	CreateStream(LStream* inStream);

	protected:

		virtual void	DrawSelf();
		virtual void	DrawBox();

		virtual void	DrawEverything();	// DrawSelf and DrawBox just need to draw everything,
											// incl. frame, shadowing, bg, text
	};

#pragma mark AGATextEditPP

//
// AGATextEditPP -------------------------------------------------------
//
// LTextEdit subclass that attaches an AGAWhiteBackgroundAttachmentPP
// and an AGABorderFrameAttachmentPP to itself.
//

class AGATextEditPP : public LTextEdit
	{
	public:

		AGATextEditPP();
		AGATextEditPP(LStream* inStream);
		virtual ~AGATextEditPP();
		
		enum { class_ID = 'GCte' };
		static AGATextEditPP*	CreateStream(LStream* inStream);
		
	protected:

		// PowerPlant overrides
		virtual void	DrawSelf();
	};

#pragma mark AGACaptionPP

//
// AGACaptionPP -------------------------------------------------------
//
// LCaption replacement subclass. Erases pane to gray before allowing
// standard LCaption drawing.
//

class AGACaptionPP : public LCaption
	{
	public:

		AGACaptionPP();
		AGACaptionPP(LStream* inStream);
		virtual ~AGACaptionPP();
		
		enum { class_ID = 'GCca' };
		static AGACaptionPP*	CreateStream(LStream* inStream);
		
	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	EnableSelf();
		virtual void	DisableSelf();

		RGBColor	mEnabledBackgroundColor;
		RGBColor	mDisabledBackgroundColor;
	};

#pragma mark AGAListBoxPP

//
// AGAListBoxPP -------------------------------------------------------
//
// LListBox replacement subclass that replaces the default LFocusBox
// with an AGAFocusBoxPP AGA-style focus frame, and attaches an
// AGAWhiteBackgroundAttachmentPP and an AGABorderFrameAttachmentPP to
// itself.
//

class AGAListBoxPP : public LListBox
	{
	public:

		AGAListBoxPP();
		AGAListBoxPP(LStream* inStream);
		virtual ~AGAListBoxPP();
		
		enum { class_ID = 'GClb' };
		static AGAListBoxPP*	CreateStream(LStream* inStream);
		
	protected:

		// PowerPlant overrides
		virtual void	FinishCreateSelf();
		virtual void	DrawSelf();
		virtual void	ActivateSelf();
		virtual void	DeactivateSelf();
		virtual void	EnableSelf();
		virtual void	DisableSelf();

		enum { kUseUpdateRgn = true, kDontUseUpdateRgn = false };	// for readability
		virtual void	DrawEverything(Boolean useUpdateRgn);	// DrawSelf, activate/deactivate, enable/disable just
																// need to draw everything, incl. frame, shadowing, bg, text
		
		Boolean	mNotchedFrame;
		Boolean	mNotchedFocusBox;
	};

#pragma mark AGANotchedListBoxPP

//
// AGANotchedListBoxPP -------------------------------------------------------
//
// An AGAListBoxPP that draws uses notched border frame and notched
// target border.
//

class AGANotchedListBoxPP : public AGAListBoxPP
	{
	public:

		AGANotchedListBoxPP();
		AGANotchedListBoxPP(LStream* inStream);
		virtual ~AGANotchedListBoxPP();
		
		enum { class_ID = 'GCnl' };
		static AGANotchedListBoxPP*	CreateStream(LStream* inStream);
	};

#pragma mark AGANotchedFocusListBoxPP

//
// AGANotchedFocusListBoxPP -------------------------------------------------------
//
// An AGAListBoxPP that draws uses notched border frame and notched
// target border.
//

class AGANotchedFocusListBoxPP : public AGANotchedListBoxPP
	{
	public:

		AGANotchedFocusListBoxPP();
		AGANotchedFocusListBoxPP(LStream* inStream);
		virtual ~AGANotchedFocusListBoxPP();
		
		enum { class_ID = 'GCnt' };
		static AGANotchedFocusListBoxPP*	CreateStream(LStream* inStream);
	};

#endif // __GCSTDCONTROLSPP__
