// =================================================================================
//	C256App.cp					1997 eli curtz. All rights reserved.
// =================================================================================

#include <LGrowZone.h>
#include <LMenu.h>
#include <LMenuBar.h>
#include <LString.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <PPobClasses.h>
#include <UDesktop.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <UReanimator.h>
#include <URegistrar.h>
#include <UTextTraits.h>

#include "Constants.h"
#include "C256Document.h"
#include "C256View.h"

#include "C256App.h"


// =================================================================================
//		 Main Program
// =================================================================================

void
main( void )
{
	// Setup the throw and signal actions.
	SetDebugThrow_( debugAction_Alert );
	SetDebugSignal_( debugAction_Alert );

	// Initialize the heap. Parameter is number
	// of master handle blocks to allocate.
	InitializeHeap( 4 );
	
	// Initialize the MacOS toolbox.
	UQDGlobals::InitializeToolbox( &qd );

	// Install a GrowZone function to catch  low memory situations.
	// Parameter is the size of the memory reserve in bytes.
	new LGrowZone( 20000 );

	// Create the application object and run it.
	C256App	theApp;
	theApp.Run();
}


// ---------------------------------------------------------------------------------
//		 C256App
// ---------------------------------------------------------------------------------

C256App::C256App()
{

	// Register PowerPlant class creator functions.
	RegisterAllPPClasses();

	// Register custom classes.
	RegisterClass_(C256View);
}


// ---------------------------------------------------------------------------------
//		 ~C256App
// ---------------------------------------------------------------------------------

C256App::~C256App()
{
}


// ---------------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------------

void
C256App::StartUp()
{
//	ObeyCommand( cmd_New, nil );
	ChooseDocument();
}


// ---------------------------------------------------------------------------------
//		 OpenDocument
// ---------------------------------------------------------------------------------

void
C256App::OpenDocument(
	FSSpec	*inMacFSSpec )
{
	// Create a new document using the file spec.
	new C256Document( this, inMacFSSpec );
}


// ---------------------------------------------------------------------------------
//		 MakeNewDocument
// ---------------------------------------------------------------------------------

LModelObject *
C256App::MakeNewDocument()
{
	// Don't make empty docs for now
	return nil;

	// Make a new empty document.
//	return new C256Document( this, nil );
}


// ---------------------------------------------------------------------------------
//		 ChooseDocument
// ---------------------------------------------------------------------------------

void
C256App::ChooseDocument()
{
	// Deactivate the desktop.
	::UDesktop::Deactivate();

	// Browse for a document.
	SFTypeList			theTypeList = {'.raw'};
	StandardFileReply	theReply;
	::StandardGetFile( nil, 1, theTypeList, &theReply );

	// Activate the desktop.
	::UDesktop::Activate();
	
	// Send an apple event to open the file.	
	if ( theReply.sfGood ) SendAEOpenDoc( theReply.sfFile );
}


// ---------------------------------------------------------------------------------
//		 PrintDocument
// ---------------------------------------------------------------------------------

void
C256App::PrintDocument(
	FSSpec	*inMacFSSpec )
{
	// Create a new document using the file spec.
	C256Document	*theDocument = new C256Document( this, inMacFSSpec );
	Assert_( theDocument != nil );

	// Tell the document to print.
	theDocument->DoPrint();
}
