#ifndef __BITMAPS_H__
#define __BITMAPS_H__

/*
BITMAPS.H
Saturday, August 20, 1994 12:08:34 PM
*/

// eli's change for compilation fun
#include "definitions.h"

#ifdef __cplusplus
extern "C" {
#endif

/* ---------- pixels */

typedef unsigned char pixel8;
typedef unsigned short pixel16;
typedef unsigned long pixel32;

enum // bit depths
{
        _8bit= 8,
        _16bit= 16,
        _32bit= 32,
        
        _16bit_555_encoding= 0,
        _16bit_565_encoding,
        _32bit_encoding,
        _16bit_88_encoding, // palette + alpha
        _8bit_encoding,

        // Hardware encodings so we can switch in our bitmap drawing routines
        _3dfx_frame_buffer_encoding,
        _rendition_frame_buffer_encoding,
        _direct3d_frame_buffer_encoding,
        _rave_frame_buffer_encoding,
        NUMBER_OF_BITMAP_ENCODINGS
};

enum
{
        PIXEL8_MAXIMUM_COLORS= 256,
        PIXEL16_555_MAXIMUM_COLORS= 32768,
        PIXEL16_565_MAXIMUM_COLORS= 65536,
        PIXEL32_MAXIMUM_COLORS= 16777216,

        PIXEL8_BITS= 8,
        PIXEL16_555_COMPONENT_BITS= 5,
        PIXEL16_565_RED_COMPONENT_BITS= 5,
        PIXEL16_565_GREEN_COMPONENT_BITS= 6,
        PIXEL16_565_BLUE_COMPONENT_BITS= 5,
        PIXEL32_COMPONENT_BITS= 8,

        PIXEL16_4444_COMPONENT_BITS= 4,
        PIXEL16_4444_MAXIMUM_COMPONENT= 1<<PIXEL16_4444_COMPONENT_BITS,
        PIXEL16_4444_COMPONENT_MASK= PIXEL16_4444_MAXIMUM_COMPONENT-1,

        NUMBER_OF_COLOR_COMPONENTS= 3,
        PIXEL16_555_MAXIMUM_COMPONENT= (1<<PIXEL16_555_COMPONENT_BITS),
        PIXEL16_565_MAXIMUM_RED_COMPONENT= (1<<PIXEL16_565_RED_COMPONENT_BITS),
        PIXEL16_565_MAXIMUM_GREEN_COMPONENT= (1<<PIXEL16_565_GREEN_COMPONENT_BITS),
        PIXEL16_565_MAXIMUM_BLUE_COMPONENT= (1<<PIXEL16_565_BLUE_COMPONENT_BITS),
        PIXEL32_MAXIMUM_COMPONENT= (1<<PIXEL32_COMPONENT_BITS),

        PIXEL8_MASK= PIXEL8_MAXIMUM_COLORS-1,
        PIXEL16_555_COMPONENT_MASK= PIXEL16_555_MAXIMUM_COMPONENT-1,
        PIXEL16_565_RED_COMPONENT_MASK= PIXEL16_565_MAXIMUM_RED_COMPONENT-1,
        PIXEL16_565_GREEN_COMPONENT_MASK= PIXEL16_565_MAXIMUM_GREEN_COMPONENT-1,
        PIXEL16_565_BLUE_COMPONENT_MASK= PIXEL16_565_MAXIMUM_BLUE_COMPONENT-1,
        PIXEL32_COMPONENT_MASK= PIXEL32_MAXIMUM_COMPONENT-1
};

#define BUILD_PIXEL16_4444(a,r,g,b) (((a)<<12) | ((r)<<8) | ((g)<<4) | (b))

#define RED16_555(p) ((p)>>(2*PIXEL16_555_COMPONENT_BITS)) // pel must be clean
#define GREEN16_555(p) (((p)>>PIXEL16_555_COMPONENT_BITS)&PIXEL16_555_COMPONENT_MASK)
#define BLUE16_555(p) ((p)&PIXEL16_555_COMPONENT_MASK)
#define BUILD_PIXEL16_555(r,g,b) (((r)<<(2*PIXEL16_555_COMPONENT_BITS))|((g)<<PIXEL16_555_COMPONENT_BITS)|(b))
#define RGBCOLOR_TO_PIXEL16_555(r,g,b) (((pixel16)((r)>>1)&(pixel16)0x7c00)|((pixel16)((g)>>6)&(pixel16)0x03e0)|((pixel16)((b)>>11)&(pixel16)0x1f))

#define RED16_565(p) ((p)>>(PIXEL16_565_BLUE_COMPONENT_BITS+PIXEL16_565_GREEN_COMPONENT_BITS))
#define GREEN16_565(p) (((p)>>(PIXEL16_565_BLUE_COMPONENT_BITS+1))&PIXEL16_555_COMPONENT_MASK)
#define BLUE16_565(p) ((p)&PIXEL16_555_COMPONENT_MASK)
#define BUILD_PIXEL16_565(r,g,b) (((r)<<(PIXEL16_565_BLUE_COMPONENT_BITS+PIXEL16_565_GREEN_COMPONENT_BITS)) | \
        ((g)<<(PIXEL16_565_BLUE_COMPONENT_BITS+1)) | \
        (b))
#define RGBCOLOR_TO_PIXEL16_565(r,g,b) (((r)&(PIXEL16_565_RED_COMPONENT_MASK<<(PIXEL16_565_BLUE_COMPONENT_BITS+PIXEL16_565_GREEN_COMPONENT_BITS))) | \
        (((g)>>(PIXEL16_565_RED_COMPONENT_BITS))&(PIXEL16_565_GREEN_COMPONENT_MASK<<PIXEL16_565_BLUE_COMPONENT_BITS)) | \
        (((b)>>(PIXEL16_565_RED_COMPONENT_BITS+PIXEL16_565_GREEN_COMPONENT_BITS))&(PIXEL16_565_BLUE_COMPONENT_MASK)))

#define PIXEL16_555_TO_565(p) (((p)&BUILD_PIXEL16_555(0, 1, PIXEL16_555_COMPONENT_MASK)) | \
        (((p)&BUILD_PIXEL16_555(PIXEL16_555_COMPONENT_MASK, PIXEL16_555_COMPONENT_MASK, 0))<<1))
#define PIXEL16_565_TO_555(p) (((p)&BUILD_PIXEL16_555(0, 0, PIXEL16_555_COMPONENT_MASK)) | \
        (((p)>>1)&BUILD_PIXEL16_555(PIXEL16_555_COMPONENT_MASK, PIXEL16_555_COMPONENT_MASK, 0)))

#define RED32(p) ((p)>>(2*PIXEL32_COMPONENT_BITS)) /* pel must be clean */
#define GREEN32(p) (((p)>>PIXEL32_COMPONENT_BITS)&PIXEL32_COMPONENT_MASK)
#define BLUE32(p) ((p)&PIXEL32_COMPONENT_MASK)
#define BUILD_PIXEL32(a,r,g,b) (((a)<<(3*PIXEL32_COMPONENT_BITS))|((r)<<(2*PIXEL32_COMPONENT_BITS))|((g)<<PIXEL32_COMPONENT_BITS)|(b))
#define RGBCOLOR_TO_PIXEL32(a,r,g,b) (((((pixel32)(a))<<16)&0xff000000) | ((((pixel32)(r))<<8)&0x00ff0000) | ((((pixel32)(g)))&0x0000ff00) | ((((pixel32)(b))>>8)&0x000000ff))

/* ---------- transparency */

enum
{
        _transparency_normal= 0, // linear blend of source and destination
        _transparency_overlay, // source component is transparency
        NUMBER_OF_TRANSPARENCY_MODES,
        
        uOPAQUE= 0,
        uTRANSPARENT= 15,
        
        TRANSPARENCY_BITS= 4,
        NUMBER_OF_TRANSPARENCY_LEVELS= 1<<TRANSPARENCY_BITS,
        TRANSPARENCY_MASK= NUMBER_OF_TRANSPARENCY_LEVELS-1,
        
        TRANSPARENCY_TABLE_SIZE= NUMBER_OF_TRANSPARENCY_LEVELS*PIXEL16_555_MAXIMUM_COMPONENT*PIXEL16_555_MAXIMUM_COMPONENT
};

#define TRANSPARENCY_TABLE_ADDRESS16(transparency) \
        ((transparency)<<(2*PIXEL16_555_COMPONENT_BITS))
#define TRANSPARENCY_TABLE_ADDRESS32(transparency) \
        ((transparency)<<(2*PIXEL32_COMPONENT_BITS))
#define FIXED_TRANSPARENCY_TABLE_ADDRESS16(transparency) \
        (((transparency)>>(FIXED_FRACTIONAL_BITS-TRANSPARENCY_BITS-2*PIXEL16_555_COMPONENT_BITS)) & \
        ((NUMBER_OF_TRANSPARENCY_LEVELS-1)<<(2*PIXEL16_555_COMPONENT_BITS)))
#define FIXED_TRANSPARENCY_TABLE_ADDRESS32(transparency) \
        (((transparency)<<(TRANSPARENCY_BITS+2*PIXEL32_COMPONENT_BITS-FIXED_FRACTIONAL_BITS)) & \
        ((NUMBER_OF_TRANSPARENCY_LEVELS-1)<<(2*PIXEL32_COMPONENT_BITS)))

#define TRANSPARENCY16_555(transparency_table, source, destination) \
        BUILD_PIXEL16_555((transparency_table)[RED16_555(source) + (((destination)>>(PIXEL16_555_COMPONENT_BITS))&(PIXEL16_555_COMPONENT_MASK<<PIXEL16_555_COMPONENT_BITS))], \
                (transparency_table)[GREEN16_555(source) + ((destination)&(PIXEL16_555_COMPONENT_MASK<<PIXEL16_555_COMPONENT_BITS))], \
                (transparency_table)[BLUE16_555(source) + (((destination)<<PIXEL16_555_COMPONENT_BITS)&(PIXEL16_555_COMPONENT_MASK<<PIXEL16_555_COMPONENT_BITS))])
#define TRANSPARENCY16_565(transparency_table, source, destination) \
        BUILD_PIXEL16_565((transparency_table)[RED16_565(source) + (((destination)>>(PIXEL16_555_COMPONENT_BITS+1))&(PIXEL16_555_COMPONENT_MASK<<PIXEL16_555_COMPONENT_BITS))], \
                (transparency_table)[GREEN16_565(source) + (((destination)>>1)&(PIXEL16_555_COMPONENT_MASK<<PIXEL16_555_COMPONENT_BITS))], \
                (transparency_table)[BLUE16_565(source) + (((destination)<<PIXEL16_555_COMPONENT_BITS)&(PIXEL16_555_COMPONENT_MASK<<PIXEL16_555_COMPONENT_BITS))])
#define TRANSPARENCY32(transparency_table, source, destination) \
        BUILD_PIXEL32((transparency_table)[RED32(source) + (((destination)>>PIXEL32_COMPONENT_BITS)&(PIXEL32_COMPONENT_MASK<<PIXEL32_COMPONENT_BITS))], \
                (transparency_table)[GREEN32(source) + ((destination)&(PIXEL32_COMPONENT_MASK<<PIXEL32_COMPONENT_BITS))], \
                (transparency_table)[BLUE32(source) + (((destination)<<PIXEL32_COMPONENT_BITS)&(PIXEL32_COMPONENT_MASK<<PIXEL32_COMPONENT_BITS))])

/* ---------- color tables */

enum // rgb_color/hsv_color flags
{
        _color_reversed_bit,
        _color_glows_bit,
        _color_unused_bit,
        
        NUMBER_OF_COLOR_FLAGS, // <=16
        
        _color_reversed_flag= FLAG(_color_reversed_bit),
        _color_glows_flag= FLAG(_color_glows_bit),
        _color_unused_flag= FLAG(_color_unused_bit)
};

struct rgb_color
{
        UInt16 red, green, blue;
        
        word flags;
};

struct hsv_color
{
        word hue, saturation, value;
        
        word flags;
};

struct color_table
{
        long color_count;
        long seed;
        
        unsigned long acceleration_cookie; // how the hardware sees this color table
        
        long unused[5];
        
        struct rgb_color colors[PIXEL8_MAXIMUM_COLORS];
};

/* ---------- bitmaps */

/*
_power_of_two means that width==bytes_per_row==1<<width_bits, height==1<<height_bits

how about a way to not have a row_address table if we dont need it (i.e., power of two)
*/

enum
{
        _power_of_two_bit, // must be set for textures
        _bitmap_color_key_transparency_bit, // pixel index zero is transparent
        _1bit_transparency_encoded_bit, // cannot be set for textures
        _4bit_light_mask_encoded_bit, // must be 16bpp
        _4bit_transparency_encoded_bit, // must be 16bpp
        _no_row_address_table_bit, // dont compute row address table (it doesnt exist)
        _word_aligned_encoding_bit, // encoded span headers are on word boundaries
        _grayscale_bitmap_bit, // 8bpp, [0,256) [black,white)
        _pixel_doubled_bitmap_bit, // will be pixel-doubled when drawn
        _postprocessed_bitmap_bit, // implementation dependant
        _protected_bitmap_bit, // used on disk; means "can't be recomputed"
        _bitmap_is_overlay_bit, // uses intensity as transparency
        _bitmap_changed_bit, // must be reuploaded to hardware
        _bitmap_in_collection_bit, // this bitmap is part of a loaded shape collection
        
        NUMBER_OF_BITMAP_FLAGS, /* <= 16 */
        
        _power_of_two_flag= FLAG(_power_of_two_bit),
        _bitmap_color_key_transparency_flag= FLAG(_bitmap_color_key_transparency_bit),
        _1bit_transparency_encoded_flag= FLAG(_1bit_transparency_encoded_bit),
        _4bit_light_mask_encoded_flag= FLAG(_4bit_light_mask_encoded_bit),
        _4bit_transparency_encoded_flag= FLAG(_4bit_transparency_encoded_bit),
        _no_row_address_table_flag= FLAG(_no_row_address_table_bit),
        _word_aligned_encoding_flag= FLAG(_word_aligned_encoding_bit),
        _grayscale_bitmap_flag= FLAG(_grayscale_bitmap_bit),
        _pixel_doubled_bitmap_flag= FLAG(_pixel_doubled_bitmap_bit),
        _postprocessed_bitmap_flag= FLAG(_postprocessed_bitmap_bit),
        _protected_bitmap_flag= FLAG(_protected_bitmap_bit),
        _bitmap_is_overlay_flag= FLAG(_bitmap_is_overlay_bit),
        _bitmap_changed_flag= FLAG(_bitmap_changed_bit),
        _bitmap_in_collection_flag= FLAG(_bitmap_in_collection_bit),

        _bitmap_mask_flags= _4bit_light_mask_encoded_flag|_4bit_transparency_encoded_flag
};

#define POWER2_TEXTURE_ADDRESS(u, u_shift, u_mask, v, v_shift, v_mask) \
        ((((v)>>v_shift)&v_mask) + (((u)>>u_shift)&u_mask))

enum
{
        ENCODED_TRANSPARENCY_BIT_OFFSET= 8,
        MAXIMUM_NUMBER_OF_SHADED_BITMAPS= 2,
        MAXIMUM_NUMBER_OF_SHADED_BITMAP_REFERENCES= 6
};

#define DECODE_PIXEL8(p) ((p)&PIXEL8_MASK)
#define DECODE_INTENSITY(p) (p)
#define DECODE_TRANSPARENCY(p) (((p)<<4)&(FIXED_ONE-1))

struct bitmap_data
{
        short width, height;
        short bytes_per_row;
        
        short flags;
        short logical_bit_depth; // logical bits per pixel
        
        short width_bits, height_bits; // only valid if _power_of_two
        short physical_byte_depth_minus_one; // physical bits per pixel divided by CHAR_BITS minus one
        
        short encoding;
        
        // these are only used if bitmap_in_collection_flag is NOT set
        short hardware_texture_cache_index;
        unsigned long hardware_address;
        void *hardware_format;

        short hardware_width_bits, hardware_height_bits;
        short hardware_height, hardware_width;
        
        short x0, y0;
        
        // only valid if bitmap_in_collection_flag is set
        short collection_index, bitmap_index;

        short shaded_hardware_addresses[MAXIMUM_NUMBER_OF_SHADED_BITMAPS];
        
        void *row_addresses[1];
};

/* ---------- encoded transparency */

#define ALIGN_SPAN_PIXEL_COUNT(size) (((size)&1) ? ((size)+1) : (size))
#define SIZEOF_ENCODED_SPAN_HEADER(header) (sizeof(struct encoded_span_header) + (header)->span_count*sizeof(struct encoded_span))

struct encoded_span
{
        short start, end; // in bitmap [0,width] coordinates
};

struct encoded_span_header
{
        short span_count, pixel_count;
        
        struct encoded_span spans[];
};

/* ---------- prototypes/BITMAPS.C */

struct bitmap_data *new_bitmap(int width, int height, int bytes_per_row, int encoding, int flags);

long sizeof_bitmap(int height, int flags);
long sizeof_color_table(int color_count);

struct rgb_color *set_rgb_color(struct rgb_color *color, int red, int green, int blue);
struct hsv_color *set_hsv_color(struct hsv_color *color, int hue, int saturation, int value);

void precalculate_bitmap_row_addresses(struct bitmap_data *bitmap); // .row_addresses[0] must be initialized!

void map_bytes(byte *buffer, byte *table, long size);

void postprocess_extracted_bitmap(struct bitmap_data *bitmap);

enum
{
        _decompress_bitmap_8, // color map only
        _decompress_bitmap_88, // alpha + color map
        NUMBER_OF_DECOMPRESS_BITMAP_MODES
};

void decompress_bitmap(struct color_table *color_table, struct bitmap_data *bitmap,
        void *buffer, int width, int height, int bytes_per_row, int mode);

struct hsv_color *rgb_color_to_hsv_color(struct rgb_color *rgb, struct hsv_color *hsv);
struct rgb_color *hsv_color_to_rgb_color(struct hsv_color *hsv, struct rgb_color *rgb);

int closest_color_in_color_table(struct color_table *colors, struct rgb_color *color);

void byte_swap_bitmap(struct bitmap_data *bitmap);
void byte_swap_bitmap_pixels(struct bitmap_data *bitmap); // .row_addresses[0] must be initialized!
void byte_swap_color_table(struct color_table *color_table);

void remap_bitmap(struct bitmap_data *bitmap, struct color_table *original_color_table,
        struct color_table *new_color_table);

int next_highest_power_of_two(int x);
int find_first_one(int k);
int get_encoding_byte_depth(int encoding);

#ifdef __cplusplus
}
#endif

#endif // __BITMAPS_H__