#ifndef __MESH_FILE_H__
#define __MESH_FILE_H__

/*
MESH_FILE.H
Tuesday May 7, 1996 2:46 PM
*/

//#include "tag_files.h"
#include "markers.h"
//#include "myth.h"

/* ---------- constants */

enum
{
	MESH_GROUP_TAG= 0x6d657368, // 'mesh' (looks like text)
	MESH_VERSION_NUMBER= 7,
	
	MESH_HEADER_EDITOR_DATA_SIZE= 16
};

/* ---------- mesh header */

enum // flags (mirrored is user data field of tag)
{
	// first 16 flags show valid netgame scoring types
	_mesh_is_single_player_entry_point_bit= MAXIMUM_NUMBER_OF_GAME_SCORING_TYPES,
	_mesh_supports_unit_trading_bit,
	_mesh_supports_veterans_bit,
	_mesh_has_limited_terrain_visibility_bit,
	_mesh_is_complete_bit,
	_mesh_can_be_used_by_demo_bit,
	_mesh_leaves_overhead_map_closed_bit,
	_mesh_is_training_map_bit,
	_mesh_can_catch_fire_bit,
	
	NUMBER_OF_MESH_HEADER_FLAGS,
	
	_mesh_is_single_player_entry_point_flag= FLAG(_mesh_is_single_player_entry_point_bit),
	_mesh_supports_unit_trading_flag= FLAG(_mesh_supports_unit_trading_bit),
	_mesh_supports_veterans_flag= FLAG(_mesh_supports_veterans_bit),
	_mesh_has_limited_terrain_visibility_flag= FLAG(_mesh_has_limited_terrain_visibility_bit),
	_mesh_is_complete_flag= FLAG(_mesh_is_complete_bit),
	_mesh_can_be_used_by_demo_flag= FLAG(_mesh_can_be_used_by_demo_bit),
	_mesh_leaves_overhead_map_closed_flag= FLAG(_mesh_leaves_overhead_map_closed_bit),
	_mesh_is_training_map_flag= FLAG(_mesh_is_training_map_bit),
	_mesh_can_catch_fire_flag= FLAG(_mesh_can_catch_fire_bit)
};

enum // next mesh array indexes
{
	_next_mesh_success,
	_next_mesh_failure,
	NUMBER_OF_NEXT_MESH_TAGS
};

enum // cutscenes array indexes
{
	_cutscene_pregame,
	_cutscene_postgame_success,
	_cutscene_postgame_failure,
	NUMBER_OF_CUTSCENE_MOVIE_TAGS
};

enum // storyline array indexes
{
	_storyline_pregame,
	_storyline_rant,
	_storyline_postgame_success,
	_storyline_postgame_failure,
	NUMBER_OF_STORYLINE_STRING_TAGS
};

enum // screens array indexes (these are just collections)
{
	_screen_loss,
	_screen_success,
	_screen_failure,
	NUMBER_OF_SCREEN_COLLECTION_TAGS
};

enum // map description string list indexes
{
	_map_name_string_index,
	_custom_rules_string_index,
	_caption_string_index,
	NUMBER_OF_DESCRIPTION_STRING_INDEXES
};

enum // sequence indexes
{
	// overhead map collection
	_overhead_map_sequence_map= 0,
	
	// pregame collection
	_pregame_sequence_story_background= 0,
	_pregame_sequence_netgame_preview,
	_pregame_sequence_story_detail_map,
	_pregame_sequence_colors,
	_pregame_sequence_story_map,
	
	// pregame colors
	_pregame_header_text_color_index= 0,
	_pregame_body_text_color_index,
	_pregame_caption_text_color_index,
	
	// postgame collection
	_postgame_sequence_win= 0,
	_postgame_sequence_loss,
	_postgame_sequence_colors,

		_postgame_color_header= 0,
		_postgame_color_column_header,
		_postgame_color_body
};

#define SIZEOF_STRUCT_MESH_HEADER 1024
struct mesh_header
{
	file_tag landscape_collection_tag;
	file_tag media_tag;
	
	short submesh_width, submesh_height;
	long mesh_offset, mesh_size;
	void *mesh;

	long data_offset, data_size;
	void *data;
	
	long marker_palette_entries, marker_palette_offset, marker_palette_size; // array of marker_data structures
	void *marker_palette; // combined with marker list in scenario_header
	
	long marker_count, markers_offset, markers_size; // array of marker_data structures
	void *markers;
	
	file_tag mesh_lighting_tag; // shadow intensity, ambient light, etc.
	file_tag render_variables_tag; // wobble speed, reflectivity, etc.
	
	unsigned long flags;
	
	file_tag particle_system_tag;

	long team_count, teams_offset, teams_size;
	void *teams;

	long annotation_count, annotation_offset, annotation_size;
	void *annotations;
	
	// this is not a rectangle; these are the buffers on each edge of the map, in cells
	rectangle2d edge_of_mesh_buffer_zones;
	
	file_tag global_ambient_sound_tag;

	long map_action_count, map_actions_offset, map_action_buffer_size;

	file_tag map_description_string_list_tag;
	file_tag postgame_collection_tag;
	file_tag pregame_collection_tag;
	file_tag overhead_map_collection_tag;
	file_tag next_mesh_tags[NUMBER_OF_NEXT_MESH_TAGS];
	file_tag cutscene_movie_tags[NUMBER_OF_CUTSCENE_MOVIE_TAGS];
	file_tag storyline_string_tags[NUMBER_OF_STORYLINE_STRING_TAGS];
	
	long media_coverage_region_offset, media_coverage_region_size;
	short *media_coverage_region;
	
	long media_coverage_polygons_offset, media_coverage_polygons_size;
	byte *media_coverage_polygons;

	struct rgb_color fog_color;
	short_fixed fog_density;
	
	word pad;
	file_tag wind_tag;

	file_tag screen_collection_tags[NUMBER_OF_SCREEN_COLLECTION_TAGS];
	
	struct rgb_color blood_color;

	file_tag picture_caption_string_list_tag;
	file_tag narration_sound_tag;

	file_tag win_ambient_sound_tag, loss_ambient_sound_tag;

	short unused[365];
	
	short map_description_string_index;
	short overhead_map_collection_index;
	short landscape_collection_index;
	short global_ambient_sound_index;
	short media_type;
	
	byte editor_data[MESH_HEADER_EDITOR_DATA_SIZE];
};

/* ---------- prototypes/MESH_FILE.C */

#endif // __MESH_FILE_H__
