#ifdef OBSOLETE

Moved to geometry_definitions.h


#ifndef __GEOMETRY_FILE_H__
#define __GEOMETRY_FILE_H__

/*
 * GEOMETRY_FILE.H
 */

#include "tag_files.h"
#include "world.h"

/* ---------- constants */

enum
{
	GEOMETRY_GROUP_TAG= 0x67656F6D,	// 'geom' looks like text
	GEOMETRY_VERSION_NUMBER= 1,
	
	MAXIMUM_MATERIALS_PER_GEOMETRY= 32,
	MAXIMUM_SURFACES_PER_GEOMETRY= 512,
	MAXIMUM_VERTICES_PER_GEOMETRY= 2048
};

/* ---------- structures */

/* ---------- geometry definition */

#define SIZEOF_STRUCT_GEOMETRY_VERTEX 6
struct geometry_vertex
{
	short_world_point3d p;
};

enum
{
	NUMBER_OF_CORNERS_PER_TRIANGLE= 3,
	NUMBER_OF_ENCODED_GEOMETRY_CORNER_NORMALS= 1024,
	NUMBER_OF_GEOMETRY_CORNER_INTERPOLANTS= 2
};

#define SIZEOF_STRUCT_GEOMETRY_CORNER 8
struct geometry_corner
{
	fixed_fraction interpolants[NUMBER_OF_GEOMETRY_CORNER_INTERPOLANTS];
	
	short vertex_index;
	short encoded_normal; // used for normal-based shading (environment mapping, etc.)
};

enum // struct geometry_surface flags
{
	_geometry_surface_is_transparent_bit,	// draw back-to-front
	_geometry_surface_no_backface_cull_bit,	// don't cull; used for glass, etc.
	_geometry_surface_under_mesh_bit,	// its just water under the bridge

	// these bits are set and reset by the model for rendering, collision detection, etc..
	_geometry_surface_negative_bit,	// for iterative traversal-> used at runtime
	_geometry_surface_does_not_stop_projectiles_bit,	// for iterative traversal-> used at runtime
	
	NUMBER_OF_GEOMETRY_SURFACE_FLAGS, // <=16
	
	_geometry_surface_is_transparent_flag= FLAG(_geometry_surface_is_transparent_bit),
	_geometry_surface_no_backface_cull_flag= FLAG(_geometry_surface_no_backface_cull_bit),
	_geometry_surface_under_mesh_flag= FLAG(_geometry_surface_under_mesh_bit),
	_geometry_surface_negative_flag= FLAG(_geometry_surface_negative_bit),
	_geometry_surface_does_not_stop_projectiles_flag= FLAG(_geometry_surface_does_not_stop_projectiles_bit)
};

#define BUILD_MESH_CELL_CONTAINMENT(x, y) (((y)<<8)|(x))
#define GET_MESH_CELL_CONTAINMENT_X(c) ((c)&0xff)
#define GET_MESH_CELL_CONTAINMENT_Y(c) (((c)>>8)&0xff)

#define SIZEOF_STRUCT_GEOMETRY_SURFACE 64
struct geometry_surface
{
	word flags;
	
	world_vector3d normal;
	world_distance d;				// of plane equation

	short_world_rectangle3d bounds;	// bounding box in geometry space for this triangle
	
	fixed_fraction transparency;
	short material_index;
	
	struct geometry_corner corners[NUMBER_OF_CORNERS_PER_TRIANGLE];

	// bsp tree shit
	short next_negative_surface_index;
	short next_parallel_surface_index;
	short next_positive_surface_index;

	word mesh_cell_contained;
	short dependency_table_index;
	short unused[1];
};

enum
{
	MAXIMUM_GEOMETRY_MATERIAL_NAME_LENGTH= 32
};

#define SIZEOF_STRUCT_GEOMETRY_MATERIAL 64
struct geometry_material
{
	char name[MAXIMUM_GEOMETRY_MATERIAL_NAME_LENGTH];	// used only in fear
	short sequence_index;
	
	// these two fields are filled in before rendering
	short collection_index;
	short color_table_index;
	short bitmap_index;
	
	short unused[12];
};

enum // struct geometry_definition flags (also user_data contents)
{
	_geometry_definition_is_mesh_aligned_bit,
	_geometry_definition_is_object_bit, // completely fits within one cell...
	
	NUMBER_OF_GEOMETRY_DEFINITION_FLAGS, // <=31
	
	_geometry_definition_is_mesh_aligned_flag= FLAG(_geometry_definition_is_mesh_aligned_bit),
	_geometry_definition_is_object_flag= FLAG(_geometry_definition_is_object_bit)
};

#define SIZEOF_STRUCT_GEOMETRY_DEFINITION 96
struct geometry_definition
{
	unsigned long flags;
	
	file_tag collection_reference_tag;
	
	short material_count;
	short vertex_count;
	short surface_count;
	short dependency_count;

	short_world_point3d center; // in geometry coordinate space.
	short height;
	long radius_squared;
	
	int materials_offset, materials_size;
	struct geometry_material *materials;	// memory-based
	
	int vertices_offset, vertices_size;
	struct geometry_vertex *vertices; // memory-based
	
	int surfaces_offset, surfaces_size;
	struct geometry_surface *surfaces; // memory-based

	int dependency_list_offset, dependency_size;
	short *dependency_list;
	
	long data_offset, data_size;
	void *data; // memory-based
	
	fixed_fraction transparency; // used for model crossfades...
	short unused[2];

	short collection_reference_index; // only valid at runtime..
};
#endif


#endif