#ifndef __MARKERS_H__
#define __MARKERS_H__

/*
MARKERS.H
Friday August 2, 1996 12:02 PM
*/

#include "world.h"
#include "tag_files.h"

/* ---------- markers */

enum // marker types
{
	_marker_team, // no tag
	_marker_scenery,
	_marker_item,
	_marker_monster, // refers to unit tag
	_marker_projectile_generator,
	_marker_sound_source,
	_marker_model,
	_marker_media_current,
	_marker_script,
	_marker_projectile,
	_marker_local_projectile_group,
	NUMBER_OF_MARKER_TYPES
};

enum // marker palette flags
{
	_marker_palette_uncontrollable_bit, // has the "ignores player control" bit set
	_marker_palette_may_be_traded_bit, // if the game allows, these units can be traded for other units
	_marker_palette_may_use_veterans_bit, // if veterans exist from the previous level they can be used as replacements for these units
	_marker_palette_must_use_veterans_bit, // veterans must exist from the previous level for these units to be placed
	
	NUMBER_OF_MARKER_PALETTE_FLAGS,
	
	_marker_palette_uncontrollable_flag= FLAG(_marker_palette_uncontrollable_bit),
	_marker_palette_may_be_traded_flag= FLAG(_marker_palette_may_be_traded_bit),
	_marker_palette_may_use_veterans_flag= FLAG(_marker_palette_may_use_veterans_bit),
	_marker_palette_must_use_veterans_flag= FLAG(_marker_palette_must_use_veterans_bit)
};

#define SIZEOF_STRUCT_MARKER_PALETTE_ENTRY 32
struct marker_palette_entry
{
	short type; // of marker (types which don't need palettes don't have them)
	word flags;
	
	file_tag tag; // subgroup tag based on type (types which don't need subgroups don't have them)
	
	short team_index; // to establish ownership; only planned to be used for monsters and observers
	
	short unused[9];
	
	// load time
	
	short count; // total references on map; temporary field constructed during palette editing
	short tag_index; // NONE if couldn't load tag
};

enum // marker flags
{
	_marker_is_invisible_bit, // not initially placed
	_marker_is_stone_bit, // stone balls/flags never change owners
	_marker_detonates_immediately_bit,
	_marker_unused3_bit,
	_marker_unused4_bit,
	_marker_unused5_bit,
	_marker_unused6_bit,
	_marker_unused7_bit,
	
	NUMBER_OF_MARKER_FLAGS,
	
	_marker_is_invisible_flag= FLAG(_marker_is_invisible_bit),
	_marker_is_stone_flag= FLAG(_marker_is_stone_bit),
	_marker_detonates_immediately_flag= FLAG(_marker_detonates_immediately_bit),
	_marker_unused3_flag= FLAG(_marker_unused3_bit),
	_marker_unused4_flag= FLAG(_marker_unused4_bit),
	_marker_unused5_flag= FLAG(_marker_unused5_bit),
	_marker_unused6_flag= FLAG(_marker_unused6_bit),
	_marker_unused7_flag= FLAG(_marker_unused7_bit)
};

#define SIZEOF_MARKER_USER_DATA 16
#define SIZEOF_STRUCT_MARKER_DATA 64
struct marker_data
{
	unsigned long flags;
	
	short type; // _marker_observer, _marker_scenery, ...
	short palette_index; // index into global marker palette list

	short identifier; // unique for each marker type
	
	short minimum_difficulty_level;
	
	world_location3d location;

	char user_data[SIZEOF_MARKER_USER_DATA];	// specific to the marker type
	
	short unused[4];

	// load time

	short data_index, data_identifier; // usually object_index/object_identifier
};

/* ---------- markers */

struct scan_markers_state
{
	short type; // NONE matches all types
	short palette_index; // NONE matches all entries of this type in the palette
	short data_index; // NONE matches against all data_indexes

	// result
	
	short result_marker_index; // of returned marker, NONE if NULL

	// internal
	
	short internal_index;
};

/* ---------- marker palettes */

struct scan_marker_palette_state
{
	short type; // NONE matches all types
	
	file_tag tag; // NONE matches all tags
	
	// result
	
	short result_palette_index; // of returned entry, NONE if NULL
	short typed_index; // same as above but only within the specified type
	
	// internal
	
	short internal_index;
};

/* ---------- prototypes/MARKERS.C */

void initialize_markers(void);

void initialize_markers_for_new_map(void);

void new_scan_markers_state(struct scan_markers_state *state, int type_filter, int palette_index_filter, int data_index_filter);
struct marker_data *scan_markers(struct scan_markers_state *state);

void new_scan_marker_palette_state(struct scan_marker_palette_state *state, int type_filter, int tag_filter);
struct marker_palette_entry *scan_marker_palette(struct scan_marker_palette_state *state);

struct marker_data *get_marker_by_identifier(int type, int identifier);
struct marker_data *get_marker_by_data_index(int type, int data_index);

int count_markers(int type);
int count_marker_palette_entries(int type);

int get_marker_array_of_type(int type, struct marker_data **markers);

struct marker_palette_entry *get_marker_palette_entry(int type, int index);

void add_markers_to_new_map(void);

void add_marker_palette_entry_for_type(int type, struct marker_palette_entry *entry);
file_tag marker_type_to_tag_group(int type);
const char *marker_type_to_group_name(int type);
int create_data_from_marker(int type, struct marker_data *marker);
void delete_data_associated_with_marker(struct marker_data *marker);

void sort_markers_before_saving(void);

void byte_swap_marker_palette(struct marker_palette_entry *marker_palette, int count);
void byte_swap_marker_list(struct marker_data *markers, int count);

#endif // __MARKERS_H__
