/*----------------------------------------------------------------------------------------
	soun.h

	Sounds are all compressed using the 'ima4' codec. This is hard-coded. Most are 16-bit,
	22.050 kHz samples, although the engine will support other formats
	
	Following is a typical Macintosh compressed sound header, after being converted from
	Myth's internal format (fields marked with  are taken from the header):

		0x0001								//	format type; all sounds are Type 1
		0x0001								//	number of synthesizers
		0x0005								//	resource ID of the sampled sound synth
		0x000000C0							//	initialization options
		0x0001								//	number of sound commands that follow
		0x8051								//	sound command - bufferCmd
		0x0000								//	param1, unused
		0x00000014							//	param2, offset to sound header
		0x00000000							//	pointer to data (incorporated)
		0x00000001							//	 number of channels in sample
		0x56220000							//	 sample rate as a Fixed, here 22.050kHz
		0x00000000							//	 loopStart
		0x00000000							//	 loopEnd
		0xFE								//	compressed header format, hard coded
		0x3C								//	baseFrequency, sampled at middle C, hard coded
		0x00000304							//	number of frames, data size divided by 0x0022
		0x00000000000000000000				//	AIFF sample rate, unused
		0x00000000							//	markerChunk (unused)
		0x696D6134							//	codec, always 'ima4', hard-coded
		0x00000000							//	futureUse2, unused
		0x00000000							//	stateVars, unused
		0x00000000							//	leftOverBlockPtr, unused
		0xFFFF								//	compression ID, -1 means use codec
		0x0000								//	packetSize
		0x0000								//	synthID
		0x0010								//	 sampleSize, 0x0010 is 16-bit

	Following are the two primary .gor structs, described here for reference. Note that
	I have formatted the soun header as a C-style struct, but it is actually not usable
	as such because of the variable lengths of parts of the header on disk.
	
	typedef struct gor_Header {
		UInt16			version;
		UInt16			dataVersion;
		char			name[32];
		UInt32			checksum;
		Int32			tagHeaderOffset;
		UInt16			headerCount;
		UInt16			headerSize;			//	64
		UInt16			flags;
		UInt16			padding;
		UInt32			modTime;
		UInt16			unused1[35];
		UInt16			unused2;
	} gor_Header;


	typedef struct dir_Header {
		char			name[32];
		FourCharCode	group;
		FourCharCode	subGroup;
		UInt32			version;
		UInt16			tagVersion;
		UInt16			flags;
		Int32			dataOffset;
		UInt32			dataSize;
		UInt32			userData;
		FourCharCode	fileTag;			//	'myth'
	} dir_Header;
*/

typedef struct soun_Header {				//	offsets are given from start of sound header
	UInt32			u1;
	UInt16			u2;
	UInt16			u3;						//	possibly flags
	UInt16			u4;						//	 0x0100	This and the following may be left
	UInt16			u5;						//	 0x0100	and right volume control
	UInt32			u6;						//	possibly flags
	UInt32			subtitleIndex;			//	1-based index into related 'stli' tag, 0 is none
	UInt32			headerSize;				//	offset to start of data
	UInt32			dataSize;				//	total data size, including core headers
	FourCharCode	stliID;					//	optional ID of related 'slti' tag
	UInt32			u7;						//	 0x00000000
	UInt32			soundCount;
	UInt32			tocOffset;				//	offset to where toc begins
	UInt32			tocSize;				//	offset from start of toc (same loc as headerOffset)
	UInt16			unused [8];
}; typedef struct soun_Header soun_Header;

struct soun_tocEntry {
	UInt16			u1;
	UInt16			u2;					//	possibly flags
	UInt16			u3;
	char			name[26];
}; typedef struct soun_tocEntry soun_tocEntry;
	
struct soun_indexEntry {
	UInt32			u1;					//	 0x00000001
	UInt16			sampleSize;			//	0x0010 is 16-bit
	UInt16			u3;					//	 0x0001
	UInt16			numChannels;		//	stored as short, converted to long for header
	UInt16			u5;					//	 0x0000
	Fixed			sampleRate;			//	0x5622000 = 22.050 kHz
	UInt32			frameCount;			//	multiply by 0x0022 to get actual size
	UInt32			loopStart;
	UInt32			loopEnd;
	UInt16			unused [2];
}; typedef struct soun_indexEntry soun_indexEntry;
