#ifndef __WORLD_H__
#define __WORLD_H__

/*
WORLD.H
Wednesday, June 17, 1992 6:40:10 PM

Friday, June 26, 1992 10:47:47 AM
	to maintain precision when squaring world coordinates, they must be in [0,32*WORLD_ONE).
Wednesday, August 18, 1993 2:59:47 PM
	added 3d world points, world_distance is now a short (because we care).
Sunday, May 22, 1994 10:48:26 PM
	added fixed_point3d.  GUESS_HYPOTENUSE() is no longer completely broken.
Tuesday September 3, 1996 8:35 AM
	angles are now 16-bit quantities with fractional parts, removed fixed_angle.
*/

#include <math.h>

/* ---------- constants */

enum
{
	MILLISECONDS_PER_SECOND= 1000,
	SECONDS_PER_MINUTE= 60,
	MINUTES_PER_HOUR= 60,
	HOURS_PER_DAY= 24,
	TICKS_PER_SECOND= 30,
	TICKS_PER_MINUTE= TICKS_PER_SECOND*SECONDS_PER_MINUTE,
	TICKS_PER_HOUR= TICKS_PER_MINUTE*MINUTES_PER_HOUR,
	TICKS_PER_DAY= TICKS_PER_HOUR*HOURS_PER_DAY,
	
	TRIG_SHIFT= 14,
	TRIG_BITS= TRIG_SHIFT,
	TRIG_MAGNITUDE= 1<<TRIG_SHIFT,

	ANGULAR_BITS= 16,
	NUMBER_OF_ANGLES= 1<<ANGULAR_BITS,
	FULL_CIRCLE= NUMBER_OF_ANGLES,
	QUARTER_CIRCLE= NUMBER_OF_ANGLES/4,
	HALF_CIRCLE= NUMBER_OF_ANGLES/2,
	THREE_QUARTER_CIRCLE= (3*NUMBER_OF_ANGLES)/4,
	EIGHTH_CIRCLE= NUMBER_OF_ANGLES/8,
	SIXTEENTH_CIRCLE= NUMBER_OF_ANGLES/16,

	SIGNIFICANT_ANGULAR_BITS= 10,
	NUMBER_OF_SIGNIFICANT_ANGLES= 1<<SIGNIFICANT_ANGULAR_BITS,
	ANGLE_ONE= 1<<(ANGULAR_BITS-SIGNIFICANT_ANGULAR_BITS),

	WORLD_FRACTIONAL_BITS= 10,
	WORLD_ONE= 1<<WORLD_FRACTIONAL_BITS,
	WORLD_ONE_HALF= WORLD_ONE/2,
	WORLD_ONE_FOURTH= WORLD_ONE/4,
	WORLD_THREE_FOURTHS= (3*WORLD_ONE)/4,

	DEFAULT_RANDOM_SEED= 0xfded
};

/* ---------- types */

typedef word angle;

typedef short short_world_distance;
typedef long world_distance;

typedef float real;

/* ---------- macros */

#define INTEGER_TO_WORLD(s) (((world_distance)(s))<<WORLD_FRACTIONAL_BITS)
#define WORLD_FRACTIONAL_PART(d) ((d)&((world_distance)(WORLD_ONE-1)))
#define WORLD_TO_INTEGER(d) ((d)>>WORLD_FRACTIONAL_BITS)

#define WORLD_TO_FIXED(w) (((fixed)(w))<<(FIXED_FRACTIONAL_BITS-WORLD_FRACTIONAL_BITS))
#define FIXED_TO_WORLD(f) ((world_distance)((f)>>(FIXED_FRACTIONAL_BITS-WORLD_FRACTIONAL_BITS)))

#define WORLD_TO_FLOAT(f) (((float)(f))/WORLD_ONE)
#define FLOAT_TO_WORLD(f) ((world_distance)((f)*WORLD_ONE))

#define ANGLE_TO_INTEGER_ANGLE(a) ((a)>>(ANGULAR_BITS-SIGNIFICANT_ANGULAR_BITS))
#define INTEGER_ANGLE_TO_ANGLE(a) ((a)<<(ANGULAR_BITS-SIGNIFICANT_ANGULAR_BITS))
#define NORMALIZE_ANGLE(a) ((a)&(FULL_CIRCLE-1))

/* arguments must be positive (!) or use guess_hypotenuse() */
#define GUESS_HYPOTENUSE(x, y) ((x)>(y) ? ((x)+((y)>>1)) : ((y)+((x)>>1)))

#if 1
// Here's the new random range calls which use the high random number bits instead of the low bits.
// The parentheses in the random radius calls have also been fixed.
	enum {
		RAND_BITS= 16
	};

	#define RANDOM_ANGLE() random()
	#define RANDOM_RANGE(lower_bound, delta) ((lower_bound) + (((delta)>0) ? (((delta)*random())>>RAND_BITS) : 0))
	#define RANDOM_RANGE_32BIT(lower_bound, delta) ((lower_bound) + (((delta)>0) ? ((delta)*random()) : 0))
	#define RANDOM_RADIUS(radius) (((radius)>0) ? (((((2*(radius))+1)*random())>>RAND_BITS) - (radius)) : 0)
	#define RANDOM_RADIUS_32BIT(radius) (((radius)>0) ? ((((2*(radius))+1)*random()) - (radius)) : 0)

	#define LOCAL_RANDOM_ANGLE() local_random()
	#define LOCAL_RANDOM_RANGE(lower_bound, delta) ((lower_bound) + (((delta)>0) ? (((delta)*local_random())>>RAND_BITS) : 0))
	#define LOCAL_RANDOM_RANGE_32BIT(lower_bound, delta) ((lower_bound) + (((delta)>0) ? ((delta)*local_random()) : 0))
	#define LOCAL_RANDOM_RADIUS(radius) (((radius)>0) ? (((((2*(radius))+1)*local_random())>>RAND_BITS) - (radius)) : 0)
	#define LOCAL_RANDOM_RADIUS_32BIT(radius) (((radius)>0) ? ((((2*(radius))+1)*local_random()) - (radius)) : 0)

#else
	#define RANDOM_ANGLE() random()
	#define RANDOM_RANGE(lower_bound, delta) ((lower_bound) + (((delta)>0) ? (random()%(delta)) : 0))
	#define RANDOM_RADIUS(radius) (((radius)>0) ? ((random()%(2*((radius)+1))) - (radius)) : 0)

	#define LOCAL_RANDOM_RANGE(lower_bound, delta) ((lower_bound) + (((delta)>0) ? (local_random()%(delta)) : 0))
	#define LOCAL_RANDOM_RADIUS(radius) (((radius)>0) ? ((local_random()%(2*((radius)+1))) - (radius)) : 0)
#endif

/* ---------- point structures */

struct world_point2d
{
	world_distance x, y;
};
typedef struct world_point2d world_point2d;

struct short_world_point2d
{
	short_world_distance x, y;
};
typedef struct short_world_point2d short_world_point2d;

struct world_point3d
{
	world_distance x, y, z;
};
typedef struct world_point3d world_point3d;

struct short_world_point3d
{
	short_world_distance x, y, z;
};
typedef struct short_world_point3d short_world_point3d;

/* ---------- rectangle structures */

struct short_world_rectangle2d
{
	short_world_point2d minimum_corner, maximum_corner;
};
typedef struct short_world_rectangle2d short_world_rectangle2d;

struct short_world_rectangle3d
{
	short_world_point3d minimum_corner, maximum_corner;
};
typedef struct short_world_rectangle3d short_world_rectangle3d;

struct world_rectangle2d
{
	world_point2d minimum_corner, maximum_corner;
};
typedef struct world_rectangle2d world_rectangle2d;

struct world_rectangle3d
{
	world_point3d minimum_corner, maximum_corner;
};
typedef struct world_rectangle3d world_rectangle3d;

/* ---------- vector structures */

struct world_vector2d
{
	short_world_distance i, j;
};
typedef struct world_vector2d world_vector2d;

struct world_vector3d
{
	short_world_distance i, j, k;
};
typedef struct world_vector3d world_vector3d;

struct fixed_world_vector3d
{
	fixed i, j, k;
};
typedef struct fixed_world_vector3d fixed_world_vector3d;

/* ---------- locations */

struct world_location3d
{
	world_point3d position;
	world_vector3d velocity;
	short_world_distance height; // height above ground (or under media?)
	
	angle yaw, pitch;
};
typedef struct world_location3d world_location3d;

/* ---------- matrices */

struct matrix3x3
{
	short n[3][3];
};
typedef struct matrix3x3 matrix3x3;

struct real_matrix3x3
{
	real n[3][3];
};
typedef struct real_matrix3x3 real_matrix3x3;

/* ---------- reals */

struct real_point2d
{
	real x, y;
};
typedef struct real_point2d real_point2d;

struct real_vector2d
{
	real i, j, k;
};
typedef struct real_vector2d real_vector2d;

struct real_point3d
{
	real x, y, z;
};
typedef struct real_point3d real_point3d;

struct real_vector3d
{
	real i, j, k;
};
typedef struct real_vector3d real_vector3d;

/* ---------- globals */

extern real _pi, _2pi;
extern real _deg2rad, _rad2deg;

/* ---------- prototypes/WORLD.C */

void initialize_world(void);

#ifdef DEBUG
int cosine(int theta);
int sine(int theta);
#else
extern short *cosine_table, *sine_table;
#define cosine(theta) cosine_table[ANGLE_TO_INTEGER_ANGLE(NORMALIZE_ANGLE((theta)+ANGLE_ONE/2))]
#define sine(theta) sine_table[ANGLE_TO_INTEGER_ANGLE(NORMALIZE_ANGLE((theta)+ANGLE_ONE/2))]
#endif

int angular_difference(int angle1, int angle2);
int signed_angular_difference(int angle1, int angle2);

int arctangent(int x, int y);
int arccosine(int value);
int arcsine(int value);

world_vector3d *set_world_vector3d(world_vector3d *v, world_distance i, world_distance j, world_distance k);
world_point2d *set_world_point2d(world_point2d *p, world_distance x, world_distance y);
short_world_point2d *set_short_world_point2d(short_world_point2d *p, short_world_distance x, short_world_distance y);
world_point3d *set_world_point3d(world_point3d *p, world_distance x, world_distance y, world_distance z);
short_world_point3d *set_short_world_point3d(short_world_point3d *p, short_world_distance x, short_world_distance y, short_world_distance z);
world_vector3d *build_world_vector3d(world_vector3d *v, world_distance magnitude, angle yaw, angle pitch);
world_vector3d *line_segment_to_vector(world_vector3d *v, world_point3d *p0, world_point3d *p1, world_distance vector_magnitude);

short_world_distance world_magnitude3d(world_vector3d *v);

world_point2d *rotate_point2d(world_point2d *point, world_point2d *origin, angle theta);
world_point2d *translate_point2d(world_point2d *point, world_distance distance, angle yaw);
short_world_point2d *translate_short_point2d(short_world_point2d *point, world_distance distance, angle yaw);
world_point3d *translate_point3d(world_point3d *point, world_distance distance, angle yaw, angle pitch);

matrix3x3 *build_rotation_matrix(matrix3x3 *m, int yaw, int pitch);
matrix3x3 *build_inverse_rotation_matrix(matrix3x3 *m, int yaw, int pitch);

short_world_point3d *transform_point3d(short_world_point3d *result, matrix3x3 *rotation, world_point3d *viewer, world_point3d *point);

matrix3x3 *build_rotation_matrix3d(matrix3x3 *m, world_vector3d *forward, world_vector3d *up);
matrix3x3 *build_inverse_rotation_matrix3d(matrix3x3 *m, world_vector3d *forward, world_vector3d *up);

world_vector3d *reflect_vector(world_vector3d *v0, world_vector3d *n, world_vector3d *v1);
world_vector3d *rotate_vector(world_vector3d *result, matrix3x3 *rotation, world_vector3d *source);

/*void set_random_seed(word seed);
word get_random_seed(void);*/
void set_random_seed(unsigned long seed);
unsigned long get_random_seed(void);

word random(void);
word local_random(void);

world_distance guess_distance2d(world_point3d *p0, world_point3d *p1);
world_distance guess_distance3d(world_point3d *p0, world_point3d *p1);
world_distance distance3d(world_point3d *p0, world_point3d *p1); // calls isqrt()
world_distance distance2d(world_point2d *p0, world_point2d *p1); // calls isqrt()

void closest_point2d_test_case(void);
boolean closest_point_on_line_segment_from_point(world_point2d *point, world_point2d *line0,
	world_point2d *line1, world_point2d *closest_point_on_line_reference,
	world_point2d *closest_point_on_line_segment_reference);
boolean closest_point_to_line_segment2d(world_point2d *A, world_point2d *B, world_point2d *C,
	world_point2d *closest_pt);

world_distance world_dot_product3d(world_vector3d *a, world_vector3d *b);
world_vector3d *world_cross_product3d(world_vector3d *a, world_vector3d *b, world_vector3d *r);

world_distance point_in_plane_test(world_vector3d *normal, world_distance d, short_world_point3d *p);

long isqrt(register unsigned long x);

void start_recording_random_seed(void);
void stop_recording_random_seed(void);

void dprintf_matrix(char *message, matrix3x3 *m); // for debugging

/* ---------- prototypes/REAL_WORLD.C */

void initialize_real_world(void);

real_vector3d *set_real_vector3d(struct real_vector3d *v, real i, real j, real k);
real_point3d *set_real_point3d(struct real_point3d *p, real x, real y, real z);

real magnitude2d(real_vector2d *a);
real dot_product2d(real_vector2d *a, real_vector2d *b);
real cross_product2d(real_vector2d *a, real_vector2d *b);

real real_distance3d(real dx, real dy, real dz);

real magnitude3d(real_vector3d *a);
real_vector3d *cross_product3d(real_vector3d *a, real_vector3d *b, real_vector3d *r);
real dot_product3d(real_vector3d *a, real_vector3d *b);
real_vector3d *normalize3d(real_vector3d *a);

real_matrix3x3 *quaternion_to_real_matrix(real_matrix3x3 *m, real_vector3d *v, real w);
real_matrix3x3 *real_matrix_multiply(real_matrix3x3 *m, real_matrix3x3 *o, real_matrix3x3 *result);
real_matrix3x3 *real_matrix_inverse(real_matrix3x3 *m, real_matrix3x3 *result);
real real_matrix_cofactor(real_matrix3x3 *m, int i, int j);
real real_matrix_determinant(real_matrix3x3 *m);
real_vector3d *real_vector_times_real_matrix(real_vector3d *p, real_matrix3x3 *m, real_vector3d *result);

#endif // __WORLD_H__
