// ===========================================================================
// CGWorldView.cp
// ===========================================================================
// CGWorldView - Base class only
// CGWorldViewTimed - Draws the GWorld in the Idle cycle, every mDrawInterval ticks
// CGWorldViewListener - Draws the GWorld in response to a message

#include <LStream.h>
#include "CGWorldView.h"
#include <UDrawingState.h>

// ---------------------------------------------------------------------------
// Default Constructor
// ---------------------------------------------------------------------------
CGWorldView::CGWorldView()
	: LView()
{
	mGWorld = nil;
};

// ---------------------------------------------------------------------------
// Stream Constructor
// ---------------------------------------------------------------------------
CGWorldView::CGWorldView(LStream *inStream)
	: LView(inStream)
{
};

// ---------------------------------------------------------------------------
// Destructor
// ---------------------------------------------------------------------------
CGWorldView::~CGWorldView(void)
{
};

// ---------------------------------------------------------------------------
// SetMyGWorld
// ---------------------------------------------------------------------------
void
CGWorldView::SetMyGWorld(LGWorld *inGW)
{
	if (inGW) {
		Rect bounds, size;
		inGW->GetBounds(bounds);
		size.left = 10;
		size.right = bounds.right - bounds.left;
		size.top = 10;
		size.bottom = bounds.bottom - bounds.top;
		ResizeImageTo(size.right, size.bottom, true);
		GrafPtr myPort = GetMacPort();
		LWindow *myWindow = LWindow::FetchWindowObject(myPort);
		myWindow->SetMinMaxSize(size);
		SDimension16 dims;
		dims.width = size.right;
		dims.height = size.bottom;
		myWindow->SetStandardSize(dims);
	}
	mGWorld = inGW;
}

// ---------------------------------------------------------------------------
// DrawSelf
// ---------------------------------------------------------------------------
void
CGWorldView::DrawSelf(void)
{
	if(!mGWorld) return;
	Rect theRect;
	mGWorld->GetBounds(theRect);
	if ((theRect.right - theRect.left != mImageSize.width) ||
		(theRect.bottom-theRect.top != mImageSize.height)) {
		ResizeImageTo(theRect.right - theRect.left, theRect.bottom-theRect.top, true);
		FocusExposed(true);
	}
	mGWorld->CopyImage(GetMacPort(), theRect);
}

#pragma mark -

// ---------------------------------------------------------------------------
// Default Constructor
// ---------------------------------------------------------------------------
CGWorldViewTimed::CGWorldViewTimed()
	: CGWorldView(), LPeriodical()
{
	mLastDraw = 0;
	mDrawInterval = 60; // in Ticks
};

// ---------------------------------------------------------------------------
// Stream Constructor
// ---------------------------------------------------------------------------
CGWorldViewTimed::CGWorldViewTimed(LStream *inStream)
	: CGWorldView(inStream)
{
	UInt8	redrawSecs;
	inStream->ReadData(&redrawSecs, sizeof(UInt8));
	mLastDraw = 0;
	mDrawInterval = redrawSecs*60;
};


// ---------------------------------------------------------------------------
// Destructor
// ---------------------------------------------------------------------------
CGWorldViewTimed::~CGWorldViewTimed(void)
{
};

// ---------------------------------------------------------------------------
// SetMyGWorld
// ---------------------------------------------------------------------------
void
CGWorldViewTimed::SetMyGWorld(LGWorld *inGW)
{
	CGWorldView::SetMyGWorld(inGW);	// call inherited
	
	if (!inGW) {
		StopIdling();
	} else {
		StartIdling();
	}
}

// ---------------------------------------------------------------------------
// SpendTime
// ---------------------------------------------------------------------------
void
CGWorldViewTimed::SpendTime(const EventRecord	&/*inMacEvent*/)
{
	Uint32 theTick = ::TickCount();
	if (theTick > mLastDraw + mDrawInterval) {
		Refresh();
		mLastDraw = theTick;
	}
}



#pragma mark -
// ---------------------------------------------------------------------------
// Default Constructor
// ---------------------------------------------------------------------------
CGWorldViewListener::CGWorldViewListener()
	: CGWorldView(), LListener()
{
};

// ---------------------------------------------------------------------------
// Stream Constructor
// ---------------------------------------------------------------------------
CGWorldViewListener::CGWorldViewListener(LStream *inStream)
	: CGWorldView(inStream)
{
};


// ---------------------------------------------------------------------------
// Destructor
// ---------------------------------------------------------------------------
CGWorldViewListener::~CGWorldViewListener(void)
{
};

// ---------------------------------------------------------------------------
// ListenToMessage
// ---------------------------------------------------------------------------
void
CGWorldViewListener::ListenToMessage(
								MessageT		inMessage,
								void			* /*ioParam*/)
{
	if (inMessage == msg_JustChanged)
		Refresh();
}
