// =================================================================================
//	C256Tag.h					 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#pragma once


#include "CTag.h"

class CCollectionHeader;
class CColorTable;
class CHueChange;
class CSequence;
class CBlendTable;
class CRemapTable;
class CBitmap;

class CTagElementArray;


class C256Tag : public CTag {
public:
						// Default constructor and destructor
						
						C256Tag();
	virtual				~C256Tag();
	
						// Read from and write to "file"
						
	virtual void		ReadStream( LStream &inStream, Int32 inOffset );
	virtual void		WriteStream( LStream &outStream, Int32 inOffset );
	
						// Returns the tag data size on file (minus header)
	
	virtual Int32		GetSize() const;
	
					// LListener
	
	virtual void		ListenToMessage( MessageT inMessage, void *ioData );

					// Accessors
				
						// The Collection Header
	
	Uint32				GetUserDataFlags() const;
	
	Int32				GetColorTableCount() const;
	Int32				GetHueChangeCount() const;
	Int32				GetSequenceCount() const;
	Int32				GetBlendTableCount() const;
	Int32				GetRemapTableCount() const;
	Int32				GetBitmapCount() const;
	Int32				GetColorBitmapCount() const;
	Int32				GetShadowBitmapCount() const;
	
						// Color Tables
	
	const CColorTable	*GetColorTable( Int32 inIndex = 0 ) const;
	CColorTable			*GetColorTable( Int32 inIndex = 0 );
	Boolean				MoveColorTable( CColorTable *inColorTable, Int32 inNewIndex );
	Boolean				InsertColorTable( CColorTable *inColorTable, Int32 inAtIndex );
	Boolean				RemoveColorTable( CColorTable *inColorTable );
	
	Boolean				GetColorTable( RGBColor *outColorTable, Int32 inIndex = 0 ) const;
	Boolean				SetColorTable( const RGBColor *inColorTable, Int32 inIndex = 0 );
						
						// Hue Changes
						
	const CHueChange	*GetHueChange( Int32 inIndex ) const;
	CHueChange			*GetHueChange( Int32 inIndex );
	Boolean				MoveHueChange( CHueChange *inHueChange, Int32 inNewIndex );
	Boolean				RemoveHueChange( CHueChange *inHueChange );
	Boolean				InsertHueChange( CHueChange *inHueChange, Int32 inAtIndex );
	
	void				GetHueChangedColorTable( RGBColor *outColorTable ) const;
	void				GetHueChangedColorIndex( Byte *outIndex ) const;
	
						// Sequences
	
	const CSequence		*GetSequence( Int32 inIndex ) const;
	CSequence			*GetSequence( Int32 inIndex );
	Boolean				MoveSequence( CSequence *inSequence, Int32 inNewIndex );
	Boolean				RemoveSequence( CSequence *inSequence );
	Boolean				InsertSequence( CSequence *inSequence, Int32 inAtIndex );
	
						// Blend Tables
	
	const CBlendTable	*GetBlendTable( Int32 inIndex = 0 ) const;
	CBlendTable			*GetBlendTable( Int32 inIndex = 0 );
	Boolean				MoveBlendTable( CBlendTable *inTable, Int32 inNewIndex );
	Boolean				RemoveBlendTable( CBlendTable *inTable );
	Boolean				InsertBlendTable( CBlendTable *inTable, Int32 inAtIndex );
	
						// Remap Tables
	
	const CRemapTable	*GetRemapTable( Int32 inIndex = 0 ) const;
	CRemapTable			*GetRemapTable( Int32 inIndex = 0 );
	Boolean				MoveRemapTable( CRemapTable *inTable, Int32 inNewIndex );
	Boolean				RemoveRemapTable( CRemapTable *inTable );
	Boolean				InsertRemapTable( CRemapTable *inTable, Int32 inAtIndex );
	
	
						// Color Bitmaps
	
	const CBitmap		*GetColorBitmap( Int32 inIndex ) const;
	CBitmap				*GetColorBitmap( Int32 inIndex );
	const CBitmap		*GetNextColorBitmap( const CBitmap *inBitmap ) const;
	CBitmap				*GetNextColorBitmap( const CBitmap *inBitmap );
	const CBitmap		*GetPrevColorBitmap( const CBitmap *inBitmap ) const;
	CBitmap				*GetPrevColorBitmap( const CBitmap *inBitmap );
	
	Boolean				MoveColorBitmap( CBitmap *inBitmap, Int32 inNewIndex );
	Boolean				RemoveColorBitmap( CBitmap *inBitmap );
	Boolean				InsertColorBitmap( CBitmap *inBitmap, Int32 inAtIndex );
	
						// Shadow Bitmaps
	
	const CBitmap		*GetShadowBitmap( Int32 inIndex ) const;
	CBitmap				*GetShadowBitmap( Int32 inIndex );
	const CBitmap		*GetNextShadowBitmap( const CBitmap *inBitmap ) const;
	CBitmap				*GetNextShadowBitmap( const CBitmap *inBitmap );
	const CBitmap		*GetPrevShadowBitmap( const CBitmap *inBitmap ) const;
	CBitmap				*GetPrevShadowBitmap( const CBitmap *inBitmap );
	
	Boolean				MoveShadowBitmap( CBitmap *inBitmap, Int32 inNewIndex );
	Boolean				RemoveShadowBitmap( CBitmap *inBitmap );
	Boolean				InsertShadowBitmap( CBitmap *inBitmap, Int32 inAtIndex );
	
private:
						// Defensive programming
						C256Tag( const C256Tag & );
	C256Tag				&operator=( const C256Tag & );
	
	virtual void		UpdateTagHeader();
	void				UpdateHueChangedColorTable();
	
	CCollectionHeader	*mCollectionHeader;
	CTagElementArray	*mColorTables;
	CTagElementArray	*mHueChanges;
	CTagElementArray	*mSequences;
	CTagElementArray	*mBlendTables;
	CTagElementArray	*mRemapTables;
	CTagElementArray	*mColorBitmaps;
	CTagElementArray	*mShadowBitmaps;
	
	RGBColor			mHueChangedColorTable[256];
	
	static const char	sCopyrightNotice[];
	
};

