// =================================================================================
//	CBitmap.cp					 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CBitmap.h"

#include "CBitmapReference.h"
#include "CBitmapData.h"

#include "256Messages.h"
#include "UGUI.h"


// =================================================================================
//	 CBitmap																[public]
// =================================================================================

CBitmap::CBitmap( void )
{
	mReference = new CBitmapReference;
	mData = new CBitmapData;
	
	mReference->AddListener( this );
	mData->AddListener( this );
}


// =================================================================================
//	 ~CBitmap																[public]
// =================================================================================
					
CBitmap::~CBitmap( void )
{
	delete mReference;
	delete mData;
}


#pragma mark -

// =================================================================================
//	 ListenToMessage														[public]
// =================================================================================

void
CBitmap::ListenToMessage( MessageT inMessage, void *ioData )
{
	#pragma unused(ioData)
	
	switch ( inMessage ) {
		case msg_256UpdateBitmapReference:
			BroadcastMessage( msg_256UpdateBitmap, this );
			break;

		case msg_256UpdateBitmapData:
			mReference->UpdateFromBitmap( mData );
			BroadcastMessage( msg_256UpdateBitmap, this );
			break;
			
	}
}


#pragma mark -

// =================================================================================
//	 ReadReference															[public]
// =================================================================================

void
CBitmap::ReadReference( const Byte *inBytes )
{
	mReference->ReadData( inBytes );
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================

void
CBitmap::ReadData( const Byte *inBytes )
{
	mData->ReadData( inBytes );
}


// =================================================================================
//	 WriteReference														[public]
// =================================================================================

void
CBitmap::WriteReference( Byte *outBytes )
{
	mReference->WriteData( outBytes );
}


// =================================================================================
//	 WriteData																[public]
// =================================================================================

void
CBitmap::WriteData( Byte *outBytes )
{
	mData->WriteData( outBytes );
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================

Int32
CBitmap::GetSize( void ) const
{
	return mReference->GetSize() + mData->GetSize();
}


// =================================================================================
//	 GetReferenceSize												 [const, public]
// =================================================================================

Int32
CBitmap::GetReferenceSize( void ) const
{
	return mReference->GetSize();
}


#pragma mark -

// =================================================================================
//	 GetName														 [const, public]
// =================================================================================

const char *
CBitmap::GetName( void ) const
{
	return mReference->GetName();
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CBitmap::SetName( const char *inName )
{
	mReference->SetName( inName );
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CBitmap::SetName( ConstStr255Param inName )
{
	mReference->SetName( inName );
}


// =================================================================================
//	 GetDataOffset													 [const, public]
// =================================================================================

Int32
CBitmap::GetDataOffset( void ) const
{
	return mReference->GetDataOffset();
}


// =================================================================================
//	 SetDataOffset															[public]
// =================================================================================

void
CBitmap::SetDataOffset( Int32 inOffset )
{
	mReference->SetDataOffset( inOffset);
}


// =================================================================================
//	 GetDataSize													 [const, public]
// =================================================================================

Int32
CBitmap::GetDataSize( void ) const
{
	return mReference->GetDataSize();
}


// =================================================================================
//	 SetDataSize															[public]
// =================================================================================

void
CBitmap::SetDataSize( Int32 inSize )
{
	mReference->SetDataSize( inSize );
}


#pragma mark -

// =================================================================================
//	 GetWidth														 [const, public]
// =================================================================================

Int16
CBitmap::GetWidth( void ) const
{
	return mData->GetWidth();
}


// =================================================================================
//	 GetHeight														 [const, public]
// =================================================================================

Int16
CBitmap::GetHeight( void ) const
{
	return mData->GetHeight();
}


// =================================================================================
//	 GetBitDepth													 [const, public]
// =================================================================================

Int16
CBitmap::GetBitDepth( void ) const
{
	return mData->GetBitDepth();
}


// =================================================================================
//	 GetFlags														 [const, public]
// =================================================================================

Uint16
CBitmap::GetFlags( void ) const
{
	return mData->GetFlags();
}


#pragma mark -

// =================================================================================
//	 DrawBitmap													 [const, public]
// =================================================================================

void
CBitmap::DrawBitmap( Byte *outPixels, Int32 inRowBytes ) const
{
	mData->DrawBitmap( outPixels, inRowBytes );
}


// =================================================================================
//	 DrawBitmap													 [const, public]
// =================================================================================

void
CBitmap::DrawBitmap(
	Byte *outPixels,
	Int32 inRowBytes,
	Boolean inFlipHorizontal,
	Boolean inFlipVertical ) const
{
	mData->DrawBitmap( outPixels, inRowBytes, inFlipHorizontal, inFlipVertical );
}


// =================================================================================
//	 DrawAlphas													 [const, public]
// =================================================================================

void
CBitmap::DrawAlphas( Byte *outPixels, Int32 inRowBytes ) const
{
	mData->DrawAlphas( outPixels, inRowBytes );
}


// =================================================================================
//	 DrawAlphas													 [const, public]
// =================================================================================

void
CBitmap::DrawAlphas(
	Byte *outPixels,
	Int32 inRowBytes,
	Boolean inFlipHorizontal,
	Boolean inFlipVertical ) const
{
	mData->DrawAlphas( outPixels, inRowBytes, inFlipHorizontal, inFlipVertical );
}


