// =================================================================================
//	CBitmap.h					 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================
// This is just an interfrace class for CBitmapReference and CBitmapData.  All
// access to these should go through this class.

#pragma once


#include "CTagElement.h"
#include <LListener.h>

class CBitmapReference;
class CBitmapData;


class CBitmap : public CTagElement, public LListener {
public:
	enum			{	// Exceptions (must correspond with CBitmapData.h)
						err_BadBitDepth				= 'BABI',
	
						// Flags (must correspond with CBitmapData.h)
						flag_PowerOfTwo				= 0x0001,
						flag_ColorKeyTransparency	= 0x0002,	// Not used
						flag_1BitTranspEncoded		= 0x0004,
						flag_4BitLightMaskEncoded	= 0x0008,	// Not used
						flag_4BitTranspEncoded		= 0x0010,
						flag_NoRowAddressTable		= 0x0020,
						flag_WordAlignedEncoding	= 0x0040,
						flag_GreyscaleBitmap		= 0x0080,
						flag_PixelDoubledBitmap		= 0x0100,	// Not used
						flag_PostprocessedBitmap	= 0x0200,	// Not used
						flag_ProtectedBitmap		= 0x0400,	// Not really used
						flag_BitmapIsOverlay		= 0x0800,
						flag_BitmapChanged			= 0x1000,	// Runtime(?)
						flag_BitmapInCollection		= 0x2000,	// Runtime(?)
						
						// Constant element size (the bitmap ref size)
						kElementSize				= 128
					};
					
					
					// Default constructor and destructor
	
					CBitmap();
					~CBitmap();
					
	void			ReadReference( const Byte *inBytes );
	void			ReadData( const Byte *inBytes );
	
	void			WriteReference( Byte *inBytes );
	void			WriteData( Byte *inBytes );
					
					// GetSize returns the size of the reference and the
					// bitmap data, GetReferenceSize just the reference size
					
	Int32			GetSize() const;
	Int32			GetReferenceSize() const;
	
				// LListener
				
	virtual void	ListenToMessage( MessageT inMessage, void *ioData );
	
				// Accessors
	
	const char		*GetName() const;
	void			SetName( const char *inName );
	void			SetName( ConstStr255Param inName );
	
	Int32			GetDataOffset() const;
	void			SetDataOffset( Int32 inOffset );
	
	Int32			GetDataSize() const;
	void			SetDataSize( Int32 inSize );
	
	Int16			GetWidth() const;
	Int16			GetHeight() const;
	Int16			GetBitDepth() const;
	
	UInt16			GetFlags() const;
	
	void			DrawBitmap( Byte *outPixels, Int32 inRowBytes ) const;
	void			DrawBitmap(
						Byte *outPixels,
						Int32 inRowBytes,
						Boolean inFlipHorizontal,
						Boolean inFlipVertical ) const;
	void			DrawAlphas( Byte *outPixels, Int32 inRowBytes ) const;
	void			DrawAlphas(
						Byte *outPixels,
						Int32 inRowBytes,
						Boolean inFlipHorizontal,
						Boolean inFlipVertical ) const;
	
private:
					// Defensive programming
					CBitmap( const CBitmap & );
	CBitmap			&operator=( const CBitmap & );
	
	CBitmapReference	*mReference;
	CBitmapData			*mData;
};


