// =================================================================================
//	CBitmapReference.cp			 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CBitmapReference.h"
#include "CopyBytes.h"

#include "256Messages.h"
#include "CBitmapData.h"


// =================================================================================
//	 CBitmapReference														[public]
// =================================================================================
					
CBitmapReference::CBitmapReference( void )
{
	for ( int i = 0; i < 64; i++ )
		mName[i] = 0;
	
	mDataOffset	= 0;
	mDataSize	= 0;
	
	mWidth		= 0;
	mHeight		= 0;
	
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================
// inBitmapReferenceData is a buffer containing the bitmap entry data as they are
// stored in the .gor files and plugin files.  Zeroes out the mem based field.
					
void
CBitmapReference::ReadData( const Byte *inBitmapReferenceData )
{
	inBitmapReferenceData += ::CopyBytes( inBitmapReferenceData, mName, 64 );
	inBitmapReferenceData += ::CopyBytes( inBitmapReferenceData, &mDataOffset, 4 );
	inBitmapReferenceData += ::CopyBytes( inBitmapReferenceData, &mDataSize, 4 );
	inBitmapReferenceData += 4;
	
	inBitmapReferenceData += 2;
	inBitmapReferenceData += 2;
	
	inBitmapReferenceData += 30;
	
	inBitmapReferenceData += 2;
	inBitmapReferenceData += 16;
}


// =================================================================================
//	 WriteData														 [const, public]
// =================================================================================
// outBitmapReferenceData is filled with the bitmap entry data as they are stored on
// file
					
void
CBitmapReference::WriteData( Byte *outBitmapReferenceData ) const
{
	const Int32 theZero = 0;
	
	outBitmapReferenceData += ::CopyBytes( mName, outBitmapReferenceData, 64 );
	outBitmapReferenceData += ::CopyBytes( &mDataOffset, outBitmapReferenceData, 4 );
	outBitmapReferenceData += ::CopyBytes( &mDataSize, outBitmapReferenceData, 4 );
	outBitmapReferenceData += ::CopyBytes( &theZero, outBitmapReferenceData, 4 );

	outBitmapReferenceData += ::CopyBytes( &mWidth, outBitmapReferenceData, 2 );
	outBitmapReferenceData += ::CopyBytes( &mHeight, outBitmapReferenceData, 2 );
	
	for ( Int32 i = 0; i < 48; i++ )
		*outBitmapReferenceData++ = 0;
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================
// Return the bitmap entry size in bytes

Int32
CBitmapReference::GetSize( void ) const
{
	return 128;
}


#pragma mark -

// =================================================================================
//	 GetName														 [const, public]
// =================================================================================

const char *
CBitmapReference::GetName( void ) const
{
	return mName;
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CBitmapReference::SetName( const char *inName )
{
	// Don't update if they're identical
	if ( LStr255(mName) == LStr255(inName) )
		return;
	
	// Copy the new name
	Int32 i = 0;
	
	for ( ; i < 64 && inName[i] != 0; i++ )
		mName[i] = inName[i];
	
	// Clear the rest of the name field
	for ( ; i < 64; i++ )
		mName[i] = 0;
	
	BroadcastMessage( msg_256UpdateBitmapReference, nil );
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CBitmapReference::SetName( ConstStr255Param inName )
{
	// Don't update if they're identical
	if ( LStr255(mName) == LStr255(inName) )
		return;
	
	// Get the length, truncate if it's too long
	Int32 theLength = inName[0];
	
	if ( theLength >= 64 )
		theLength = 63;
	
	// Copy the new name
	::BlockMove( &inName[1], mName, theLength );
	
	// Clear the rest of the name field
	for ( Int32 i = theLength; i < 64; i++ )
		mName[i] = 0;
	
	BroadcastMessage( msg_256UpdateBitmapReference, nil );
}


// =================================================================================
//	 GetDataOffset													 [const, public]
// =================================================================================

Int32
CBitmapReference::GetDataOffset( void ) const
{
	return mDataOffset;
}


// =================================================================================
//	 SetDataOffset															[public]
// =================================================================================

void
CBitmapReference::SetDataOffset( Int32 inDataOffset )
{
	if ( mDataOffset != inDataOffset ) {
		mDataOffset = inDataOffset;
//		BroadcastMessage( msg_256UpdateBitmapReference, nil );
	}
}


// =================================================================================
//	 GetDataSize													 [const, public]
// =================================================================================

Int32
CBitmapReference::GetDataSize( void ) const
{
	return mDataSize;
}


// =================================================================================
//	 SetDataSize															[public]
// =================================================================================

void
CBitmapReference::SetDataSize( Int32 inDataSize )
{
	if ( mDataSize != inDataSize ) {
		mDataSize = inDataSize;
//		BroadcastMessage( msg_256UpdateBitmapReference, nil );
	}
}


// =================================================================================
//	 UpdateFromBitmap														[public]
// =================================================================================

void
CBitmapReference::UpdateFromBitmap( const CBitmapData *inBitmap )
{
	mDataSize = inBitmap->GetSize();
	mWidth = inBitmap->GetWidth();
	mHeight = inBitmap->GetHeight();
}


