// =================================================================================
//	CBitmapReference.h			 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#pragma once

#include "CTagElement.h"
class CBitmapData;


// =================================================================================
//	CBitmapReference													 [128 bytes]
// =================================================================================
// Now we get to some important stuff... the bitmap references. Each
// of these entries contains the location for a given bitmap.

class CBitmapReference : public CTagElement {
friend				class CBitmap;

protected:
					// Default constructor
	
					CBitmapReference();
	
					// inBitmapReferenceData is a buffer containing the bitmap
					// entry data as they are stored in the .gor files and plugin
					// files.
					
	void			ReadData( const Byte *inBitmapReferenceData );
	
					// outBitmapReferenceData is filled with the bitmap entry
					// data as they are stored on file
					
	void			WriteData( Byte *outBitmapReferenceData ) const;
	
					// Return the bitmap reference size on disk
					
	Int32			GetSize() const;
	
				// Accessors
	
	const char		*GetName() const;
	void			SetName( const char *inName );
	void			SetName( ConstStr255Param inName );
	
	Int32			GetDataOffset() const;
	void			SetDataOffset( Int32 inOffset );
	
	Int32			GetDataSize() const;
	void			SetDataSize( Int32 inSize );
	
	void			UpdateFromBitmap( const CBitmapData *inBitmap );
	
private:
					// Defensive programming
					CBitmapReference( const CBitmapReference & );
	CBitmapReference &operator=( const CBitmapReference & );
	
	char	mName[64];			// name of the bitmap
	Int32	mDataOffset;		// offset to the bitmap data
	Int32	mDataSize;			// size of the bitmap data
	
	// The rest can be zeroed out, according to Eli Curtz
	// (Checked, it can't for sprite tags - the bitmaps disappear :)
	
//	void	*mBitmapData;		// memory based, zero this out
	Int16	mWidth;				// width of the bitmap in pixels
	Int16	mHeight;			// height of the bitmap in pixels
	
//	Byte	mUnused[30];		// filler?
	
//	Int16	mVirtualID;			// ???
//	Byte	mExtractorData[16];	// Data used and set by Extractor
};


