// =================================================================================
//	CCollectionHeader.cp		 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CCollectionHeader.h"
#include "CopyBytes.h"


// =================================================================================
//	 CCollectionHeader														[public]
// =================================================================================
// Default constructor
					
CCollectionHeader::CCollectionHeader( void )
{
	mFlags					= 0;
	mUserDataFlags			= 0;
	
	mColorTableCount		= 0;
	mColorTablesOffset		= 0;
	mColorTablesSize		= 0;
	
	mHueChangeCount			= 0;
	mHueChangesOffset		= 0;
	mHueChangesSize			= 0;
	
	mBitmapCount			= 0;
	mBitmapsOffset			= 0;
	mBitmapsSize			= 0;
	
	mBitmapInstanceCount	= 0;
	mBitmapInstancesOffset	= 0;
	mBitmapInstancesSize	= 0;
	
	mSequenceCount			= 0;
	mSequencesOffset		= 0;
	mSequencesSize			= 0;
	
	mShadowMapCount			= 0;
	mShadowMapsOffset		= 0;
	mShadowMapsSize			= 0;
	
	mBlendTableCount		= 0;
	mBlendTablesOffset		= 0;
	mBlendTablesSize		= 0;
	
	mRemapTableCount		= 0;
	mRemapTablesOffset		= 0;
	mRemapTablesSize		= 0;
	
	mShadingTableCount		= 0;
	
	mDataOffset				= 320;
	mDataSize				= 0;
	mDataDelta				= 0;
	
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================
// inHeaderData is a buffer containing the header data as they are stored in the
// .gor files and plugin files.  Zeroes out the mem-based fields.
					
void
CCollectionHeader::ReadData( Byte *inHeaderData )
{
	inHeaderData += ::CopyBytes( inHeaderData, &mFlags, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mUserDataFlags, 4 );
	inHeaderData += 56;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mColorTableCount, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mColorTablesOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mColorTablesSize, 4 );
	inHeaderData += 4;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mHueChangeCount, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mHueChangesOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mHueChangesSize, 4 );
	inHeaderData += 4;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mBitmapCount, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mBitmapsOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mBitmapsSize, 4 );
	inHeaderData += 4;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mBitmapInstanceCount, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mBitmapInstancesOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mBitmapInstancesSize, 4 );
	inHeaderData += 4;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mSequenceCount, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mSequencesOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mSequencesSize, 4 );
	inHeaderData += 4;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mShadowMapCount, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mShadowMapsOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mShadowMapsSize, 4 );
	inHeaderData += 4;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mBlendTableCount, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mBlendTablesOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mBlendTablesSize, 4 );
	inHeaderData += 4;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mRemapTableCount, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mRemapTablesOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mRemapTablesSize, 4 );
	inHeaderData += 4;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mShadingTableCount, 4 );
	inHeaderData += 4;
	
	inHeaderData += 48;
	
	inHeaderData += ::CopyBytes( inHeaderData, &mDataOffset, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mDataSize, 4 );
	inHeaderData += ::CopyBytes( inHeaderData, &mDataDelta, 4 );
	inHeaderData += 4;

}


// =================================================================================
//	 WriteData														 [const, public]
// =================================================================================
// outHeaderData is filled with the 320 bytes of the header data as they are
// stored on file
					
void
CCollectionHeader::WriteData( Byte *outHeaderData ) const
{
	const Int32 theZero = 0;
	
	outHeaderData += ::CopyBytes( &mFlags, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mUserDataFlags, outHeaderData, 4 );
	
	for ( Int32 i = 0; i < 56; i++ )
		*outHeaderData++ = 0;
	
	outHeaderData += ::CopyBytes( &mColorTableCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mColorTablesOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mColorTablesSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	outHeaderData += ::CopyBytes( &mHueChangeCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mHueChangesOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mHueChangesSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	outHeaderData += ::CopyBytes( &mBitmapCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mBitmapsOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mBitmapsSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	outHeaderData += ::CopyBytes( &mBitmapInstanceCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mBitmapInstancesOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mBitmapInstancesSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	outHeaderData += ::CopyBytes( &mSequenceCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mSequencesOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mSequencesSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	outHeaderData += ::CopyBytes( &mShadowMapCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mShadowMapsOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mShadowMapsSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	outHeaderData += ::CopyBytes( &mBlendTableCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mBlendTablesOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mBlendTablesSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	outHeaderData += ::CopyBytes( &mRemapTableCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mRemapTablesOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mRemapTablesSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	outHeaderData += ::CopyBytes( &mShadingTableCount, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	for ( Int32 i = 0; i < 48; i++ )
		*outHeaderData++ = 0;
	
	outHeaderData += ::CopyBytes( &mDataOffset, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mDataSize, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &mDataDelta, outHeaderData, 4 );
	outHeaderData += ::CopyBytes( &theZero, outHeaderData, 4 );
	
	for ( Int32 i = 0; i < 40; i++ )
		*outHeaderData++ = 0;
	
	for ( Int32 i = 0; i < 16; i++ )
		*outHeaderData++ = 0;
	
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================
// Return the header size in bytes

Int32
CCollectionHeader::GetSize( void ) const
{
	return 320;
}


#pragma mark -

// =================================================================================
//	 GetUserDataFlags												 [const, public]
// =================================================================================
// Return the user data flags

Uint32
CCollectionHeader::GetUserDataFlags( void ) const
{
	return mUserDataFlags;
}


// =================================================================================
//	 SetUserDataFlags														[public]
// =================================================================================
// Set the user data flags

void
CCollectionHeader::SetUserDataFlags( Uint32 inFlags )
{
	mUserDataFlags = inFlags;
}


#pragma mark -

// =================================================================================
//	 GetColorTableCount											 [const, public]
// =================================================================================
// Return the count

Int32
CCollectionHeader::GetColorTableCount( void ) const
{
	return mColorTableCount;
}


// =================================================================================
//	 GetColorTablesOffset											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetColorTablesOffset( void ) const
{
	return mColorTablesOffset;
}


// =================================================================================
//	 GetColorTablesSize											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetColorTablesSize( void ) const
{
	return mColorTablesSize;
}


// =================================================================================
//	 SetColorTableCount													[public]
// =================================================================================
// Set the count

void
CCollectionHeader::SetColorTableCount( Int32 inCount )
{
	mColorTableCount = inCount;
}


// =================================================================================
//	 SetColorTablesOffset													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetColorTablesOffset( Int32 inOffset )
{
	mColorTablesOffset = inOffset;
}


// =================================================================================
//	 SetColorTablesSize													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetColorTablesSize( Int32 inSize )
{
	mColorTablesSize = inSize;
}


#pragma mark -

// =================================================================================
//	 GetHueChangeCount												 [const, public]
// =================================================================================
// Return the count

Int32
CCollectionHeader::GetHueChangeCount( void ) const
{
	return mHueChangeCount;
}


// =================================================================================
//	 GetHueChangesOffset											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetHueChangesOffset( void ) const
{
	return mHueChangesOffset;
}


// =================================================================================
//	 GetHueChangesSize												 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetHueChangesSize( void ) const
{
	return mHueChangesSize;
}


// =================================================================================
//	 SetHueChangeCount													[public]
// =================================================================================
// Set the count

void
CCollectionHeader::SetHueChangeCount( Int32 inCount )
{
	mHueChangeCount = inCount;
}


// =================================================================================
//	 SetHueChangesOffset													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetHueChangesOffset( Int32 inOffset )
{
	mHueChangesOffset = inOffset;
}


// =================================================================================
//	 SetHueChangesSize														[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetHueChangesSize( Int32 inSize )
{
	mHueChangesSize = inSize;
}


#pragma mark -

// =================================================================================
//	 GetBitmapCount												 [const, public]
// =================================================================================
// Return the count

Int32
CCollectionHeader::GetBitmapCount( void ) const
{
	return mBitmapCount;
}


// =================================================================================
//	 GetBitmapsOffset												 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetBitmapsOffset( void ) const
{
	return mBitmapsOffset;
}


// =================================================================================
//	 GetBitmapsSize												 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetBitmapsSize( void ) const
{
	return mBitmapsSize;
}


// =================================================================================
//	 SetBitmapCount														[public]
// =================================================================================
// Set the count

void
CCollectionHeader::SetBitmapCount( Int32 inCount )
{
	mBitmapCount = inCount;
}


// =================================================================================
//	 SetBitmapsOffset														[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetBitmapsOffset( Int32 inOffset )
{
	mBitmapsOffset = inOffset;
}


// =================================================================================
//	 SetBitmapsSize														[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetBitmapsSize( Int32 inSize )
{
	mBitmapsSize = inSize;
}


#pragma mark -

// =================================================================================
//	 GetBitmapInstanceCount										 [const, public]
// =================================================================================
// Return the count

Int32
CCollectionHeader::GetBitmapInstanceCount( void ) const
{
	return mBitmapInstanceCount;
}


// =================================================================================
//	 GetBitmapInstancesOffset										 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetBitmapInstancesOffset( void ) const
{
	return mBitmapInstancesOffset;
}


// =================================================================================
//	 GetBitmapInstancesSize										 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetBitmapInstancesSize( void ) const
{
	return mBitmapInstancesSize;
}


// =================================================================================
//	 SetBitmapInstanceCount												[public]
// =================================================================================
// Set the count

void
CCollectionHeader::SetBitmapInstanceCount( Int32 inCount )
{
	mBitmapInstanceCount = inCount;
}


// =================================================================================
//	 SetBitmapInstancesOffset												[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetBitmapInstancesOffset( Int32 inOffset )
{
	mBitmapInstancesOffset = inOffset;
}


// =================================================================================
//	 SetBitmapInstancesSize												[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetBitmapInstancesSize( Int32 inSize )
{
	mBitmapInstancesSize = inSize;
}


#pragma mark -

// =================================================================================
//	 GetSequenceCount												 [const, public]
// =================================================================================
// Return the count

Int32
CCollectionHeader::GetSequenceCount( void ) const
{
	return mSequenceCount;
}


// =================================================================================
//	 GetSequencesOffset											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetSequencesOffset( void ) const
{
	return mSequencesOffset;
}


// =================================================================================
//	 GetSequencesSize												 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetSequencesSize( void ) const
{
	return mSequencesSize;
}


// =================================================================================
//	 SetSequenceCount														[public]
// =================================================================================
// Set the count

void
CCollectionHeader::SetSequenceCount( Int32 inCount )
{
	mSequenceCount = inCount;
}


// =================================================================================
//	 SetSequencesOffset													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetSequencesOffset( Int32 inOffset )
{
	mSequencesOffset = inOffset;
}


// =================================================================================
//	 SetSequencesSize														[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetSequencesSize( Int32 inSize )
{
	mSequencesSize = inSize;
}


#pragma mark -

// =================================================================================
//	 GetShadowMapCount												 [const, public]
// =================================================================================
// Return the count

Int32
CCollectionHeader::GetShadowMapCount( void ) const
{
	return mShadowMapCount;
}


// =================================================================================
//	 GetShadowMapsOffset											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetShadowMapsOffset( void ) const
{
	return mShadowMapsOffset;
}


// =================================================================================
//	 GetShadowMapsSize												 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetShadowMapsSize( void ) const
{
	return mShadowMapsSize;
}


// =================================================================================
//	 SetShadowMapCount														[public]
// =================================================================================
// Set the count

void
CCollectionHeader::SetShadowMapCount( Int32 inCount )
{
	mShadowMapCount = inCount;
}


// =================================================================================
//	 SetShadowMapsOffset													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetShadowMapsOffset( Int32 inOffset )
{
	mShadowMapsOffset = inOffset;
}


// =================================================================================
//	 SetShadowMapsSize														[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetShadowMapsSize( Int32 inSize )
{
	mShadowMapsSize = inSize;
}


#pragma mark -

// =================================================================================
//	 GetBlendTableCount											 [const, public]
// =================================================================================
// Return the count

Int32
CCollectionHeader::GetBlendTableCount( void ) const
{
	return mBlendTableCount;
}


// =================================================================================
//	 GetBlendTablesOffset											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetBlendTablesOffset( void ) const
{
	return mBlendTablesOffset;
}


// =================================================================================
//	 GetBlendTablesSize											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetBlendTablesSize( void ) const
{
	return mBlendTablesSize;
}


// =================================================================================
//	 SetBlendTableCount													[public]
// =================================================================================
// Set the count

void
CCollectionHeader::SetBlendTableCount( Int32 inCount )
{
	mBlendTableCount = inCount;
}


// =================================================================================
//	 SetBlendTablesOffset													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetBlendTablesOffset( Int32 inOffset )
{
	mBlendTablesOffset = inOffset;
}


// =================================================================================
//	 SetBlendTablesSize													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetBlendTablesSize( Int32 inSize )
{
	mBlendTablesSize = inSize;
}


#pragma mark -

// =================================================================================
//	 GetRemapTableCount											 [const, public]
// =================================================================================
// Return the count

Int32
CCollectionHeader::GetRemapTableCount( void ) const
{
	return mRemapTableCount;
}


// =================================================================================
//	 GetRemapTablesOffset											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetRemapTablesOffset( void ) const
{
	return mRemapTablesOffset;
}


// =================================================================================
//	 GetRemapTablesSize											 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetRemapTablesSize( void ) const
{
	return mRemapTablesSize;
}


// =================================================================================
//	 SetRemapTableCount													[public]
// =================================================================================
// Set the count

void
CCollectionHeader::SetRemapTableCount( Int32 inCount )
{
	mRemapTableCount = inCount;
}


// =================================================================================
//	 SetRemapTablesOffset													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetRemapTablesOffset( Int32 inOffset )
{
	mRemapTablesOffset = inOffset;
}


// =================================================================================
//	 SetRemapTablesSize													[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetRemapTablesSize( Int32 inSize )
{
	mRemapTablesSize = inSize;
}


#pragma mark -

// =================================================================================
//	 GetDataOffset													 [const, public]
// =================================================================================
// Return the offset

Int32
CCollectionHeader::GetDataOffset( void ) const
{
	return mDataOffset;
}


// =================================================================================
//	 GetDataSize													 [const, public]
// =================================================================================
// Return the size

Int32
CCollectionHeader::GetDataSize( void ) const
{
	return mDataSize;
}


// =================================================================================
//	 SetDataOffset															[public]
// =================================================================================
// Set the offset

void
CCollectionHeader::SetDataOffset( Int32 inOffset )
{
	mDataOffset = inOffset;
}


// =================================================================================
//	 SetDataSize															[public]
// =================================================================================
// Set the size

void
CCollectionHeader::SetDataSize( Int32 inSize )
{
	mDataSize = inSize;
}


