// =================================================================================
//	CCollectionHeader.h			 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#pragma once


// =================================================================================
//	CCollectionHeader													 [320 bytes]
// =================================================================================

class CCollectionHeader {
public:
	enum				{
							flag_IsInitialized			= 0x01,
							
							flag_UseForTextures			= 0x01,
							flag_UseForMonsters			= 0x02,
							flag_useForProjectiles		= 0x04,
							flag_UseForScenery			= 0x08,
							flag_UseForAtmospheric		= 0x10,
							flag_UseForInterface		= 0x20,
							
							flag_IsTextureMap			= 0x0100
						};
	
						// Default constructor
	
						CCollectionHeader();
	
						// inHeaderData is a buffer containing the header data
						// as they are stored in the .gor files and plugin files.
	
	void				ReadData( Byte *inHeaderData );
	
						// outHeaderData is filled with the 320 bytes of the
						// collection header data as they are stored on file
	
	void				WriteData( Byte *outHeaderData ) const;
	
						// Return size of the header on file
	
	Int32				GetSize() const;
	
					// Accessors
	
	UInt32				GetUserDataFlags() const;
	void				SetUserDataFlags( Uint32 inFlags );
	
	Int32				GetColorTableCount() const;
	Int32				GetColorTablesOffset() const;
	Int32				GetColorTablesSize() const;
	
	void				SetColorTableCount( Int32 inCount );
	void				SetColorTablesOffset( Int32 inOffset );
	void				SetColorTablesSize( Int32 inSize );
	
	Int32				GetHueChangeCount() const;
	Int32				GetHueChangesOffset() const;
	Int32				GetHueChangesSize() const;
	
	void				SetHueChangeCount( Int32 inCount );
	void				SetHueChangesOffset( Int32 inOffset );
	void				SetHueChangesSize( Int32 inSize );
	
	Int32				GetBitmapCount() const;
	Int32				GetBitmapsOffset() const;
	Int32				GetBitmapsSize() const;
	
	void				SetBitmapCount( Int32 inCount );
	void				SetBitmapsOffset( Int32 inOffset );
	void				SetBitmapsSize( Int32 inSize );
	
	Int32				GetBitmapInstanceCount() const;
	Int32				GetBitmapInstancesOffset() const;
	Int32				GetBitmapInstancesSize() const;
	
	void				SetBitmapInstanceCount( Int32 inCount );
	void				SetBitmapInstancesOffset( Int32 inOffset );
	void				SetBitmapInstancesSize( Int32 inSize );
	
	Int32				GetSequenceCount() const;
	Int32				GetSequencesOffset() const;
	Int32				GetSequencesSize() const;
	
	void				SetSequenceCount( Int32 inCount );
	void				SetSequencesOffset( Int32 inOffset );
	void				SetSequencesSize( Int32 inSize );
	
	Int32				GetShadowMapCount() const;
	Int32				GetShadowMapsOffset() const;
	Int32				GetShadowMapsSize() const;
	
	void				SetShadowMapCount( Int32 inCount );
	void				SetShadowMapsOffset( Int32 inOffset );
	void				SetShadowMapsSize( Int32 inSize );
	
	Int32				GetBlendTableCount() const;
	Int32				GetBlendTablesOffset() const;
	Int32				GetBlendTablesSize() const;
	
	void				SetBlendTableCount( Int32 inCount );
	void				SetBlendTablesOffset( Int32 inOffset );
	void				SetBlendTablesSize( Int32 inSize );
	
	Int32				GetRemapTableCount() const;
	Int32				GetRemapTablesOffset() const;
	Int32				GetRemapTablesSize() const;
	
	void				SetRemapTableCount( Int32 inCount );
	void				SetRemapTablesOffset( Int32 inOffset );
	void				SetRemapTablesSize( Int32 inSize );
	
	Int32				GetDataOffset() const;
	Int32				GetDataSize() const;
	
	void				SetDataOffset( Int32 inOffset );
	void				SetDataSize( Int32 inSize );
	
private:
						// Defensive programming
						CCollectionHeader( const CCollectionHeader & );
	CCollectionHeader	&operator=( const CCollectionHeader & );
	
	Uint32	mFlags;
	Uint32	mUserDataFlags;

//	Int16	mUnused1[28];

	Int32	mColorTableCount;		// number of color tables (usually 1)
	Int32	mColorTablesOffset;		// offset to the clut data
	Int32	mColorTablesSize;		// size in bytes of the clut data (usually 2080)
//	void	*mColorTables;			// memory based, zero this out

	Int32	mHueChangeCount;		// number of HueChange entries
	Int32	mHueChangesOffset;		// offset to the hueChanges table
	Int32	mHueChangesSize;		// size of the hueChanges table
//	void	*mHueChanges;			// memory based, zero this out

	Int32	mBitmapCount;			// number of individual images in the .256 data
	Int32	mBitmapsOffset;			// offset to the bitmap reference table
	Int32	mBitmapsSize;			// size of bitmap reference table info 
//	void	*mBitmapRefs;			// memory based, zero this out

	Int32	mBitmapInstanceCount;	// number of bitmap descriptors
	Int32	mBitmapInstancesOffset;	// offset to the bitmap instance table
	Int32	mBitmapInstancesSize;	// size of the bitmap instance table
//	void	*mBitmapInstances;		// memory based, zero this out

	Int32	mSequenceCount;			// number of sequences
	Int32	mSequencesOffset;		// offset to the sequence table
	Int32	mSequencesSize;			// size of the sequence table
//	void	*mSequences;			// memory based, zero this out

	Int32	mShadowMapCount;		// number of shadow maps
	Int32	mShadowMapsOffset;		// offset to the shadow map table
	Int32	mShadowMapsSize;		// size of the shadow map table
//	void	*mShadowMaps;			// memory based, zero this out
	
	Int32	mBlendTableCount;		// number of blending tables
	Int32	mBlendTablesOffset;		// offset to the blending table info
	Int32	mBlendTablesSize;		// size of the blending table info
//	void	*mBlendTables;			// memory based, zero this out
	
	Int32	mRemapTableCount;		// number of (color?) remapping tables
	Int32	mRemapTablesOffset;		// offset to the remapping tables info
	Int32	mRemapTablesSize;		// size of the remapping tables info
//	void	*mRemapTables;			// memory based, zero this out
	
	Int32	mShadingTableCount;		// number of shading tables
//	void	**mShadingTables;		// memory based (zero out)
	
//	Int16	mUnused2[24];
	
	Int32	mDataOffset;			// offset to tag data
	Int32	mDataSize;				// length of tag data
	Int32	mDataDelta;				// ??
//	void	*mData;					// Memory based (zero out)
	
//	Int16	mUnused3[20];
//	Byte	mExtractorData[16];		// Used by Extractor (collection editor)
};

