// =================================================================================
//	CColorTable.cp				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CColorTable.h"
#include "CopyBytes.h"

#include "256Messages.h"


// =================================================================================
//	 CColorTable															[public]
// =================================================================================
					
CColorTable::CColorTable( void )
{
	mColorCount = 256;
	
//	for ( int i = 0; i < 7; i++ )
//		mUnknown[i] = 0;
	
	// Clear the color table
	Byte *theColors = (Byte *)mColors;
	
	for ( int i = 1; i < sizeof(mColors); i++ )
		theColors[i] = 0;
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================
// inColorTableData is a buffer containing the palette data as they are stored in
// the .gor files and plugin files.
					
void
CColorTable::ReadData( Byte *inColorTableData )
{
	inColorTableData += ::CopyBytes( inColorTableData, &mColorCount, 4 );
	inColorTableData += 28;
	
	if ( mColorCount > 256 )
		throw( err_MoreThan256Colors );
	
	inColorTableData +=
		::CopyBytes( inColorTableData, mColors, mColorCount*sizeof(RGBInfo) );
}


// =================================================================================
//	 WriteData														 [const, public]
// =================================================================================
// outColorTableData is filled with the palette data as they are stored on file.
					
void
CColorTable::WriteData( Byte *outColorTableData ) const
{
	const Int32 theZero = 0;
	
	outColorTableData += ::CopyBytes( &mColorCount, outColorTableData, 4 );
	
//	outColorTableData += ::CopyBytes( mUnknown, outColorTableData, 28 );

	for ( Int32 i = 0; i < 7; i++ )
		outColorTableData += ::CopyBytes( &theZero, outColorTableData, 4 );
	
	outColorTableData +=
		::CopyBytes( mColors, outColorTableData, mColorCount*sizeof(RGBInfo) );
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================
// Returns the color table size in bytes

Int32
CColorTable::GetSize( void ) const
{
	return 32 + sizeof(RGBInfo)*mColorCount;
}


#pragma mark -

// =================================================================================
//	 GetColorCount													 [const, public]
// =================================================================================
// Returns the # of colors in the color table

Int32
CColorTable::GetColorCount( void ) const
{
	return mColorCount;
}


// =================================================================================
//	 GetColorTable													 [const, public]
// =================================================================================
// outColorTable points to an RGBColor array with at least mColorCount entries.

void
CColorTable::GetColorTable( RGBColor *outColorTable ) const
{
	for ( Int32 i = 0; i < mColorCount; i++ )
		outColorTable[i] = *(RGBColor *)(mColors+i);
}


// =================================================================================
//	 SetColorTable															[public]
// =================================================================================
// inColorTable points to an RGBColor array with 256.

void
CColorTable::SetColorTable( const RGBColor *inColorTable )
{
	Boolean isDirty = false;
	
	for ( Int32 i = 0; i < 256; i++ ) {
		if ( mColors[i].redValue != inColorTable[i].red
		||   mColors[i].greenValue != inColorTable[i].green
		||   mColors[i].blueValue != inColorTable[i].blue ) {
			*(RGBColor *)(mColors+i) = inColorTable[i];
			mColors[i].flags = 0;
			isDirty = true;
		}
	}
	
	if ( isDirty )
		BroadcastMessage( msg_256UpdateColorTable, this );
}


// =================================================================================
//	 GetColor														 [const, public]
// =================================================================================

RGBColor
CColorTable::GetColor( Int32 inIndex ) const
{
	RGBColor theColor = { 0, 0, 0 };
	GetColor( theColor, inIndex );
	return theColor;
}


// =================================================================================
//	 GetColor														 [const, public]
// =================================================================================

void
CColorTable::GetColor( RGBColor &outColor, Int32 inIndex ) const
{
	if ( inIndex >= 0 && inIndex < mColorCount )
		outColor = *(RGBColor *)(mColors+inIndex);
}


// =================================================================================
//	 SetColor																[public]
// =================================================================================

void
CColorTable::SetColor( const RGBColor &inColor, Int32 inIndex )
{
	if ( inIndex < 0 || inIndex >= 256 )
		return;
	
	Boolean isDirty = false;
	
	if ( inIndex >= mColorCount ) {
		mColorCount = inIndex + 1;
		isDirty = true;
	}
	
	if ( mColors[inIndex].redValue != inColor.red
	||   mColors[inIndex].greenValue != inColor.green
	||   mColors[inIndex].blueValue != inColor.blue ) {
		*(RGBColor *)(mColors+inIndex) = inColor;
		mColors[inIndex].flags = 0;
		isDirty = true;
	}
	
	if ( isDirty )
		BroadcastMessage( msg_256UpdateColorTable, this );
}


