// =================================================================================
//	CColorTable.h				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#pragma once

#include "CTagElement.h"
#include "RGBInfo.h"


// =================================================================================
//	CColorTable															[2080 bytes]
// =================================================================================
// All .256 tags from bungie have 1 color table with 256 colors.
// Usually 2080 bytes (8 * 256 colors + 32 byte header)

class CColorTable : public CTagElement {
public:
	enum			{	// Exceptions
						err_MoreThan256Colors		= '>256'
					};
					
					// Default constructor
					
					CColorTable();
	
					// inColorTableData is a buffer containing the color table data
					// as they are stored in the .gor files and plugin files.
					
	void			ReadData( Byte *inColorTableData );
	
					// outColorTableData is filled with the color table data as they
					// are stored on file
					
	void			WriteData( Byte *outColorTableData ) const;
	
					// Return size of the color table on file
					
	Int32			GetSize() const;
	
				// Accessors
	
	Int32			GetColorCount() const;
	
	void			GetColorTable( RGBColor *outColorTable ) const;
	void			SetColorTable( const RGBColor *inColorTable );
	
	RGBColor		GetColor( Int32 inIndex ) const;
	void			GetColor( RGBColor &outColor, Int32 inIndex ) const;
	void			SetColor( const RGBColor &inColor, Int32 inIndex );
	
private:
					// Defensive programming
					CColorTable( const CColorTable & );
	CColorTable		&operator=( const CColorTable & );
	
	Int32			mColorCount;		// number of colors in the color table
//	Int32			mUnknown[7];		// filler?
	RGBInfo			mColors[256];

};


