// =================================================================================
//	CHueChange.cp				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CHueChange.h"
#include "CopyBytes.h"

#include "256Messages.h"


// =================================================================================
//	 CHueChange															[public]
// =================================================================================

CHueChange::CHueChange( void )
{
	for ( int i = 0; i < 64; i++ )
		mName[i] = 0;
	
	mStartHue = 0;
	mEndHue = 0;
	
	for ( int i = 0; i < 32; i++ )
		mAffectedColors[i] = 0;
	
	mMinSaturation = 0;
	mMedianHue = 0;
	mFlags = 0;
	
	mNewColor.red = 0;
	mNewColor.green = 0;
	mNewColor.blue = 0;
	mReversed = 0;
	mIsUsedFlag = flag_IsUsed;
	
	for ( int i = 0; i < 6; i++ )
		mUnknown3[i] = 0;
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================
// inHueChangeData is a buffer containing the HueChange data as they are stored
// in the .gor files and plugin files.

void
CHueChange::ReadData( Byte *inHueChangeData )
{
	inHueChangeData += ::CopyBytes( inHueChangeData, mName, 64 );
	inHueChangeData += ::CopyBytes( inHueChangeData, &mStartHue, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, &mEndHue, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, mAffectedColors, 32 );
	
	inHueChangeData += ::CopyBytes( inHueChangeData, &mMinSaturation, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, &mMedianHue, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, &mFlags, 2 );
	inHueChangeData += 6;
	
	inHueChangeData += ::CopyBytes( inHueChangeData, &mNewColor.red, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, &mNewColor.green, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, &mNewColor.blue, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, &mReversed, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, &mIsUsedFlag, 2 );
	inHueChangeData += ::CopyBytes( inHueChangeData, mUnknown3, 6 );
}


// =================================================================================
//	 WriteData														 [const, public]
// =================================================================================
// outHueChangeData is filled with the HueChange data as they are stored on file
					
void
CHueChange::WriteData( Byte *outHueChangeData ) const
{
	outHueChangeData += ::CopyBytes( mName, outHueChangeData, 64 );
	outHueChangeData += ::CopyBytes( &mStartHue, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( &mEndHue, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( mAffectedColors, outHueChangeData, 32 );
	
	outHueChangeData += ::CopyBytes( &mMinSaturation, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( &mMedianHue, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( &mFlags, outHueChangeData, 2 );
	
	for ( Int32 i = 0; i < 6; i++ )
		*outHueChangeData++ = 0;
	
	outHueChangeData += ::CopyBytes( &mNewColor.red, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( &mNewColor.green, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( &mNewColor.blue, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( &mReversed, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( &mIsUsedFlag, outHueChangeData, 2 );
	outHueChangeData += ::CopyBytes( mUnknown3, outHueChangeData, 6 );
}


// =================================================================================
//	 GetLength														 [const, public]
// =================================================================================
// Return the HueChange size in bytes

Int32
CHueChange::GetSize( void ) const
{
	return 128;
}

#pragma mark -

// =================================================================================
//	 GetName														 [const, public]
// =================================================================================

const char *
CHueChange::GetName( void ) const
{
	return mName;
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CHueChange::SetName( const char *inName )
{
	// Don't update if they're identical
	if ( LStr255(mName) == LStr255(inName) )
		return;
	
	// Copy the new name
	Int32 i = 0;
	
	for ( ; i < 64 && inName[i] != 0; i++ )
		mName[i] = inName[i];
	
	// Clear the rest of the name field
	for ( ; i < 64; i++ )
		mName[i] = 0;
	
	BroadcastMessage( msg_256UpdateHueChange, this );
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CHueChange::SetName( ConstStr255Param inName )
{
	// Don't update if they're identical
	if ( LStr255(mName) == LStr255(inName) )
		return;
	
	// Get the length, truncate if it's too long
	Int32 theLength = inName[0];
	
	if ( theLength >= 64 )
		theLength = 63;
	
	// Copy the new name
	::BlockMove( &inName[1], mName, theLength );
	
	// Clear the rest of the name field
	for ( Int32 i = theLength; i < 64; i++ )
		mName[i] = 0;
	
	BroadcastMessage( msg_256UpdateHueChange, this );
}


// =================================================================================
//	 GetStartHue													 [const, public]
// =================================================================================

Uint16
CHueChange::GetStartHue( void ) const
{
	return mStartHue;
}


// =================================================================================
//	 SetStartHue															[public]
// =================================================================================

void
CHueChange::SetStartHue( Uint16 inStartHue )
{
	if ( mStartHue != inStartHue ) {
		mStartHue = inStartHue;
		BroadcastMessage( msg_256UpdateHueChange, this );
	}
}


// =================================================================================
//	 GetEndHue														 [const, public]
// =================================================================================

Uint16
CHueChange::GetEndHue( void ) const
{
	return mEndHue;
}


// =================================================================================
//	 SetEndHue																[public]
// =================================================================================

void
CHueChange::SetEndHue( Uint16 inEndHue )
{
	if ( mEndHue != inEndHue ) {
		mEndHue = inEndHue;
		BroadcastMessage( msg_256UpdateHueChange, this );
	}
}


// =================================================================================
//	 GetMedianHue													 [const, public]
// =================================================================================

Uint16
CHueChange::GetMedianHue( void ) const
{
	return mMedianHue;
}


// =================================================================================
//	 SetMedianHue															[public]
// =================================================================================

void
CHueChange::SetMedianHue( Uint16 inMedianHue )
{
	if ( mMedianHue != inMedianHue ) {
		mMedianHue = inMedianHue;
		BroadcastMessage( msg_256UpdateHueChange, this );
	}
}


// =================================================================================
//	 GetMinSaturation												 [const, public]
// =================================================================================

Uint16
CHueChange::GetMinSaturation( void ) const
{
	return mMinSaturation;
}


// =================================================================================
//	 SetMinSaturation														[public]
// =================================================================================

void
CHueChange::SetMinSaturation( Uint16 inSaturation )
{
	if ( mMinSaturation != inSaturation ) {
		mMinSaturation = inSaturation;
		BroadcastMessage( msg_256UpdateHueChange, this );
	}
}


// =================================================================================
//	 GetFlags														 [const, public]
// =================================================================================

Uint16
CHueChange::GetFlags( void ) const
{
	return mFlags;
}


// =================================================================================
//	 SetFlags																[public]
// =================================================================================

void
CHueChange::SetFlags( Uint16 inFlags )
{
	if ( mFlags != inFlags ) {
		mFlags = inFlags;
		BroadcastMessage( msg_256UpdateHueChange, this );
	}
}


// =================================================================================
//	 GetUsedFlags													 [const, public]
// =================================================================================

Boolean
CHueChange::GetUsedFlag( void ) const
{
	return ((mIsUsedFlag & flag_IsUsed) != 0);
}


// =================================================================================
//	 SetUsedFlags															[public]
// =================================================================================

void
CHueChange::SetUsedFlag( Boolean inIsUsed )
{
	if ( inIsUsed != GetUsedFlag() ) {
		if ( inIsUsed )
			mIsUsedFlag |= flag_IsUsed;
		else
			mIsUsedFlag &= ~flag_IsUsed;
		BroadcastMessage( msg_256UpdateHueChange, this );
	}
}


// =================================================================================
//	 GetReplacementColor											 [const, public]
// =================================================================================

RGBColor
CHueChange::GetReplacementColor( void ) const
{
	return mNewColor;
}


// =================================================================================
//	 SetReplacementColor													[public]
// =================================================================================

void
CHueChange::SetReplacementColor( const RGBColor &inColor )
{
	if ( inColor.red != mNewColor.red
	||   inColor.green != mNewColor.green
	||   inColor.blue != mNewColor.blue ) {
		mNewColor = inColor;
		BroadcastMessage( msg_256UpdateHueChange, this );
	}
}


// =================================================================================
//	 GetReversedFlag												 [const, public]
// =================================================================================

Boolean
CHueChange::GetReversedFlag( void ) const
{
	return (mReversed > 0 );
}


// =================================================================================
//	 SetReversedFlag														[public]
// =================================================================================

void
CHueChange::SetReversedFlag( Boolean inReversed )
{
	if ( mReversed != (inReversed == true) ) {
		mReversed = (inReversed == true);
		BroadcastMessage( msg_256UpdateHueChange, this );
	}
}


// =================================================================================
//	 GetAffectedColorBits											 [const, public]
// =================================================================================
// Copy the mAffectedColorBits into outBits

void
CHueChange::GetAffectedColorBits( Byte *outBits ) const
{
	for ( Int32 i = 0; i < sizeof(mAffectedColors); i++ )
		outBits[i] = mAffectedColors[i];
}


// =================================================================================
//	 UpdateAffectedColorBits												[public]
// =================================================================================
// Update which colors in the given color table will be affected.
// Returns whether the hue change is active.

Boolean
CHueChange::UpdateAffectedColorBits( const RGBColor *inColorTable )
{
	HSVColor theHSV;
	
	// Clear the affected color indicies
	for ( Int32 i = 0; i < 32; i++ )
		mAffectedColors[i] = 0;
	
	// Bail if we're not active
	if ( (mIsUsedFlag & flag_IsUsed) == 0 )
		return false;
	
	// Compute affected colors
	for ( Int32 i = 0; i < 256; i++ ) {
		Boolean isAffected = false;
		Byte &theByte = mAffectedColors[i/8];
		Byte theBit = 1 << (i%8);
		
		::RGB2HSV( inColorTable+i, &theHSV );
		
		if ( mStartHue <= mEndHue ) {
			if ( mStartHue <= theHSV.hue && theHSV.hue <= mEndHue
			&&   theHSV.saturation >= mMinSaturation )
				isAffected = true;
				
		} else if ( theHSV.hue <= mEndHue || mStartHue <= theHSV.hue
		&&          theHSV.saturation >= mMinSaturation )
			isAffected = true;
		
		if ( isAffected )
			theByte |= theBit;
		else
			theByte &= ~theBit;
		
	}
	
//	BroadcastMessage( msg_256UpdateHueChange, this );
	return true;
}


// =================================================================================
//	 AffectColorTable														[public]
// =================================================================================
// Apply the hue change to a color table.  Uses the precalculated affected bits.

void
CHueChange::AffectColorTable( RGBColor *ioColorTable )
{
	HSVColor theReplacement;
	::RGB2HSV( &mNewColor, &theReplacement );
	
	for ( Int32 i = 0; i < 256; i++ ) {
		Byte &theByte = mAffectedColors[i/8];
		Byte theBit = 1 << (i%8);
		
		if ( (theByte & theBit) == 0 )
			continue;
		
		// We now have an affected color index
		HSVColor theColor;
		::RGB2HSV( ioColorTable+i, &theColor );
		
		Int32 theHueOffset = theColor.hue - mMedianHue;
		
		if ( GetReversedFlag() )
			theHueOffset = -theHueOffset;
		
		theColor.hue = theReplacement.hue + theHueOffset;
		theColor.saturation = (theColor.saturation*(Uint32)theReplacement.saturation)/0xFFFFU;
		theColor.value = (theColor.value*(Uint32)theReplacement.value)/0xFFFFU;
		
		::HSV2RGB( &theColor, ioColorTable+i );
	}
}


