// =================================================================================
//	CHueChange.h				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#pragma once


#include "CTagElement.h"


// =================================================================================
//	CHueChange															 [128 bytes]
// =================================================================================
// After the color table comes the HueChange data. This is the place where the
// colors that can be changed by a team's primary and secondary colors, or be
// affected by a core tag, are stored.  It consists of a series of HueChange
// entries, each 128 bytes long.

class CHueChange : public CTagElement {
public:
	enum				{	// Flags
							flag_PrimaryHueChange	= 0x01,
							flag_SecondaryHueChange	= 0x02,
						
							// Used flag
							flag_IsUsed				= 0x0100,
							
							// Constant element size
							kElementSize			= 128
						};
					
						// Default constructor
					
						CHueChange();
	
						// inHueChangeData is a buffer containing the swap
						// color data as they are stored in the .gor files and
						// plugin files.
					
	void				ReadData( Byte *inHueChangeData );
	
						// outHueChangeData is filled with the swap color
						// data as they are stored on file
					
	void				WriteData( Byte *outHueChangeData ) const;
					
						// Return size of the hue change on file
					
	Int32				GetSize() const;
	
					// Accessors
	
	const char			*GetName() const;
	void				SetName( const char *inName );
	void				SetName( ConstStr255Param inName );
	
	Uint16				GetStartHue() const;
	void				SetStartHue( Uint16 inStartHue );
	
	Uint16				GetEndHue() const;
	void				SetEndHue( Uint16 inEndHue );
	
	Uint16				GetMedianHue() const;
	void				SetMedianHue( Uint16 inMedianHue );
	
	Uint16				GetMinSaturation() const;
	void				SetMinSaturation( Uint16 inSaturation );
	
	Uint16				GetFlags() const;
	void				SetFlags( Uint16 inFlags );
	
	Boolean				GetUsedFlag() const;
	void				SetUsedFlag( Boolean inIsUsed );
	
	RGBColor			GetReplacementColor() const;
	void				SetReplacementColor( const RGBColor &inColor );
	
	Boolean				GetReversedFlag() const;
	void				SetReversedFlag( Boolean inReversed );
	
	void				GetAffectedColorBits( Byte *outBits ) const;
	Boolean				UpdateAffectedColorBits( const RGBColor *inColorTable );
	void				AffectColorTable( RGBColor *ioColorTable );
	
private:
						// Defensive programming
						CHueChange( const CHueChange & );
	CHueChange			&operator=( const CHueChange & );
	

	char mName[64];			// Body region where hue change takes place
	Uint16 mStartHue;		// Start of hue-range to be changed
	Uint16 mEndHue;			// End of hue-range to be changed
	Byte mAffectedColors[32];// 256 bits marking affected/unaffected color indices
							// mAffectedColors[index/8] = 1 << (index % 8)
	
	Uint16 mMinSaturation;	// Minimum color saturation to be affected
	Uint16 mMedianHue;		// Median hue (Changes around this value)
	Uint16 mFlags;			// 0001 - Primary color, 0002 - secondary
//	Byte mUnused[6];
	
	// The last 16 bytes are used by Extractor (and Amber)
	// We're preserving the unknown bytes to be nice to Extractor
	RGBColor mNewColor;		// New color (rgb)
	Uint16 mReversed;		// 1 - "color is reversed", 0000 otherwise
	Uint16 mIsUsedFlag;		// 0x0100 when used, 0 otherwise
	Byte mUnknown3[6];
};

