// =================================================================================
//	CBitmapInstance.cp			 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CBitmapInstance.h"
#include "CopyBytes.h"

#include "AmberMessages.h"
#include "256Messages.h"
#include "CBitmap.h"


// =================================================================================
//	 CBitmapInstance														[public]
// =================================================================================

CBitmapInstance::CBitmapInstance( void )
{
	mFlags				= 0;
	mMinLightIntensity	= 0;
	mMaxLightIntensity	= -1;
	mTransparency		= 0;
	mRegPointX			= 0;
	mRegPointY			= 0;
	mKeyPointX			= 0;
	mKeyPointY			= 0;
	mBitmapIndex		= -1;
	
	mBitmap = nil;
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================
// inInstanceData is a buffer containing the bitmap instance data as they are
// stored in the .gor files and plugin files.
					
void
CBitmapInstance::ReadData( Byte *inInstanceData )
{
	inInstanceData += ::CopyBytes( inInstanceData, &mFlags, 4 );
	inInstanceData += ::CopyBytes( inInstanceData, &mMinLightIntensity, 2 );
	inInstanceData += ::CopyBytes( inInstanceData, &mMaxLightIntensity, 2 );
	inInstanceData += ::CopyBytes( inInstanceData, &mTransparency, 2 );
	inInstanceData += 2;
	inInstanceData += ::CopyBytes( inInstanceData, &mRegPointX, 2 );
	inInstanceData += ::CopyBytes( inInstanceData, &mRegPointY, 2 );
	inInstanceData += 8;
	inInstanceData += ::CopyBytes( inInstanceData, &mKeyPointX, 2 );
	inInstanceData += ::CopyBytes( inInstanceData, &mKeyPointY, 2 );
	inInstanceData += ::CopyBytes( inInstanceData, &mBitmapIndex, 2 );
	inInstanceData += 18;
	inInstanceData += 16;
}
	

// =================================================================================
//	 WriteData														 [const, public]
// =================================================================================
// outInstanceData is filled with the bitmap instance data as they are stored
// on file
					
void
CBitmapInstance::WriteData( Byte *outInstanceData ) const
{
	const Int32 theZero = 0;
	
	outInstanceData += ::CopyBytes( &mFlags, outInstanceData, 4 );
	outInstanceData += ::CopyBytes( &mMinLightIntensity, outInstanceData, 2 );
	outInstanceData += ::CopyBytes( &mMaxLightIntensity, outInstanceData, 2 );
	outInstanceData += ::CopyBytes( &mTransparency, outInstanceData, 2 );
	outInstanceData += ::CopyBytes( &theZero, outInstanceData, 2 );
	outInstanceData += ::CopyBytes( &mRegPointX, outInstanceData, 2 );
	outInstanceData += ::CopyBytes( &mRegPointY, outInstanceData, 2 );
	
	for ( Int32 i = 0; i < 8; i++ )
		*outInstanceData++ = 0;
	
	outInstanceData += ::CopyBytes( &mKeyPointX, outInstanceData, 2 );
	outInstanceData += ::CopyBytes( &mKeyPointY, outInstanceData, 2 );
	outInstanceData += ::CopyBytes( &mBitmapIndex, outInstanceData, 2 );
	
	for ( Int32 i = 0; i < 18; i++ )
		*outInstanceData++ = 0;
	
	for ( Int32 i = 0; i < 16; i++ )
		*outInstanceData++ = 0;
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================
// Return the bitmap instance size in bytes

Int32
CBitmapInstance::GetSize( void ) const
{
	return kElementSize;
}


#pragma mark -

// =================================================================================
//	 ListenToMessage														[public]
// =================================================================================

void
CBitmapInstance::ListenToMessage( MessageT inMessage, void *ioData )
{
	switch ( inMessage ) {
		case msg_TagDeleteElement:
			if ( ioData == mBitmap )
				SetBitmap( nil );
			break;
		
		case msg_256UpdateBitmap:
			if ( ioData == mBitmap )
				UpdateFromBitmap();
			break;
	}
}


#pragma mark -

// =================================================================================
//	 GetBitmap														 [const, public]
// =================================================================================

const CBitmap *
CBitmapInstance::GetBitmap( void ) const
{
	return mBitmap;
}


// =================================================================================
//	 GetBitmap																[public]
// =================================================================================

CBitmap *
CBitmapInstance::GetBitmap( void )
{
	return mBitmap;
}


// =================================================================================
//	 GetFlags														 [const, public]
// =================================================================================

Uint32
CBitmapInstance::GetFlags( void ) const
{
	return mFlags;
}


// =================================================================================
//	 SetFlags																[public]
// =================================================================================

void
CBitmapInstance::SetFlags( Uint32 inFlags )
{
	if ( mFlags != inFlags ) {
		mFlags = inFlags;
		BroadcastMessage( msg_256UpdateBitmapInstance, this );
	}
}


// =================================================================================
//	 GetMinLightIntensity											 [const, public]
// =================================================================================

Uint16
CBitmapInstance::GetMinLightIntensity( void ) const
{
	return mMinLightIntensity;
}


// =================================================================================
//	 SetMinLightIntensity													[public]
// =================================================================================

void
CBitmapInstance::SetMinLightIntensity( Uint16 inIntensity )
{
	if ( mMinLightIntensity != inIntensity ) {
		mMinLightIntensity = inIntensity;
		BroadcastMessage( msg_256UpdateBitmapInstance, this );
	}
}


// =================================================================================
//	 GetMaxLightIntensity											 [const, public]
// =================================================================================

Uint16
CBitmapInstance::GetMaxLightIntensity( void ) const
{
	return mMaxLightIntensity;
}


// =================================================================================
//	 SetMaxLightIntensity													[public]
// =================================================================================

void
CBitmapInstance::SetMaxLightIntensity( Uint16 inIntensity )
{
	if ( mMaxLightIntensity != inIntensity ) {
		mMaxLightIntensity = inIntensity;
		BroadcastMessage( msg_256UpdateBitmapInstance, this );
	}
}


// =================================================================================
//	 GetRegPointX													 [const, public]
// =================================================================================

Int16
CBitmapInstance::GetRegPointX( void ) const
{
	return mRegPointX;
}


// =================================================================================
//	 SetRegPointX															[public]
// =================================================================================

void
CBitmapInstance::SetRegPointX( Int16 inRegPointX )
{
	if ( mRegPointX != inRegPointX ) {
		mRegPointX = inRegPointX;
		BroadcastMessage( msg_256UpdateBitmapInstance, this );
	}
}


// =================================================================================
//	 GetRegPointY													 [const, public]
// =================================================================================

Int16
CBitmapInstance::GetRegPointY( void ) const
{
	return mRegPointY;
}


// =================================================================================
//	 SetRegPointY															[public]
// =================================================================================

void
CBitmapInstance::SetRegPointY( Int16 inRegPointY )
{
	if ( mRegPointY != inRegPointY ) {
		mRegPointY = inRegPointY;
		BroadcastMessage( msg_256UpdateBitmapInstance, this );
	}
}


// =================================================================================
//	 GetKeyPointX													 [const, public]
// =================================================================================

Int16
CBitmapInstance::GetKeyPointX( void ) const
{
	return mKeyPointX;
}


// =================================================================================
//	 SetKeyPointX															[public]
// =================================================================================

void
CBitmapInstance::SetKeyPointX( Int16 inKeyPointX )
{
	if ( mKeyPointX != inKeyPointX ) {
		mKeyPointX = inKeyPointX;
		BroadcastMessage( msg_256UpdateBitmapInstance, this );
	}
}


// =================================================================================
//	 GetKeyPointY													 [const, public]
// =================================================================================

Int16
CBitmapInstance::GetKeyPointY( void ) const
{
	return mKeyPointY;
}


// =================================================================================
//	 SetKeyPointY															[public]
// =================================================================================

void
CBitmapInstance::SetKeyPointY( Int16 inKeyPointY )
{
	if ( mKeyPointY != inKeyPointY ) {
		mKeyPointY = inKeyPointY;
		BroadcastMessage( msg_256UpdateBitmapInstance, this );
	}
}


#pragma mark -

// =================================================================================
//	 SetBitmap															 [protected]
// =================================================================================

void
CBitmapInstance::SetBitmap( CBitmap *inBitmap )
{
	if ( mBitmap == inBitmap )
		return;
	
	if ( mBitmap != nil )
		mBitmap->RemoveListener( this );
	
	mBitmap = inBitmap;
	
	if ( mBitmap != nil )
		mBitmap->AddListener( this );
	
	UpdateFromBitmap();
	BroadcastMessage( msg_256UpdateBitmapInstance, this );
}


// =================================================================================
//	 GetBitmapIndex											  [const, protected]
// =================================================================================
// The bitmap index is only valid right after a read operation and right before
// a write operation (when the tag/sequence data has updated it).

Int16
CBitmapInstance::GetBitmapIndex( void ) const
{
	return mBitmapIndex;
}


// =================================================================================
//	 SetBitmapIndex													 [protected]
// =================================================================================
// Does not send an update message, since it should only be used by the sequence
// data and the bitmap index isn't interesting anywhere else.

void
CBitmapInstance::SetBitmapIndex( Int16 inIndex )
{
	mBitmapIndex = inIndex;
}


#pragma mark -

// =================================================================================
//	 UpdateFromBitmap													   [private]
// =================================================================================

void
CBitmapInstance::UpdateFromBitmap( void )
{
	Boolean isDirty = false;
	
	if ( mBitmap != nil ) {
		
		// Set or clear transparent flags
		if ( mBitmap->GetBitDepth() == 16 )
			mFlags |= flag_Transparent;
		else
			mFlags &= ~flag_Transparent;
		
		// Texture bitmaps must be power of two
		if ( (mBitmap->GetFlags() & CBitmap::flag_PowerOfTwo) == 0 )
			mFlags &= ~flag_Texture;
	
	} else {
		mFlags				= 0;
		mMinLightIntensity	= 0;
		mMaxLightIntensity	= -1;
		mTransparency		= 0;
		mRegPointX			= 0;
		mRegPointY			= 0;
		mKeyPointX			= 0;
		mKeyPointY			= 0;
		mBitmapIndex		= -1;
	}
	
	if ( isDirty )
		BroadcastMessage( msg_256UpdateBitmapInstance, this );
}


