// =================================================================================
//	CBitmapInstance.h			 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#pragma once

#include "CTagElement.h"
#include "LListener.h"

class CBitmap;


// =================================================================================
//	CBitmapInstance														  [64 bytes]
// =================================================================================
// After the HueChange entries come the bitmap instance listings.  Each instance
// is 64 bytes long.  The info for this came from Eli Curtz and is NON-DISTRIBUTE
// please.
//
// [Bungie header] if a bitmap instance references a bitmap with a 4-bit alpha
// channel, it is always transparent, otherwise it may be marked transparent and
// will use the _transparency_overlay where the source color component is used
// as the transparency level.
//
// The texture flag (0x10) has this comment:  This is a texture, not a 1-bit
// transparency encoded rectangle (?)

class CBitmapInstance : public CTagElement, public LListener {
friend				class CSequenceData;
friend				class CFrameView;

public:
	enum			{	// Flags
						flag_MirrorHorizontal	= 0x01,
						flag_MirrorVertical		= 0x02,
						flag_Transparent		= 0x04,
						flag_KeypointObscured	= 0x08,
						flag_Texture			= 0x10,
						
						// Constant element size
						kElementSize			= 64
					};
	
					// Default constructor
	
					CBitmapInstance();
					
					// inInstanceData contains the bitmap instance data as
					// they are stored in tag and .gor files
					
	void			ReadData( Byte *inInstanceData );
	
					// outInstanceData is filled with the bitmap instance
					// data as they are stored in tag and .gor files
					
	void			WriteData( Byte *outInstanceData ) const;
	
					// Return the size of the bitmap instance on file
	
	Int32			GetSize() const;
	
				// LListener
	
	virtual void	ListenToMessage( MessageT inMessage, void *ioData );
	
				// Accessors
	
	const CBitmap	*GetBitmap() const;
	CBitmap			*GetBitmap();
	
	Uint32			GetFlags() const;
	void			SetFlags( Uint32 inFlags );
	
	Uint16			GetMinLightIntensity() const;
	void			SetMinLightIntensity( Uint16 inIntensity );
	
	Uint16			GetMaxLightIntensity() const;
	void			SetMaxLightIntensity( Uint16 inIntensity );
	
	Int16			GetRegPointX() const;
	void			SetRegPointX( Int16 inRegPointX );
	
	Int16			GetRegPointY() const;
	void			SetRegPointY( Int16 inRegPointY );
	
	Int16			GetKeyPointX() const;
	void			SetKeyPointX( Int16 inKeyPointX );
	
	Int16			GetKeyPointY() const;
	void			SetKeyPointY( Int16 inKeyPointY );
	
protected:
	void			SetBitmap( CBitmap *inBitmap );
	Int16			GetBitmapIndex() const;
	void			SetBitmapIndex( Int16 inIndex );

private:
					// Defensive programming
					CBitmapInstance( const CBitmapInstance & );
	CBitmapInstance	&operator=( const CBitmapInstance & );
	
	void			UpdateFromBitmap();
	
	Uint32 mFlags;
	Uint16 mMinLightIntensity;	// In [0,kFixedOne)
	Uint16 mMaxLightIntensity;	// In [0,kFixedOne)
	Uint16 mTransparency;		// 0 if !flag_Transparent
//	Int16 mUnused1;
	Int16 mRegPointX;			// Animation centering, in pixels from
	Int16 mRegPointY;			// the top left corner
//	Int16 mWorldTop;			// Runtime (?)
//	Int16 mWorldLeft;			// Runtime (?)
//	Int16 mWorldRight;			// Runtime (?)
//	Int16 mWorldBottom;			// Runtime (?)
	Int16 mKeyPointX;			// Projectile creation etc.
	Int16 mKeyPointY;			// (pixels from top left corner)
	Int16 mBitmapIndex;			// The bitmap described
//	Int16 mUnused2[9];
//	Byte mExtractorData[16];	// Data set and used by Extractor
	
	CBitmap *mBitmap;
}; 


