// =================================================================================
//	CSequence.cp				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CSequence.h"
#include "CSequenceReference.h"
#include "CSequenceData.h"

#include "256Messages.h"
#include "UGUI.h"


// =================================================================================
//	 CSequence																[public]
// =================================================================================

CSequence::CSequence( void )
{
	mReference = new CSequenceReference;
	mData = new CSequenceData;
	
	mReference->AddListener( this );
	mData->AddListener( this );
}


// =================================================================================
//	 ~CSequence															[public]
// =================================================================================

CSequence::~CSequence( void )
{
	delete mReference;
	delete mData;
}


// =================================================================================
//	 ReadReference															[public]
// =================================================================================

void
CSequence::ReadReference( const Byte *inBytes )
{
	mReference->ReadData( inBytes );
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================

void
CSequence::ReadData( const Byte *inBytes )
{
	mData->ReadData( inBytes );
}


// =================================================================================
//	 WriteReference														[public]
// =================================================================================

void
CSequence::WriteReference( Byte *outBytes )
{
	mReference->WriteData( outBytes );
}


// =================================================================================
//	 WriteData																[public]
// =================================================================================

void
CSequence::WriteData( Byte *outBytes )
{
	mData->WriteData( outBytes );
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================
// Returns the size of all sequence elements (reference, data, instances, shadows)

Int32
CSequence::GetSize( void ) const
{
	return mReference->GetSize() + mData->GetTotalSize();
}


// =================================================================================
//	 GetReferenceSize												 [const, public]
// =================================================================================

Int32
CSequence::GetReferenceSize( void ) const
{
	return mReference->GetSize();
}


// =================================================================================
//	 GetRefDataSize												 [const, public]
// =================================================================================

Int32
CSequence::GetRefDataSize( void ) const
{
	return mReference->GetDataSize();
}


// =================================================================================
//	 GetDataSize													 [const, public]
// =================================================================================

Int32
CSequence::GetDataSize( void ) const
{
	return mData->GetSize();
}


#pragma mark -

// =================================================================================
//	 ListenToMessage														[public]
// =================================================================================

void
CSequence::ListenToMessage( MessageT inMessage, void *ioData )
{
	#pragma unused(ioData)
	
	switch ( inMessage ) {
		default:
			// Pass it on
			BroadcastMessage( inMessage, ioData );
			break;
		
		case msg_256UpdateSequence:
			BroadcastMessage( msg_256UpdateSequence, this );
			break;
	
	}
}


#pragma mark -

// =================================================================================
//	 GetName														 [const, public]
// =================================================================================

const char *
CSequence::GetName( void ) const
{
	return mReference->GetName();
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CSequence::SetName( ConstStr255Param inName )
{
	mReference->SetName( inName );
}


// =================================================================================
//	 GetDataOffset													 [const, public]
// =================================================================================

Int32
CSequence::GetDataOffset( void ) const
{
	return mReference->GetDataOffset();
}


// =================================================================================
//	 SetDataOffset															[public]
// =================================================================================

void
CSequence::SetDataOffset( Int32 inOffset)
{
	mReference->SetDataOffset( inOffset );
}


#pragma mark -

// =================================================================================
//	 GetFlags														 [const, public]
// =================================================================================

Uint32
CSequence::GetFlags( void ) const
{
	return mData->GetFlags();
}


// =================================================================================
//	 SetFlags																[public]
// =================================================================================

void
CSequence::SetFlags( Uint32 inFlags )
{
	mData->SetFlags( inFlags );
}


// =================================================================================
//	 GetViewCount													 [const, public]
// =================================================================================

Int16
CSequence::GetViewCount( void ) const
{
	return mData->GetViewCount();
}


// =================================================================================
//	 SetViewCount															[public]
// =================================================================================

void
CSequence::SetViewCount( Int16 inCount )
{
	mData->SetViewCount( inCount );
}


// =================================================================================
//	 GetFrameCount													 [const, public]
// =================================================================================

Int16
CSequence::GetFrameCount( void ) const
{
	return mData->GetFrameCount();
}


// =================================================================================
//	 SetFrameCount															[public]
// =================================================================================

void
CSequence::SetFrameCount( Int16 inCount )
{
	mData->SetFrameCount( inCount );
}


// =================================================================================
//	 GetFrameTicks													 [const, public]
// =================================================================================

Int16
CSequence::GetFrameTicks( void ) const
{
	return mData->GetFrameTicks();
}


// =================================================================================
//	 SetFrameTicks															[public]
// =================================================================================

void
CSequence::SetFrameTicks( Int16 inTicks )
{
	mData->SetFrameTicks( inTicks );
}


// =================================================================================
//	 GetKeyFrame													 [const, public]
// =================================================================================

Int16
CSequence::GetKeyFrame( void ) const
{
	return mData->GetKeyFrame();
}


// =================================================================================
//	 SetKeyFrame															[public]
// =================================================================================

void
CSequence::SetKeyFrame( Int16 inFrame )
{
	mData->SetKeyFrame( inFrame );
}


// =================================================================================
//	 GetLoopFrame													 [const, public]
// =================================================================================

Int16
CSequence::GetLoopFrame( void ) const
{
	return mData->GetLoopFrame();
}


// =================================================================================
//	 SetLoopFrame															[public]
// =================================================================================

void
CSequence::SetLoopFrame( Int16 inFrame )
{
	mData->SetLoopFrame( inFrame );
}


// =================================================================================
//	 GetTransferPeriod												 [const, public]
// =================================================================================

Int16
CSequence::GetTransferPeriod( void ) const
{
	return mData->GetTransferPeriod();
}


// =================================================================================
//	 SetTransferPeriod														[public]
// =================================================================================

void
CSequence::SetTransferPeriod( Int16 inPeriod )
{
	mData->SetTransferPeriod( inPeriod );
}


// =================================================================================
//	 GetTransferMode												 [const, public]
// =================================================================================

Uint16
CSequence::GetTransferMode( void ) const
{
	return mData->GetTransferMode();
}


// =================================================================================
//	 SetTransferMode														[public]
// =================================================================================

void
CSequence::SetTransferMode( Uint16 inMode )
{
	mData->SetTransferMode( inMode );
}


// =================================================================================
//	 GetFirstSound													 [const, public]
// =================================================================================

Int32
CSequence::GetFirstSound( void ) const
{
	return mData->GetFirstSound();
}


// =================================================================================
//	 GetFirstSoundIndex											 [const, public]
// =================================================================================

Int16
CSequence::GetFirstSoundIndex( void ) const
{
	return mData->GetFirstSoundIndex();
}


// =================================================================================
//	 SetFirstSound															[public]
// =================================================================================

void
CSequence::SetFirstSound( Int32 inSound, Int16 inIndex )
{
	mData->SetFirstSound( inSound, inIndex );
}


// =================================================================================
//	 GetKeySound													 [const, public]
// =================================================================================

Int32
CSequence::GetKeySound( void ) const
{
	return mData->GetKeySound();
}


// =================================================================================
//	 GetKeySoundIndex												 [const, public]
// =================================================================================

Int16
CSequence::GetKeySoundIndex( void ) const
{
	return mData->GetKeySoundIndex();
}


// =================================================================================
//	 SetKeySound															[public]
// =================================================================================

void
CSequence::SetKeySound( Int32 inSound, Int16 inIndex )
{
	mData->SetKeySound( inSound, inIndex );
}


// =================================================================================
//	 GetLastSound													 [const, public]
// =================================================================================

Int32
CSequence::GetLastSound( void ) const
{
	return mData->GetLastSound();
}


// =================================================================================
//	 GetLastSoundIndex												 [const, public]
// =================================================================================

Int16
CSequence::GetLastSoundIndex( void ) const
{
	return mData->GetLastSoundIndex();
}


// =================================================================================
//	 SetLastSound															[public]
// =================================================================================

void
CSequence::SetLastSound( Int32 inSound, Int16 inIndex )
{
	mData->SetLastSound( inSound, inIndex );
}


// =================================================================================
//	 GetBBoxRadius													 [const, public]
// =================================================================================

Int32
CSequence::GetBBoxRadius( void ) const
{
	return mData->GetBBoxRadius();
}


// =================================================================================
//	 SetBBoxRadius															[public]
// =================================================================================

void
CSequence::SetBBoxRadius( Int32 inRadius )
{
	mData->SetBBoxRadius( inRadius );
}


// =================================================================================
//	 GetBBoxBottom													 [const, public]
// =================================================================================

Int32
CSequence::GetBBoxBottom( void ) const
{
	return mData->GetBBoxBottom();
}


// =================================================================================
//	 SetBBoxBottom															[public]
// =================================================================================

void
CSequence::SetBBoxBottom( Int32 inBottom )
{
	mData->SetBBoxBottom( inBottom );
}


// =================================================================================
//	 GetBBoxTop													 [const, public]
// =================================================================================

Int32
CSequence::GetBBoxTop( void ) const
{
	return mData->GetBBoxTop();
}


// =================================================================================
//	 SetBBoxTop															[public]
// =================================================================================

void
CSequence::SetBBoxTop( Int32 inTop )
{
	mData->SetBBoxTop( inTop );
}


#pragma mark -

// =================================================================================
//	 GetInstance													 [const, public]
// =================================================================================

const CBitmapInstance *
CSequence::GetInstance( Int16 inFrame, Int16 inView ) const
{
	return mData->GetInstance( inFrame, inView );
}


// =================================================================================
//	 GetInstance															[public]
// =================================================================================

CBitmapInstance *
CSequence::GetInstance( Int16 inFrame, Int16 inView )
{
	return mData->GetInstance( inFrame, inView );
}


// =================================================================================
//	 GetBitmap														 [const, public]
// =================================================================================

const CBitmap *
CSequence::GetBitmap( Int16 inFrame, Int16 inView ) const
{
	return mData->GetBitmap( inFrame, inView );
}


// =================================================================================
//	 GetBitmap																[public]
// =================================================================================

CBitmap *
CSequence::GetBitmap( Int16 inFrame, Int16 inView )
{
	return mData->GetBitmap( inFrame, inView );
}


#pragma mark -

// =================================================================================
//	 GetBitmap														 [const, public]
// =================================================================================

const CBitmap *
CSequence::GetBitmap( Int16 inFrame ) const
{
	return mData->GetBitmap( inFrame );
}


// =================================================================================
//	 GetBitmap																[public]
// =================================================================================

CBitmap *
CSequence::GetBitmap( Int16 inFrame )
{
	return mData->GetBitmap( inFrame );
}


// =================================================================================
//	 GetShadowMap													 [const, public]
// =================================================================================

const CShadowMap *
CSequence::GetShadowMap( Int16 inFrame ) const
{
	return mData->GetShadowMap( inFrame );
}


// =================================================================================
//	 GetShadowMap															[public]
// =================================================================================

CShadowMap *
CSequence::GetShadowMap( Int16 inFrame )
{
	return mData->GetShadowMap( inFrame );
}


#pragma mark -

// =================================================================================
//	 GetFrameView															[public]
// =================================================================================

Boolean
CSequence::GetFrameView( const CBitmapInstance *inInstance, SFrameView &outFrameView )
{
	Int16 theFrame;
	Int16 theView;
	
	if ( mData->GetFrameView( inInstance, theFrame, theView ) ) {
		outFrameView.sSequence = this;
		outFrameView.sFrame = theFrame;
		outFrameView.sView = theView;
		return true;
	}
	
	return false;
}


// =================================================================================
//	 GetFrameView															[public]
// =================================================================================

Boolean
CSequence::GetFrameView( const CShadowMap *inShadowMap, SFrameView &outFrameView )
{
	Int16 theFrame;
	
	if ( mData->GetFrameView( inShadowMap, theFrame ) ) {
		outFrameView.sSequence = this;
		outFrameView.sFrame = theFrame;
		outFrameView.sView = -1;
		return true;
	}
	
	return false;
}


// =================================================================================
//	 GetFrameView															[public]
// =================================================================================

Boolean
CSequence::GetFrameView( const CBitmap *inBitmap, SFrameView &outFrameView )
{
	Int16 theFrame;
	Int16 theView;
	
	if ( mData->GetFrameView( inBitmap, theFrame, theView ) ) {
		outFrameView.sSequence = this;
		outFrameView.sFrame = theFrame;
		outFrameView.sView = theView;
		return true;
	}
	
	return false;
}


#pragma mark -

// =================================================================================
//	 AssignBitmapInstances													[public]
// =================================================================================

Boolean
CSequence::AssignBitmapInstances( Byte *inBytes, Int32 inCount, LArray &inBitmaps )
{
	return mData->AssignBitmapInstances( inBytes, inCount, inBitmaps );
}


// =================================================================================
//	 AssignShadowMaps														[public]
// =================================================================================

Boolean
CSequence::AssignShadowMaps( Byte *inBytes, Int32 inCount, LArray &inBitmaps )
{
	return mData->AssignShadowMaps( inBytes, inCount, inBitmaps );
}


// =================================================================================
//	 DeclareElements														[public]
// =================================================================================

void
CSequence::DeclareElements(
	LArray &inInstances,
	LArray &inShadowMaps,
	LArray &inBitmaps )
{
	mData->DeclareElements( inInstances, inShadowMaps, inBitmaps );
}


