// =================================================================================
//	CSequence.h					 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================
// This is just an interface class for the sequences.  All access to the
// CSequenceReference and CSequenceData should go through this class.

#pragma once


#include "CTagElement.h"
#include "LListener.h"

class CSequence;
class CSequenceReference;
class CSequenceData;
class CBitmapInstance;
class CShadowMap;
class CBitmap;

struct SFrameView {
	CSequence	*sSequence;
	Int16		sFrame;
	Int16		sView;
};

typedef struct SFrameView SFrameView;


class CSequence : public CTagElement, public LListener {
friend					class C256Tag;

public:
	enum				{	// Flags (must correspond to CSequenceData.h)
							flag_UsesMirroring		= 0x01,
							flag_AlwaysSet			= 0x02,
							flag_HasNoRotation		= 0x04,
		
							// Transfer modes (must correspond to CSequenceData.h)
							transferMode_Normal		= 0,
							transferMode_FadeOut	= 1,
							transferMode_FadeIn		= 2,
							transferMode_ShrinkOut	= 3,	// Not used?
							transferMode_ShrinkIn	= 4,	// Not used?
							
							// Constant element sizes
							kElementSize	= 128
						};
	
	
						// Default constructor and destructor
					
						CSequence();
						~CSequence();
	
	void				ReadReference( const Byte *inBytes );
	void				ReadData( const Byte *inBytes );
					
	void				WriteReference( Byte *outBytes );
	void				WriteData( Byte *outBytes );
					
						// GetSize returns the combined size of all
						// sequence elements, GetReference size returns
						// the size of just the sequence reference struct.
						// GetRefDataSize returns just the size of the
						// sequence data struct as given by the reference,
						// while GetDataSize gives the size of the sequence
						// data struct "in real life"
					
	Int32				GetSize() const;
	Int32				GetReferenceSize() const;
	Int32				GetRefDataSize() const;
	Int32				GetDataSize() const;
					
					// LListener
	
	virtual void		ListenToMessage( MessageT inMessage, void *ioData );
	
					// Accessors
					
	const char			*GetName() const;
	void				SetName( ConstStr255Param inName );
	
	Int32				GetDataOffset() const;
	void				SetDataOffset( Int32 inOffset );
	
	
	Uint32				GetFlags() const;
	void				SetFlags( Uint32 inFlags );
	
	Int16				GetViewCount() const;
	void				SetViewCount( Int16 inCount );
	
	Int16				GetFrameCount() const;
	void				SetFrameCount( Int16 inCount );
	
	Int16				GetFrameTicks() const;
	void				SetFrameTicks( Int16 inTicks );
	
	Int16				GetKeyFrame() const;
	void				SetKeyFrame( Int16 inFrame );
	
	Int16				GetLoopFrame() const;
	void				SetLoopFrame( Int16 inFrame );
	
	Int16				GetTransferPeriod() const;
	void				SetTransferPeriod( Int16 inPeriod );
	
	Uint16				GetTransferMode() const;
	void				SetTransferMode( Uint16 inMode );
	
	Int32				GetFirstSound() const;
	Int16				GetFirstSoundIndex() const;
	void				SetFirstSound( Int32 inSound = -1, Int16 inIndex = -1 );
	
	Int32				GetKeySound() const;
	Int16				GetKeySoundIndex() const;
	void				SetKeySound( Int32 inSound = -1, Int16 inIndex = -1 );
	
	Int32				GetLastSound() const;
	Int16				GetLastSoundIndex() const;
	void				SetLastSound( Int32 inSound = -1, Int16 inIndex = -1 );
	
	Int32				GetBBoxRadius() const;
	void				SetBBoxRadius( Int32 inRadius );
	
	Int32				GetBBoxBottom() const;
	void				SetBBoxBottom( Int32 inBottom );
	
	Int32				GetBBoxTop() const;
	void				SetBBoxTop( Int32 inTop );
	
					// Instance accessors
	
	const CBitmapInstance *GetInstance( Int16 inFrame, Int16 inView ) const;
	CBitmapInstance		*GetInstance( Int16 inFrame, Int16 inView );
	
	const CBitmap		*GetBitmap( Int16 inFrame, Int16 inView ) const;
	CBitmap				*GetBitmap( Int16 inFrame, Int16 inView );
	
					// Shadow map accessors
	
	const CShadowMap	*GetShadowMap( Int16 inFrame ) const;
	CShadowMap			*GetShadowMap( Int16 inFrame );
	
	const CBitmap		*GetBitmap( Int16 inFrame ) const;
	CBitmap				*GetBitmap( Int16 inFrame );
	
					// Misc accessors
	
	Boolean				GetFrameView( const CBitmapInstance *inInstance, SFrameView &outFrameView );
	Boolean				GetFrameView( const CShadowMap *inShadowMap, SFrameView &outFrameView );
	Boolean				GetFrameView( const CBitmap *inBitmap, SFrameView &outFrameView );

protected:
	Boolean				AssignBitmapInstances(
							Byte *inBytes,
							Int32 inCount,
							LArray &inBitmaps );
	Boolean				AssignShadowMaps(
							Byte *inBytes,
							Int32 inCount,
							LArray &inBitmaps );
	
	void				DeclareElements(
							LArray &inInstances,
							LArray &inShadowMaps,
							LArray &inBitmaps );
	
private:
						// Defensive programming
						CSequence( const CSequence & );
	CSequence			&operator=( const CSequence & );
	
	CSequenceReference	*mReference;
	CSequenceData		*mData;
};


