// =================================================================================
//	CSequenceData.cp			 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CSequenceData.h"
#include "CBitmapInstance.h"
#include "CShadowMap.h"

#include "256Messages.h"
#include "CopyBytes.h"
#include "StNewBlock.h"

#include <TArray.h>


#pragma mark CFrameView

// =================================================================================
//	 CFrameView															 [local]
// =================================================================================
// The indices are only used to initialize the pointers after a read operation
// and for write operations.  They are updated right before a write and are
// otherwise to be considered invalid.

class CFrameView {
public:
					CFrameView();
					~CFrameView();
	
	void			SetFrameViewCount(
						Int16 inFrameCount,
						Int16 inViewCount,
						LListener *inListener );
	Int32			ReadData( const Byte *inData );
	Int32			WriteData( Byte *outData );
	
	Int32			GetSize() const;
	
	Int16			GetFrameCount() const;
	void			SetFrameCount( Int16 inCount, LListener *inListener );
	
	Int16			GetViewCount() const;
	void			SetViewCount( Int16 inCount, LListener *inListener );
	
					// Misc accessors
	
	const CBitmapInstance	*GetInstance( Int16 inFrame, Int16 inView ) const;
	CBitmapInstance			*GetInstance( Int16 inFrame, Int16 inView );
	
	const CShadowMap		*GetShadowMap( Int16 inFrame ) const;
	CShadowMap				*GetShadowMap( Int16 inFrame );
	
	Boolean				AssignBitmapInstances(
							Byte *inBytes,
							Int32 inCount,
							LArray &inBitmaps );
	Boolean				AssignShadowMaps(
							Byte *inBytes,
							Int32 inCount,
							LArray &inBitmaps );
	
	void				DeclareElements(
							LArray &inInstances,
							LArray &inShadowMaps,
							LArray &inBitmaps );

private:
	typedef struct SView {
		CBitmapInstance	*mInstance;
		Int16			mIndex;
	};
	
	typedef struct SFrame {
		CShadowMap		*mShadowMap;
		Int16			mShadowIndex;
		SView			mViews[];
	};
	
	SFrame				*GetFrame( Int16 inFrame );
	const SFrame		*GetFrame( Int16 inFrame ) const;
	Int32				GetFrameSize() const;
	
	SFrame			**mFrameH;
	Int16			mFrameCount;
	Int16			mViewCount;
};


#pragma mark -

// =================================================================================
//	 CSequenceData														 [protected]
// =================================================================================
					
CSequenceData::CSequenceData( void )
{
	mFlags			= flag_AlwaysSet;
	mPixelsToWorld	= 0;
	mFrameTicks		= 0;
	mKeyFrame		= 0;
	mLoopFrame		= 0;
	mFirstSoundIndex= -1;
	mKeySoundIndex	= -1;
	mLastSoundIndex	= -1;
	mFirstFrameSound= -1;
	mKeyFrameSound	= -1;
	mLastFrameSound	= -1;
	mTransferPeriod	= 4;
	mTransferMode	= 0;
	mBBoxRadius		= 0;
	mBBoxBottom 	= 0;
	mBBoxTop		= 0;
	
	mFrameView = new CFrameView;
}


// =================================================================================
//	 ~CSequenceData													 [protected]
// =================================================================================
					
CSequenceData::~CSequenceData( void )
{
	delete mFrameView;
}


// =================================================================================
//	 ReadData															 [protected]
// =================================================================================
// inSequenceData is a buffer containing the sequence data as they are
// stored in the .gor files and plugin files.
					
void
CSequenceData::ReadData( const Byte *inSequenceData )
{
	Int16 theFrameCount;
	Int16 theViewCount;
	
	inSequenceData += ::CopyBytes( inSequenceData, &mFlags, 4 );
	inSequenceData += ::CopyBytes( inSequenceData, &mPixelsToWorld, 4 );
	inSequenceData += ::CopyBytes( inSequenceData, &theViewCount, 2 );
	inSequenceData += ::CopyBytes( inSequenceData, &theFrameCount, 2 );
	inSequenceData += ::CopyBytes( inSequenceData, &mFrameTicks, 2 );
	inSequenceData += ::CopyBytes( inSequenceData, &mKeyFrame, 2 );
	inSequenceData += ::CopyBytes( inSequenceData, &mLoopFrame, 2 );
	
	inSequenceData += ::CopyBytes( inSequenceData, &mFirstSoundIndex, 2 );
	inSequenceData += ::CopyBytes( inSequenceData, &mKeySoundIndex, 2 );
	inSequenceData += ::CopyBytes( inSequenceData, &mLastSoundIndex, 2 );
	
	inSequenceData += ::CopyBytes( inSequenceData, &mFirstFrameSound, 4 );
	inSequenceData += ::CopyBytes( inSequenceData, &mKeyFrameSound, 4 );
	inSequenceData += ::CopyBytes( inSequenceData, &mLastFrameSound, 4 );
	
	inSequenceData += ::CopyBytes( inSequenceData, &mTransferMode, 2 );
	inSequenceData += ::CopyBytes( inSequenceData, &mTransferPeriod, 2 );
	
	inSequenceData += ::CopyBytes( inSequenceData, &mBBoxRadius, 4 );
	inSequenceData += ::CopyBytes( inSequenceData, &mBBoxBottom, 4 );
	inSequenceData += ::CopyBytes( inSequenceData, &mBBoxTop, 4 );

	inSequenceData += 2;
	inSequenceData += 2;
	inSequenceData += 2;
	
	inSequenceData += 6;
	
	mFrameView->SetFrameViewCount( theFrameCount, theViewCount, this );
	inSequenceData += mFrameView->ReadData( inSequenceData );
}
	

// =================================================================================
//	 WriteData													  [const, protected]
// =================================================================================
// outSequenceData is filled with the sequence data as they are stored on
// file
					
void
CSequenceData::WriteData( Byte *outSequenceData ) const
{
	Int16 theFrameCount = GetFrameCount();
	Int16 theViewCount = GetViewCount();
	
	outSequenceData += ::CopyBytes( &mFlags, outSequenceData, 4 );
	outSequenceData += ::CopyBytes( &mPixelsToWorld, outSequenceData, 4 );
	outSequenceData += ::CopyBytes( &theViewCount, outSequenceData, 2 );
	outSequenceData += ::CopyBytes( &theFrameCount, outSequenceData, 2 );
	outSequenceData += ::CopyBytes( &mFrameTicks, outSequenceData, 2 );
	outSequenceData += ::CopyBytes( &mKeyFrame, outSequenceData, 2 );
	outSequenceData += ::CopyBytes( &mLoopFrame, outSequenceData, 2 );
	
	outSequenceData += ::CopyBytes( &mFirstSoundIndex, outSequenceData, 2 );
	outSequenceData += ::CopyBytes( &mKeySoundIndex, outSequenceData, 2 );
	outSequenceData += ::CopyBytes( &mLastSoundIndex, outSequenceData, 2 );
	
	outSequenceData += ::CopyBytes( &mFirstFrameSound, outSequenceData, 4 );
	outSequenceData += ::CopyBytes( &mKeyFrameSound, outSequenceData, 4 );
	outSequenceData += ::CopyBytes( &mLastFrameSound, outSequenceData, 4 );
	
	outSequenceData += ::CopyBytes( &mTransferMode, outSequenceData, 2 );
	outSequenceData += ::CopyBytes( &mTransferPeriod, outSequenceData, 2 );
	
	outSequenceData += ::CopyBytes( &mBBoxRadius, outSequenceData, 4 );
	outSequenceData += ::CopyBytes( &mBBoxBottom, outSequenceData, 4 );
	outSequenceData += ::CopyBytes( &mBBoxTop, outSequenceData, 4 );
	
	for ( Int32 i = 0; i < 12; i++ )
		*outSequenceData++ = 0;
	
	outSequenceData += mFrameView->WriteData( outSequenceData );
}


// =================================================================================
//	 GetSize													  [const, protected]
// =================================================================================
// Return the sequence size in bytes

Int32
CSequenceData::GetSize( void ) const
{
	return 64 + mFrameView->GetSize();
}


// =================================================================================
//	 GetTotalSize												  [const, protected]
// =================================================================================
// Return the sequence size in bytes, including instances and shadow maps

Int32
CSequenceData::GetTotalSize( void ) const
{
	Int32 theSize = GetSize();
	
	// Add size of bitmap instance elements
	theSize += GetFrameCount()*GetViewCount()*CBitmapInstance::kElementSize;
	
	// Add size of shdow map elements
	theSize += GetFrameCount()*CShadowMap::kElementSize;
	
	return theSize;
}


#pragma mark -

// =================================================================================
//	 ListenToMessage													 [protected]
// =================================================================================

void
CSequenceData::ListenToMessage( MessageT inMessage, void *ioData )
{
	switch ( inMessage ) {
		default:
			// Pass it on
			BroadcastMessage( inMessage, ioData );
			break;
	}
}


#pragma mark -

// =================================================================================
//	 GetFlags													  [const, protected]
// =================================================================================

Uint32
CSequenceData::GetFlags( void ) const
{
	return mFlags;
}


// =================================================================================
//	 SetFlags															 [protected]
// =================================================================================

void
CSequenceData::SetFlags( Uint32 inFlags )
{
	if ( mFlags != inFlags ) {
		mFlags = inFlags;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


#pragma mark -

// =================================================================================
//	 GetViewCount												  [const, protected]
// =================================================================================

Int16
CSequenceData::GetViewCount( void ) const
{
	return mFrameView->GetViewCount();
}


// =================================================================================
//	 SetViewCount														 [protected]
// =================================================================================

void
CSequenceData::SetViewCount( Int16 inCount )
{
	if ( GetViewCount() != inCount ) {
		mFrameView->SetViewCount( inCount, this );
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetFrameCount												  [const, protected]
// =================================================================================

Int16
CSequenceData::GetFrameCount( void ) const
{
	return mFrameView->GetFrameCount();
}


// =================================================================================
//	 SetFrameCount														 [protected]
// =================================================================================

void
CSequenceData::SetFrameCount( Int16 inCount )
{
	if ( GetFrameCount() != inCount ) {
		mFrameView->SetFrameCount( inCount, this );
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetFrameTicks												  [const, protected]
// =================================================================================

Int16
CSequenceData::GetFrameTicks( void ) const
{
	return mFrameTicks;
}


// =================================================================================
//	 SetFrameTicks														 [protected]
// =================================================================================

void
CSequenceData::SetFrameTicks( Int16 inTicks )
{
	if ( mFrameTicks != inTicks ) {
		mFrameTicks = inTicks;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


#pragma mark -

// =================================================================================
//	 GetKeyFrame												  [const, protected]
// =================================================================================

Int16
CSequenceData::GetKeyFrame( void ) const
{
	return mKeyFrame;
}


// =================================================================================
//	 SetKeyFrame														 [protected]
// =================================================================================

void
CSequenceData::SetKeyFrame( Int16 inFrame )
{
	if ( mKeyFrame != inFrame ) {
		mKeyFrame = inFrame;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetLoopFrame												  [const, protected]
// =================================================================================

Int16
CSequenceData::GetLoopFrame( void ) const
{
	return mLoopFrame;
}


// =================================================================================
//	 SetLoopFrame														 [protected]
// =================================================================================

void
CSequenceData::SetLoopFrame( Int16 inFrame )
{
	if ( mLoopFrame != inFrame ) {
		mLoopFrame = inFrame;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetTransferPeriod											  [const, protected]
// =================================================================================

Int16
CSequenceData::GetTransferPeriod( void ) const
{
	return mTransferPeriod;
}


// =================================================================================
//	 SetTransferPeriod													 [protected]
// =================================================================================

void
CSequenceData::SetTransferPeriod( Int16 inPeriod )
{
	if ( mTransferPeriod != inPeriod ) {
		mTransferPeriod = inPeriod;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetTransferMode											  [const, protected]
// =================================================================================

Uint16
CSequenceData::GetTransferMode( void ) const
{
	return mTransferMode;
}


// =================================================================================
//	 SetTransferMode													 [protected]
// =================================================================================

void
CSequenceData::SetTransferMode( Uint16 inMode )
{
	if ( mTransferMode != inMode ) {
		mTransferMode = inMode;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


#pragma mark -

// =================================================================================
//	 GetFirstSound												  [const, protected]
// =================================================================================

Int32
CSequenceData::GetFirstSound( void ) const
{
	return mFirstFrameSound;
}


// =================================================================================
//	 GetFirstSoundIndex										  [const, protected]
// =================================================================================

Int16
CSequenceData::GetFirstSoundIndex( void ) const
{
	return mFirstSoundIndex;
}


// =================================================================================
//	 SetFirstSound														 [protected]
// =================================================================================

void
CSequenceData::SetFirstSound( Int32 inSound, Int16 inIndex )
{
	if ( mFirstFrameSound != inSound || mFirstSoundIndex != inIndex ) {
		mFirstFrameSound = inSound;
		mFirstSoundIndex = inIndex;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetKeySound												  [const, protected]
// =================================================================================

Int32
CSequenceData::GetKeySound( void ) const
{
	return mKeyFrameSound;
}


// =================================================================================
//	 GetKeySoundIndex											  [const, protected]
// =================================================================================

Int16
CSequenceData::GetKeySoundIndex( void ) const
{
	return mKeySoundIndex;
}


// =================================================================================
//	 SetKeySound														 [protected]
// =================================================================================

void
CSequenceData::SetKeySound( Int32 inSound, Int16 inIndex )
{
	if ( mKeyFrameSound != inSound || mKeySoundIndex != inIndex ) {
		mKeyFrameSound = inSound;
		mKeySoundIndex = inIndex;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetLastSound												  [const, protected]
// =================================================================================

Int32
CSequenceData::GetLastSound( void ) const
{
	return mLastFrameSound;
}


// =================================================================================
//	 GetLastSoundIndex											  [const, protected]
// =================================================================================

Int16
CSequenceData::GetLastSoundIndex( void ) const
{
	return mLastSoundIndex;
}


// =================================================================================
//	 SetLastSound														 [protected]
// =================================================================================

void
CSequenceData::SetLastSound( Int32 inSound, Int16 inIndex )
{
	if ( mLastFrameSound != inSound || mLastSoundIndex != inIndex ) {
		mLastFrameSound = inSound;
		mLastSoundIndex = inIndex;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


#pragma mark -

// =================================================================================
//	 GetBBoxRadius												  [const, protected]
// =================================================================================

Int32
CSequenceData::GetBBoxRadius( void ) const
{
	return mBBoxRadius;
}


// =================================================================================
//	 SetBBoxRadius														 [protected]
// =================================================================================

void
CSequenceData::SetBBoxRadius( Int32 inRadius )
{
	if ( mBBoxRadius != inRadius ) {
		mBBoxRadius = inRadius;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetBBoxBottom												  [const, protected]
// =================================================================================

Int32
CSequenceData::GetBBoxBottom( void ) const
{
	return mBBoxBottom;
}


// =================================================================================
//	 SetBBoxBottom														 [protected]
// =================================================================================

void
CSequenceData::SetBBoxBottom( Int32 inBottom )
{
	if ( mBBoxBottom != inBottom ) {
		mBBoxBottom = inBottom;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


// =================================================================================
//	 GetBBoxTop												  [const, protected]
// =================================================================================

Int32
CSequenceData::GetBBoxTop( void ) const
{
	return mBBoxTop;
}


// =================================================================================
//	 SetBBoxTop														 [protected]
// =================================================================================

void
CSequenceData::SetBBoxTop( Int32 inTop )
{
	if ( mBBoxTop != inTop ) {
		mBBoxTop = inTop;
		BroadcastMessage( msg_256UpdateSequence, nil );
	}
}


#pragma mark -

// =================================================================================
//	 GetInstance												  [const, protected]
// =================================================================================

const CBitmapInstance *
CSequenceData::GetInstance( Int16 inFrame, Int16 inView ) const
{
	return mFrameView->GetInstance( inFrame, inView );
}


// =================================================================================
//	 GetInstance														 [protected]
// =================================================================================

CBitmapInstance *
CSequenceData::GetInstance( Int16 inFrame, Int16 inView )
{
	return mFrameView->GetInstance( inFrame, inView );
}

// =================================================================================
//	 GetBitmap													  [const, protected]
// =================================================================================

const CBitmap *
CSequenceData::GetBitmap( Int16 inFrame, Int16 inView ) const
{
	const CBitmapInstance *theInstance = mFrameView->GetInstance( inFrame, inView );
	const CBitmap *theBitmap = nil;
	
	if ( theInstance != nil )
		theBitmap = theInstance->GetBitmap();
	
	return theBitmap;
}


// =================================================================================
//	 GetBitmap															 [protected]
// =================================================================================

CBitmap *
CSequenceData::GetBitmap( Int16 inFrame, Int16 inView )
{
	CBitmapInstance *theInstance = mFrameView->GetInstance( inFrame, inView );
	CBitmap *theBitmap = nil;
	
	if ( theInstance != nil )
		theBitmap = theInstance->GetBitmap();
	
	return theBitmap;
}


#pragma mark -

// =================================================================================
//	 GetShadowMap												  [const, protected]
// =================================================================================

const CShadowMap *
CSequenceData::GetShadowMap( Int16 inFrame ) const
{
	return mFrameView->GetShadowMap( inFrame );
}


// =================================================================================
//	 GetShadowMap														 [protected]
// =================================================================================

CShadowMap *
CSequenceData::GetShadowMap( Int16 inFrame )
{
	return mFrameView->GetShadowMap( inFrame );
}


// =================================================================================
//	 GetBitmap													  [const, protected]
// =================================================================================

const CBitmap *
CSequenceData::GetBitmap( Int16 inFrame ) const
{
	const CShadowMap *theShadowMap = mFrameView->GetShadowMap( inFrame );
	const CBitmap *theBitmap = nil;
	
	if ( theShadowMap != nil )
		theBitmap = theShadowMap->GetBitmap();
	
	return theBitmap;
}


// =================================================================================
//	 GetBitmap															 [protected]
// =================================================================================

CBitmap *
CSequenceData::GetBitmap( Int16 inFrame )
{
	CShadowMap *theShadowMap = mFrameView->GetShadowMap( inFrame );
	CBitmap *theBitmap = nil;
	
	if ( theShadowMap != nil )
		theBitmap = theShadowMap->GetBitmap();
	
	return theBitmap;
}


#pragma mark -

// =================================================================================
//	 GetFrameView												  [const, protected]
// =================================================================================

Boolean
CSequenceData::GetFrameView(
	const CBitmapInstance *inInstance,
	Int16 &outFrame,
	Int16 &outView ) const
{
	Int16 theFrameCount = mFrameView->GetFrameCount();
	Int16 theViewCount = mFrameView->GetViewCount();
	Int16 theFrame;
	Int16 theView;
	
	for ( theFrame = 0; theFrame < theFrameCount; theFrame++ ) {
		for ( theView = 0; theView < theViewCount; theView ++ ) {
			if ( inInstance == mFrameView->GetInstance( theFrame, theView ) ) {
				outFrame = theFrame;
				outView = theView;
				return true;
			}
		}
	}
	
	return false;
}


// =================================================================================
//	 GetFrameView												  [const, protected]
// =================================================================================

Boolean
CSequenceData::GetFrameView( const CShadowMap *inShadowMap, Int16 &outFrame ) const
{
	Int16 theFrameCount = mFrameView->GetFrameCount();
	Int16 theFrame;
	
	for ( theFrame = 0; theFrame < theFrameCount; theFrame++ ) {
		if ( inShadowMap == mFrameView->GetShadowMap( theFrame ) ) {
			outFrame = theFrame;
			return true;
		}
	}
	
	return false;
}


// =================================================================================
//	 GetFrameView												  [const, protected]
// =================================================================================

Boolean
CSequenceData::GetFrameView(
	const CBitmap *inBitmap,
	Int16 &outFrame,
	Int16 &outView ) const
{
	Int16 theFrameCount = mFrameView->GetFrameCount();
	Int16 theViewCount = mFrameView->GetViewCount();
	Int16 theFrame;
	Int16 theView;
	
	for ( theFrame = 0; theFrame < theFrameCount; theFrame++ ) {
		for ( theView = 0; theView < theViewCount; theView ++ ) {
			if ( inBitmap == GetBitmap( theFrame, theView ) ) {
				outFrame = theFrame;
				outView = theView;
				return true;
			}
		}
	}
	
	return false;
}


#pragma mark -

// =================================================================================
//	 AssignBitmapInstances												 [protected]
// =================================================================================

Boolean
CSequenceData::AssignBitmapInstances( Byte *inBytes, Int32 inCount, LArray &inBitmaps )
{
	return mFrameView->AssignBitmapInstances( inBytes, inCount, inBitmaps );
}


// =================================================================================
//	 AssignShadowMaps													 [protected]
// =================================================================================

Boolean
CSequenceData::AssignShadowMaps( Byte *inBytes, Int32 inCount, LArray &inBitmaps )
{
	return mFrameView->AssignShadowMaps( inBytes, inCount, inBitmaps );
}


// =================================================================================
//	 DeclareElements													 [protected]
// =================================================================================

void
CSequenceData::DeclareElements(
	LArray &inInstances,
	LArray &inShadowMaps,
	LArray &inBitmaps )
{
	mFrameView->DeclareElements( inInstances, inShadowMaps, inBitmaps );
}


#pragma mark -

// =================================================================================
//	 CFrameView															[public]
// =================================================================================

CFrameView::CFrameView( void )
{
	mFrameH = (SFrame **)::NewHandle(0);
	mFrameCount = 0;
	mViewCount = 0;
}


// =================================================================================
//	 ~CFrameView															[public]
// =================================================================================

CFrameView::~CFrameView( void )
{
	// Delete the bitmap instances and shadow maps
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		CShadowMap *theShadowMap = GetShadowMap( i );
		delete theShadowMap;
		
		for ( Int32 j = 0; j < mViewCount; j++ ) {
			CBitmapInstance *theInstance = GetInstance( i, j );
			delete theInstance;
		}
	}
	
	// Dispose of the frame handle
	::DisposeHandle( (Handle)mFrameH );
}


// =================================================================================
//	 SetFrameViewCount														[public]
// =================================================================================
// Can throw mem errors

void
CFrameView::SetFrameViewCount( Int16 inFrameCount, Int16 inViewCount, LListener *inListener )
{
	if ( inFrameCount < 0 )
		inFrameCount = 0;
	
	if ( inViewCount < 0 )
		inViewCount = 0;
	
	// Delete the old instances and shadow maps
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		CShadowMap *theShadowMap = GetShadowMap( i );
		delete theShadowMap;
		
		for ( Int32 j = 0; j < mViewCount; j++ ) {
			CBitmapInstance *theInstance = GetInstance( i, j );
			delete theInstance;
		}
	}
	
	mFrameCount = inFrameCount;
	mViewCount = inViewCount;
	
	::SetHandleSize( (Handle)mFrameH, mFrameCount*(sizeof(SFrame) + mViewCount*sizeof(SView)) );
	ThrowIfMemError_();
	
	// Clear the frames/views
	StHandleLocker theLock( (Handle)mFrameH );
	
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		SFrame *theFrame = GetFrame( i );
		
		theFrame->mShadowIndex = -1;
		theFrame->mShadowMap = new CShadowMap;
		theFrame->mShadowMap->AddListener( inListener );
		
		SView *theViews = theFrame->mViews;
		
		for ( Int32 j = 0; j < mViewCount; j++ ) {
			theViews[j].mIndex = -1;
			theViews[j].mInstance = new CBitmapInstance;
			theViews[j].mInstance->AddListener( inListener );
		}
	}
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================
// Assumes that mFrameCount, mViewCount and particularly mFrameH are initialized
// to the correct number of frames and views.

Int32
CFrameView::ReadData( const Byte *inData )
{
	StHandleLocker theLock( (Handle)mFrameH );
	SFrame *theFrame;
	
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		theFrame = GetFrame( i );
		
		inData += ::CopyBytes( inData, &theFrame->mShadowIndex, 2 );
		inData += 44;
		
		SView *theViews = theFrame->mViews;
		
		for ( Int32 j = 0; j < mViewCount; j++ )
			inData += ::CopyBytes( inData, &theViews[j].mIndex, 2 );
	}
	
	return GetSize();
}


// =================================================================================
//	 WriteData																[public]
// =================================================================================

Int32
CFrameView::WriteData( Byte *outData )
{
	StHandleLocker theLock( (Handle)mFrameH );
	const SFrame *theFrame;
	
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		theFrame = GetFrame( i );
		
		outData += ::CopyBytes( &theFrame->mShadowIndex, outData, 2 );
		
		for ( Int32 i = 0; i < 44; i++ )
			*outData++ = 0;
		
		const SView *theViews = theFrame->mViews;
		
		for ( Int32 j = 0; j < mViewCount; j++ )
			outData += ::CopyBytes( &theViews[j].mIndex, outData, 2 );
	}
	
	return GetSize();
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================

Int32
CFrameView::GetSize( void ) const
{
	return mFrameCount*(46 + 2*mViewCount);
}


#pragma mark -

// =================================================================================
//	 GetFrameCount													 [const, public]
// =================================================================================

Int16
CFrameView::GetFrameCount( void ) const
{
	return mFrameCount;
}


// =================================================================================
//	 SetFrameCount															[public]
// =================================================================================
// Can throw mem errors

void
CFrameView::SetFrameCount( Int16 inCount, LListener *inListener )
{
	Int16 theOldCount = mFrameCount;
	
	if ( inCount < 0 )
		inCount = 0;
	
	// Delete removed instances and shadows
	if ( inCount < mFrameCount ) {
		StHandleLocker theLock( (Handle)mFrameH );
		
		for ( Int32 i = inCount; i < mFrameCount; i++ ) {
			SFrame *theFrame = GetFrame( i );
			delete theFrame->mShadowMap;
			
			for ( Int32 j = 0; j < mViewCount; j++ )
				delete theFrame->mViews[j].mInstance;
		}
	}
	
	mFrameCount = inCount;
	
	::SetHandleSize( (Handle)mFrameH, mFrameCount*GetFrameSize() );
	ThrowIfMemError_();
	
	// Initialize the new frames
	StHandleLocker theLock( (Handle)mFrameH );
	
	for ( Int32 i = theOldCount; i < mFrameCount; i++ ) {
		SFrame *theFrame = GetFrame( i );
		
		theFrame->mShadowIndex = -1;
		theFrame->mShadowMap = new CShadowMap;
		theFrame->mShadowMap->AddListener( inListener );
		
		SView *theViews = theFrame->mViews;
		
		for ( Int32 j = 0; j < mViewCount; j++ ) {
			theViews[j].mIndex = -1;
			theViews[j].mInstance = new CBitmapInstance;
			theViews[j].mInstance->AddListener( inListener );
		}
	}
}


// =================================================================================
//	 GetViewCount													 [const, public]
// =================================================================================

Int16
CFrameView::GetViewCount( void ) const
{
	return mViewCount;
}


// =================================================================================
//	 SetViewCount															[public]
// =================================================================================
// Can throw mem errors

void
CFrameView::SetViewCount( Int16 inCount, LListener *inListener )
{
	if ( inCount < 0 )
		inCount = 0;
	
	if ( inCount == mViewCount )
		return;
	
	// Save the old data
	Int16 theOldCount = mViewCount;
	Int32 theOldFrameSize = GetFrameSize();
	Int32 theDataSize = mFrameCount*theOldFrameSize;
	StNewBlock theData( theDataSize );
	
	if ( mFrameCount*mViewCount != 0 ) {
		StHandleLocker theLock( (Handle)mFrameH );
		::BlockMove( *mFrameH, theData, theDataSize );
	}
		
	// Delete removed instances
	if ( inCount < mViewCount ) {
		StHandleLocker theLock( (Handle)mFrameH );
		
		for ( Int32 i = 0; i < mFrameCount; i++ ) {
			SFrame *theFrame = GetFrame( i );
			
			for ( Int32 j = inCount; j < mViewCount; j++ )
				delete theFrame->mViews[j].mInstance;
		}
	}
	
	mViewCount = inCount;
	
	// Resize the handle
	::SetHandleSize( (Handle)mFrameH, mFrameCount*(sizeof(SFrame) + mViewCount*sizeof(SView)) );
	ThrowIfMemError_();
	
	// Restore the old/clear the new frames and views
	SFrame *theOldFrame = (SFrame *)((char *)theData);
	StHandleLocker theLock( (Handle)mFrameH );
	
	Int32 theFrameSize = GetFrameSize();
	Int32 theMinSize = theFrameSize < theOldFrameSize ? theFrameSize : theOldFrameSize;
	
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		SFrame *theFrame = GetFrame( i );
		::BlockMove( theOldFrame, theFrame, theMinSize );
		
		// Add new views
		for ( Int32 j = theOldCount; j < mViewCount; j++ ) {
			theFrame->mViews[j].mInstance = new CBitmapInstance;
			theFrame->mViews[j].mInstance->AddListener( inListener );
			theFrame->mViews[j].mIndex = -1;
		}
		
		(char *)theOldFrame += theOldFrameSize;
	}
}


#pragma mark -

// =================================================================================
//	 GetInstance													 [const, public]
// =================================================================================

const CBitmapInstance *
CFrameView::GetInstance( Int16 inFrame, Int16 inView ) const
{
	ThrowIf_(inFrame < 0 || inFrame >= mFrameCount);
	ThrowIf_(inView < 0  || inView >= mViewCount);
	StHandleLocker theLock( (Handle)mFrameH );
	
	const SFrame *theFrame = GetFrame( inFrame );
	return theFrame->mViews[inView].mInstance;
}


// =================================================================================
//	 GetInstance															[public]
// =================================================================================

CBitmapInstance *
CFrameView::GetInstance( Int16 inFrame, Int16 inView )
{
	ThrowIf_(inFrame < 0 || inFrame >= mFrameCount);
	ThrowIf_(inView < 0  || inView >= mViewCount);
	StHandleLocker theLock( (Handle)mFrameH );
	
	SFrame *theFrame = GetFrame( inFrame );
	return theFrame->mViews[inView].mInstance;
}


// =================================================================================
//	 GetShadowMap													 [const, public]
// =================================================================================

const CShadowMap *
CFrameView::GetShadowMap( Int16 inFrame ) const
{
	ThrowIf_(inFrame < 0 || inFrame >= mFrameCount);
	StHandleLocker theLock( (Handle)mFrameH );
	
	const SFrame *theFrame = GetFrame( inFrame );
	return theFrame->mShadowMap;
}


// =================================================================================
//	 GetShadowMap															[public]
// =================================================================================

CShadowMap *
CFrameView::GetShadowMap( Int16 inFrame )
{
	ThrowIf_(inFrame < 0 || inFrame >= mFrameCount);
	StHandleLocker theLock( (Handle)mFrameH );
	
	SFrame *theFrame = GetFrame( inFrame );
	return theFrame->mShadowMap;
}


#pragma mark -

// =================================================================================
//	 AssignBitmapInstances													[public]
// =================================================================================

Boolean
CFrameView::AssignBitmapInstances( Byte *inBytes, Int32 inCount, LArray &inBitmaps )
{
	StHandleLocker theLock( (Handle)mFrameH );
	
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		SFrame *theFrame = GetFrame( i );
		
		for ( Int32 j = 0; j < mViewCount; j++ ) {
			SView *theView = &theFrame->mViews[j];
			
			// Read the instance data and do error checking
			if ( theView->mIndex >= inCount )
				return false;
			
			if ( theView->mIndex < 0 )
				continue;
			
			Byte *theData = inBytes + theView->mIndex*CBitmapInstance::kElementSize;
			theView->mInstance->ReadData( theData );
			
			// Assign the correct bitmap to the instance and do error checking
			Int16 theBitmapIndex = theView->mInstance->GetBitmapIndex();
			CBitmap *theBitmap;
			
			if ( theBitmapIndex < 0 )
				continue;
			
			if ( !inBitmaps.FetchItemAt( theBitmapIndex+1, &theBitmap ) )
				return false;
			
			theView->mInstance->SetBitmap( theBitmap );
		}
	}
	
	return true;
}


// =================================================================================
//	 AssignShadowMaps														[public]
// =================================================================================

Boolean
CFrameView::AssignShadowMaps( Byte *inBytes, Int32 inCount, LArray &inBitmaps )
{
	StHandleLocker theLock( (Handle)mFrameH );
	
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		SFrame *theFrame = GetFrame( i );
		
		// Read the shadow map and do error checking
		if ( theFrame->mShadowIndex >= inCount )
			return false;
		
		if ( theFrame->mShadowIndex < 0 )
			continue;
		
		Byte *theData = inBytes + theFrame->mShadowIndex*CShadowMap::kElementSize;
		theFrame->mShadowMap->ReadData( theData );
		
		// Assign the correct bitmap to the shadow map and do error checking
		Int16 theBitmapIndex = theFrame->mShadowMap->GetBitmapIndex();
		CBitmap *theBitmap;
		
		if ( theBitmapIndex < 0 )
			continue;
		
		if ( !inBitmaps.FetchItemAt( theBitmapIndex+1, &theBitmap ) )
			return false;
		
		theFrame->mShadowMap->SetBitmap( theBitmap );
	}
	
	return true;
}


// =================================================================================
//	 DeclareElements														[public]
// =================================================================================
// Append instance and shadow map pointers to the array, and note indices

void
CFrameView::DeclareElements(
	LArray &inInstances,
	LArray &inShadowMaps,
	LArray &inBitmaps )
{
	StHandleLocker theLock( (Handle)mFrameH );
	
	for ( Int32 i = 0; i < mFrameCount; i++ ) {
		SFrame *theFrame = GetFrame( i );
		CShadowMap *theShadow = theFrame->mShadowMap;
		
		// Note the shadow bitmap index
		CBitmap *theBitmap = theShadow->GetBitmap();
		Int16 theBitmapIndex = inBitmaps.FetchIndexOf( &theBitmap );
		
		if ( theBitmapIndex == LArray::index_Bad )
			theBitmapIndex = -1;
		else
			theBitmapIndex -= 1;
		
		theShadow->SetBitmapIndex( theBitmapIndex );
		
		// Note the shadow map index
		theFrame->mShadowIndex = inShadowMaps.GetCount();
		
		// Append the shadow map
		inShadowMaps.InsertItemsAt( 1, LArray::index_Last, &theShadow );
		
		// Handle bitmap instances
		for ( Int32 j = 0; j < mViewCount; j++ ) {
			SView *theView = &theFrame->mViews[j];
			CBitmapInstance *theInstance = theView->mInstance;
			
			// Note the bitmap index
			theBitmap = theInstance->GetBitmap();
			Int16 theBitmapIndex = inBitmaps.FetchIndexOf( &theBitmap );
			
			if ( theBitmapIndex == LArray::index_Bad )
				theBitmapIndex = -1;
			else
				theBitmapIndex -= 1;
			
			theInstance->SetBitmapIndex( theBitmapIndex );
			
			// Note the instance index
			theView->mIndex = inInstances.GetCount();
			
			// Append the instance
			inInstances.InsertItemsAt( 1, LArray::index_Last, &theInstance );
		}
	}
}


#pragma mark -

// =================================================================================
//	 GetFrame														[const, private]
// =================================================================================
// NOTE: mFrameH should probably be locked before this function is called
// Returns a valid SFrame pointer unless an exception is thrown

const CFrameView::SFrame *
CFrameView::GetFrame( Int16 inFrame ) const
{
	ThrowIf_(inFrame < 0 || inFrame >= mFrameCount);
	return (SFrame *)(*(Handle)mFrameH + sizeof(SFrame) + mViewCount*sizeof(SView));
}


// =================================================================================
//	 GetFrame															   [private]
// =================================================================================
// NOTE: mFrameH should probably be locked before this function is called
// Returns a valid SFrame pointer unless an exception is thrown

CFrameView::SFrame *
CFrameView::GetFrame( Int16 inFrame )
{
	ThrowIf_(inFrame < 0 || inFrame >= mFrameCount);
	return (SFrame *)(*(Handle)mFrameH + inFrame*GetFrameSize());
}


// =================================================================================
//	 GetFrameSize													[const, private]
// =================================================================================

Int32
CFrameView::GetFrameSize( void ) const
{
	return sizeof(SFrame) + mViewCount*sizeof(SView);
}


