// =================================================================================
//	CSequenceReference.cp		 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CSequenceReference.h"
#include "CopyBytes.h"

#include "256Messages.h"
#include "CSequenceData.h"


// =================================================================================
//	 CSequenceReference													[public]
// =================================================================================
					
CSequenceReference::CSequenceReference( void )
{
	for ( int i = 0; i < 64; i++ )
		mName[i] = 0;
	
	mDataOffset	= 0;
	mDataSize	= 0;
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================
// inSequenceData is a buffer containing the sequence data as they are
// stored in the .gor files and plugin files.  Zeroes out the mem based field.
					
void
CSequenceReference::ReadData( const Byte *inSequenceData )
{
	inSequenceData += ::CopyBytes( inSequenceData, mName, 64 );
	inSequenceData += ::CopyBytes( inSequenceData, &mDataOffset, 4 );
	inSequenceData += ::CopyBytes( inSequenceData, &mDataSize, 4 );
	inSequenceData += 4;

	inSequenceData += 52;
}
	

// =================================================================================
//	 WriteData														 [const, public]
// =================================================================================
// outSequenceData is filled with the sequence data as they are stored on file
					
void
CSequenceReference::WriteData( Byte *outSequenceData ) const
{
	const Int32 theZero = 0;
	
	outSequenceData += ::CopyBytes( mName, outSequenceData, 64 );
	outSequenceData += ::CopyBytes( &mDataOffset, outSequenceData, 4 );
	outSequenceData += ::CopyBytes( &mDataSize, outSequenceData, 4 );
	outSequenceData += ::CopyBytes( &theZero, outSequenceData, 4 );
	
	for ( int i = 0; i < 52; i++ )
		*outSequenceData++ = 0;

}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================
// Return the bitmap group entry size in bytes

Int32
CSequenceReference::GetSize( void ) const
{
	return 128;
}


#pragma mark -

// =================================================================================
//	 GetName														 [const, public]
// =================================================================================

const char *
CSequenceReference::GetName( void ) const
{
	return mName;
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CSequenceReference::SetName( const char *inName )
{
	// Don't update if they're identical
	if ( LStr255(mName) == LStr255(inName) )
		return;
	
	// Copy the new name
	Int32 i = 0;
	
	for ( ; i < 64 && inName[i] != 0; i++ )
		mName[i] = inName[i];
	
	// Clear the rest of the name field
	for ( ; i < 64; i++ )
		mName[i] = 0;
	
	BroadcastMessage( msg_256UpdateSequenceReference, nil );
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CSequenceReference::SetName( ConstStr255Param inName )
{
	// Don't update if they're identical
	if ( LStr255(mName) == LStr255(inName) )
		return;
	
	// Get the length, truncate if it's too long
	Int32 theLength = inName[0];
	
	if ( theLength >= 64 )
		theLength = 63;
	
	// Copy the new name
	::BlockMove( &inName[1], mName, theLength );
	
	// Clear the rest of the name field
	for ( Int32 i = theLength; i < 64; i++ )
		mName[i] = 0;
	
	BroadcastMessage( msg_256UpdateSequenceReference, nil );
}


// =================================================================================
//	 GetDataOffset													 [const, public]
// =================================================================================

Int32
CSequenceReference::GetDataOffset( void ) const
{
	return mDataOffset;
}


// =================================================================================
//	 SetDataOffset															[public]
// =================================================================================

void
CSequenceReference::SetDataOffset( Int32 inDataOffset )
{
	if ( mDataOffset != inDataOffset ) {
		mDataOffset = inDataOffset;
//		BroadcastMessage( msg_256UpdateSequenceReference, nil );
	}
}


// =================================================================================
//	 GetDataSize													 [const, public]
// =================================================================================

Int32
CSequenceReference::GetDataSize( void ) const
{
	return mDataSize;
}


// =================================================================================
//	 SetDataSize															[public]
// =================================================================================

void
CSequenceReference::SetDataSize( Int32 inDataSize )
{
	if ( mDataSize != inDataSize ) {
		mDataSize = inDataSize;
//		BroadcastMessage( msg_256UpdateSequenceReference, nil );
	}
}


// =================================================================================
//	 UpdateFromData														[public]
// =================================================================================

void
CSequenceReference::UpdateFromData( const CSequenceData *inData )
{
	mDataSize = inData->GetSize();
}


