// =================================================================================
//	CShadowMap.cp				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CShadowMap.h"
#include "CopyBytes.h"

#include "AmberMessages.h"
#include "256Messages.h"
#include "CBitmap.h"


// =================================================================================
//	 CShadowMap															[public]
// =================================================================================
					
CShadowMap::CShadowMap( void )
{
	mFlags			= 0;
	mRegPointX		= 0;
	mRegPointY		= 0;
	mBitmapIndex	= -1;
	
	mBitmap = nil;
}


// =================================================================================
//	 ReadData																[public]
// =================================================================================
// inShadowMapData is a buffer containing the shadow map data as they
// are stored in the .gor files and plugin files.

void
CShadowMap::ReadData( Byte *inShadowMapData )
{
	inShadowMapData += ::CopyBytes( inShadowMapData, &mFlags, 4 );
	inShadowMapData += ::CopyBytes( inShadowMapData, &mRegPointX, 4 );
	inShadowMapData += ::CopyBytes( inShadowMapData, &mRegPointY, 4 );
	inShadowMapData += ::CopyBytes( inShadowMapData, &mBitmapIndex, 2 );
}
	

// =================================================================================
//	 WriteData														 [const, public]
// =================================================================================
// outShadowMapData is filled with the shadow map data as they are stored on
// file
					
void
CShadowMap::WriteData( Byte *outShadowMapData ) const
{
	outShadowMapData += ::CopyBytes( &mFlags, outShadowMapData, 4 );
	outShadowMapData += ::CopyBytes( &mRegPointX, outShadowMapData, 4 );
	outShadowMapData += ::CopyBytes( &mRegPointY, outShadowMapData, 4 );
	outShadowMapData += ::CopyBytes( &mBitmapIndex, outShadowMapData, 2 );
	
	for ( Int32 i = 0; i < 18; i++ )
		*outShadowMapData++ = 0;
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================
// Return the GodKnowsWhat entry size in bytes

Int32
CShadowMap::GetSize( void ) const
{
	return 32;
}


#pragma mark -

// =================================================================================
//	 ListenToMessage														[public]
// =================================================================================

void
CShadowMap::ListenToMessage( MessageT inMessage, void *ioData )
{
	switch ( inMessage ) {
		case msg_TagDeleteElement:
			if ( ioData == mBitmap )
				SetBitmap( nil );
			break;
	}
}


#pragma mark -

// =================================================================================
//	 GetBitmap														 [const, public]
// =================================================================================

const CBitmap *
CShadowMap::GetBitmap( void ) const
{
	return mBitmap;
}


// =================================================================================
//	 GetBitmap																[public]
// =================================================================================

CBitmap *
CShadowMap::GetBitmap( void )
{
	return mBitmap;
}


// =================================================================================
//	 GetFlags														 [const, public]
// =================================================================================

Uint32
CShadowMap::GetFlags( void ) const
{
	return mFlags;
}


// =================================================================================
//	 SetFlags																[public]
// =================================================================================

void
CShadowMap::SetFlags( Uint32 inFlags )
{
	if ( mFlags != inFlags ) {
		mFlags = inFlags;
		BroadcastMessage( msg_256UpdateShadowMap, this );
	}
}


// =================================================================================
//	 GetRegPointX													 [const, public]
// =================================================================================

Int32
CShadowMap::GetRegPointX( void ) const
{
	return mRegPointX;
}


// =================================================================================
//	 SetRegPointX															[public]
// =================================================================================

void
CShadowMap::SetRegPointX( Int32 inRegPointX )
{
	if ( mRegPointX != inRegPointX ) {
		mRegPointX = inRegPointX;
		BroadcastMessage( msg_256UpdateShadowMap, this );
	}
}


// =================================================================================
//	 GetRegPointY													 [const, public]
// =================================================================================

Int32
CShadowMap::GetRegPointY( void ) const
{
	return mRegPointY;
}


// =================================================================================
//	 SetRegPointX															[public]
// =================================================================================

void
CShadowMap::SetRegPointY( Int32 inRegPointY )
{
	if ( mRegPointY != inRegPointY ) {
		mRegPointY = inRegPointY;
		BroadcastMessage( msg_256UpdateShadowMap, this );
	}
}


#pragma mark -

// =================================================================================
//	 SetBitmap															 [protected]
// =================================================================================

void
CShadowMap::SetBitmap( CBitmap *inBitmap )
{
	if ( mBitmap == inBitmap )
		return;
	
	if ( mBitmap != nil )
		mBitmap->RemoveListener( this );
	
	mBitmap = inBitmap;
	
	if ( mBitmap != nil )
		mBitmap->AddListener( this );
	
//	BroadcastMessage( msg_256UpdateShadowMap, this );
}


// =================================================================================
//	 GetBitmapIndex											  [const, protected]
// =================================================================================
// The bitmap index is only valid right after a read operation and right before
// a write operation (when the tag/sequence data has updated it).

Int16
CShadowMap::GetBitmapIndex( void ) const
{
	return mBitmapIndex;
}


// =================================================================================
//	 SetBitmapIndex													 [protected]
// =================================================================================
// Does not send an update message, since it should only be used by the sequence
// data and the bitmap index isn't interesting anywhere else.

void
CShadowMap::SetBitmapIndex( Int16 inIndex )
{
	mBitmapIndex = inIndex;
}


