// =================================================================================
//	CShadowMap.h				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#pragma once

#include "CTagElement.h"
#include <LListener.h>
class CBitmap;


// =================================================================================
//	CShadowMap															  [32 bytes]
// =================================================================================
// CShadowMaps are references to the bitmaps used for shadows (for instance for
// unit sprites and scenery), and contains information about the shadow.

class CShadowMap : public CTagElement, public LListener {
friend				class CSequenceData;
friend				class CFrameView;

public:
	enum			{	// Flags
						flag_ShadowIsLight		= 0x01,
						flag_FlipHorizontal		= 0x02,
						flag_FlipVertical		= 0x04,
						
						// Constant element size
						kElementSize			= 32
					};
					
					// Default constructor
					CShadowMap();
	
					// inShadowMapData is a buffer containing the shadow map
					// data as they are stored in the .gor files and plugin
					// files.
					
	void			ReadData( Byte *inShadowMapData );
	
					// outShadowMapData is filled with the shadow map
					// data as they are stored on file
					
	void			WriteData( Byte *outShadowMapData ) const;
	
					// Return the shadow map size on file
					
	Int32			GetSize() const;
	
				// LListener
	
	virtual void	ListenToMessage( MessageT inMessage, void *ioData );
	
				// Accessors

	const CBitmap	*GetBitmap() const;
	CBitmap			*GetBitmap();
	
	Uint32			GetFlags() const;
	void			SetFlags( Uint32 inFlags );
	
	Int32			GetRegPointX() const;
	void			SetRegPointX( Int32 inRegPointX );
	
	Int32			GetRegPointY() const;
	void			SetRegPointY( Int32 inRegPointY );

protected:
	void			SetBitmap( CBitmap *inBitmap );
	Int16			GetBitmapIndex() const;
	void			SetBitmapIndex( Int16 inIndex );
	
private:
					// Defensive programming
					CShadowMap( const CShadowMap &inShadowMap );
	CShadowMap		&operator=( const CShadowMap &inShadowMap );
	
	Uint32	mFlags;			// Shadow map flags (flip and light)
	Int32	mRegPointX;		// From top, left pixel, matched with the
	Int32	mRegPointY;		// registration point of the frame.
	Int16	mBitmapIndex;	// Bitmap index for shadow bitmap
//	Int16	mUnused;
//	Byte	mExtractorData[16];	// Usually zero?

	CBitmap *mBitmap;
};

