// =================================================================================
//	CTagElement.h				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================
// Implements basic behavior for a part of a Myth tag
// - Support for broadcasting a message when the element is deleted to allow objects
//   that rely on the element to take appropriate action.
// - Supports an "update" mode where outbound messages are stored
//   and sent as a batch when the element leaves update mode.
//
// This class also marks a definitive end to the illusion of keeping the tag
// structure classes platform independent.
//
// File created November 14, 1998

#pragma once

#include <LBroadcaster.h>

class LArray;


// =================================================================================
//	 CTagElement
// =================================================================================

class CTagElement : public LBroadcaster {
public:
						CTagElement();
	virtual				~CTagElement();
	
	void				BroadcastMessage(
							MessageT inMessage,
							void *ioParam = nil);
	
	void				BeginUpdate();
	void				EndUpdate();
	
private:
						// Defensive programming
						CTagElement( const CTagElement & );
	CTagElement			&operator=( const CTagElement & );
	
	Int32				mUpdating;
	LArray				*mMessages;
 	
};


// =================================================================================
//	 StElementUpdater
// =================================================================================
// Helper class to make updating easier

class StElementUpdater {
public:
						StElementUpdater( CTagElement *inElement );
	virtual				~StElementUpdater();

private:
						StElementUpdater();
	CTagElement			*mElement;
};