// =================================================================================
//	GorHeader.cp				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#pragma mark Change Log

/*	Change Log:
	
	Sept 15 1998:
		 File created.  Pythos posted a struct for the .gor header, pasted
		  this in here as a base for development.
*/


#pragma mark -
#pragma mark Headers and Declarations
#pragma mark -

// =================================================================================
//	Headers
// =================================================================================

#include "GorHeader.h"
#include "CopyBytes.h"


// =================================================================================
//	 GorHeader( Byte * )
// =================================================================================
// inGorHeaderData is a buffer containing the gor data as they are stored on file.
// Only the first 128 bytes (the header) are read.

GorHeader::GorHeader( Byte *inGorHeaderData )
{
	ReadData( inGorHeaderData );
}


// =================================================================================
//	 ReadData
// =================================================================================
// inGorHeaderData is a buffer containing the header data as they are stored in the
// file.
					
void
GorHeader::ReadData( Byte *inGorHeaderData )
{
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &version, 2 );
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &data_version, 2 );
	inGorHeaderData += ::CopyBytes( inGorHeaderData, description, 32 );
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &checksum, 4 );
	
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &headers_offset, 4 );
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &headers_count, 2 );
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &headers_size, 2 );
	
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &flags, 2 );
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &pad, 2 );
	inGorHeaderData += ::CopyBytes( inGorHeaderData, &mod_time, 4 );
	inGorHeaderData += ::CopyBytes( inGorHeaderData, unused, 36 );
}


// =================================================================================
//	 WriteData
// =================================================================================
// outGorHeaderData is filled with the 128 bytes of the header data as they are
// stored on file
					
void
GorHeader::WriteData( Byte *outGorHeaderData )
{
	outGorHeaderData += ::CopyBytes( &version, outGorHeaderData, 2 );
	outGorHeaderData += ::CopyBytes( &data_version, outGorHeaderData, 2 );
	outGorHeaderData += ::CopyBytes( description, outGorHeaderData, 32 );
	outGorHeaderData += ::CopyBytes( &checksum, outGorHeaderData, 4 );
	
	outGorHeaderData += ::CopyBytes( &headers_offset, outGorHeaderData, 4 );
	outGorHeaderData += ::CopyBytes( &headers_count, outGorHeaderData, 2 );
	outGorHeaderData += ::CopyBytes( &headers_size, outGorHeaderData, 2 );
	
	outGorHeaderData += ::CopyBytes( &flags, outGorHeaderData, 2 );
	outGorHeaderData += ::CopyBytes( &pad, outGorHeaderData, 2 );
	outGorHeaderData += ::CopyBytes( &mod_time, outGorHeaderData, 4 );
	outGorHeaderData += ::CopyBytes( unused, outGorHeaderData, 36 );
}


// =================================================================================
//	 GetLength
// =================================================================================
// Return the header data length in bytes

Int32
GorHeader::GetLength( void )
{
	return 128;
}
