// =================================================================================
//	GorHeader.h					 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================


#pragma mark Change Log

/*	Change Log:
	
	Oct  20 1998:
		 Changed char, short and long (and unsigned versions) to Byte,
		  Int16 and Int32 (Uint16, Uint32)
	Sept 29 1998:
		 Added comment on the flags variable from the FutureMyth forum
		  (Steve Ramsey)
	Sept 15 1998:
		 File created.  Pythos posted a struct for the .gor header, pasted
		  this in here as a base for development.
*/

#pragma mark Pyth's comments

// The typical .gor file is made up of three sections:
//
//   - the 128 byte .gor header
//   - the actual tag data
//   - a list of tag headers
//
// The tag headers are nearly identical to the ones in a so called plugin file,
// the only difference being that the offset changes from 0x40 to the address
// within the .gor file. One thing to note is that Local.gor contains headers only,
// there's no data. In such a case, the flags variable will be set to 0x1,
// otherwise, it'll be zero.
//
// Anyways, I used this struct in my PC apps, but it should work on anything.
// Just make sure your compiler is byte packing stuff.
// -Pyth

// Steve Ramsey:
// Hey, Pyth, you got that backwards. .gor files with
// tag data in them (as opposed to index files) have their
// flag field set to 0x0001. It's the index files (such as
// local.gor) that have their flag field set to 0x0000. 


#pragma mark class GorHeader

class GorHeader {
public:
					// in_GorHeaderData is a buffer containing the header data
					// as they are stored on file.
					
					GorHeader( Byte *in_GorHeaderData );
	void			ReadData( Byte *in_GorHeaderData );
	
					// out_GorHeaderData is filled with the 128 bytes of the
					// header data as they are stored on file
					
	void			WriteData( Byte *out_GorHeaderData );
	
					// Return the header data length in bytes
					
	long			GetLength();

public:
	Uint16	version;
	Uint16	data_version;
	char	description[32];
	Uint16	checksum;
	
	Int32	headers_offset;
	Uint16	headers_count;
	Uint16	headers_size;		// Size of individual header (0x40) ?
	
	Uint16	flags;				// 0x0000 for headers only, 0x0001 otherwise
	Uint16	pad;
	Uint32	mod_time;
	Uint16	unused[36];
};
