// =================================================================================
//	CTag.cp						 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================

#include "CTag.h"
#include "CTagHeader.h"
#include "AmberMessages.h"


// =================================================================================
//	 CTag
// =================================================================================

CTag::CTag( void )
{
	mTagHeader = new CTagHeader();
	mDirty = false;
}


// =================================================================================
//	 ~CTag
// =================================================================================

CTag::~CTag( void )
{
	delete mTagHeader;
}


#pragma mark -

// =================================================================================
//	 ReadTagHeader
// =================================================================================
// inTagData is a buffer containing the tag data as they are stored in the .gor
// files and plugin files.  Only reads the tag header.

void
CTag::ReadTagHeader( LStream &inStream )
{
	mTagHeader->ReadStream( inStream );

	if ( !mTagHeader->IsValidTag() )
		Throw_( err_WrongTagFormat );

	BroadcastMessage( msg_UpdateTagHeader, nil );
}


// =================================================================================
//	 WriteTagHeader
// =================================================================================

Int32
CTag::WriteTagHeader( LStream &outStream )
{
	UpdateTagHeader();
	return mTagHeader->WriteStream( outStream );
}


// =================================================================================
//	 GetTagHeaderSize
// =================================================================================

Int32
CTag::GetTagHeaderSize( void ) const
{
	if ( mTagHeader == nil )
		Throw_( err_HasNoTagHeader );
	
	return mTagHeader->GetSize();
}


#pragma mark -

// =================================================================================
//	 GetTagHeader
// =================================================================================
// Return a pointer to the tag header.

const CTagHeader *
CTag::GetTagHeader( void ) const
{
	return mTagHeader;
}


// =================================================================================
//	 GetTagHeader
// =================================================================================
// Return a pointer to the tag header.

CTagHeader *
CTag::GetTagHeader( void )
{
	return mTagHeader;
}


// =================================================================================
//	 GetTagGroup
// =================================================================================
// Return the tag type of this tag

OSType
CTag::GetTagGroup( void ) const
{
	if ( mTagHeader != nil )
		return mTagHeader->GetTagGroup();
	
	return '????';
}


#pragma mark -

// =================================================================================
//	 GetFileCreator
// =================================================================================

OSType
CTag::GetFileCreator( void ) const
{
	if ( mTagHeader != nil ) {
		if ( mTagHeader->GetMythCode() == 'myth' )
			return 'Myth';
		
		return 'M2SB';
	}
	
	return '????';
}


// =================================================================================
//	 GetFileType
// =================================================================================

OSType
CTag::GetFileType( void ) const
{
	if ( mTagHeader != nil ) {
		if ( mTagHeader->GetMythCode() == 'myth' )
			mTagHeader->GetTagGroup();
		
		return 'File';
	}
	
	return '????';
}


#pragma mark -

// =================================================================================
//	 IsDirty
// =================================================================================
// Return whether the tag has been changed

Boolean
CTag::IsDirty( void ) const
{
	return mDirty;
}


// =================================================================================
//	 SetDirty
// =================================================================================
// Set dirty state

void
CTag::SetDirty( Boolean inDirty )
{
	if ( inDirty != mDirty ) {
		mDirty = inDirty;
		BroadcastMessage( msg_UpdateTagDirtyState, &inDirty );
	}
}


// =================================================================================
//	 UpdateTagHeader
// =================================================================================
// Override in subclasses to syncronize the header with the tag

void
CTag::UpdateTagHeader( void )
{
}


