// =================================================================================
//	CTagHeader.cp				 1998 Hans Hvidsten Birkeland. All rights reserved.
// =================================================================================
// File created Jun 17, 1998

#include "CTagHeader.h"


// =================================================================================
//	Constants																 [local]
// =================================================================================

const Int32		kMythCodeOffset		= 0x3C;


// =================================================================================
//	 CTagHeader															[public]
// =================================================================================

CTagHeader::CTagHeader( void )
{
	#pragma verify that these are sensible default settings...
	
	for ( Int32 i = 0; i < 32; i++ )
		mName[i] = 0;
	
	mTagGroup		= 0;
	mTagID			= 'vodi';
	mVersionM1		= 4;
	mTagVersionM1	= 0;
	mFlagsM1		= 0;
	mTagOffset		= 0x40;
	mTagSize		= 0;
	mUserData		= 0;
	mMythCode		= 'mth2';
	
	#pragma let the user edit foundation?
//	mIdentifierM2	= 0;
//	mFlagsM2		= 0;
//	mTypeM2			= 0;
	mVersionM2		= 4;
	mFoundationM2	= foundation_LargeInstall;
//	mOwnerIndexM2	= -1;
}


// =================================================================================
//	 ReadStream															[public]
// =================================================================================

Int32
CTagHeader::ReadStream( LStream &inStream )
{
	Int32 theMythCode;
	
	inStream.SetMarker( kMythCodeOffset, streamFrom_Start );
	inStream.ReadBlock( &theMythCode, 4 );
	inStream.SetMarker( 0, streamFrom_Start );
	
	if ( theMythCode == 'myth' ) {
		inStream.ReadBlock( mName, 32 );
		inStream.ReadBlock( &mTagGroup, 4 );
		inStream.ReadBlock( &mTagID, 4 );
		inStream.ReadBlock( &mVersionM1, 4 );
		inStream.ReadBlock( &mTagVersionM1, 2 );
		inStream.ReadBlock( &mFlagsM1, 2 );
		inStream.ReadBlock( &mTagOffset, 4 );
		inStream.ReadBlock( &mTagSize, 4 );
		inStream.ReadBlock( &mUserData, 4 );
		inStream.ReadBlock( &mMythCode, 4 );
		
		// Set m2 fields to match
		mVersionM2 = mVersionM1;
		mFoundationM2 = foundation_LargeInstall;
		
	} else if ( theMythCode == 'mth2' ) {
//		inStream.ReadBlock( &mIdentifierM2, 2 );
//		inStream.ReadBlock( &mFlagsM2, 1 );
//		inStream.ReadBlock( &mTypeM2, 1 );
		inStream.SetMarker( 4, streamFrom_Marker );
		inStream.ReadBlock( mName, 32 );
		inStream.ReadBlock( &mTagGroup, 4 );
		inStream.ReadBlock( &mTagID, 4 );
		inStream.ReadBlock( &mTagOffset, 4 );
		inStream.ReadBlock( &mTagSize, 4 );
		inStream.ReadBlock( &mUserData, 4 );
		inStream.ReadBlock( &mVersionM2, 2 );
		inStream.ReadBlock( &mFoundationM2, 1 );
//		inStream.ReadBlock( &mOwnerIndexM2, 1 );
		inStream.SetMarker( 1, streamFrom_Marker );
		inStream.ReadBlock( &mMythCode, 4 );
		
		// Set m1 fields to match
		mVersionM1 = mVersionM2;
		mTagVersionM1 = 0;
		mFlagsM1 = 0;
		
	} else {
		Throw_(err_BadMythVersion);
	}
	
	return GetSize();
}


// =================================================================================
//	 WriteStream															[public]
// =================================================================================

Int32
CTagHeader::WriteStream( LStream &outStream )
{
	if ( mMythCode == 'myth' ) {
		// Increment tag version
		++mTagVersionM1;
		
		outStream.WriteBlock( mName, 32 );
		outStream.WriteBlock( &mTagGroup, 4 );
		outStream.WriteBlock( &mTagID, 4 );
		outStream.WriteBlock( &mVersionM1, 4 );
		outStream.WriteBlock( &mTagVersionM1, 2 );
		outStream.WriteBlock( &mFlagsM1, 2 );
		outStream.WriteBlock( &mTagOffset, 4 );
		outStream.WriteBlock( &mTagSize, 4 );
		outStream.WriteBlock( &mUserData, 4 );
		outStream.WriteBlock( &mMythCode, 4 );
	
	} else if ( mMythCode == 'mth2' ) {
		Int32 theZero = 0;
		Int8 theNone = -1;
		
//		outStream.WriteBlock( &mIdentifierM2, outHeaderData, 2 );
//		outStream.WriteBlock( &mFlagsM2, outHeaderData, 1 );
//		outStream.WriteBlock( &mTypeM2, outHeaderData, 1 );
		outStream.WriteBlock( &theZero, 4 );
		outStream.WriteBlock( mName, 32 );
		outStream.WriteBlock( &mTagGroup, 4 );
		outStream.WriteBlock( &mTagID, 4 );
		outStream.WriteBlock( &mTagOffset, 4 );
		outStream.WriteBlock( &mTagSize, 4 );
		outStream.WriteBlock( &mUserData, 4 );
		outStream.WriteBlock( &mVersionM2, 2 );
		outStream.WriteBlock( &mFoundationM2, 1 );
//		outStream.WriteBlock( &mOwnerIndexM2, 1 );
		outStream.WriteBlock( &theNone, 1 );
		outStream.WriteBlock( &mMythCode, 4 );
	
	} else {
		Throw_(err_BadMythVersion);
	}
	
	return GetSize();
}


// =================================================================================
//	 IsValidTag													 [const, public]
// =================================================================================

Boolean
CTagHeader::IsValidTag( Int32 inTagGroup ) const
{
	if ( inTagGroup != 0 && mTagID != inTagGroup )
		return false;
	
	return (mMythCode == 'myth' | mMythCode == 'mth2');
}


// =================================================================================
//	 GetSize														 [const, public]
// =================================================================================

Int32
CTagHeader::GetSize( void ) const
{
	return 0x40;
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CTagHeader::SetName( const StringPtr inName )
{
	::BlockMove( &inName[1], mName, inName[0] );
	mName[inName[0]] = 0;
}


// =================================================================================
//	 SetName																[public]
// =================================================================================

void
CTagHeader::SetName( const char *inName )
{
	Int32 i = 0;
	
	do mName[i] = inName[i];
	while ( i < 32 && inName[i++] != 0 );
}


// =================================================================================
//	 GetName														 [const, public]
// =================================================================================

const char *
CTagHeader::GetName( void ) const
{
	return mName;
}


// =================================================================================
//	 SetTagGroup															[public]
// =================================================================================

void
CTagHeader::SetTagGroup( Int32 inTagGroup )
{
	mTagGroup = inTagGroup;
}


// =================================================================================
//	 GetMythCode													 [const, public]
// =================================================================================

Int32
CTagHeader::GetMythCode( void ) const
{
	return mMythCode;
}


// =================================================================================
//	 SetMythCode															[public]
// =================================================================================

void
CTagHeader::SetMythCode( Int32 inMythCode )
{
	Assert_(inMythCode == 'myth' || inMythCode == 'mth2');
	mMythCode = inMythCode;
}


// =================================================================================
//	 GetTagGroup													 [const, public]
// =================================================================================

Int32
CTagHeader::GetTagGroup( void ) const
{
	return mTagGroup;
}


// =================================================================================
//	 SetTagID																[public]
// =================================================================================

void
CTagHeader::SetTagID( Int32 inTagID )
{
	mTagID = inTagID;
}


// =================================================================================
//	 GetTagID														 [const, public]
// =================================================================================

Int32
CTagHeader::GetTagID( void ) const
{
	return mTagID;
}


// =================================================================================
//	 SetTagOffset															[public]
// =================================================================================

void
CTagHeader::SetTagOffset( Int32 inTagOffset )
{
	mTagOffset = inTagOffset;
}


// =================================================================================
//	 GetTagOffset													 [const, public]
// =================================================================================

Int32
CTagHeader::GetTagOffset( void ) const
{
	return mTagOffset;
}


// =================================================================================
//	 SetUserData															[public]
// =================================================================================

void
CTagHeader::SetUserData( Int32 inUserData )
{
	mUserData = inUserData;
}


// =================================================================================
//	 GetUserData													 [const, public]
// =================================================================================

Int32
CTagHeader::GetUserData( void ) const
{
	return mUserData;
}


// =================================================================================
//	 SetTagSize															[public]
// =================================================================================

void
CTagHeader::SetTagSize( Int32 inSize )
{
	mTagSize = inSize;
}


// =================================================================================
//	 GetTagSize													 [const, public]
// =================================================================================

Int32
CTagHeader::GetTagSize( void ) const
{
	return mTagSize;
}


