// =================================================================================
//	C3dmfDocument.h					1997 eli curtz. All rights reserved.
// =================================================================================

#pragma once

#include <LSingleDoc.h>
#include <LListener.h>
#include <UStandardDialogs.h>

#include "gorStuff.h"
#include "gorStructs.h"

#include "definitions.h"
#include "geometry_definitions.h"
#include "model_definitions.h"

#include <QD3DGeometry.h>
#include <PictUtils.h>
#include <LEditField.h>
#include <LBroadcaster.h>

class CgeomView;

const MessageT message_TextChanged = 'Ntfy';

class CNotifyField : public LEditField, public LBroadcaster {
public:
	enum { class_ID = FOUR_CHAR_CODE('Ntfy') };

						CNotifyField();
						CNotifyField( LStream *inStream );
	virtual				~CNotifyField() {};

	virtual void		UserChangedText();
	virtual void		DontBeTarget();
	
protected:
	Boolean		m_bChanged;
};

class C3dmfDocument : public LSingleDoc, public LListener {
public:
						C3dmfDocument( LCommander *inSuper, FSSpec *inFileSpec );
						~C3dmfDocument();

	virtual Boolean		IsModified();
	
	virtual void		DoAESave( FSSpec &inFileSpec, OSType inFileType );
	virtual void		DoSave();
	virtual void		DoRevert();
	virtual void		DoPrint();

	virtual void		ListenToMessage(MessageT inMessage, void *ioParam);
	
	virtual Boolean		ValidSurfaceData() { return ( ( mGeometry.surface_count > 0 ) && ( mGeometry.surface_count < MAXIMUM_SURFACES_PER_GEOMETRY ) ); };
	virtual void		SubmitPickSurfaces( TQ3ViewObject inView, UInt32 inInitialID = 0 );
	virtual void		SubmitDrawSurfaces( TQ3ViewObject inView, UInt32 inFlag );
	virtual void		SetSurfaceFlag( UInt32 inSurface, UInt32 inFlag );

	virtual Boolean		ValidMeshData() { return ( ( mModel.model_cell_count > 0 ) && ( mModel.model_cell_count < MAXIMUM_CELLS_PER_MODEL ) ); };
	virtual void		SubmitPickMesh( TQ3ViewObject inView, UInt32 inInitialID = 0 );
	virtual void		SubmitDrawMesh( TQ3ViewObject inView, UInt32 inFlag );
	virtual void		SetMeshFlag( UInt32 inSurface, UInt32 inFlag );
	
	virtual Boolean		ValidVertexData() { return ( ( mGeometry.vertex_count > 0 ) && ( mGeometry.vertex_count < MAXIMUM_VERTICES_PER_GEOMETRY ) ); };
	virtual void		SubmitPickVertices( TQ3ViewObject inView, UInt32 inInitialID = 0 );
	virtual void		SubmitDrawVertices( TQ3ViewObject inView, UInt32 inFlag );
	virtual void		SetVertexFlag( UInt32 inVertex, UInt32 inFlag );
	
	virtual void		SetTransparencyLevel( fixed_fraction inTransparency )
							{ mTransparency = inTransparency; };
	virtual void		ResetDependencySurface()
							{ mDependencySurface = -1; };

protected:
	CgeomView *			mGeomView;

	void				NameNewDoc();
	void				OpenFile( FSSpec &inFileSpec );
	void				SetPrintFrameSize(void);

	virtual void		DoExport();
	
	virtual void		AdjustEditFields();
	virtual void		AdjustMythVertices();

private:
	LFileStream *		mFileStream; // screw casting mFile all the time
	
	CNotifyField *		mScaleXField;
	CNotifyField *		mScaleYField;
	CNotifyField *		mScaleZField;
	
	CNotifyField *		mAlignXField;
	CNotifyField *		mAlignYField;
	CNotifyField *		mAlignZField;
	
	float				mMythScale;
	Int32				mMeshOffsetX;
	Int32				mMeshOffsetY;
	Int32				mMeshOffsetZ;
	float				mAlignmentX;
	float				mAlignmentY;
	float				mAlignmentZ;
	
	fixed_fraction		mTransparency;
	SInt16				mDependencySurface;
	
	LArray*				mDependencyArrays;

	CNotifyField *		mTagField;
	CNotifyField *		mNameField;
	
	geometry_definition mGeometry;
	model_definition	mModel;
	TQ3BoundingBox		mBoundingBox;
	TQ3Matrix4x4		mTransformMatrix;
	
	geometry_vertex		mVertices[MAXIMUM_VERTICES_PER_GEOMETRY];
	geometry_surface	mSurfaces[MAXIMUM_SURFACES_PER_GEOMETRY];
	geometry_material	mMaterials[MAXIMUM_MATERIALS_PER_GEOMETRY];
	
	model_mesh_cell_data	mCells[MAXIMUM_CELLS_PER_MODEL];
	geometry_vertex_data	mVertexData[MAXIMUM_VERTICES_PER_GEOMETRY];

	TQ3Point3D			mPoints[MAXIMUM_VERTICES_PER_GEOMETRY];
	TQ3TriangleData		mTriangles[MAXIMUM_SURFACES_PER_GEOMETRY];
	TQ3ShaderObject		mShaders[MAXIMUM_MATERIALS_PER_GEOMETRY];
	
	PictInfoID			mPictInfoID;
	LGWorld*			mGWorlds[MAXIMUM_MATERIALS_PER_GEOMETRY];
	
	model_normal		mVertexNormals[NUMBER_OF_PRECALCULATED_MODEL_NORMALS];
	
//	void				TestTriMeshObject(TQ3Object& inTriMesh);
	void				AddTriMeshObject(TQ3Object& inTriMesh);
	void				SetCorner(TQ3TriMeshData& inData, UInt16 inVertex, geometry_corner& ioCorner);

	inline void			SetTransparency(TQ3ViewObject inView, UInt16 inTriangle);
};