// ===========================================================================
// CNew3DView.cp
// ===========================================================================


#include "CNew3DView.h"

#include <QD3DGroup.h>
#include <QD3DLight.h>
#include <QD3DCamera.h>
#include <QD3DShader.h>
#include <QD3DGeometry.h>
#include <QD3DDrawContext.h>
#include <QD3DRenderer.h>
#include <QD3DView.h>
#include <QD3DMath.h>
#include <QD3DStorage.h>
#include <QD3DPick.h>
#include <QD3DTransform.h>

#include <math.h>

#include <LToggleButton.h>

// ---------------------------------------------------------------------------
// CNew3DView
// ---------------------------------------------------------------------------

CNew3DView::CNew3DView()
		  : LPane(), LCommander(), LListener()
{
	mQD3DView			= NULL;
	mQD3DGroup			= NULL;
	mQD3DPointPicker	= NULL;
	mQD3DRectPicker		= NULL;
	
	TQ3Point3D theZeroPoint = { 0.0, 0.0, 0.0 };
	Q3BoundingBox_Set( &mBoundingBox, &theZeroPoint, &theZeroPoint, kQ3True );
	
	mCurrentTool		= kToolNone;
	mScale				= 1.0;
	
	mButtons			= NULL;
}

CNew3DView::CNew3DView( LStream *inStream )
		  : LPane( inStream ), LCommander(), LListener()
{
	mQD3DView			= NULL;
	mQD3DGroup			= NULL;
	mQD3DPointPicker	= NULL;
	mQD3DRectPicker		= NULL;
	
	TQ3Point3D theZeroPoint = { 0.0, 0.0, 0.0 };
	Q3BoundingBox_Set( &mBoundingBox, &theZeroPoint, &theZeroPoint, kQ3True );
	
	mCurrentTool		= kToolNone;
	
	mButtons			= NULL;
}


// ---------------------------------------------------------------------------
// ~CNew3DView
// ---------------------------------------------------------------------------

CNew3DView::~CNew3DView()
{
	::Q3Object_Dispose( mQD3DView );
	::Q3Object_Dispose( mQD3DGroup );
	::Q3Object_Dispose( mQD3DPointPicker );
	::Q3Object_Dispose( mQD3DRectPicker );

	if ( mButtons )
	{
		delete [] mButtons;
		mButtons = NULL;
	}
}


// ---------------------------------------------------------------------------
// FinishCreateSelf
// ---------------------------------------------------------------------------

void CNew3DView::FinishCreateSelf()
{
	LPane::FinishCreateSelf();

	this->InitializeQD3DView();
	
	mButtons = new LToggleButton*[NUMBER_OF_3D_BUTTONS];
	
	mButtons[kButtonCamera]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDCamera ));
	mButtons[kButtonTruck]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDTruck ));
	mButtons[kButtonOrbit]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDOrbit ));
	mButtons[kButtonZoom]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDZoom ));
	mButtons[kButtonDolly]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDDolly ));
	mButtons[kButtonReset]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDReset ));
	
	for ( int iButton = 0; iButton < NUMBER_OF_3D_BUTTONS; iButton++ )
	{
		if ( mButtons[iButton] ) mButtons[iButton]->AddListener( this );
	}
	
	LCommander::SwitchTarget( this );
}


// ---------------------------------------------------------------------------
// InitializeQD3DView
// ---------------------------------------------------------------------------

void CNew3DView::InitializeQD3DView()
{
	TQ3Status theStatus;
	Rect thePortRect;
	Assert_( CalcPortFrameRect( thePortRect ) );

	mQD3DView = ::Q3View_New();
	ThrowIfNULL_( mQD3DView );
	
// Draw Context
	// Build the generic draw context
	TQ3DrawContextData theContextData;
	TQ3DrawContextObject theDrawContext = NULL;
	
	// Erase Color
	TQ3ColorARGB theWhiteColor;	
	::Q3ColorARGB_Set( &theWhiteColor, 1.0, 1.0, 1.0, 1.0 );
	
	// Clip Rect
	TQ3Area thePaneArea;
	TQ3Point2D theMinPanePoint;
	TQ3Point2D theMaxPanePoint;
	theMinPanePoint.x 	= thePortRect.left;
	theMinPanePoint.y 	= thePortRect.top;
	theMaxPanePoint.x 	= thePortRect.right;
	theMaxPanePoint.y 	= thePortRect.bottom;
	thePaneArea.min 	= theMinPanePoint;
	thePaneArea.max 	= theMaxPanePoint;

	// Mask
	TQ3Bitmap theNullBitmap = { NULL, 0, 0, 0, kQ3EndianBig};
	
	theContextData.clearImageMethod 	= kQ3ClearMethodWithColor;
	theContextData.clearImageColor 		= theWhiteColor;
	theContextData.pane 				= thePaneArea;
	theContextData.paneState 			= kQ3True;
	theContextData.mask 				= theNullBitmap;
	theContextData.maskState 			= kQ3False;
	theContextData.doubleBufferState 	= kQ3True;

	// Build the macintosh draw context
	TQ3MacDrawContextData theMacContextData;
	
	theMacContextData.drawContextData 	= theContextData;
	theMacContextData.window 			= NULL;
	theMacContextData.library 			= kQ3Mac2DLibraryQuickDraw;	// Do we need this for clipping?
	theMacContextData.viewPort 			= NULL;
	theMacContextData.grafPort 			= (CGrafPtr) GetMacPort();

	// Set the draw context in the view
	theDrawContext = ::Q3MacDrawContext_New( &theMacContextData );
	ThrowIfNULL_( theDrawContext );
	theStatus = ::Q3View_SetDrawContext( mQD3DView, theDrawContext );
	ThrowIf_( theStatus == kQ3Failure );
	::Q3Object_Dispose( theDrawContext );
	

// Lights
	// Build the light group
	TQ3GroupObject theLightList;
	TQ3LightData theLightData;
	TQ3LightObject theAmbientLight;
	TQ3GroupPosition theGroupPosition;
	
	// Light color
	TQ3ColorRGB	theWhiteLightColor;
	::Q3ColorRGB_Set( &theWhiteLightColor, 1.0, 1.0, 1.0 );
	
	// Create ambient light
	theLightData.isOn 		= kQ3True;
	theLightData.brightness = 0.2;
	theLightData.color		= theWhiteLightColor;
	
	theAmbientLight = ::Q3AmbientLight_New( &theLightData );
	ThrowIfNULL_( theAmbientLight );

	// Create light group and add lights to the view
	theLightList = ::Q3LightGroup_New();
	ThrowIfNULL_( theLightList );
	
	theGroupPosition = ::Q3Group_AddObject( theLightList, theAmbientLight );
	ThrowIfNULL_( theGroupPosition );
	::Q3Object_Dispose( theAmbientLight );
	
	theStatus = ::Q3View_SetLightGroup( mQD3DView, theLightList );
	ThrowIf_( theStatus == kQ3Failure );
	::Q3Object_Dispose( theLightList );
	

// Camera	
	// Build the camera
	TQ3CameraObject	theCamera;
	TQ3CameraData theCameraData;
	
	// Generic camera data
	TQ3Point3D cameraLocation 	= { 0.0, 0.0, 1.0 };
	TQ3Point3D cameraFocus		= { 0.0, 0.0, 0.0 };
	TQ3Vector3D cameraUp 		= { 0.0, 1.0, 0.0 };

	float hitherValue 	= 0.01;
	float yonValue 		= 100.0;

	theCameraData.placement.cameraLocation 	= cameraLocation;
	theCameraData.placement.pointOfInterest = cameraFocus;
	theCameraData.placement.upVector 		= cameraUp;

	theCameraData.range.hither 	= hitherValue;
	theCameraData.range.yon 	= yonValue;

	theCameraData.viewPort.origin.x 	= -1.0;
	theCameraData.viewPort.origin.y 	= 1.0;
	theCameraData.viewPort.width 		= 2.0;
	theCameraData.viewPort.height 		= 2.0;
	
	// Angle Aspect camera data
	TQ3ViewAngleAspectCameraData theAngleAspectCameraData;
	
	float fieldOfView = 1.0;

	theAngleAspectCameraData.cameraData 		= theCameraData;
	theAngleAspectCameraData.fov 				= fieldOfView;
	theAngleAspectCameraData.aspectRatioXToY 	= (float) ( thePortRect.right - thePortRect.left ) / 
													(float) ( thePortRect.bottom - thePortRect.top );
	
	// Create the camera and add it to the view
	theCamera = ::Q3ViewAngleAspectCamera_New( &theAngleAspectCameraData );
	ThrowIfNULL_( theCamera );
	
	theStatus = ::Q3View_SetCamera( mQD3DView, theCamera );
	ThrowIf_( theStatus == kQ3Failure );
	::Q3Object_Dispose( theCamera );
	

// Renderer	
	// Build the renderer and add it to view
	TQ3RendererObject theRenderer;
	theRenderer = Q3Renderer_NewFromType( kQ3RendererTypeInteractive );
	ThrowIfNULL_( theRenderer );	
	
	theStatus = ::Q3View_SetRenderer( mQD3DView, theRenderer );
	ThrowIf_( theStatus == kQ3Failure );
	::Q3Object_Dispose( theRenderer );

// Pickers
	TQ3WindowPointPickData thePointData;
	
	thePointData.data.sort 				= kQ3PickSortNearToFar;
	thePointData.data.mask 				= kQ3PickDetailMaskPickID | kQ3PickDetailMaskXYZ | kQ3PickDetailMaskUV;
	thePointData.data.numHitsToReturn 	= kQ3ReturnAllHits;
	thePointData.point.x 				= 0;
	thePointData.point.y 				= 0;
	thePointData.vertexTolerance 		= 0.1;
	thePointData.edgeTolerance			= 0.1;

	mQD3DPointPicker = ::Q3WindowPointPick_New( &thePointData );
	ThrowIfNULL_( mQD3DPointPicker );
	
	TQ3WindowRectPickData theRectData;
	
	theRectData.data.sort 				= kQ3PickSortNone;
	theRectData.data.mask 				= kQ3PickDetailMaskPickID;
	theRectData.data.numHitsToReturn 	= kQ3ReturnAllHits;
	theRectData.rect.min.x				= 0.0;
	theRectData.rect.min.y				= 0.0;
	theRectData.rect.max.x				= 0.0;
	theRectData.rect.max.y				= 0.0;

	mQD3DRectPicker = ::Q3WindowRectPick_New( &theRectData );
	ThrowIfNULL_( mQD3DRectPicker );
}


// ---------------------------------------------------------------------------
// SetSourceFile
// ---------------------------------------------------------------------------

void CNew3DView::SetSourceFile(LFile* inSourceFile)
{
	ResetHomeCamera();
}


// ---------------------------------------------------------------------------
// SetSourceData
// ---------------------------------------------------------------------------

void CNew3DView::SetSourceData(Handle inSourceData)
{
	ResetHomeCamera();
}


// ---------------------------------------------------------------------------
// SetGroup
// ---------------------------------------------------------------------------

void CNew3DView::SetGroup(TQ3GroupObject inGroup)
{
	if ( mQD3DGroup )
	{
		::Q3Object_Dispose( mQD3DGroup );
	}
	
	mQD3DGroup = inGroup;
	
	ResetHomeCamera();
}

// ---------------------------------------------------------------------------
// SetBoundingBox
// ---------------------------------------------------------------------------

void CNew3DView::SetBoundingBox(const TQ3BoundingBox* inBBox)
{
	if ( inBBox )
	{
		::Q3BoundingBox_Copy( inBBox, &mBoundingBox );
	}
}


// ---------------------------------------------------------------------------
// ResetHomeCamera
// ---------------------------------------------------------------------------

void CNew3DView::ResetHomeCamera()
{
	if ( mQD3DGroup )
	{
		Try_ {
			TQ3Status status = ::Q3DisplayGroup_CalcAndUseBoundingBox( mQD3DGroup, kQ3ComputeBoundsApproximate, mQD3DView );
			if ( status == kQ3Success )
			{
				status = ::Q3DisplayGroup_GetBoundingBox(mQD3DGroup, &mBoundingBox);
				ThrowIf_( status == kQ3Failure );
			}

			TQ3CameraObject theCamera;
			status = ::Q3View_GetCamera( mQD3DView, &theCamera );
			ThrowIf_( status == kQ3Failure );

			TQ3CameraPlacement theCameraPlacement;
			status = ::Q3Camera_GetPlacement( theCamera, &theCameraPlacement );
			ThrowIf_( status == kQ3Failure );

			::Q3Point3D_Set( &theCameraPlacement.pointOfInterest, (mBoundingBox.min.x + mBoundingBox.max.x) / 2.0, (mBoundingBox.min.y + mBoundingBox.max.y) / 2.0, (mBoundingBox.min.z + mBoundingBox.max.z) / 2.0 );
			::Q3Point3D_Set( &theCameraPlacement.cameraLocation, 0.0, mBoundingBox.max.y * 1.5, mBoundingBox.max.z * 2.5);

			status = ::Q3Camera_SetPlacement( theCamera, &theCameraPlacement );
			ThrowIf_( status == kQ3Failure );

			float theRange = ::Q3Point3D_Distance(&theCameraPlacement.pointOfInterest, &theCameraPlacement.cameraLocation);
			mScale = theRange / 50.0;
				
			TQ3CameraRange theCameraRange;
			status = ::Q3Camera_GetRange( theCamera, &theCameraRange );
			ThrowIf_( status == kQ3Failure );
			if (theCameraRange.yon < (theRange * 5.0))
			{
				theCameraRange.yon = theRange * 5.0;
				theCameraRange.hither = theCameraRange.yon / 10000.0;
				
				status = ::Q3Camera_SetRange( theCamera, &theCameraRange );
				ThrowIf_( status == kQ3Failure );
			}
			
			::Q3Object_Dispose( theCamera );
		} Catch_( inErr ) {
			// Doesn't matter that we failed, camera didn't get moved
		} EndCatch_
	}
}


// ----------------------------------------------------------------------------
// ResizeFrameBy									
// ----------------------------------------------------------------------------

void CNew3DView::ResizeFrameBy(Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh)
{
	#pragma unused( inRefresh )
	
	LPane::ResizeFrameBy( inWidthDelta, inHeightDelta, false );

	if ( mQD3DView )
	{
		Rect thePortRect;
		CalcPortFrameRect( thePortRect );
		
		TQ3Status theStatus;
		TQ3DrawContextObject theDrawContext;
		
		// Update the draw context
		theStatus = ::Q3View_GetDrawContext( mQD3DView, &theDrawContext );
		ThrowIf_( theStatus == kQ3Failure );
		
		TQ3Area thePaneArea;
		TQ3Point2D theMinPanePoint;
		TQ3Point2D theMaxPanePoint;
		
		theMinPanePoint.x 	= thePortRect.left;
		theMinPanePoint.y 	= thePortRect.top;
		theMaxPanePoint.x 	= thePortRect.right;
		theMaxPanePoint.y 	= thePortRect.bottom;
		thePaneArea.min 	= theMinPanePoint;
		thePaneArea.max 	= theMaxPanePoint;
		
		theStatus = ::Q3DrawContext_SetPane( theDrawContext, &thePaneArea );
		ThrowIf_( theStatus == kQ3Failure );
		::Q3Object_Dispose( theDrawContext );
		
		// update the camera
		TQ3CameraObject theCamera;
		theStatus = ::Q3View_GetCamera( mQD3DView, &theCamera );
		ThrowIf_( theStatus == kQ3Failure );
		
		float aspectRatioXToY = (float) ( thePortRect.right - thePortRect.left ) / 
								(float)  (thePortRect.bottom - thePortRect.top );
		
		theStatus = ::Q3ViewAngleAspectCamera_SetAspectRatio( theCamera, aspectRatioXToY );
		ThrowIf_( theStatus == kQ3Failure );
		::Q3Object_Dispose( theCamera );
	}
	
	if ( inRefresh )
	{
		Refresh();
	}
}


// ---------------------------------------------------------------------------
// FocusDraw
// ---------------------------------------------------------------------------
// Restore the port origin to reduce confusion for QuickDraw3D drawing and clicking
Boolean CNew3DView::FocusDraw( LPane *inSubPane )
{
	Boolean superFocused = LPane::FocusDraw( inSubPane );

	if ( mQD3DView )
	{
		// Restore the origin
		::SetOrigin( 0, 0 );
	
		LView::OutOfFocus( nil );
	}

	return superFocused;
}


// ---------------------------------------------------------------------------
// DrawSelf
// ---------------------------------------------------------------------------

void CNew3DView::DrawSelf()
{
	if ( mQD3DView )
	{
		TQ3Status theStatus;
		
		theStatus = ::Q3View_StartRendering( mQD3DView );
		ThrowIf_( theStatus == kQ3Failure );

		do
		{
			// Turn on anti-aliasing
			/*
			TQ3AntiAliasStyleData theAntiAliasStyle;
			theAntiAliasStyle.state = kQ3On;
			theAntiAliasStyle.mode = kQ3AntiAliasModeMaskEdges;
			theAntiAliasStyle.quality = 1.0;	// Small models, may as well...
			
			theStatus = ::Q3AntiAliasStyle_Submit( &theAntiAliasStyle, mQD3DView );
			*/
			
			// Render the group
			theStatus = this->RenderSelf();
			ThrowIf_( theStatus == kQ3Failure );
		}
		while ( ::Q3View_EndRendering( mQD3DView ) == kQ3ViewStatusRetraverse );
	}
}


// ---------------------------------------------------------------------------
// RenderSelf
// ---------------------------------------------------------------------------

TQ3Status CNew3DView::RenderSelf()
{
	TQ3Status theStatus = kQ3Success;
	
	if ( mQD3DGroup )
	{
		theStatus = ::Q3DisplayGroup_Submit( mQD3DGroup, mQD3DView );
	}
	
	return theStatus;
}


// ---------------------------------------------------------------------------
//	AdjustCursorSelf
// ---------------------------------------------------------------------------

void CNew3DView::AdjustCursorSelf( Point /* inPortPt */, const EventRecord& /* inMacEvent */ )
{
	switch( mCurrentTool )
	{
		case kToolTruck:
			UCursor::SetTheCursor( kCursorTruck );
			break;
		case kToolOrbit:
			UCursor::SetTheCursor( kCursorOrbit );
			break;
		case kToolZoom:
			UCursor::SetTheCursor( kCursorZoom );
			break;
		case kToolDolly:
			UCursor::SetTheCursor( kCursorDolly );
			break;
	}
}


// ---------------------------------------------------------------------------
// ClickSelf
// ---------------------------------------------------------------------------

void CNew3DView::ClickSelf( const SMouseDownEvent &inMouseDown )
{
	// GetMouse() uses port coordinates, so save ourselves trouble later on...
	this->FocusDraw();
	
	float theCameraIncrement = 0.0;
	if ( IS_VIEWER_TOOL( mCurrentTool ) )
	{
		TQ3CameraObject theCamera;
		TQ3Status status = ::Q3View_GetCamera( mQD3DView, &theCamera );
		ThrowIf_( status == kQ3Failure );

		TQ3CameraPlacement theCameraPlacement;
		status = ::Q3Camera_GetPlacement( theCamera, &theCameraPlacement );
		ThrowIf_( status == kQ3Failure );

		TQ3Vector3D theCameraVector;
		::Q3Point3D_Subtract( &theCameraPlacement.pointOfInterest, &theCameraPlacement.cameraLocation, &theCameraVector );
		
		float theCameraDistance = ::Q3Vector3D_Length( &theCameraVector );

		::Q3Object_Dispose( theCamera );
		
		SDimension16 frameSize;
		GetFrameSize( frameSize );
		theCameraIncrement = theCameraDistance / frameSize.height;
	}
	
	switch ( mCurrentTool )
	{
		case kToolTruck:
			if ( ::WaitMouseMoved( inMouseDown.macEvent.where ) )
			{
				StCursor theCursor( kCursorTruckActive, true );
				
				Point ptLast;
				::GetMouse( &ptLast );
				Point ptMouse;
				while ( ::StillDown() )
				{
					::GetMouse( &ptMouse );

					if ( ptMouse.v != ptLast.v )
					{
						CameraZoom( theCameraIncrement * ( ptMouse.v - ptLast.v ) );
						
						Refresh();
						UpdatePort();
						
						ptLast = ptMouse;
					}
				}
			}
			break;
			
		case kToolOrbit:
			if ( ::WaitMouseMoved( inMouseDown.macEvent.where ) )
			{
				StCursor theCursor( kCursorOrbitActive, true );
				
				SDimension16 frameSize;
				GetFrameSize( frameSize );
				
				SPoint32 frameLoc;
				GetFrameLocation( frameLoc );
				
				Point ptCenter;
				ptCenter.h = frameLoc.h + ( frameSize.width / 2 );
				ptCenter.v = frameLoc.v + ( frameSize.height / 2 );
				
				float theRadius = fmin( ( frameSize.width / 2 ), ( frameSize.height / 2 ));
				theRadius -= 2.0;	// Try to avoid "the flip"
				
				Point ptLast;
				::GetMouse( &ptLast );
				float angleLastH = Q3Math_RadiansToDegrees( asin( fmax( fmin( ( ptLast.h - ptCenter.h ), theRadius ), -theRadius ) / theRadius ) );
				float angleLastV = Q3Math_RadiansToDegrees( acos( fmax( fmin( ( ptLast.v - ptCenter.v ), theRadius ), -theRadius ) / theRadius ) );
				Point ptMouse;
				float angleMouseH;
				float angleMouseV;
				while ( ::StillDown() )
				{
					::GetMouse( &ptMouse );
					angleMouseH = Q3Math_RadiansToDegrees( asin( fmax( fmin( ( ptMouse.h - ptCenter.h ), theRadius ), -theRadius ) / theRadius ) );
					angleMouseV = Q3Math_RadiansToDegrees( acos( fmax( fmin( ( ptMouse.v - ptCenter.v ), theRadius ), -theRadius ) / theRadius ) );

					if ( ( angleMouseH != angleLastH ) || ( angleMouseV != angleLastV ) )
					{
						CameraOrbit( -( angleMouseH - angleLastH ), -( angleMouseV - angleLastV ) );
						
						Refresh();
						UpdatePort();
						
						angleLastH = angleMouseH;
						angleLastV = angleMouseV;
					}
				}
			}
			break;
			
		case kToolZoom:
			if ( ::WaitMouseMoved( inMouseDown.macEvent.where ) )
			{
				StCursor theCursor( kCursorZoomActive, true );
				
				Point ptLast;
				::GetMouse( &ptLast );
				Point ptMouse;
				while ( ::StillDown() )
				{
					::GetMouse( &ptMouse );

					if ( ptMouse.v != ptLast.v )
					{
						CameraZoom( theCameraIncrement * ( ptMouse.v - ptLast.v ) );
						
						Refresh();
						UpdatePort();
						
						ptLast = ptMouse;
					}
				}
			}
			break;
			
		case kToolDolly:
			if ( ::WaitMouseMoved( inMouseDown.macEvent.where ) )
			{
				StCursor theCursor( kCursorDollyActive, true );
				
				Point ptLast;
				::GetMouse( &ptLast );
				Point ptMouse;
				while ( ::StillDown() )
				{
					::GetMouse( &ptMouse );

					// The newest QD3D viewer goes both directions with the dolly
					if ( ( ptMouse.h != ptLast.h ) || ( ptMouse.v != ptLast.v ) )
					{
						CameraMove( 0.0, theCameraIncrement * ( ptMouse.h - ptLast.h ), theCameraIncrement * ( ptMouse.v - ptLast.v ) );
						
						Refresh();
						UpdatePort();
						
						ptLast = ptMouse;
					}
				}
			}
			break;

		case kToolNone:
		default:
			LPane::ClickSelf( inMouseDown );
			break;
	}
}


// ---------------------------------------------------------------------------
//	CameraOrbit						
// ---------------------------------------------------------------------------

void CNew3DView::CameraOrbit( float inDegrees, float inVerticalDegrees )
{
	TQ3CameraObject theCamera;
	TQ3Status status = ::Q3View_GetCamera( mQD3DView, &theCamera );
	ThrowIf_( status == kQ3Failure );

	TQ3CameraPlacement theCameraPlacement;
	status = ::Q3Camera_GetPlacement( theCamera, &theCameraPlacement );
	ThrowIf_( status == kQ3Failure );

	float xAngle = 0.0;
	float yAngle = Q3Math_DegreesToRadians( inDegrees );
	float zAngle = 0.0;
	
	TQ3Matrix4x4 cameraMatrix;
	::Q3Matrix4x4_SetRotateAboutPoint( &cameraMatrix, &theCameraPlacement.pointOfInterest, xAngle, yAngle, zAngle );
	
	::Q3Point3D_Transform( &theCameraPlacement.cameraLocation, &cameraMatrix, &theCameraPlacement.cameraLocation );

	if ( inVerticalDegrees != 0.0 )
	{
		float verticalAngle = Q3Math_DegreesToRadians( inVerticalDegrees );
		TQ3Vector3D theCameraVector;
		::Q3Point3D_Subtract( &theCameraPlacement.pointOfInterest, &theCameraPlacement.cameraLocation, &theCameraVector );
		
		TQ3Vector3D theOrthogonalVector;
		theOrthogonalVector.x = theCameraVector.z;
		theOrthogonalVector.y = 0.0;
		theOrthogonalVector.z = 0.0 - theCameraVector.x;
		::Q3Vector3D_Normalize( &theOrthogonalVector, &theOrthogonalVector );

		TQ3Matrix4x4 verticalMatrix;
		::Q3Matrix4x4_SetRotateAboutAxis( &verticalMatrix, &theCameraPlacement.pointOfInterest, &theOrthogonalVector, verticalAngle );

		::Q3Point3D_Transform( &theCameraPlacement.cameraLocation, &verticalMatrix, &theCameraPlacement.cameraLocation );
	}

	status = ::Q3Camera_SetPlacement( theCamera, &theCameraPlacement );
	ThrowIf_( status == kQ3Failure );
	
	::Q3Object_Dispose( theCamera );
}


// ---------------------------------------------------------------------------
//	CameraPan						
// ---------------------------------------------------------------------------

void CNew3DView::CameraPan( float inDegrees )
{
	TQ3CameraObject theCamera;
	TQ3Status status = ::Q3View_GetCamera( mQD3DView, &theCamera );
	ThrowIf_( status == kQ3Failure );

	TQ3CameraPlacement theCameraPlacement;
	status = ::Q3Camera_GetPlacement( theCamera, &theCameraPlacement );
	ThrowIf_( status == kQ3Failure );

	float xAngle = 0.0;
	float yAngle = Q3Math_DegreesToRadians( inDegrees );
	float zAngle = 0.0;
	
	TQ3Matrix4x4 cameraMatrix;
	::Q3Matrix4x4_SetRotateAboutPoint( &cameraMatrix, &theCameraPlacement.cameraLocation, xAngle, yAngle, zAngle );
	::Q3Point3D_Transform( &theCameraPlacement.pointOfInterest, &cameraMatrix, &theCameraPlacement.pointOfInterest );

	status = ::Q3Camera_SetPlacement( theCamera, &theCameraPlacement );
	ThrowIf_( status == kQ3Failure );
	
	::Q3Object_Dispose( theCamera );
}


// ---------------------------------------------------------------------------
//	CameraZoom						
// ---------------------------------------------------------------------------

void CNew3DView::CameraZoom( float inDistance )
{
	TQ3CameraObject theCamera;
	TQ3Status status = ::Q3View_GetCamera( mQD3DView, &theCamera );
	ThrowIf_( status == kQ3Failure );

	TQ3CameraPlacement theCameraPlacement;
	status = ::Q3Camera_GetPlacement( theCamera, &theCameraPlacement );
	ThrowIf_( status == kQ3Failure );

	TQ3Vector3D theCameraVector;
	::Q3Point3D_Subtract( &theCameraPlacement.pointOfInterest, &theCameraPlacement.cameraLocation, &theCameraVector );
	
	float theCameraDistance = ::Q3Vector3D_Length( &theCameraVector );
	
	if ( inDistance < theCameraDistance )
	{
		::Q3Vector3D_Normalize( &theCameraVector, &theCameraVector );
		::Q3Vector3D_Scale( &theCameraVector, inDistance, &theCameraVector );
		
		::Q3Point3D_Vector3D_Add( &theCameraPlacement.cameraLocation, &theCameraVector, &theCameraPlacement.cameraLocation );
		
		status = ::Q3Camera_SetPlacement( theCamera, &theCameraPlacement );
		ThrowIf_( status == kQ3Failure );
	}
	
	::Q3Object_Dispose( theCamera );
}


// ---------------------------------------------------------------------------
//	CameraMove						
// ---------------------------------------------------------------------------

void CNew3DView::CameraMove( float inDistance, float inOrthogonalDistance, float inVerticalDistance )
{
	TQ3CameraObject theCamera;
	TQ3Status status = ::Q3View_GetCamera( mQD3DView, &theCamera );
	ThrowIf_( status == kQ3Failure );

	TQ3CameraPlacement theCameraPlacement;
	status = ::Q3Camera_GetPlacement( theCamera, &theCameraPlacement );
	ThrowIf_( status == kQ3Failure );

	TQ3Vector3D theCameraVector;
	::Q3Point3D_Subtract( &theCameraPlacement.pointOfInterest, &theCameraPlacement.cameraLocation, &theCameraVector );
	theCameraVector.y = 0.0;
	
	TQ3Vector3D theOrthogonalVector;
	::Q3Vector3D_Normalize( &theCameraVector, &theCameraVector );
	theOrthogonalVector.x = theCameraVector.z;
	theOrthogonalVector.y = 0.0;
	theOrthogonalVector.z = 0.0 - theCameraVector.x;
	
	TQ3Vector3D theVerticalVector;
	::Q3Vector3D_Cross( &theCameraVector, &theOrthogonalVector, &theVerticalVector);
	
	::Q3Vector3D_Scale( &theCameraVector, inDistance, &theCameraVector );
	::Q3Vector3D_Scale( &theOrthogonalVector, inOrthogonalDistance, &theOrthogonalVector );
	::Q3Vector3D_Scale( &theVerticalVector, inVerticalDistance, &theVerticalVector );
	::Q3Vector3D_Add( &theCameraVector, &theOrthogonalVector, &theCameraVector );
	::Q3Vector3D_Add( &theCameraVector, &theVerticalVector, &theCameraVector );
		
	::Q3Point3D_Vector3D_Add( &theCameraPlacement.pointOfInterest, &theCameraVector, &theCameraPlacement.pointOfInterest );
	::Q3Point3D_Vector3D_Add( &theCameraPlacement.cameraLocation, &theCameraVector, &theCameraPlacement.cameraLocation );
		
	status = ::Q3Camera_SetPlacement( theCamera, &theCameraPlacement );
	ThrowIf_( status == kQ3Failure );
	
	::Q3Object_Dispose( theCamera );
}


// ---------------------------------------------------------------------------
//	ListenToMessage						
// ---------------------------------------------------------------------------

void CNew3DView::ListenToMessage( MessageT inMessage, void* ioParam )
{
	switch( inMessage )
	{
		case kMsgCamera:
			break;
		case kMsgTruck:
			mCurrentTool = kToolTruck;
			break;
		case kMsgOrbit:
			mCurrentTool = kToolOrbit;
			break;
		case kMsgZoom:
			mCurrentTool = kToolZoom;
			break;
		case kMsgDolly:
			mCurrentTool = kToolDolly;
			break;
		case kMsgReset:
			ResetHomeCamera();
			mButtons[kButtonReset]->SetValue( 0 );
			Refresh();
			break;
	}
}
