// ===========================================================================
// CNew3DView.h
// ===========================================================================


#pragma once


#include <QD3D.h>
#include <QD3DGeometry.h>

enum
{
	kButtonCamera		= 0,
	kButtonTruck,
	kButtonOrbit,
	kButtonZoom,
	kButtonDolly,
	kButtonReset,
	NUMBER_OF_3D_BUTTONS
};

enum
{
	kToolNone			= 0,
	kToolTruck			= kButtonTruck,
	kToolOrbit			= kButtonOrbit,
	kToolZoom			= kButtonZoom,
	kToolDolly			= kButtonDolly
};

const PaneIDT kIDCamera = '3D00';
const PaneIDT kIDTruck	= '3D01';
const PaneIDT kIDOrbit 	= '3D02';
const PaneIDT kIDZoom 	= '3D03';
const PaneIDT kIDDolly 	= '3D04';
const PaneIDT kIDReset 	= '3D05';

const MessageT kMsgCamera 	= kIDCamera;
const MessageT kMsgTruck	= kIDTruck;
const MessageT kMsgOrbit 	= kIDOrbit;
const MessageT kMsgZoom 	= kIDZoom;
const MessageT kMsgDolly 	= kIDDolly;
const MessageT kMsgReset 	= kIDReset;

const ResIDT kCursorTruck		= 2380;
const ResIDT kCursorTruckActive	= 2381;
const ResIDT kCursorOrbit		= 2382;
const ResIDT kCursorOrbitActive	= 2383;
const ResIDT kCursorZoom		= 2384;
const ResIDT kCursorZoomActive	= 2385;
const ResIDT kCursorDolly		= 2386;
const ResIDT kCursorDollyActive	= 2387;

enum
{
	kCameraRestore		= 0,
	kCameraFit,
	kCameraFront,
	kCameraBack,
	kCameraLeft,
	kCameraRight,
	kCameraTop,
	kCameraBottom,
	NUMBER_OF_CAMERA_POSITIONS
};

#define IS_VIEWER_TOOL(x)		( ((x) >= kToolTruck) && ((x) <= kToolDolly) )

class LToggleButton;

// ---------------------------------------------------------------------------
// CNew3DView
// ---------------------------------------------------------------------------

class CNew3DView :	public LPane,
				   	public LCommander,
				   	public LListener
{
public:
	enum { class_ID = 'Nu3d' };
	
							CNew3DView();
							CNew3DView( LStream *inStream );
	virtual					~CNew3DView();
	
	virtual void			ResizeFrameBy( Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
	virtual Boolean			FocusDraw( LPane *inSubPane = nil );

	virtual void			AdjustCursorSelf(Point inPortPt, const EventRecord &inMacEvent);
	virtual	void			ClickSelf( const SMouseDownEvent &inMouseDown );
	virtual void			ListenToMessage( MessageT inMessage, void* ioParam );
			
	virtual void			SetSourceFile(LFile* inSourceFile);
	virtual void			SetSourceData(Handle inSourceData);
	virtual void			SetGroup(TQ3GroupObject inGroup);
	virtual void			SetBoundingBox(const TQ3BoundingBox* inBBox);
	
	virtual TQ3GroupObject	GetGroup()	{ return mQD3DGroup; };
	virtual TQ3ViewObject	GetView()	{ return mQD3DView; };
	
	virtual void			ResetHomeCamera();

//	virtual void			SaveToFile(LFile* inDestinationFile = nil);
								
protected:
			float			mScale;
			
			UInt32			mCurrentTool;
			
			TQ3GroupObject	mQD3DGroup;
			TQ3ViewObject	mQD3DView;
			TQ3PickObject	mQD3DPointPicker;
			TQ3PickObject	mQD3DRectPicker;
			TQ3BoundingBox	mBoundingBox;
			
			LToggleButton**	mButtons;
			
	virtual	void			FinishCreateSelf();
			
	virtual	void			DrawSelf();
	virtual TQ3Status		RenderSelf();
	
	virtual void			InitializeQD3DView();

			void			CameraOrbit( float inDegrees, float inVerticalDegrees = 0.0 );
			void			CameraPan( float inDegrees );
			void			CameraZoom( float inDistance );
			void			CameraMove( float inDistance, float inOrthogonalDistance, float inVerticalDistance = 0.0 );
};