// ===========================================================================
//	CPopupToggle.cp	
// ===========================================================================

#include "CPopupToggle.h"

#include <LToggleButton.h>
#include <LStream.h>
#include <UDrawingState.h>
#include <PP_Messages.h>
#include <PP_Resources.h>

const	SInt32	delay_Animation	= 6;

// ---------------------------------------------------------------------------
//	 LToggleButton							Default Constructor		  [public]
// ---------------------------------------------------------------------------

CPopupToggle::CPopupToggle()
	: LToggleButton()
{
	mMenuID = resID_Undefined;
	
	mMenuRef = nil;
}


// ---------------------------------------------------------------------------
//	 CPopupToggle							Stream Constructor		  [public]
// ---------------------------------------------------------------------------
//
//	Stream data must be:
//		ResIDT		Resource ID of popup menu

CPopupToggle::CPopupToggle(
	LStream*	inStream)
	
	: LToggleButton(inStream)
{
	*inStream >> mMenuID;
	
	mMenuRef = ::GetMenu( mMenuID );
	if ( mMenuRef )
	{
		::InsertMenu( mMenuRef, -1 );
	}
}


// ---------------------------------------------------------------------------
//	 ~LToggleButton						Destructor				  [public]
// ---------------------------------------------------------------------------

CPopupToggle::~CPopupToggle()
{
}


// ---------------------------------------------------------------------------
//	 TrackHotSpot												   [protected]
// ---------------------------------------------------------------------------
//	Track the mouse while it is down after clicking in a Control HotSpot
//
//	Returns whether the mouse is released within the HotSpot

Boolean
CPopupToggle::TrackHotSpot(
	SInt16	/* inHotSpot */,
	Point 	/* inPoint */,
	SInt16	/* inModifiers */)
{
	SInt32 result = 0;
	
	Rect rectFrame;
	if ( mMenuRef && CalcPortFrameRect( rectFrame ) )
	{
		Point globalPoint = { rectFrame.top, rectFrame.right };
		PortToGlobalPoint( globalPoint );
		
		short initialItem = (short) GetValue();
		
		result = ::PopUpMenuSelect( mMenuRef , globalPoint.v, globalPoint.h, initialItem );
	}
				
	SetValue( result & 0x0000FFFF );
	
	return ( result != 0 );
}


// ---------------------------------------------------------------------------
//	 HotSpotResult
// ---------------------------------------------------------------------------

void
CPopupToggle::HotSpotResult(
	SInt16	/* inHotSpot */)
{	
	// Animate Button from current state to a transition state
	// and then to the end state

	if (mTransitionID != resID_Undefined)
	{
		FocusDraw();
		
		DrawGraphic(mTransitionID);
		UInt32 ticks;
		::Delay(delay_Animation, &ticks);
		
		// We've already set the value
		// so make the "normal" transition
		ResIDT	endClickState = mOffClickID;
		if (mValue != 0)
		{
			endClickState = mOnClickID;
		}
		DrawGraphic(endClickState);
	}
}


