// ===========================================================================
//	CRaveEngine.cp			   1999 eli curtz. All rights reserved.
// ===========================================================================


#include <QD3D.h>
#include <QD3DGeometry.h>
#include <QD3DRenderer.h>
#include <QD3DExtension.h>

#include "CRaveEngine.h" 

CRaveEngine::sEngine = nil;



void
CRaveEngine::Initialize()
{
	ThrowIfOSErr_(::QARegisterEngine(CRaveEngine::GetEngineMethod));
}

TQAError
CRaveEngine::GetEngineMethod(
	TQAEngineMethodTag methodTag,
	TQAEngineMethod *method)
{
	switch (methodTag) {
	case kQADrawPrivateNew:
		method->drawPrivateNew = MyDrawPrivateNew;
		break;
	
	case kQADrawPrivateDelete:
		method->drawPrivateDelete = MyDrawPrivateDelete;
		break;
	
	case kQAEngineCheckDevice:
		method->engineCheckDevice = MyEngineCheckDevice;
		break;
	
	case kQAEngineGestalt:
		method->engineGestalt = MyEngineGestalt;
		break;
	
	case kQABitmapNew:
		method->bitmapNew = MyBitmapNew;
		break;
	
	case kQABitmapDetach:
		method->bitmapDetach = MyBitmapDetach;
		break;
	
	case kQABitmapDelete:
		method->bitmapDelete = MyBitmapDelete;
		break;
	
	case kQAEngineCheckDevice:	
	case kQATextureNew:
	case kQATextureDetach:
	case kQATextureDelete:
	case kQAColorTableNew:
	case kQAColorTableDelete:
	case kQATextureBindColorTable:
	case kQABitmapBindColorTable:
	default:
		return(kQANotSupported);
	}
}	

TQAError
CRaveEngine::DrawPrivateNew (
	TQADrawContext *drawContext,
	const TQADevice *device,
	const TQARect *rect,
	const TQAClip *clip,
	unsigned long flags)
{
	TQAError theQAError = kQANoErr;
	
	sEngine = new CRaveEngine();
	if (sEngine == NULL)
		theQAError = kQAOutOfMemory;

	drawContext->drawPrivate = static_cast<TQADrawPrivate *>(sEngine);

	drawContext->setFloat;					/* Method: Set a float state variable */
	drawContext->setInt;					/* Method: Set an unsigned long state variable */
	drawContext->setPtr;					/* Method: Set an unsigned long state variable */
	drawContext->getFloat;					/* Method: Get a float state variable */
	drawContext->getInt;					/* Method: Get an unsigned long state variable */
	drawContext->getPtr;					/* Method: Get an pointer state variable */
	drawContext->drawPoint;					/* Method: Draw a point */
	drawContext->drawLine;					/* Method: Draw a line */
	drawContext->drawTriGouraud;			/* Method: Draw a Gouraud shaded triangle */
	drawContext->drawTriTexture;			/* Method: Draw a texture mapped triangle */
	drawContext->drawVGouraud;				/* Method: Draw Gouraud vertices */
	drawContext->drawVTexture;				/* Method: Draw texture vertices */
	drawContext->drawBitmap;				/* Method: Draw a bitmap */
	drawContext->renderStart;				/* Method: Initialize for rendering */
	drawContext->renderEnd;					/* Method: Complete rendering and display */
	drawContext->renderAbort;				/* Method: Abort any outstanding rendering (blocking) */
	drawContext->flush;						/* Method: Start render of any queued commands (non-blocking) */
	drawContext->sync;						/* Method: Wait for completion of all rendering (blocking) */
	drawContext->submitVerticesGouraud;		/* Method: Submit Gouraud vertices for trimesh */
	drawContext->submitVerticesTexture;		/* Method: Submit Texture vertices for trimesh */
	drawContext->drawTriMeshGouraud;		/* Method: Draw a Gouraud triangle mesh */
	drawContext->drawTriMeshTexture;		/* Method: Draw a Texture triangle mesh */
	drawContext->setNoticeMethod;			/* Method: Set a notice method */
	drawContext->getNoticeMethod;			/* Method: Get a notice method */
	
	return theQAError;
}

void
CRaveEngine::EngineSetFloat (
	TQADrawContext *drawContext,
	TQATagFloat tag,
	float newValue)
{
	return (static_cast<CRaveEngine*>(drawContext->drawPrivate))->DoSetFloat(tag, newValue);
}
void
CRaveEngine::DoSetFloat (
	TQATagFloat tag,
	float newValue)
{
	switch (tag) {
		// These variables are required by all drawing engines:
		case kQATag_ColorBG_a:			// Background color alpha
		case kQATag_ColorBG_r:			// Background color red
		case kQATag_ColorBG_g:			// Background color green
		case kQATag_ColorBG_b:			// Background color blue
		case kQATag_Width:				// Line and point width (pixels)
		case kQATag_ZMinOffset:			// Min offset to Z to guarantee visibility (Read only!)
		case kQATag_ZMinScale:			// Min scale to Z to guarantee visibility (Read only!)
		// These variables are used for OpenGL support:
		case kQATagGL_DepthBG:			// Background Z
		case kQATagGL_TextureBorder_a:	// Texture border color alpha
		case kQATagGL_TextureBorder_r:	// Texture border color red
		case kQATagGL_TextureBorder_g:	// Texture border color green
		case kQATagGL_TextureBorder_b:	// Texture border color blue
		default:
	}
}

void
CRaveEngine::EngineSetInt (
	TQADrawContext *drawContext,
	TQATagInt tag,
	unsigned long newValue)
{
}

void
CRaveEngine::EngineSetPtr (
	TQADrawContext *drawContext,
	TQATagPtr tag,
	const void *newValue)
{
}

float
CRaveEngine::EngineGetFloat (
	TQADrawContext *drawContext,
	TQATagFloat tag)
{
}

unsigned long
CRaveEngine::EngineGetInt (
	TQADrawContext *drawContext,
	TQATagInt tag)
{
}

void *
CRaveEngine::EngineGetPtr (
	const TQADrawContext *drawContext,
	TQATagPtr tag)
{
}

void 
CRaveEngine::EngineDrawPoint (
	const TQADrawContext *drawContext,
	const TQAVGouraud *v)
{
}

void
CRaveEngine::EngineDrawLine (
	const TQADrawContext *drawContext,
	const TQAVGouraud *v0,
	const TQAVGouraud *v1)

{
}

void
CRaveEngine::EngineDrawTriGouraud (
	const TQADrawContext *drawContext,
	const TQAVGouraud *v0,
	const TQAVGouraud *v1,
	const TQAVGouraud *v2,
	unsigned long flags)
{
}

void
CRaveEngine::EngineDrawTriTexture (
	const TQADrawContext *drawContext,
	const TQAVTexture *v0,
	const TQAVTexture *v1,
	const TQAVTexture *v2,
	unsigned long flags)
{
}

void
CRaveEngine::EngineSubmitVerticesGouraud (
	const TQADrawContext *drawContext,
	unsigned long nVertices,
	const TQAVGouraud *vertices)
{
}

void
CRaveEngine::EngineSubmitVerticesTexture (
	const TQADrawContext *drawContext,
	unsigned long nVertices,
	const TQAVTexture *vertices)
{
}

void
CRaveEngine::EngineDrawTriMeshGouraud (
	const TQADrawContext *drawContext,
	unsigned long nTriangles,
	const TQAIndexedTriangle *triangles)
{
}

void
CRaveEngine::EngineDrawTriMeshTexture (
	const TQADrawContext *drawContext,
	unsigned long nTriangles,
	const TQAIndexedTriangle *triangles)
{
}

void
CRaveEngine::EngineDrawVGouraud (
	const TQADrawContext *drawContext,
	unsigned long nVertices,
	TQAVertexMode vertexMode,
	const TQAVGouraud vertices[],
	const unsigned long flags[])
{
}

void
CRaveEngine::EngineDrawVTexture (
	const TQADrawContext *drawContext,
	unsigned long nVertices,
	TQAVertexMode vertexMode,
	const TQAVTexture vertices[],
	const unsigned long flags[])
{
}

void
CRaveEngine::EngineDrawBitmap (
	const TQADrawContext *drawContext,
	const TQAVGouraud *v,
	TQABitmap *bitmap)
{
}

void
CRaveEngine::EngineRenderStart (
	const TQADrawContext *drawContext,
	const TQARect *dirtyRect,
	const TQADrawContext *initialContext)
{
}

TQAError
CRaveEngine::EngineRenderEnd (
	const TQADrawContext *drawContext,
	const TQARect *modifiedRect)
{
}

TQAError
CRaveEngine::EngineRenderAbort (
	const TQADrawContext *drawContext)
{
}

TQAError
CRaveEngine::EngineFlush (
	const TQADrawContext *drawContext)
{
}

TQAError
CRaveEngine::EngineSync (
	const TQADrawContext *drawContext)
{
}

TQAError
CRaveEngine::EngineSetNoticeMethod (
	const TQADrawContext *drawContext,
	TQAMethodSelector method,
	TQANoticeMethod completionCallBack,
	void *refCon)
{
}

TQAError
CRaveEngine::EngineGetNoticeMethod (
	const TQADrawContext *drawContext,
	TQAMethodSelector method,
	TQANoticeMethod *completionCallBack,
	void **refCon)
{
}
