// =================================================================================
//	CgeomApp.cp					1997 eli curtz. All rights reserved.
// =================================================================================

#include <LGrowZone.h>
#include <LMenu.h>
#include <LToggleButton.h>
#include <LMenuBar.h>
#include <LString.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <UDesktop.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <UReanimator.h>
#include <URegistrar.h>
#include <UTextTraits.h>
#include <UStandardDialogs.h>

#include <QuickTimeComponents.h>

#include "UQuickDraw3D.h"
#include "Constants.h"
#include "CgeomDocument.h"
#include "C3dmfDocument.h"
#include "CgeomView.h"
#include "CPopupToggle.h"

#include "CNew3DView.h"

#include "CgeomApp.h"


//debugging
#include "CGWorldView.h"


// =================================================================================
//		 Main Program
// =================================================================================

void
main( void )
{
	// Setup the throw and signal actions.
//	SetDebugThrow_( debugAction_Alert );
//	SetDebugSignal_( debugAction_Alert );
	SetDebugThrow_( debugAction_SourceDebugger );
	SetDebugSignal_( debugAction_SourceDebugger );

	// Initialize the heap. Parameter is number
	// of master handle blocks to allocate.
	InitializeHeap( 4 );
	
	// Initialize the MacOS toolbox.
	UQDGlobals::InitializeToolbox( &qd );

	// Install a GrowZone function to catch  low memory situations.
	// Parameter is the size of the memory reserve in bytes.
	new LGrowZone( 20000 );

	// Create the application object and run it.
	CgeomApp	theApp;
	theApp.Run();
}


// ---------------------------------------------------------------------------------
//		 CgeomApp
// ---------------------------------------------------------------------------------

CgeomApp::CgeomApp()
{
	// Initialize QD3D
	UQuickDraw3D::Initialize();
	Assert_(UQuickDraw3D::QuickDraw3DIsPresent());
	
	// Register functions to create core PowerPlant classes	
	RegisterClass_(LPane);
	RegisterClass_(LView);
	RegisterClass_(LWindow);
	RegisterClass_(LDialogBox);
	
	// Register other PowerPlant classes
	RegisterClass_(LCaption);
	RegisterClass_(LEditField);
	RegisterClass_(LTabGroup);
	RegisterClass_(LStdControl);
	RegisterClass_(LStdButton);
	RegisterClass_(LStdCheckBox);
	RegisterClass_(LButton);
	RegisterClass_(LPaintAttachment);
	RegisterClass_(LControl);
	RegisterClass_(LToggleButton);
	RegisterClass_(LRadioGroup);
	RegisterClass_(LPicture);
	RegisterClass_(LCaption);

	// Register custom classes.
	RegisterClass_(LQD3DViewer);
	RegisterClass_(CNew3DView);
	RegisterClass_(CgeomView);
	RegisterClass_(CNotifyField);
	RegisterClass_(CPopupToggle);
	RegisterClass_(CCenteredView);
	
	// debugging
//	RegisterClass_(CGWorldView);
//	RegisterClass_(CGWorldViewTimed);
//	RegisterClass_(CGWorldViewListener);
}


// ---------------------------------------------------------------------------------
//		 ~CgeomApp
// ---------------------------------------------------------------------------------

CgeomApp::~CgeomApp()
{
}


// ---------------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------------

void
CgeomApp::StartUp()
{
//	ObeyCommand( cmd_New, nil );
	ChooseDocument();
}


// ---------------------------------------------------------------------------
//	 ShowAboutBox													  [public]
// ---------------------------------------------------------------------------
//	Display the About Box for the Application

void
CgeomApp::ShowAboutBox()
{
	LDialogBox* pAboutBox = (LDialogBox *) LWindow::CreateWindow( 1003, this );
	Assert_( pAboutBox != nil );
	
	LButton* pCloseButton = (LButton *) pAboutBox->FindPaneByID( kCloseButton );
	Assert_( pCloseButton != nil );
	LButton* pThanksButton = (LButton *) pAboutBox->FindPaneByID( kThanksButton );
	Assert_( pThanksButton != nil );
	
	StDialogHandler aboutHandler( pAboutBox, this );	
	pCloseButton->AddListener( &aboutHandler );
	pThanksButton->AddListener( &aboutHandler );

	CAboutIdler updateIdler( msg_IdleUpdate, 480 );
	updateIdler.StartIdling();
	updateIdler.AddListener( &aboutHandler );
	
	MessageT message = aboutHandler.DoDialog();
	while ( message != msg_Cancel )
	{
		if ( message == msg_IdleUpdate )
		{
			pThanksButton->Show();
			updateIdler.StopIdling();
		}

		message = aboutHandler.DoDialog();
	};
}


// ---------------------------------------------------------------------------------
//		 OpenDocument
// ---------------------------------------------------------------------------------

void
CgeomApp::OpenDocument(
	FSSpec	*inMacFSSpec )
{
	FInfo theFinderInfo;
	
	::FSpGetFInfo(inMacFSSpec, &theFinderInfo);
	
	if (theFinderInfo.fdType == kQTFileType3DMF)
	{
		new C3dmfDocument( this, inMacFSSpec );
	}
	else
	{
		// Create a new document using the file spec.
		new CgeomDocument( this, inMacFSSpec );
	}
}


// ---------------------------------------------------------------------------------
//		 MakeNewDocument
// ---------------------------------------------------------------------------------

LModelObject *
CgeomApp::MakeNewDocument()
{
	// Don't make empty docs for now
	return nil;

	// Make a new empty document.
//	return new CgeomDocument( this, nil );
}


// ---------------------------------------------------------------------------------
//		 ChooseDocument
// ---------------------------------------------------------------------------------

void
CgeomApp::ChooseDocument()
{
	// Deactivate the desktop.
	::UDesktop::Deactivate();

	// Browse for a document.
	FSSpec theFSSpec;
	OSType theOSTypes[3] = {kQTFileType3DMF, 'GEOM', 'File' };
//	LFileTypeList theFileTypes(3, theOSTypes);
	LFileTypeList theFileTypes(1, theOSTypes);	// Only '3DMF' for now
	PP_StandardDialogs::LFileChooser theChooser;
	Boolean choseFile = theChooser.AskChooseOneFile(theFileTypes, theFSSpec);

	// Activate the desktop.
	::UDesktop::Activate();
	
	// Send an apple event to open the file.	
	if ( choseFile ) SendAEOpenDoc( theFSSpec );
}


// ---------------------------------------------------------------------------------
//		 PrintDocument
// ---------------------------------------------------------------------------------

void
CgeomApp::PrintDocument(
	FSSpec	*inMacFSSpec )
{
	// Create a new document using the file spec.
	CgeomDocument	*theDocument = new CgeomDocument( this, inMacFSSpec );
	Assert_( theDocument != nil );

	// Tell the document to print.
	theDocument->DoPrint();
}



// ---------------------------------------------------------------------------
// CAboutIdler
// ---------------------------------------------------------------------------

CAboutIdler::CAboutIdler( MessageT inMessage, UInt32 inFrequency )
			: LPeriodical(), LBroadcaster()
{
	mMessage = inMessage;
	
	mFrequency = inFrequency;
	mLastBroadcast = 0;
}


// ---------------------------------------------------------------------------
// ~CAboutIdler
// ---------------------------------------------------------------------------

CAboutIdler::~CAboutIdler()
{
}

	
// ---------------------------------------------------------------------------
// SpendTime
// ---------------------------------------------------------------------------

void
CAboutIdler::SpendTime( const EventRecord& inMacEvent )
{
	if ( mLastBroadcast == 0 )
	{
		mLastBroadcast = inMacEvent.when;
	}
	else if ( ( inMacEvent.when - mFrequency ) > mLastBroadcast )
	{
		mLastBroadcast = inMacEvent.when;
		BroadcastMessage( mMessage );
	}
}



// ---------------------------------------------------------------------------
// CCenteredView
// ---------------------------------------------------------------------------

CCenteredView::CCenteredView()
			: LView()
{
}

CCenteredView::CCenteredView( LStream *inStream )
			: LView( inStream )
{
}


// ---------------------------------------------------------------------------
// ~CCenteredView
// ---------------------------------------------------------------------------

CCenteredView::~CCenteredView()
{
}


// ---------------------------------------------------------------------------
//	AdaptToSuperFrameSize
// ---------------------------------------------------------------------------
//	Adjust state of Pane when size of SuperView's Frame changes by the
//	specified amounts

void
CCenteredView::AdaptToSuperFrameSize(
	SInt32	inSurrWidthDelta,			// Change in width of SuperView
	SInt32	inSurrHeightDelta,			// Change in height of SuperView
	Boolean	inRefresh)
{
		// When SuperView changes size, a Pane may move or change
		// size, depending on how each of its sides is "bound" to
		// that of its SuperView. A side that is bound always remains
		// the same distance from the corresponding side of its SuperView.
		
	SInt32		horizDelta  = inSurrWidthDelta / 2;

	if ( horizDelta != 0 ) {
		MoveBy(horizDelta, 0, false);
	}

	LView::AdaptToSuperFrameSize( 0, inSurrHeightDelta, inRefresh );
}
