// ===========================================================================
//	CgeomView.cp				   1993-1997 eli curtz. All rights reserved.
// ===========================================================================
//

#ifdef PowerPlant_PCH
#include PowerPlant_PCH
#endif

#include <LPicture.h>
#include <LStream.h>
#include <LToggleButton.h>
#include <UDrawingState.h>

#include <QD3DView.h>
#include <QD3DMath.h>
#include <QD3DCamera.h>
#include <QD3DGroup.h>
#include <QD3DIO.h>
#include <QD3DShader.h>
#include <QD3DTransform.h>

#include "CgeomView.h"
#include "C3dmfDocument.h"
#include "geometry_definitions.h"
#include "CPopupToggle.h"
#include "Constants.h"

// ---------------------------------------------------------------------------
//	 CgeomView						Default Constructor		  [public]
// ---------------------------------------------------------------------------

CgeomView::CgeomView()
		: CNew3DView()
{
	mCameraKeys[kCamera_MoveForward]= 0x0D;	// 'W'
	mCameraKeys[kCamera_MoveBack]	= 0x01;	// 'S'
	mCameraKeys[kCamera_MoveLeft]	= 0x06;	// 'Z'
	mCameraKeys[kCamera_MoveRight]	= 0x07;	// 'X'
	mCameraKeys[kCamera_PanLeft]	= 0x00;	// 'A'
	mCameraKeys[kCamera_PanRight]	= 0x02;	// 'D'
	mCameraKeys[kCamera_OrbitLeft]	= 0x0C;	// 'Q'
	mCameraKeys[kCamera_OrbitRight]	= 0x0E;	// 'E'
	mCameraKeys[kCamera_OrbitUp]	= 0x0F;	// 'R'
	mCameraKeys[kCamera_OrbitDown]	= 0x03;	// 'F'
	mCameraKeys[kCamera_ZoomIn]		= 0x08;	// 'C'
	mCameraKeys[kCamera_ZoomOut]	= 0x09;	// 'V'
	
	mVertexPicker = nil;
	mDocument = nil;
	mCaption = nil;

	mLastViewTool = kToolNone;
	mLastEditTool = kToolNone;
	
	mDisplayedMeshAlert = false;
}


// ---------------------------------------------------------------------------
//	 CgeomView(LStream*)			Stream Constructor	  [public]
// ---------------------------------------------------------------------------

CgeomView::CgeomView(
	LStream	*inStream)
		: CNew3DView(inStream)
{
	mCameraKeys[kCamera_MoveForward]= 0x0D;	// 'W'
	mCameraKeys[kCamera_MoveBack]	= 0x01;	// 'S'
	mCameraKeys[kCamera_MoveLeft]	= 0x06;	// 'Z'
	mCameraKeys[kCamera_MoveRight]	= 0x07;	// 'X'
	mCameraKeys[kCamera_PanLeft]	= 0x00;	// 'A'
	mCameraKeys[kCamera_PanRight]	= 0x02;	// 'D'
	mCameraKeys[kCamera_OrbitLeft]	= 0x0C;	// 'Q'
	mCameraKeys[kCamera_OrbitRight]	= 0x0E;	// 'E'
	mCameraKeys[kCamera_OrbitUp]	= 0x0F;	// 'R'
	mCameraKeys[kCamera_OrbitDown]	= 0x03;	// 'F'
	mCameraKeys[kCamera_ZoomIn]		= 0x08;	// 'C'
	mCameraKeys[kCamera_ZoomOut]	= 0x09;	// 'V'
	
	mVertexPicker = nil;
	mDocument = nil;
	mCaption = nil;

	mLastViewTool = kToolNone;
	mLastEditTool = kToolNone;
	
	mDisplayedMeshAlert = false;
}


// ----------------------------------------------------------------------------
//   FinishCreateSelf												[public]
// ----------------------------------------------------------------------------

void
CgeomView::FinishCreateSelf()	
{
	CNew3DView::FinishCreateSelf();
	
	LToggleButton** pCurrentButtons = mButtons;
	
	mButtons = new LToggleButton*[NUMBER_OF_GEOM_BUTTONS];
	
	mButtons[kButtonTransparent]	= dynamic_cast<CPopupToggle*>( GetSuperView()->FindPaneByID( kIDTransparent ));
	
	mButtons[kButtonNoCull]			= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDNoCull ));
	mButtons[kButtonUnderMesh]		= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDUnderMesh ));
	mButtons[kButtonReflective]		= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDReflective ));
	mButtons[kButtonClosestZ]		= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDClosestZ ));
	mButtons[kButtonDependencies]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDDependencies ));
	mButtons[kButtonReplacesMesh]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDReplacesMesh ));
	mButtons[kButtonNoWalk]			= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDNoWalk ));
	mButtons[kButtonNoFly]			= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDNoFly ));
	mButtons[kButtonOverlapped]		= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDOverlapped ));
	mButtons[kButtonDeformsMesh]	= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDDeformsMesh ));
	mButtons[kButtonMedia]			= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDMedia ));
	mButtons[kButtonWobbles]		= dynamic_cast<LToggleButton*>( GetSuperView()->FindPaneByID( kIDWobbles ));
	
	// Copy over the existing buttons
	for ( int iButton = 0; iButton < NUMBER_OF_3D_BUTTONS; iButton++ )
	{
		mButtons[iButton] = pCurrentButtons[iButton];
	}
	delete [] pCurrentButtons;
	
	for ( int iButton = NUMBER_OF_3D_BUTTONS; iButton < NUMBER_OF_GEOM_BUTTONS; iButton++ )
	{
		if ( mButtons[iButton] ) mButtons[iButton]->AddListener( this );
	}

	TQ3WindowPointPickData theData;
	theData.data.sort = kQ3PickSortNearToFar;
	theData.data.mask = kQ3PickDetailMaskPickID;
	theData.data.numHitsToReturn = kQ3ReturnAllHits;
	theData.point.x = 0;
	theData.point.y = 0;
	theData.vertexTolerance = 5.0;
	theData.edgeTolerance = 5.0;

	mVertexPicker = ::Q3WindowPointPick_New(&theData);
	if (mVertexPicker == nil)	{
		SignalPStr_("\pCouldn't create vertex picker");
	}

	mCaption = dynamic_cast<LCaption*>( GetSuperView()->FindPaneByID( kIDCaption ));;
	Assert_( mCaption != nil );
	
	ListenToMessage( kMsgOrbit, this );
}


// ----------------------------------------------------------------------------
//	 ClickSelf												[public]
// ----------------------------------------------------------------------------

void
CgeomView::ClickSelf( const SMouseDownEvent &inMouseDown )	
{
	if ( IS_GEOM_TOOL( mCurrentTool ) && mQD3DView && mQD3DGroup)
	{
		LCommander::SwitchTarget( this );

		this->FocusDraw();
		
		TQ3Point2D qd3DWhere;
		qd3DWhere.x = inMouseDown.wherePort.h;
		qd3DWhere.y = inMouseDown.wherePort.v;
	
		TQ3Status qd3DStatus = ::Q3WindowPointPick_SetPoint(mVertexPicker, &qd3DWhere);
		if ( qd3DStatus != kQ3Success )
			SignalPStr_("\pCouldn't set picker point for Peter");
	
		qd3DStatus = ::Q3View_StartPicking( mQD3DView, mVertexPicker );
		if ( qd3DStatus != kQ3Success )
			SignalPStr_("\pCouldn't start picking");

		do
		{
			if ( mDocument )
			{
				if ( IS_SURFACE_TOOL( mCurrentTool ) )
				{
					mDocument->SubmitPickSurfaces( mQD3DView );
				}
				else if ( IS_MESH_TOOL( mCurrentTool ) )
				{
					mDocument->SubmitPickMesh( mQD3DView );
				}
				else if ( IS_VERTEX_TOOL( mCurrentTool ) )
				{
					mDocument->SubmitPickVertices( mQD3DView );
				}
			}
		}
		while ( ::Q3View_EndPicking( mQD3DView ) == kQ3ViewStatusRetraverse );
		
		unsigned long hitCount = 0;
		qd3DStatus = ::Q3Pick_GetNumHits(mVertexPicker, &hitCount);
		if (qd3DStatus != kQ3Success)
			SignalPStr_("\pCouldn't get hit list");

		TQ3PickDetail qd3DHitDetail;
		TQ3Int32 q3dHitID;
		// We only want the closest hit, so only look at 1
		if ( hitCount > 1 )
		{
			hitCount = 1;
		}
		for ( unsigned long hitIndex = 0; hitIndex < hitCount ; hitIndex++ )
		{
			qd3DStatus = ::Q3Pick_GetPickDetailValidMask( mVertexPicker, hitIndex, &qd3DHitDetail );
			if (qd3DStatus != kQ3Success)
				SignalPStr_("\pBad hit data");
			
			if ( qd3DHitDetail & kQ3PickDetailMaskPickID )
			{
				qd3DStatus = ::Q3Pick_GetPickDetailData( mVertexPicker, hitIndex, kQ3PickDetailMaskPickID, &q3dHitID );
				if (qd3DStatus != kQ3Success)
					SignalPStr_("\pBad PickID from hit");
				
				if ( mDocument )
				{
					if ( IS_SURFACE_TOOL( mCurrentTool ) )
					{
						mDocument->SetSurfaceFlag( q3dHitID, DataFlagFromTool( mCurrentTool ) );
					}
					else if ( IS_MESH_TOOL( mCurrentTool ) )
					{
						mDocument->SetMeshFlag( q3dHitID, DataFlagFromTool( mCurrentTool ) );
					}
					else if ( IS_VERTEX_TOOL( mCurrentTool ) )
					{
						mDocument->SetVertexFlag( q3dHitID, DataFlagFromTool( mCurrentTool ) );
					}
				}
			}			
		}
		
		this->Refresh();

		::Q3Pick_EmptyHitList(mVertexPicker);
	}
	else
	{
		CNew3DView::ClickSelf(inMouseDown);
	}
}


// ---------------------------------------------------------------------------
//	HandleKeyPress (with TestKeyBit	utility)		
// ---------------------------------------------------------------------------

inline Boolean TestKeyBit(KeyMap& inKeyMap, UInt16 inTestBit)
{
	UInt8* theRealArray = (UInt8*) inKeyMap;
	
	return (theRealArray[inTestBit / 8] & (0x01 << (inTestBit % 8)));	
}

const UInt16 kKey_Control 	= 0x3B;
const UInt16 kKey_Shift		= 0x38;
const UInt16 kKey_Space		= 0x20;

Boolean CgeomView::HandleKeyPress( const EventRecord& inKeyEvent )
{
	Boolean keyHandled 	= false;
	
	if ( inKeyEvent.modifiers & cmdKey )
	{
		keyHandled = LCommander::HandleKeyPress( inKeyEvent );
	}
	else
	{
		UInt8 theChar = (UInt8) ( inKeyEvent.message & charCodeMask );

		KeyMap theKeyState;
		Boolean movementKey = false;

		do
		{
			::GetKeys( theKeyState );
			movementKey = false;
			float modifierScale = 1.0;

			if ( TestKeyBit(theKeyState, kKey_Control) ) {
				modifierScale *= 0.5;
			}
			if ( TestKeyBit(theKeyState, kKey_Shift) ) {
				modifierScale *= 0.2;
			}

			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_PanLeft]) ) {
				CameraPan(3.0 * modifierScale);
				movementKey = true;
			}
							
			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_PanRight]) ) {
				CameraPan(-3.0 * modifierScale);
				movementKey = true;
			}

			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_OrbitLeft]) ) {
				CameraOrbit(-3.0 * modifierScale);
				movementKey = true;
			}
				
			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_OrbitRight]) ) {
				CameraOrbit(3.0 * modifierScale);
				movementKey = true;
			}

			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_OrbitUp]) ) {
				CameraOrbit(0.0, 3.0 * modifierScale);
				movementKey = true;
			}
				
			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_OrbitDown]) ) {
				CameraOrbit(0.0, -3.0 * modifierScale);
				movementKey = true;
			}

			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_ZoomIn]) ) {
				CameraZoom(0.3 * modifierScale * mScale);
				movementKey = true;
			}
				
			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_ZoomOut]) ) {
				CameraZoom(-0.3 * modifierScale * mScale);
				movementKey = true;
			}
				
			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_MoveForward]) ) {
				CameraMove(0.3 * modifierScale * mScale, 0.0);
				movementKey = true;
			}
				
			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_MoveBack]) ) {
				CameraMove(-0.3 * modifierScale * mScale, 0.0);
				movementKey = true;
			}
				
			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_MoveLeft]) ) {
				CameraMove(0.0, 0.3 * modifierScale * mScale);
				movementKey = true;
			}
				
			if ( TestKeyBit(theKeyState, mCameraKeys[kCamera_MoveRight]) ) {
				CameraMove(0.0, -0.3 * modifierScale * mScale);
				movementKey = true;
			}
			 
			if ( movementKey )
			{
				this->Refresh();
				this->UpdatePort();
				keyHandled = true;
			}
		}
		while ( movementKey );
		
		if ( ( theChar == kKey_Space ) && ( inKeyEvent.what == keyDown ) )
		{
			int iSwitchTool = kToolNone;
			
			if ( IS_VIEWER_TOOL( mCurrentTool ) )
			{
				iSwitchTool = mLastEditTool;
			}
			if ( IS_GEOM_TOOL( mCurrentTool ) )
			{
				iSwitchTool = mLastViewTool;
			}
			
			if ( iSwitchTool != kToolNone )
			{
				if ( mButtons[mCurrentTool] )
				{
					mButtons[mCurrentTool]->SetValue( 0 );
				}
			
				if ( mButtons[iSwitchTool] )
				{
					mButtons[iSwitchTool]->SetValue( 1 );
				}
				
				mCurrentTool = iSwitchTool;
			}

			this->Refresh();
			
			mCaption->SetDescriptor( LStr255( 1000, mCurrentTool ) );

			keyHandled = true;
		}
		
		if ( !keyHandled )
			keyHandled = LCommander::HandleKeyPress( inKeyEvent );
	}
	
	return keyHandled;
}


// ---------------------------------------------------------------------------
//	ListenToMessage						
// ---------------------------------------------------------------------------

void CgeomView::ListenToMessage( MessageT inMessage, void* ioParam )
{
	int iPreviousTool = mCurrentTool;
	
	switch( inMessage )
	{
		case kMsgTransparent:
			mCurrentTool = kToolTransparent;
			if ( mDocument )
			{
				mDocument->SetTransparencyLevel( (float) mButtons[kButtonTransparent]->GetValue() / 10.0 * FIXED_ONE );
			}
			break;
		case kMsgNoCull:
			mCurrentTool = kToolNoCull;
			break;
		case kMsgUnderMesh:
			mCurrentTool = kToolUnderMesh;
			break;
		case kMsgReflective:
			mCurrentTool = kToolReflective;
			break;
		case kMsgClosestZ:
			mCurrentTool = kToolClosestZ;
			break;
		case kMsgDependencies:
			mCurrentTool = kToolDependencies;
			if ( mDocument )
			{
				mDocument->ResetDependencySurface();
			}
			break;
		case kMsgReplacesMesh:
			if ( mDocument && mDocument->ValidMeshData() )
			{
				mCurrentTool = kToolReplacesMesh;
			}
			else
			{
			 	if ( !mDisplayedMeshAlert )
			 	{
					mDisplayedMeshAlert = true;
					::ParamText( LStr255( rSTRl_Alerts, eAlertModelTooLarge ), nil, nil, nil );
					UModalAlerts::NoteAlert( ALRT_Generic );
				}
				mButtons[kToolReplacesMesh]->SetValue( 0 );
			}
			break;
		case kMsgNoWalk:
			if ( mDocument && mDocument->ValidMeshData() )
			{
				mCurrentTool = kToolNoWalk;
			}
			else
			{
			 	if ( !mDisplayedMeshAlert )
			 	{
					mDisplayedMeshAlert = true;
					::ParamText( LStr255( rSTRl_Alerts, eAlertModelTooLarge ), nil, nil, nil );
					UModalAlerts::NoteAlert( ALRT_Generic );
				}
				mButtons[kToolNoWalk]->SetValue( 0 );
			}
			break;
		case kMsgNoFly:
			if ( mDocument && mDocument->ValidMeshData() )
			{
				mCurrentTool = kToolNoFly;
			}
			else
			{
			 	if ( !mDisplayedMeshAlert )
			 	{
					mDisplayedMeshAlert = true;
					::ParamText( LStr255( rSTRl_Alerts, eAlertModelTooLarge ), nil, nil, nil );
					UModalAlerts::NoteAlert( ALRT_Generic );
				}
				mButtons[kToolNoFly]->SetValue( 0 );
			}
			break;
		case kMsgOverlapped:
			if ( mDocument && mDocument->ValidMeshData() )
			{
				mCurrentTool = kToolOverlapped;
			}
			else
			{
			 	if ( !mDisplayedMeshAlert )
			 	{
					mDisplayedMeshAlert = true;
					::ParamText( LStr255( rSTRl_Alerts, eAlertModelTooLarge ), nil, nil, nil );
					UModalAlerts::NoteAlert( ALRT_Generic );
				}
				mButtons[kToolOverlapped]->SetValue( 0 );
			}
			break;
		case kMsgDeformsMesh:
			mCurrentTool = kToolDeformsMesh;
			break;
		case kMsgMedia:
			mCurrentTool = kToolMedia;
			break;
		case kMsgWobbles:
			mCurrentTool = kToolWobbles;
			break;
		
		default:
			CNew3DView::ListenToMessage( inMessage, ioParam );
	}
	
	if ( mCurrentTool != iPreviousTool )
	{
		if ( mButtons[iPreviousTool] )
		{
			mButtons[iPreviousTool]->SetValue( 0 );
		}
		if ( mButtons[mCurrentTool] )
		{
			mButtons[mCurrentTool]->SetValue( 1 );
		}

		this->Refresh();
		
		mCaption->SetDescriptor( LStr255( 1000, mCurrentTool ) );
	}

	if ( IS_VIEWER_TOOL( mCurrentTool ) )
	{
		mLastViewTool = mCurrentTool;
	}
	if ( IS_GEOM_TOOL( mCurrentTool ) )
	{
		mLastEditTool = mCurrentTool;
	}
}


// ---------------------------------------------------------------------------
// RenderSelf
// ---------------------------------------------------------------------------

TQ3Status CgeomView::RenderSelf()
{
	TQ3Status theStatus = kQ3Success;
	if ( IS_MESH_TOOL( mLastEditTool ) || IS_VERTEX_TOOL( mLastEditTool ) )
	{
		theStatus = ::Q3Push_Submit( mQD3DView );
		
		TQ3ColorRGB theColorValue;
		theColorValue.r = 0.3;
		theColorValue.g = 0.3;
		theColorValue.b = 0.3;
		::Q3Attribute_Submit( kQ3AttributeTypeDiffuseColor, &theColorValue, mQD3DView );
		theStatus = ::Q3FillStyle_Submit( kQ3FillStyleEdges, mQD3DView );
		
		if ( theStatus == kQ3Success )
		{
 			mDocument->SubmitDrawSurfaces( mQD3DView, 0 );
		}
		
		theStatus = ::Q3Pop_Submit( mQD3DView );
		
		if ( IS_MESH_TOOL( mLastEditTool ) )
		{
			mDocument->SubmitDrawMesh( mQD3DView, DataFlagFromTool( mLastEditTool ) );
		}
		else if ( IS_VERTEX_TOOL( mLastEditTool ) )
		{
			mDocument->SubmitDrawVertices( mQD3DView, DataFlagFromTool( mLastEditTool ) );
		}
	}
	else if ( IS_SURFACE_TOOL( mLastEditTool ) )
	{
 		mDocument->SubmitDrawSurfaces( mQD3DView, DataFlagFromTool( mLastEditTool ) );
	}
	else
	{
 		mDocument->SubmitDrawSurfaces( mQD3DView, 0 );
	}
	
	return theStatus;
}


// ---------------------------------------------------------------------------
// DataFlagFromTool
// ---------------------------------------------------------------------------

 UInt32	CgeomView::DataFlagFromTool( UInt32 inTool )
 {
 	UInt32 flag = 0;
 	
 	switch ( inTool )
 	{
		case kToolTransparent:
			flag = _geometry_surface_is_transparent_flag;
			break;
		case kToolNoCull:
			flag = _geometry_surface_no_backface_cull_flag;
			break;
		case kToolUnderMesh:
			flag = _geometry_surface_under_mesh_flag;
			break;
		case kToolReflective:
			flag = _geometry_surface_reflective_flag;
			break;
		case kToolClosestZ:
			flag = _geometry_surface_uses_closest_z_flag;
			break;
		case kToolDependencies:
			flag = _GEOMETRY_DEPENDENCY_FAKE_FLAG;
			break;
		case kToolReplacesMesh:
			flag = _model_mesh_cell_replaces_mesh_flag;
			break;
		case kToolNoWalk:
			flag = _mesh_cell_is_impassable_walking_flag;
			break;
		case kToolNoFly:
			flag = _mesh_cell_is_impassable_flying_flag;
			break;
		case kToolOverlapped:
			flag = _mesh_cell_overlapped_by_model_flag;
			break;
		case kToolDeformsMesh:
			flag = _vertex_deforms_mesh_flag;
			break;
		case kToolMedia:
			flag = _vertex_is_media_flag;
			break;
		case kToolWobbles:
			flag = _vertex_wobbles_when_media_flag;
			break;
 		default:
 			flag = 0;
 	}
 	
 	return flag;
 }
