/*******************************************************************************\
|																				|
|  CVersCaption.h 1997-1998 John C. Daub.  All rights reserved					|
|																				|
|  See the file "CVersCaption README" for full details, instructions, changes,	|
|  licensing agreement, etc.  Due to the important information included in that	|
|  file, if you did not receive a copy of it, please contact the author for		|
|  a copy.																		|
|																				|
|  John C. Daub <mailto:hsoi@eden.com>											|
|  <http://www.eden.com/~hsoi/>  <http://www.eden.com/~hsoi/prog.html>			|
|																				|
\*******************************************************************************/

#ifndef _H_CVersCaption
#define _H_CVersCaption
#pragma once

#include <PP_Prefix.h>
#include <LCaption.h>

#if PP_Uses_Pragma_Import
	#pragma import on
#endif


enum {
	STR_dot = 9500,			// the period/seperator between numbers
	STR_devel = 9501,		// "1.1d1"
	STR_alpha = 9502,		// "1.1a1"
	STR_beta = 9503,		// "1.1b1"
	STRx_countries = 9500	// the STR# with the country codes, index == code
	
};

class CVersCaption : public LCaption {

public:

		enum { class_ID = 'VCpt' };
	
								CVersCaption();
								CVersCaption( const CVersCaption	&inOriginal );
								CVersCaption( const SPaneInfo		&inPaneInfo,
													ConstStringPtr	inString,
													ResIDT			inTextTraitsID,
													bool			inUseShortString = false,
													bool			inUseCountryCode = false );
								CVersCaption( LStream *inStream );

		virtual					~CVersCaption();

				CVersCaption&	operator=( const CVersCaption &inOriginal );
		
		
		// these are public in case someone might need them, but I
		// generally wouldn't recommend their use
		
		virtual	Uint32			GetVersion() const;
		virtual void			SetVersion( const Uint32 inVersion );


protected:

		virtual	void			DrawSelf();

		virtual void			ActivateSelf();
		virtual void			DeactivateSelf();

		
		// these accessors made protected. I can't see the outside
		// needing this information, but subclasses might
		
		virtual	bool			GetUseShortString() const;
		virtual void			SetUseShortString( const bool inUseShortString );
		
		virtual bool			GetUseCountryCode() const;
		virtual void			SetUseCountryCode( const bool inUseCountryCode );
		
		
private:

				void			InitVersCaption();

				Uint32			mVersion;
				Boolean			mUseShortString;
				Boolean			mUseCountryCode;


};

//========================================================================
//	 CVersCaption inline functions
//========================================================================


//========================================================================
//	 GetVersion						[public]
//========================================================================
//	Returns the version as a Uint32 (NumVersion)

inline
Uint32
CVersCaption::GetVersion() const
{
	return mVersion;
}


//========================================================================
//	 SetVersion						[public]
//========================================================================
//	Sets the version to the given Uint32 (NumVersion)

inline
void
CVersCaption::SetVersion(
	const Uint32 inVersion )
{
	mVersion = inVersion;
}


//========================================================================
//	 GetUseShortString					[protected]
//========================================================================
//	Returns if we're using the shortString or not

inline
bool
CVersCaption::GetUseShortString() const
{
	return mUseShortString;
}


//========================================================================
//	 SetUseShortString					[protected]
//========================================================================
//	Sets the use of the short string to true or false

inline
void
CVersCaption::SetUseShortString(
	const bool inUseShortString )
{
	mUseShortString = inUseShortString;
}


//========================================================================
//	 GetUseCountryCode					[protected]
//========================================================================
//	Returns if we're printing country codes or not

inline
bool
CVersCaption::GetUseCountryCode() const
{
	return mUseCountryCode;
}


//========================================================================
//	 SetUseCountryCode					[protected]
//========================================================================
//	Sets the use of the country code to true or false

inline
void
CVersCaption::SetUseCountryCode(
	const bool inUseCountryCode )
{
	mUseCountryCode = inUseCountryCode;
}



#if PP_Uses_Pragma_Import
	#pragma import reset
#endif

#endif // _H_CVersCaption