/*
Copyright  1997 Sailmaker Software Ltd. All rights reserved.
			<mailto:richardb@sailmaker.co.uk>
			<http://www.sailmaker.co.uk/>

File:		CDecimalEditField.cp
Author:		Richard Buckle
Purpose:	subclass of CInputField, for decimal numbers
Status:		Copyrighted freeware

See read me for instructions and conditions of use.

*/

#include "CDecimalEditField.h"

#pragma mark ----- Instance methods -----
// constructor
CDecimalEditField::CDecimalEditField( LStream *inStream )
	: CInputField( inStream ),
	CDecimalDescription( inStream ),
	mStripTrailingDecimal(false),
	mDoubleValue(0.0)
	{
	inStream->ReadData(&mStripTrailingDecimal, sizeof(Boolean));
	}

// destructor	
CDecimalEditField::~CDecimalEditField()
	{}

#pragma mark -
#pragma mark ----- Public accessors -----
// return the value as a double
double CDecimalEditField::GetDoubleValue()
	{
	return mDoubleValue;
	}

// set the value to the given double
void CDecimalEditField::SetDoubleValue( const double iValue )
	{
	// if underlying value changed, note it
	if(iValue!=mDoubleValue)
		{
		mIsChanged = true;
		}
	mDoubleValue = iValue;
	
	WriteDoubleValue( iValue );
	}
	
#pragma mark -
#pragma mark ----- Conform format -----
// conform our display format
void CDecimalEditField::DoReformatSelf()
	{
	// reformat ourselves
	WriteDoubleValue( mDoubleValue );
	mIsChanged = false;
	}
	
#pragma mark -
#pragma mark ----- PowerPlant overrides -----
// user typed something: update mDoubleValue
void CDecimalEditField::UserChangedText( void )
	{
	// did the underlying value change?
	double newValue = ReadDoubleValue();
	if(newValue!=mDoubleValue)
		{
		mDoubleValue = newValue;
		CInputField::UserChangedText(); // this will broadcast
		}
	}

// we're losing the focus - reformat ourselves
void CDecimalEditField::DontBeTarget( void )
	{
	CInputField::DontBeTarget();
	
	if( mIsChanged )
		{
		DoReformatSelf();
		}
	}

#pragma mark -
#pragma mark ----- Implementation -----
// Get the double value from the text
double CDecimalEditField::ReadDoubleValue()
	{
	Str255 theStr;
	GetDescriptor( theStr );
	return StrToDoubleValue( theStr );
	}
	
// Put the double value to the text
void CDecimalEditField::WriteDoubleValue( const double iValue )
	{
	Str255 theStr;
	DoubleValueToStr( theStr, iValue, mStripTrailingDecimal );
	SetDescriptor( theStr );
	}

