/*
Copyright  1997 Sailmaker Software Ltd. All rights reserved.
			<mailto:richardb@sailmaker.co.uk>
			<http://www.sailmaker.co.uk/>

File:		CDecimalEditText.cp
Author:		Richard Buckle
Purpose:	subclass of LEditText, for decimal numbers
Status:		Copyrighted freeware

See read me for instructions and conditions of use.

*/

#include "CDecimalEditText.h"

#pragma mark ----- Instance methods -----
// constructor
CDecimalEditText::CDecimalEditText( LStream *inStream )
	: 
	LEditText( inStream ),
	CDecimalDescription( inStream ),
	mStripTrailingDecimal(false),
	mIsChanged(false),
	mDoubleValue(0.0)
	{
	inStream->ReadData(&mStripTrailingDecimal, sizeof(Boolean));
	}
	
// destructor	
CDecimalEditText::~CDecimalEditText()
	{}

#pragma mark -
#pragma mark ----- Public accessors -----
// return the value as a double
double CDecimalEditText::GetDoubleValue()
	{
	return mDoubleValue;
	}
	
// set the value to the given double
void CDecimalEditText::SetDoubleValue( const double iValue )
	{
	// if underlying value changed, note it
	if(iValue!=mDoubleValue)
		{
		mIsChanged = true;
		}
	mDoubleValue = iValue;
	
	WriteDoubleValue( iValue );
	}

#pragma mark -
#pragma mark ----- Conform format -----
// conform our display format
void CDecimalEditText::DoReformatSelf()
	{
	// reformat ourselves
	WriteDoubleValue( mDoubleValue );
	mIsChanged = false;
	}

#pragma mark -
#pragma mark ----- PowerPlant overrides -----
// user typed something: update mDoubleValue
void CDecimalEditText::UserChangedText( void )
	{
	// did the underlying value change?
	double newValue = ReadDoubleValue();
	if(newValue!=mDoubleValue)
		{
		mDoubleValue = newValue;
		mIsChanged = true;
		LEditText::UserChangedText(); // this will broadcast
		}
	}

// we're losing the focus - reformat ourselves
void CDecimalEditText::DontBeTarget( void )
	{
	LEditText::DontBeTarget();
	
	if( mIsChanged )
		{
		DoReformatSelf();
		}
	}

// 
Boolean CDecimalEditText::ObeyCommand(
									CommandT	inCommand,
									void*		ioParam)
	{
	switch (inCommand) 
		{
		// for commands that LEditText handles, just pass them up
		case cmd_Cut:
		case cmd_Copy:
		case cmd_Paste:
		case cmd_Clear:
		case cmd_ActionCut:
		case cmd_ActionPaste:
		case cmd_ActionClear:
		case cmd_ActionTyping:
		case cmd_ActionDeleted:
		case msg_TabSelect:
		case cmd_SelectAll:
			break;
		
		// for other commands, reformat ourselves first
		default:
			if( mIsChanged )
				{
				DoReformatSelf();
				}
			break;
		}
	
	return LEditText::ObeyCommand(inCommand, ioParam);
	}

#pragma mark -
#pragma mark ----- Implementation -----
// Get the double value from the text
double CDecimalEditText::ReadDoubleValue()
	{
	Str255 theStr;
	GetDescriptor( theStr );
	return StrToDoubleValue( theStr );
	}
	
// Put the double value to the text
void CDecimalEditText::WriteDoubleValue( const double iValue )
	{
	Str255 theStr;
	DoubleValueToStr( theStr, iValue, mStripTrailingDecimal );
	SetDescriptor( theStr );
	}
	
