/*
Copyright  1997 Sailmaker Software Ltd. All rights reserved.
			<mailto:richardb@sailmaker.co.uk>
			<http://www.sailmaker.co.uk/>

File:		CInputField.cp
Author:		Richard Buckle
Purpose:	subclass of LEditField, broadcasts a message when changed
Status:		Copyrighted freeware

See read me for instructions and conditions of use.

*/

#include "CInputField.h"

CInputField::CInputField( LStream *inStream )
	:LEditField( inStream ),
	mIsChanged(false)
	{
	}

CInputField::~CInputField(void)
	{
	}
	
void CInputField::UserChangedText( void )
	{
	LEditField::UserChangedText();
	mIsChanged = true;
	AnnounceChange();
	}


Boolean CInputField::ObeyCommand(
									CommandT	inCommand,
									void*		ioParam)
	{
	switch (inCommand) 
		{
		// for commands that LEditField handles, just pass them up
		case cmd_Cut:
		case cmd_Copy:
		case cmd_Paste:
		case cmd_Clear:
		case cmd_ActionCut:
		case cmd_ActionPaste:
		case cmd_ActionClear:
		case cmd_ActionTyping:
		case cmd_ActionDeleted:
		case msg_TabSelect:
		case cmd_SelectAll:
			break;
		
		// for other commands, announce any changes first
		default:
			if( mIsChanged )
				{
				AnnounceChange();
				}
			break;
		}
	
	return LEditField::ObeyCommand(inCommand, ioParam);
	}

void CInputField::AnnounceChange()
	{
	// broadcast our message
	BroadcastMessage( (MessageT)GetPaneID() );
	mIsChanged = false;
	}