/*
Copyright  1997 Sailmaker Software Ltd. All rights reserved.
			<mailto:richardb@sailmaker.co.uk>
			<http://www.sailmaker.co.uk/>

File:		CDecimalTestWindow.h
Author:		Richard Buckle
Version:	1.0
Commenced:	25/1/98
Purpose:	test window for decimal edit classes

*/

#include "CDecimalTestWindow.h"
#include "CDecimalCaption.h"
#include "CDecimalEditField.h"
#include "CDecimalStaticText.h"
#include "CDecimalEditText.h"
#include <LSlider.h>

Boolean		CDecimalTestWindow::sIsRegistered = false;

const ResIDT	PPob_DecimalTestWindowID	= 1000;
const ResIDT	RidL_DecimalTestWindowID	= 1000;

const PaneIDT	Pane_LinkedStaticText		= 'Capt';
const PaneIDT	Pane_LinkedEditText			= 'Edit';
const PaneIDT	Pane_LinkedSlider			= 'Slid';

const MessageT	msg_LinkedEditText			= 'Edit';
const MessageT	msg_LinkedSlider			= 'Slid';

//----------
void
CDecimalTestWindow::RegisterClass ()
{
	RegisterClass_(CDecimalTestWindow); 

	// register the pane classes we use
	RegisterClass_(LWindow);
	RegisterClass_(CDecimalCaption);
	RegisterClass_(CDecimalEditField);
	RegisterClass_(CDecimalStaticText);
	RegisterClass_(CDecimalEditText);

	sIsRegistered = true;
}

//----------
CDecimalTestWindow*
CDecimalTestWindow::CreateDecimalTestWindow(
	LCommander			*inSuperCommander)
{
	if (!sIsRegistered) {
		RegisterClass ();
	}

	CDecimalTestWindow		*window;
	window = (CDecimalTestWindow *)CreateWindow(PPob_DecimalTestWindowID, inSuperCommander);
	return window;
}


//----------
CDecimalTestWindow::CDecimalTestWindow()
	: 
	mLinkedStaticText(nil),
	mLinkedEditText(nil),
	mLinkedSlider(nil)
{
}

//----------
CDecimalTestWindow::CDecimalTestWindow (
	LStream*	inStream)
	: LWindow (inStream)
{
}

//----------
CDecimalTestWindow::~CDecimalTestWindow()
{
}

//----------
//	This member function gets called once the containment hierarchy that contains
//	this pane has been built. It gives us a chance to get data members for
//	interesting subviews, and to do other operations now that our subviews exist.
void
CDecimalTestWindow::FinishCreateSelf()
{
	LWindow::FinishCreateSelf();

	UReanimator::LinkListenerToControls(this, this, RidL_DecimalTestWindowID);
		// the purpose is to "connect" self to whatever controls
		// that we want to "listen" to

	// any additional initialization for your window:
	mLinkedStaticText = dynamic_cast<CDecimalStaticText*>(FindPaneByID(Pane_LinkedStaticText));
	Assert_(mLinkedStaticText);
	
	mLinkedEditText = dynamic_cast<CDecimalEditText*>(FindPaneByID(Pane_LinkedEditText));
	Assert_(mLinkedEditText);
	
	mLinkedSlider = dynamic_cast<LSlider*>(FindPaneByID(Pane_LinkedSlider));
	Assert_(mLinkedSlider);
	
	// set some values
	double	theDouble = 123456.789123;
	
	{
	CDecimalCaption* theCaption
		= dynamic_cast<CDecimalCaption*>(FindPaneByID(1));
	Assert_(theCaption);
	theCaption->SetDoubleValue(theDouble);
	}{
	CDecimalEditField* theCaption
		= dynamic_cast<CDecimalEditField*>(FindPaneByID(2));
	Assert_(theCaption);
	theCaption->SetDoubleValue(theDouble);
	}{
	CDecimalStaticText* theCaption
		= dynamic_cast<CDecimalStaticText*>(FindPaneByID(3));
	Assert_(theCaption);
	theCaption->SetDoubleValue(theDouble);
	}{
	CDecimalEditText* theCaption
		= dynamic_cast<CDecimalEditText*>(FindPaneByID(4));
	Assert_(theCaption);
	theCaption->SetDoubleValue(theDouble);
	}{
	CDecimalCaption* theCaption
		= dynamic_cast<CDecimalCaption*>(FindPaneByID(5));
	Assert_(theCaption);
	theCaption->SetDoubleValue(theDouble);
	}{
	CDecimalEditField* theCaption
		= dynamic_cast<CDecimalEditField*>(FindPaneByID(6));
	Assert_(theCaption);
	theCaption->SetDoubleValue(theDouble);
	}{
	CDecimalStaticText* theCaption
		= dynamic_cast<CDecimalStaticText*>(FindPaneByID(7));
	Assert_(theCaption);
	theCaption->SetDoubleValue(theDouble);
	}{
	CDecimalEditText* theCaption
		= dynamic_cast<CDecimalEditText*>(FindPaneByID(8));
	Assert_(theCaption);
	theCaption->SetDoubleValue(theDouble);
	}
}

//----------
void
CDecimalTestWindow::ListenToMessage(
	MessageT	inMessage,
	void		* /*ioParam*/ )
{
	switch (inMessage) {
	
	case msg_LinkedEditText:
		{
			Assert_(mLinkedStaticText);
			Assert_(mLinkedEditText);
			Assert_(mLinkedSlider);
			
			// we stop listening or else the slider will call us back
			StopListening();
			double theValue = mLinkedEditText->GetDoubleValue();
			mLinkedStaticText->SetDoubleValue(theValue);
			mLinkedSlider->SetValue(theValue * 10L);
			StartListening();
		}
		break;
	
	case Pane_LinkedSlider:
		{
			Assert_(mLinkedStaticText);
			Assert_(mLinkedEditText);
			Assert_(mLinkedSlider);
			double theValue = mLinkedSlider->GetValue() / 10.0;
			mLinkedStaticText->SetDoubleValue(theValue);
			mLinkedEditText->SetDoubleValue(theValue);
			// LEditText needs a Draw() for live tracking
			mLinkedEditText->Draw(nil);
		}
		break;
	
	default:
		SignalCStr_("CDecimalTestWindow received unknown message");
		// do nothing
		break;
	}
}

