#ifndef __MODELS_H__
#define __MODELS_H__

#include "data.h"
#include "markers.h"

enum
{
	MAXIMUM_MODELS_PER_MAP= 128,
	MAXIMUM_DESTROYED_MATERIALS_PER_MODEL= 2
};

/* --------- structures */

struct model_marker_user_data
{
	short permutation;
	
	char unused[SIZEOF_MARKER_USER_DATA-sizeof(short)];
};

enum {
	_model_has_reflection_bit,
	NUMBER_OF_MODEL_BITS,
	
	_model_has_reflection_flag= FLAG(_model_has_reflection_bit)
};

#define SIZEOF_MODEL_DATA (96)
struct model_data
{
	DATUM_HEADER;
	word flags;
	
	short type;
	short model_permutation;
	
	world_point3d location;
	world_vector3d up_vector;
	world_vector3d forward_vector;
	world_vector3d scale_vector;
	int x0, x1, y0, y1; 	// bounding rect in cell units
	short destroyed_materials[MAXIMUM_DESTROYED_MATERIALS_PER_MODEL];
	
	short desired_model_permutation;
	fixed_fraction permutation_crossfade_amount; // from [0, FIXED_ONE-1]
	angle current_yaw, current_pitch;
	fixed_fraction transparency;
	
	short unused[13];
};

/* ---------- globals */

extern struct datum_globals model_data;

/* --------- prototypes */

void initialize_models(void);
void initialize_models_for_new_map(void);
void dispose_models_from_old_map(void);

int get_model_definition_index(file_tag tag);
short new_model(struct marker_data *marker);
void add_models_to_new_map(void);
void new_model_palette_entry(struct marker_palette_entry *entry);
void delete_model(short index);
int get_model_count(void);

// match is only external for fear.
int match_precalculated_model_normal(world_vector3d *n);
void get_model_normal(int normal_index, world_vector3d *normal);

boolean object_hit_model(world_point3d *old_position, world_point3d *new_position,
	world_vector3d *collision_normal, int *model_collided_index, world_distance radius);

void byte_swap_model_list(void *models, int count);

int model_identifier_to_model_index(int model_identifier);
void get_model_centerpoint(int model_index, world_point2d *centerpoint);

short get_model_index_from_mesh_cell(short x, short y);

#ifdef DEBUG
#define get_model_data(index) ((struct model_data *)get_datum(&model_data, (index)))
#else
#define get_model_data(index) (((struct model_data *)model_data.data)+(index))
#endif

#endif