// =================================================================================
//	CMapDocument.cp				1997 eli curtz. All rights reserved.
// =================================================================================

#include <LFile.h>
#include <LPlaceHolder.h>
#include <LPrintout.h>
#include <LString.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <UMemoryMgr.h>
#include <UWindows.h>
#include <UModalDialogs.h>
#include <LTableArrayStorage.h>
#include <LString.h>

#include <QD3DGroup.h>
#include <QD3DGeometry.h>

#include "Constants.h"
#include "CMapView.h"
#include "C3DMapView.h"
#include "CMiniIconTable.h"
#include "CGorFile.h"
#include "CMapObject.h"

#include "CMapDocument.h"

// ---------------------------------------------------------------------------------
//		 CMapDocument(LCommander*, FSSpec*)
// ---------------------------------------------------------------------------------


CMapDocument::CMapDocument(
	LCommander	*inSuper,
	FSSpec		*inFileSpec,
	LArray		*inFileArray )
		: LSingleDoc( inSuper ), LListener()
{
	mObjects = nil;
	mMapView = nil;
	m3DMapView = nil;
	
	if (inFileArray)
		mGlobalSourceFiles = inFileArray;
	else
		mGlobalSourceFiles = new LArray(sizeof(CGorFile *));
	mLocalSourceFiles = new LArray(sizeof(CGorFile *));
	
	mObjectIndex = LArray::index_Bad;
	
	// Create window for our document.
	mWindow = LWindow::CreateWindow( rPPob_MapWindow, this );
	Assert_( mWindow != nil );
	
	mTextureGWorld = nil;
	
	// Find the active panes
	LStdControl* theControl;
	theControl = (LStdControl*) mWindow->FindPaneByID(kSceneryControl); Assert_(theControl);
	theControl->AddListener(this); CMapObject::SetTypeVisible(_marker_scenery, theControl->GetValue());

	theControl = (LStdControl*) mWindow->FindPaneByID(kUnitControl); Assert_(theControl);
	theControl->AddListener(this); CMapObject::SetTypeVisible(_marker_monster, theControl->GetValue());

	theControl = (LStdControl*) mWindow->FindPaneByID(kSoundControl); Assert_(theControl);
	theControl->AddListener(this); CMapObject::SetTypeVisible(_marker_sound_source, theControl->GetValue());

	theControl = (LStdControl*) mWindow->FindPaneByID(kModelControl); Assert_(theControl);
	theControl->AddListener(this); CMapObject::SetTypeVisible(_marker_model, theControl->GetValue());

	theControl = (LStdControl*) mWindow->FindPaneByID(kProjectileControl); Assert_(theControl);
	theControl->AddListener(this); CMapObject::SetTypeVisible(_marker_projectile, theControl->GetValue());

	theControl = (LStdControl*) mWindow->FindPaneByID(kLocalProjControl); Assert_(theControl);
	theControl->AddListener(this); CMapObject::SetTypeVisible(_marker_local_projectile_group, theControl->GetValue());

	((LStdControl*) mWindow->FindPaneByID('invs'))->AddListener(this); 
	((LStdControl*) mWindow->FindPaneByID('Zin '))->AddListener(this); 
	((LStdControl*) mWindow->FindPaneByID('Zout'))->AddListener(this);

//hack
	((LStdControl*) mWindow->FindPaneByID('SWAP'))->AddListener(this);
	((LStdControl*) mWindow->FindPaneByID('TXTR'))->AddListener(this);
		
	if ( inFileSpec == nil ) {

		// Set the name of new window.
		NameNewDoc();
		
	} else {
	
		// Display contents of file in window.
		OpenFile( *inFileSpec );
	
	}

	// Make the window visible.
	mWindow->Show();
}

CMapDocument::~CMapDocument()
{
	delete mObjects;

	LArrayIterator theFiles(*mLocalSourceFiles);
	CGorFile* currentFile;
	while(theFiles.Next(&currentFile))
		delete currentFile;

	delete mLocalSourceFiles;
	mGlobalSourceFiles = nil;

	if (mQD3DMapMesh)
		::Q3Object_Dispose(mQD3DMapMesh);
	
	delete mTextureGWorld;
}


// ---------------------------------------------------------------------------------
//		 NameNewDoc
// ---------------------------------------------------------------------------------

void
CMapDocument::NameNewDoc()
{
	// Setup the window title. Start with the default title.
	LStr255	theTitle( "\pUntitled" );

	// Find the first available title. We could also check the window
	// pane id if we wanted to make sure we didn't collide with other
	// window types.
	Int32	theNumber = 0;
	while ( UWindows::FindNamedWindow( theTitle ) != nil ) {

		// An existing window has the current name
		// Increment counter and try again.
		++theNumber;
		theTitle = "\pUntitled ";
		theTitle += (LStr255) theNumber;

	}		
	
	// Finally, set window title.
	mWindow->SetDescriptor( theTitle );
}


// ---------------------------------------------------------------------------------
//		 OpenFile
// ---------------------------------------------------------------------------------

void
CMapDocument::OpenFile(
	FSSpec	&inFileSpec )
{
	Try_ {

		// Create a new file object.
		mFileStream = new LFileStream( inFileSpec );
		mFile = mFileStream;	// In case the superclass is using it
		
		// Open the data fork.
		mFileStream->OpenDataFork( fsRdPerm );
		
		// Set the window title to the name of the file and
		// flag that the document has an associated file.
		mWindow->SetDescriptor( inFileSpec.name );
		mIsSpecified = true;

		// Skip the file header and read the mesh header
		dir_Header theFileHeader;
		Int32 byteCount = sizeof(dir_Header);
		mFileStream->GetBytes(&theFileHeader, byteCount);
		Assert_(theFileHeader.group == 'mesh');
		
		byteCount = sizeof(mesh_Header);
		mFileStream->GetBytes(&mMeshHeader, byteCount);
		mHeaderOffset = sizeof(dir_Header) + mMeshHeader.headerSize;
		
// HACK
		CMapObject::SetMapDimensions(mMeshHeader.height, mMeshHeader.width);

		// Read the object types present on map
		LArray* theTypes[NUMBER_OF_MARKER_TYPES];
		for (Int16 typeCount = 0; typeCount < NUMBER_OF_MARKER_TYPES; typeCount++) {
			theTypes[typeCount] = new LArray(sizeof(mesh_ObjectType));
		}

		mFileStream->SetMarker(mHeaderOffset + mMeshHeader.objectTypeOffset, streamFrom_Start);
		mesh_ObjectType theNewType;
		for (Int16 typeCount = 0; typeCount < mMeshHeader.objectTypeCount; typeCount++) {
			byteCount = sizeof(mesh_ObjectType);
			mFileStream->GetBytes(&theNewType, byteCount);
			theTypes[theNewType.typeFlag]->InsertItemsAt(1, LArray::index_Last, &theNewType);
		}
		CMapObject::SetObjectTypes(theTypes);
		
		// Read the objects
		mFileStream->SetMarker(mHeaderOffset + mMeshHeader.objectInstanceOffset, streamFrom_Start);
		Handle objectHandle = ::NewHandle(mMeshHeader.objectInstanceSize);
		ThrowIfMemFail_(objectHandle);
		byteCount = mMeshHeader.objectInstanceSize;
		{
			StHandleLocker lock(objectHandle);
			mFileStream->GetBytes(*objectHandle, byteCount);	
		}
		LArray tempObjects(sizeof(mesh_ObjectInstance), objectHandle);
		mObjects = new LArray(sizeof(CMapObject*));
		mObjects->AdjustAllocation(tempObjects.GetCount());
		LArrayIterator tempIter(tempObjects);
		mesh_ObjectInstance theObject;
		while(tempIter.Next(&theObject)) {
// HACK
			CMapObject* theNewObject = CMapObject::MakeObject(theObject, mGlobalSourceFiles);
			mObjects->InsertItemsAt(1, LArray::index_Last, &theNewObject);
		}
		
		// Build the GWorld
		BuildGWorld();
		
		// Build the QuickDraw3D map mesh
		BuildQD3DGroup();
		
// HACK
		// Set up the map view
		mMapView = (CMapView*) mWindow->FindPaneByID(kMapView);
		Assert_(mMapView);
		mMapView->SetGWorld(mTextureGWorld);
		mMapView->SetObjects(mObjects);
		mMapView->SetScale(2);
		mMapView->AddListener(this);
		
		// Set up the 3D map view
		m3DMapView = (C3DMapView*) mWindow->FindPaneByID(k3DMapView);
		Assert_(mMapView);
		m3DMapView->SetMapMesh(mQD3DMapMesh);
		m3DMapView->SetGWorld(mTextureGWorld);
		m3DMapView->SetObjects(mObjects);
		m3DMapView->AddListener(this);

		m3DMapView->SetSuperCommander(mWindow);
		mWindow->SetLatentSub(m3DMapView);

	} Catch_( inErr ) {

		// Cleanup and rethrow the error.
		delete this;
		Throw_( inErr );
	
	} EndCatch_
}


// ---------------------------------------------------------------------------------
//		 BuildGWorld()
// ---------------------------------------------------------------------------------

void
CMapDocument::BuildGWorld()
{
	// Locate the map texture
	dir_Header theSegmentHeader;
	LArrayIterator theFiles(*mGlobalSourceFiles);
	CGorFile* mapTextureFile;
	Boolean foundMatch = false;
	
	while(!foundMatch && (theFiles.Next(&mapTextureFile))) {
		foundMatch = mapTextureFile->GetSegmentHeader('.256', mMeshHeader.textureRef, &theSegmentHeader);
	}
	Assert_(foundMatch);

	mapTextureFile->OpenDataFork(fsRdPerm);
	mapTextureFile->SetMarker(theSegmentHeader.dataOffset, streamFrom_Start);

	collection_header theCollectionHeader;
	Int32 byteCount = sizeof(collection_header);
	mapTextureFile->GetBytes(&theCollectionHeader, byteCount);
	Int32 textureHeaderOffset = mapTextureFile->GetMarker();
	Assert_(theCollectionHeader.user_data_flags & _collection_is_color_map_flag);

	// Read the color table (only handle one for now)
	Assert_(theCollectionHeader.color_table_count == 1);
	Ptr theColorData = ::NewPtr(theCollectionHeader.color_tables_size);
	ThrowIfMemFail_(theColorData);
	mapTextureFile->SetMarker(textureHeaderOffset + theCollectionHeader.color_tables_offset, streamFrom_Start);
	byteCount = theCollectionHeader.color_tables_size;
	mapTextureFile->GetBytes(theColorData, byteCount);

	CTabHandle theColorTable = ::GetCTable(40);		// default 8-bit grey color table
	::HLock((Handle) theColorTable);
	Ptr colorSourcePtr = theColorData + 32;			// skip table header
	for (Int16 colorCount = 0; colorCount <= (*theColorTable)->ctSize; colorCount++) {
		(*theColorTable)->ctTable[colorCount].rgb = *((RGBColor*) colorSourcePtr);
		colorSourcePtr += 8;
	}
	::HUnlock((Handle) theColorTable);
	
	::CTabChanged(theColorTable);
	::DisposePtr(theColorData);

	// Make the GWorld
	Rect textureBounds = { 0, 0, 0, 0 };
	textureBounds.bottom = mMeshHeader.height * 256;// 256 pixel sections
	textureBounds.right = mMeshHeader.width * 256;

	LGWorld* theNewGWorld = nil;
	
	theNewGWorld = new LGWorld(textureBounds, 8, 0, theColorTable);
	Assert_(theNewGWorld != nil);

	// Write data into the new GWorld
	GWorldPtr macGWorld = theNewGWorld->GetMacGWorld();	
	if (theNewGWorld->BeginDrawing()) {
		Int32 bitmapRefMarker = textureHeaderOffset + theCollectionHeader.bitmap_references_offset;
		PixMapHandle thePixH = ::GetGWorldPixMap(macGWorld);
		UInt32 realPixRowBytes = (*thePixH)->rowBytes & 0x3FFF;

		for (Uint8 hCount = 0; hCount < mMeshHeader.height; hCount++) {
		for (Uint8 wCount = 0; wCount < mMeshHeader.width; wCount++) {
			// Read the bitmap "reference"
			bitmap_reference theBitmapRef;
			mapTextureFile->SetMarker(bitmapRefMarker, streamFrom_Start);
			byteCount = sizeof(bitmap_reference);
			mapTextureFile->GetBytes(&theBitmapRef, byteCount);
			
			// Read the bitmap data proper
			mapTextureFile->SetMarker(textureHeaderOffset + theBitmapRef.offset, streamFrom_Start);

			Ptr theBitmapData = ::NewPtr(theBitmapRef.size);
			ThrowIfMemFail_(theBitmapData);
			byteCount = theBitmapRef.size;
			mapTextureFile->GetBytes(theBitmapData, byteCount);
				
			bitmapData_Header *bitmapHeaderPtr = (bitmapData_Header*) theBitmapData;
			Assert_(bitmapHeaderPtr->height == 256);
			Assert_(bitmapHeaderPtr->width == 256);
			Ptr bitmapDataPtr = theBitmapData + sizeof(bitmapData_Header);
			bitmapDataPtr += 4;							// We know its map data (test above)
			Assert_(bitmapHeaderPtr->rowBytes != -1);	// No encoded textures

			Ptr realPixPtr = ::GetPixBaseAddr(thePixH);
			realPixPtr += hCount * 256 * realPixRowBytes;

			// Copy the pixel data in the pixmap in reversed (mirrored) order
			// We do this because the stored data is mirrored because of requirements
			// of the 3D display engine which uses a (U, 1-V) coordinate system
			for (UInt16 rowCount = 1; rowCount <= bitmapHeaderPtr->height; rowCount++) {
				Ptr ScratchPtr = realPixPtr + ((mMeshHeader.width - wCount) * 256) - 1;
				for (UInt16 pixelCount = 1; pixelCount <= bitmapHeaderPtr->rowBytes; pixelCount++)
					*ScratchPtr-- = *bitmapDataPtr++;
				
				realPixPtr += realPixRowBytes;
			}

			::DisposePtr(theBitmapData);
			bitmapRefMarker += 2 * sizeof(bitmap_reference);
		}
		}
	}
	theNewGWorld->EndDrawing();

	// Because we can't guarentee that the GWorld will update if we switch it
	// to 16 bit we'll make a new one and force the extra memory. Oh well
	LGWorld* the16BitGWorld = nil;
	the16BitGWorld = new LGWorld(textureBounds, 16, 0, nil);
	Assert_(the16BitGWorld != nil);
	
	theNewGWorld->CopyImage((GrafPort *) the16BitGWorld->GetMacGWorld(), textureBounds, srcCopy, nil);
	delete theNewGWorld;
	
	// Clean up
	mapTextureFile->CloseDataFork;

	// Assign it to our texture GWorld
	if (mTextureGWorld)
		delete mTextureGWorld;

	mTextureGWorld = the16BitGWorld;
}


// ---------------------------------------------------------------------------------
//		 BuildQD3DGroup()
// ---------------------------------------------------------------------------------

// Modified from Bungie's mesh.h
struct mesh_cell {
	Int16 height; 	// (might have a few free bits here)
	UInt16 normal; 	// two 8-bit indexes into the precalculated normal table
	UInt16 flags; 	// includes terrain type of each triangle

	Int16 first_object_index;
};

void
CMapDocument::BuildQD3DGroup()
{
	// Read the mesh
	Int32 byteCount = mMeshHeader.meshSize;
	Ptr theMeshData = ::NewPtr(byteCount);
	ThrowIfMemFail_(theMeshData);

	mFileStream->SetMarker(mHeaderOffset + mMeshHeader.meshOffset, streamFrom_Start);
	mFileStream->GetBytes(theMeshData, byteCount);
	mesh_cell* tempPtr = (mesh_cell*) theMeshData;

	// Make the QD3D Group
	TQ3GroupObject qd3DNewDisplayGroup = ::Q3OrderedDisplayGroup_New();
	if (qd3DNewDisplayGroup == nil)
		SignalPStr_("\pCouldn't allocate display group.");

	TQ3TriGridData theTriGridData;
	TQ3AttributeSet theVertexAttributes;
	UInt16 meshRows = mMeshHeader.height * 16;
	theTriGridData.numRows = meshRows;
	UInt16 meshCols = mMeshHeader.width * 16;
	theTriGridData.numColumns = meshCols;
	theTriGridData.facetAttributeSet = nil;
	theTriGridData.triGridAttributeSet = nil;

	// Read the height map into the TriGrid
	const float heightScale = 1.0 / 1024;
	TQ3Vertex3D* theVertices = new TQ3Vertex3D[meshRows * meshCols];
	for (unsigned long rowIndex = 0; rowIndex < meshRows; rowIndex++) {
		for (unsigned long colIndex = 0; colIndex < meshCols; colIndex++) {
// hack! shouldn't this be -1?
			theVertices[colIndex + rowIndex * meshCols].point.x = meshCols - colIndex;
			theVertices[colIndex + rowIndex * meshCols].point.y = tempPtr->height * heightScale;
			theVertices[colIndex + rowIndex * meshCols].point.z = rowIndex;
			theVertices[colIndex + rowIndex * meshCols].attributeSet = nil;

// HACK - save the mesh flags, we'll need them at some point
			tempPtr++;
		}
	}

	theTriGridData.vertices = theVertices;
	TQ3GeometryObject theNewTriGrid = ::Q3TriGrid_New(&theTriGridData);

	delete [] theVertices;

// HACK what the hell is going on with all these damn scales?!?!
	// Parameterize the surface
	TQ3Param2D theVertexUV;
	float paramUScale = 1.0 / meshCols;
	float paramVScale = 1.0 / meshRows;
	for (unsigned long rowIndex = 0; rowIndex < meshRows; rowIndex++) {
		for (unsigned long colIndex = 0; colIndex < meshCols; colIndex++) {
			theVertexAttributes = ::Q3AttributeSet_New();
//			theVertexUV.u = ((meshCols - 1) - colIndex) * paramUScale;
//			theVertexUV.v = ((meshRows - 1) - rowIndex) * paramVScale;
			theVertexUV.u = ((meshCols) - colIndex) * paramUScale;
			theVertexUV.v = ((meshRows) - rowIndex) * paramVScale;
			::Q3AttributeSet_Add(theVertexAttributes, kQ3AttributeTypeSurfaceUV, &theVertexUV);
			::Q3TriGrid_SetVertexAttributeSet(theNewTriGrid, rowIndex, colIndex, theVertexAttributes);
			::Q3Object_Dispose(theVertexAttributes);
		}
	}
	
	// Set the mesh
	::Q3Group_AddObject(qd3DNewDisplayGroup, theNewTriGrid);
	::Q3Object_Dispose(theNewTriGrid);
	
	mQD3DMapMesh = qd3DNewDisplayGroup;
}


// ---------------------------------------------------------------------------------
//		 ImportHeightMap()
// ---------------------------------------------------------------------------------
pascal Boolean CheckFileType(CInfoPBPtr thePBInfo, Ptr ourData);

pascal Boolean
CheckFileType(CInfoPBPtr thePBInfo, Ptr /* ourData */) {
	Boolean filterOut = true;
	GraphicsImportComponent theImporter;
	FSSpec theFSSpec;
	theFSSpec.vRefNum = thePBInfo->hFileInfo.ioVRefNum;
	theFSSpec.parID = thePBInfo->hFileInfo.ioFlParID;
	::BlockMoveData(thePBInfo->hFileInfo.ioNamePtr, &theFSSpec.name, sizeof(Str63));

	OSErr theErr = ::GetGraphicsImporterForFile(&theFSSpec, &theImporter);
	if (theImporter != nil) {
		filterOut = false;
		theErr = CloseComponent(theImporter);
	}
	
	return filterOut;
}

void
CMapDocument::ImportHeightMap()
{
	StandardFileReply theResults;
	::StandardGetFile(NewFileFilterProc(CheckFileType), -1, nil, &theResults);
	
	if (theResults.sfGood) {
		// Get the TriGrid
		TQ3GroupPosition thePosition;
		TQ3Status qd3dStatus = ::Q3Group_GetFirstPositionOfType(mQD3DMapMesh, kQ3GeometryTypeTriGrid, &thePosition);
		ThrowIf_((qd3dStatus == kQ3Failure) || (thePosition == nil));
		
		TQ3GeometryObject theTriGrid;
		qd3dStatus = ::Q3Group_GetPositionObject(mQD3DMapMesh, thePosition, &theTriGrid);
		ThrowIf_(qd3dStatus == kQ3Failure);
		
		TQ3TriGridData theTriGridData;
		qd3dStatus = ::Q3TriGrid_GetData(theTriGrid, &theTriGridData);
		ThrowIf_(qd3dStatus == kQ3Failure);

		// Load the graphic into a grayscale GWorld
		OSErr theErr = noErr;
		ComponentResult importerErr = noErr;
		Rect boundsRect = { 0, 0, 0, 0 };
		boundsRect.bottom = theTriGridData.numRows;
		boundsRect.right = theTriGridData.numColumns;
		
		GraphicsImportComponent theImporter;
		theErr = ::GetGraphicsImporterForFile(&theResults.sfFile, &theImporter);
		importerErr = ::GraphicsImportSetBoundsRect(theImporter, &boundsRect);
		
		CTabHandle theColorTable = ::GetCTable(40);		// default 8-bit grey color table
		LGWorld theGWorld(boundsRect, 8, 0, theColorTable);
		
		importerErr = ::GraphicsImportSetGWorld(theImporter, theGWorld.GetMacGWorld(), nil);
		importerErr = ::GraphicsImportDraw(theImporter);
		importerErr = CloseComponent(theImporter);

// HACK
float range = 3.0 / 256;
float offset = 0.0;

		// Adjust the TriGrid
		TQ3Point3D thePoint;
		GWorldPtr macGWorld = theGWorld.GetMacGWorld();
		PixMapHandle thePixH = ::GetGWorldPixMap(macGWorld);
		UInt32 realPixRowBytes = (*thePixH)->rowBytes & 0x3FFF;
		::LockPixels(thePixH);

		for (UInt16 rowCount = 0; rowCount < theTriGridData.numRows; rowCount++) {
			UInt8 *realPixPtr = (UInt8 *) ::GetPixBaseAddr(thePixH);
			realPixPtr += rowCount * realPixRowBytes;

			for (UInt16 colCount = 0; colCount < theTriGridData.numColumns; colCount++) {
				qd3dStatus = Q3TriGrid_GetVertexPosition(theTriGrid, rowCount, colCount, &thePoint);
				ThrowIf_(qd3dStatus == kQ3Failure);
				
				// New height value
				thePoint.y = (255 - *realPixPtr++) * range - offset;

				qd3dStatus = Q3TriGrid_SetVertexPosition(theTriGrid, rowCount, colCount, &thePoint);
				ThrowIf_(qd3dStatus == kQ3Failure);
			}
		}
		::UnlockPixels(thePixH);
		
		::Q3Object_Dispose(theTriGrid);	// Q3Group_GetPositionObject() ref counted it
		
		m3DMapView->Refresh();
	}
}


// ---------------------------------------------------------------------------------
//		 IsModified
// ---------------------------------------------------------------------------------

Boolean
CMapDocument::IsModified()
{
	return false;
}


// ---------------------------------------------------------------------------------
//		 DoAESave
// ---------------------------------------------------------------------------------

void
CMapDocument::DoAESave(
	FSSpec	& /*  inFileSpec */,
	OSType	/* inFileType */)
{
/*
	// Delete the existing file object.
	// Note: this does nothing to the actual file on disk.
	delete mFileStream;
	
	// Make a new file object.
	mFileStream = new LFile( inFileSpec );
	
	// Get the proper file type.
	OSType	theFileType = 'TEXT';
	if ( inFileType != fileType_Default ) theFileType = inFileType;

	// Make new file on disk (we'll use
	// SimpleText's creator for this example).
	mFileStream->CreateNewDataFile( 'ttxt', theFileType );
	
	// Write out the data.
	DoSave();

	// Change window title to reflect the new name.
	mWindow->SetDescriptor( inFileSpec.name );

	// Document now has a specified file.
	mIsSpecified = true;
*/
}


// ---------------------------------------------------------------------------------
//		 DoSave
// ---------------------------------------------------------------------------------

void
CMapDocument::DoSave()
{
/*
	// Open the data fork.
	mFileStream->OpenDataFork( fsRdWrPerm );

	// Get the text from the text view.
	Handle	theTextH = mTextView->GetTextHandle();
	
	// Lock the text handle.
	StHandleLocker	theLock( theTextH );
	
	// Write the text to the file.
	mFileStream->WriteDataFork( *theTextH, ::GetHandleSize( theTextH ) );

	// Close the data fork.
	mFileStream->CloseDataFork();

	// Saving makes doc un-dirty.
	mTextView->SetDirty( false );
*/
}


// ---------------------------------------------------------------------------------
//		 DoRevert
// ---------------------------------------------------------------------------------

void
CMapDocument::DoRevert()
{
/*
	// Open the data fork.
	mFileStream->OpenDataFork( fsRdWrPerm );

	// Read the entire file contents and close the file.
	Handle	theTextH = mFileStream->ReadDataFork();
	mFileStream->CloseDataFork();
	
	// Put the contents in the text view
	// and clear the dirty flag.
	mTextView->SetTextHandle( theTextH );
	mTextView->SetDirty( false );
	
	// Dispose of the text.
	::DisposeHandle( theTextH );

	// Refresh the text view.
	mTextView->Refresh();
*/
}


// ---------------------------------------------------------------------------------
//		 DoPrint
// ---------------------------------------------------------------------------------

void
CMapDocument::DoPrint()
{
/*
	// Create the printout.
	LPrintout	*thePrintout = LPrintout::CreatePrintout( rPPob_TextPrintout );
	Assert_( thePrintout != nil );
	
	// Set the print record.
	
	if (mPrintRecordH)
	{
		thePrintout->SetPrintRecord( mPrintRecordH );
	}
	
	// Get the text placeholder.
	LPlaceHolder	*thePlaceholder;
	thePlaceholder = (LPlaceHolder*) thePrintout->FindPaneByID( kTextPlaceholder );
	Assert_( thePlaceholder != nil );
	
	// Install the text view in the placeholder.
	thePlaceholder->InstallOccupant( mTextView, atNone );
	
	// Set the frame size.
	SetPrintFrameSize();
	
	// Print.
	thePrintout->DoPrintJob();
	
	// Delete the printout. The text view is returned
	// to the window when the placeholder is destroyed.
	delete thePrintout;
*/
}


// ---------------------------------------------------------------------------------
//		 SetPrintFrameSize
// ---------------------------------------------------------------------------------

void
CMapDocument::SetPrintFrameSize()
{
/*
	// Get the frame size.
	SDimension16	theFrameSize;
	mTextView->GetFrameSize( theFrameSize );
	
	// Get the text edit record handle.
	TEHandle	theTextEditH = mTextView->GetMacTEH();
	
	// Calculate the number of lines per page.
	Int16	theLinesPerPage;
	theLinesPerPage = theFrameSize.height / (**theTextEditH).lineHeight;

	// Resize the frame to an integral number of lines.
	mTextView->ResizeFrameTo( theFrameSize.width,
		(**theTextEditH).lineHeight * theLinesPerPage, false );

*/
}


// ---------------------------------------------------------------------------
//		 ShowPreferencesDialog
// ---------------------------------------------------------------------------

void
CMapDocument::ShowPreferencesDialog()
{
	StDialogHandler theHandler( rPPob_PrefDialog, this );
	MessageT theDialogResult;
	
	LWindow	*theDialog = theHandler.GetDialog();
	CMiniIconTable *theTable = (CMiniIconTable*) theDialog->FindPaneByID(kGORTable); Assert_(theTable);
	theTable->InsertCols(1, 0);
	
	LStdButton *theAddButton = (LStdButton*) theDialog->FindPaneByID(kAddButton); Assert_(theAddButton);
	theAddButton->AddListener(&theHandler);
	LStdButton *theRemoveButton = (LStdButton*) theDialog->FindPaneByID(kRemoveButton); Assert_(theRemoveButton);
	theRemoveButton->AddListener(&theHandler);

	
	LArrayIterator theFiles(*mGlobalSourceFiles);
	CGorFile* currentFile;
	FSSpec currentFileSpec;
	MIconTableRec theNewItem;
	theNewItem.iconID = rICM_GORFileIcon;
	while(theFiles.Next(&currentFile)) {
		currentFile->GetSpecifier(currentFileSpec);
		
		TString<Str31> fileName = currentFileSpec.name;
		::BlockMoveData(&fileName[0], &theNewItem.name[0], sizeof(Str31));
		
		TableIndexT	rowCount, colCount;
		theTable->GetTableSize(rowCount, colCount);
		theTable->InsertRows(1, rowCount, &theNewItem, 0, true);
	}
	
	Boolean dialogDone = false;
	do {
		while ( (theDialogResult = theHandler.DoDialog()) == msg_Nothing ) {
			if ( theTable->GetValue() == -1 )
				theRemoveButton->Disable();
			else
				theRemoveButton->Enable();
		}; // wait for result
		
		if ( theDialogResult == msg_OK ) {
			dialogDone = true;
		}
		else if ( theDialogResult == msg_Cancel ) {
			dialogDone = true;
		}
		else if ( theDialogResult == msg_Add ) {
			// Deactivate the desktop.
			::UDesktop::Deactivate();

			// Browse for a document.
			SFTypeList			theTypeList = { kMythGorFileType };
			StandardFileReply	theReply;
			::StandardGetFile( nil, 1, theTypeList, &theReply );

			// Activate the desktop.
			::UDesktop::Activate();
			
			// Add the new file
			if ( theReply.sfGood ) {
				MIconTableRec theNewItem;
				theNewItem.iconID = rICM_GORFileIcon;
				TString<Str31> fileName = theReply.sfFile.name;
				::BlockMoveData(&fileName[0], &theNewItem.name[0], sizeof(Str31));
				
				TableIndexT	rowCount, colCount;
				theTable->GetTableSize(rowCount, colCount);
				theTable->InsertRows(1, rowCount, &theNewItem, 0, true);
				
				CGorFile* theNewSource = new CGorFile(theReply.sfFile);
				mGlobalSourceFiles->InsertItemsAt(1, LArray::index_Last, &theNewSource);
			}
		}
		else if ( theDialogResult == msg_Remove ) {
			if (theTable->GetValue() >= 0) {
			}
		}
	} while(!dialogDone);
}


// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
CMapDocument::ObeyCommand(
	CommandT	inCommand,
	void		*ioParam)
{
	Boolean		cmdHandled = true;

	switch (inCommand) {
	
		case cmd_Preferences:
			ShowPreferencesDialog();
			break;
			
		case cmd_ImportHeightMap:
			ImportHeightMap();
			break;

		default:
			cmdHandled = LSingleDoc::ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}

// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	Pass back whether a Command is enabled and/or marked (in a Menu)

void
CMapDocument::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{
	switch (inCommand) {
	
		case cmd_Preferences:
			outEnabled = true;
			break;
			
		case cmd_ImportSubMenu:
			outEnabled = true;
			break;
			
		case cmd_ImportHeightMap:
			outEnabled = true;
			break;
			
		default:
			LSingleDoc::FindCommandStatus(inCommand, outEnabled,
									outUsesMark, outMark, outName);
			break;
	}
}


// ---------------------------------------------------------------------------
//		 ListenToMessage
// ---------------------------------------------------------------------------

void
CMapDocument::ListenToMessage(
	MessageT inMessage,
	void * /* ioParam */)
{
	switch (inMessage) {

		case msg_SceneryToggle:
			CMapObject::SetTypeVisible(_marker_scenery, ((LStdControl*) mWindow->FindPaneByID(kSceneryControl))->GetValue());
			mMapView->Refresh();
			m3DMapView->Refresh();
			break;
		
		case msg_UnitToggle:
			CMapObject::SetTypeVisible(_marker_monster, ((LStdControl*) mWindow->FindPaneByID(kUnitControl))->GetValue());
			mMapView->Refresh();
			m3DMapView->Refresh();
			break;
		
		case msg_SoundToggle:
			CMapObject::SetTypeVisible(_marker_sound_source, ((LStdControl*) mWindow->FindPaneByID(kSoundControl))->GetValue());
			mMapView->Refresh();
			m3DMapView->Refresh();
			break;
		
		case msg_ModelToggle:
			CMapObject::SetTypeVisible(_marker_model, ((LStdControl*) mWindow->FindPaneByID(kModelControl))->GetValue());
			mMapView->Refresh();
			m3DMapView->Refresh();
			break;
		
		case msg_ProjectileToggle:
			CMapObject::SetTypeVisible(_marker_projectile, ((LStdControl*) mWindow->FindPaneByID(kProjectileControl))->GetValue());
			mMapView->Refresh();
			m3DMapView->Refresh();
			break;
		
		case msg_LocalProjToggle:
			CMapObject::SetTypeVisible(_marker_local_projectile_group, ((LStdControl*) mWindow->FindPaneByID(kLocalProjControl))->GetValue());
			mMapView->Refresh();
			m3DMapView->Refresh();
			break;
		
		case msg_ZoomIn: {
			UInt8 scale = mMapView->GetScale();
			if (scale > 1)
				mMapView->SetScale(scale / 2);
			}
			break;
		
		case msg_ZoomOut: {
			UInt8 scale = mMapView->GetScale();
			if (scale < 4)
				mMapView->SetScale(scale * 2);
			}
			break;
		
		case msg_InvisibleToggle:
			CMapObject::SetShowInvisible(((LStdControl*) mWindow->FindPaneByID('invs'))->GetValue());
			mMapView->Refresh();
			m3DMapView->Refresh();
			break;
//HACK
		case 8888:
			if (mMapView->IsVisible()) {
				mMapView->Hide();
				m3DMapView->Show();
			}
			else {
				mMapView->Show();
				m3DMapView->Hide();
			}
			break;

		case 7777:
			if (m3DMapView->IsVisible()) {
				m3DMapView->SetUseTexture(!m3DMapView->GetUseTexture());
			}
			break;
	}
	/* no inherited to call */
}
