// =================================================================================
//	CMapModel.cp				1997 eli curtz. All rights reserved.
// =================================================================================

#include "mode.h"
#include "Constants.h"
#include "CGorFile.h"

#include <QD3DMath.h>
#include <QD3DGroup.h>
#include <QD3DGeometry.h>
#include <QD3DTransform.h>

#include "CMapModel.h"

// hack
struct model_permutation_data {
	short collection_reference_permutation;	

	byte permutations[/*MAXIMUM_MATERIALS_PER_GEOMETRY*/ 32]; // permutations for the given material

	short unused[13];
	short collection_index; // loaded at run time
	short color_table_index; // at run time...
};


// ---------------------------------------------------------------------------
//	 CMapModel						
// ---------------------------------------------------------------------------

CMapModel::CMapModel(mesh_ObjectInstance inObject, LArray* inSourceFiles)
	: CMapObject(inObject)
{
	mDisplayGroup = nil;

	// Locate the model data
	FourCharCode modelGroup, modelSubGroup;
	Int16 ignoreTeam;
	GetObjectInfo(modelGroup, modelSubGroup, ignoreTeam);

	dir_Header theSegmentHeader;
	LArrayIterator theFiles(*inSourceFiles);
	CGorFile* modelFile;
	Boolean foundMatch = false;
	
	while(!foundMatch && (theFiles.Next(&modelFile))) {
		foundMatch = modelFile->GetSegmentHeader(modelGroup, modelSubGroup, &theSegmentHeader);
	}
	Assert_(foundMatch);

	// Read the model information
	modelFile->OpenDataFork(fsRdPerm);
	modelFile->SetMarker(theSegmentHeader.dataOffset, streamFrom_Start);
	
	mode_Header theModelHeader;
	Int32 byteCount = sizeof(mode_Header);
	modelFile->GetBytes(&theModelHeader, byteCount);
	Int32 modelHeaderOffset = modelFile->GetMarker();
	
	modelFile->SetMarker(modelHeaderOffset + theModelHeader.offset2, streamFrom_Start);
	model_permutation_data* thePerms = new model_permutation_data[theModelHeader.variationCount];
	byteCount = theModelHeader.size2;
	modelFile->GetBytes(thePerms, byteCount);
		
	modelFile->CloseDataFork();
	
	// Locate the geometry data
	theFiles.ResetTo(LArrayIterator::index_BeforeStart);
	CGorFile* geometryFile;
	foundMatch = false;
	
	while(!foundMatch && (theFiles.Next(&geometryFile))) {
		foundMatch = geometryFile->GetSegmentHeader('geom', theModelHeader.geomID, &theSegmentHeader);
	}
	Assert_(foundMatch);

	geometryFile->OpenDataFork(fsRdPerm);
	geometryFile->SetMarker(theSegmentHeader.dataOffset, streamFrom_Start);

	geom_Header theGeometryHeader;
	byteCount = sizeof(geom_Header);
	geometryFile->GetBytes(&theGeometryHeader, byteCount);
	Int32 headerOffset = geometryFile->GetMarker();

	// Read the section references
	geometryFile->SetMarker(headerOffset + theGeometryHeader.refOffset, streamFrom_Start);
	geom_Ref* theReferences = new geom_Ref[theGeometryHeader.refCount];
	byteCount = theGeometryHeader.refSize;
	geometryFile->GetBytes(theReferences, byteCount);
	
	// Read the geometry data
	mDisplayGroup = ::Q3OrderedDisplayGroup_New();
	ThrowIfNil_(mDisplayGroup);

	TQ3PolyhedronData thePolyhedronData;
	thePolyhedronData.numVertices = theGeometryHeader.vertexCount;
	thePolyhedronData.vertices = nil;
	thePolyhedronData.numEdges = 0;
	thePolyhedronData.edges = nil;
	thePolyhedronData.numTriangles = theGeometryHeader.triangleCount;
	thePolyhedronData.triangles = nil;
	thePolyhedronData.polyhedronAttributeSet = nil;
	
	geometryFile->SetMarker(headerOffset + theGeometryHeader.vertexOffset, streamFrom_Start);
	TQ3Vertex3D** theVertexSets = new TQ3Vertex3D*[theGeometryHeader.refCount];
	for (Int32 refCounter = 0; refCounter < theGeometryHeader.refCount; refCounter++) {
		theVertexSets[refCounter] = new TQ3Vertex3D[thePolyhedronData.numVertices];
	}

	float scale = 1.0 / 1024;
	geom_Vertex theVertex;
	for (Int32 vertexCount = 0; vertexCount < thePolyhedronData.numVertices; vertexCount++) {
		byteCount = sizeof(geom_Vertex);
		geometryFile->GetBytes(&theVertex, byteCount);
		for (Int32 refCounter = 0; refCounter < theGeometryHeader.refCount; refCounter++) {
//			theVertexSets[refCounter][vertexCount].point.x = theVertex.x * scale;
// HACK coordinate nastiness again
			theVertexSets[refCounter][vertexCount].point.x = 0.0 - (theVertex.x * scale);
// HACK - Coordinate system hack
			theVertexSets[refCounter][vertexCount].point.y = theVertex.z * scale;
			theVertexSets[refCounter][vertexCount].point.z = theVertex.y * scale;
			theVertexSets[refCounter][vertexCount].attributeSet = ::Q3AttributeSet_New();
			ThrowIfNil_(theVertexSets[refCounter][vertexCount].attributeSet);
		}
	}

	LArray** theTriangleSets = new LArray*[theGeometryHeader.refCount];
	for (Int32 refCounter = 0; refCounter < theGeometryHeader.refCount; refCounter++) {
		theTriangleSets[refCounter] = new LArray(sizeof(TQ3PolyhedronTriangleData));
		ThrowIfNil_(theTriangleSets[refCounter]);
	}

	geometryFile->SetMarker(headerOffset + theGeometryHeader.triangleOffset, streamFrom_Start);
	TQ3PolyhedronTriangleData theQD3DTriangle;
	geom_Triangle theTriangle;
	for (Int32 triangleCount = 0; triangleCount < thePolyhedronData.numTriangles; triangleCount++) {
		byteCount = sizeof(geom_Triangle);
		geometryFile->GetBytes(&theTriangle, byteCount);
		theQD3DTriangle.vertexIndices[0] = theTriangle.vertices[0].vertexIndex;
		theQD3DTriangle.vertexIndices[1] = theTriangle.vertices[1].vertexIndex;
		theQD3DTriangle.vertexIndices[2] = theTriangle.vertices[2].vertexIndex;
		theQD3DTriangle.edgeFlag = kQ3PolyhedronEdgeAll;
		theQD3DTriangle.triangleAttributeSet = nil;
		Assert_(theTriangle.refIDNumber >= 0);
		Assert_(theTriangle.refIDNumber < theGeometryHeader.refCount);
		theTriangleSets[theTriangle.refIDNumber]->InsertItemsAt(1, LArray::index_Last, &theQD3DTriangle);

		TQ3Param2D theVertexUV;
		for (Int16 vertexCount = 0; vertexCount < 3; vertexCount++) {
			float scratchU = theTriangle.vertices[vertexCount].textureU;
			float scratchV = theTriangle.vertices[vertexCount].textureV;
			theVertexUV.v = 1 - (scratchU / 0xFFFF);
			theVertexUV.u = scratchV / 0xFFFF;
			Assert_((theVertexUV.u >= 0.0) && (theVertexUV.u <= 1.0));
			Assert_((theVertexUV.v >= 0.0) && (theVertexUV.v <= 1.0));

			TQ3AttributeSet theVertexAttributes = theVertexSets[theTriangle.refIDNumber][theTriangle.vertices[vertexCount].vertexIndex].attributeSet;
			if (::Q3AttributeSet_Contains(theVertexAttributes, kQ3AttributeTypeSurfaceUV) == kQ3False)
				::Q3AttributeSet_Add(theVertexAttributes, kQ3AttributeTypeSurfaceUV, &theVertexUV);
		}
	}
	
	CGorFile* coreFile;
	foundMatch = false;
	theFiles.ResetTo(LArrayIterator::index_BeforeStart);
	
	while(!foundMatch && (theFiles.Next(&coreFile))) {
		foundMatch = coreFile->GetSegmentHeader('core', theGeometryHeader.coreId, &theSegmentHeader);
	}
	Assert_(foundMatch);

	// Read the core information
	coreFile->OpenDataFork(fsRdPerm);
	coreFile->SetMarker(theSegmentHeader.dataOffset, streamFrom_Start);
	FourCharCode theTextureID;
	byteCount = sizeof(FourCharCode);
	coreFile->GetBytes(&theTextureID, byteCount);
	coreFile->CloseDataFork();
	
	CGorFile* textureFile;
	foundMatch = false;
	theFiles.ResetTo(LArrayIterator::index_BeforeStart);
	
	while(!foundMatch && (theFiles.Next(&textureFile))) {
		foundMatch = textureFile->GetSegmentHeader('.256', theTextureID, &theSegmentHeader);
	}
	Assert_(foundMatch);

	textureFile->OpenDataFork(fsRdPerm);
	textureFile->SetMarker(theSegmentHeader.dataOffset, streamFrom_Start);

	collection_header theCollectionHeader;
	byteCount = sizeof(collection_header);
	textureFile->GetBytes(&theCollectionHeader, byteCount);
	Int32 textureHeaderOffset = textureFile->GetMarker();
	
	textureFile->SetMarker(textureHeaderOffset + theCollectionHeader.sequence_references_offset, streamFrom_Start);
	sequence_reference* theSequences = new sequence_reference[theCollectionHeader.sequence_count];
	for (UInt8 seqCount = 0; seqCount < theCollectionHeader.sequence_count; seqCount++) {
		byteCount = sizeof(sequence_reference);
		textureFile->GetBytes(&theSequences[seqCount], byteCount);
	}

	for (UInt8 seqCount = 0; seqCount < theCollectionHeader.sequence_count; seqCount++) {
		byteCount = theSequences[seqCount].size;
		sequence_data* theSequence = (sequence_data*) ::NewPtr(byteCount);
		textureFile->SetMarker(textureHeaderOffset + theSequences[seqCount].offset, streamFrom_Start);
		textureFile->GetBytes(theSequence, byteCount);
	}
	
	Assert_(sizeof(bitmap_instance_data) == SIZEOF_STRUCT_BITMAP_INSTANCE_DATA);
	textureFile->SetMarker(textureHeaderOffset + theCollectionHeader.bitmap_instances_offset, streamFrom_Start);
	bitmap_instance_data* theInstances = new bitmap_instance_data[theCollectionHeader.bitmap_instance_count];
	for (UInt8 instCount = 0; instCount < theCollectionHeader.bitmap_instance_count; instCount++) {
		byteCount = sizeof(bitmap_instance_data);
		textureFile->GetBytes(&theInstances[instCount], byteCount);
	}
	
	
	textureFile->CloseDataFork();

	// Build the polygons
	for (Int32 refCounter = 0; refCounter < theGeometryHeader.refCount; refCounter++) {
		thePolyhedronData.vertices = theVertexSets[refCounter];
		theTriangleSets[refCounter]->Lock();
		Handle theTriangleH = theTriangleSets[refCounter]->GetItemsHandle();
		thePolyhedronData.numTriangles = theTriangleSets[refCounter]->GetCount();
		thePolyhedronData.triangles = (TQ3PolyhedronTriangleData*) *theTriangleH;

//		thePolyhedronData.polyhedronAttributeSet = ::Q3AttributeSet_New();
// HAck - color the model so I can see it
thePolyhedronData.polyhedronAttributeSet = CMapObject::sColorAttributes[_marker_model];
		TQ3GeometryObject theNewPolyhedron = ::Q3Polyhedron_New(&thePolyhedronData);

		// Add the texture
//		if (theReferences[refCounter].flags == 1)
//			AddTexture(theNewPolyhedron, theTextureFile, theReferences[refCounter].textureIndex);
//			else if (refCounter == 0)
//				Debugger();

		// Put it into the group
		::Q3Group_AddObject(mDisplayGroup, theNewPolyhedron);
		::Q3Object_Dispose(theNewPolyhedron);

		theTriangleSets[refCounter]->Unlock();
		delete theTriangleSets[refCounter];
		delete theVertexSets[refCounter];
	}
	
	delete [] theTriangleSets;
	delete [] theVertexSets;
	
	geometryFile->CloseDataFork();
	

// Hack - try to position the model
TQ3Matrix4x4 theMatrix;
::Q3Matrix4x4_SetIdentity(&theMatrix);

SPosition32 objectPos = GetPositionXYZ();
float transX = (float) objectPos.x / 1024;
float transY = (float) objectPos.y / 1024;
float transZ = (float) objectPos.z / 1024;

::Q3Matrix4x4_SetTranslate(&theMatrix, transX, transY, transZ);
TQ3TransformObject theTransform = ::Q3MatrixTransform_New(&theMatrix);
::Q3Group_AddObject(mDisplayGroup, theTransform);

float scaleAngle = 2 * kQ3Pi / 0xFFFF;
float rotationAngle = (float) mObject.facingAngle * scaleAngle - (kQ3Pi / 2);
::Q3Matrix4x4_SetRotate_Y(&theMatrix, rotationAngle);
theTransform = ::Q3MatrixTransform_New(&theMatrix);
::Q3Group_AddObject(mDisplayGroup, theTransform);

::Q3Object_Dispose(theTransform);
}


// ---------------------------------------------------------------------------
//	 ~CMapModel						
// ---------------------------------------------------------------------------

CMapModel::~CMapModel()
{
	if (mDisplayGroup)
		::Q3Object_Dispose(mDisplayGroup);
}


// ---------------------------------------------------------------------------
//	 GetDisplayGroup()					
// ---------------------------------------------------------------------------

TQ3GroupObject
CMapModel::GetDisplayGroup()
{
	return mDisplayGroup;
}
