// =================================================================================
//	CMapObject.cp				1997 eli curtz. All rights reserved.
// =================================================================================

#include <fp.h>

#include <QD3DSet.h>

#include "Constants.h"
#include "CMapModel.h"

#include "CMapObject.h"

Boolean CMapObject::sTypeVisible[] = { false, false, false, false, false, false, false, false, false, false, false };
Boolean CMapObject::sShowInvisible = false;
SDimension32 CMapObject::sMapDimensions = { 0, 0 };
LArray* CMapObject::sObjectTypes[] = { nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil };
TQ3AttributeSet CMapObject::sColorAttributes[] = { nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil };
RGBColor CMapObject::sColors[] = {
	{ 0x0000, 0x0000, 0x0000 },
	{ 0x4FFF, 0x00FF, 0xFFFF },
	{ 0x0000, 0x0000, 0x0000 },
	{ 0xFFFF, 0x0FFF, 0x0FFF },
	{ 0x0000, 0x0000, 0x0000 },
	{ 0x00FF, 0x4FFF, 0x2FFF },
	{ 0x0FFF, 0x0FFF, 0xFFFF },
	{ 0x0000, 0x0000, 0x0000 },
	{ 0x0000, 0x0000, 0x0000 },
	{ 0xFFFF, 0xFFFF, 0x0000 },
	{ 0x0000, 0xFFFF, 0xFFFF },
};

// ---------------------------------------------------------------------------
//	 InitializeColors()						
// ---------------------------------------------------------------------------

void
CMapObject::InitializeColors()
{
	TQ3ColorRGB theColorValue;
	for (Int8 colorCount = 0; colorCount < NUMBER_OF_MARKER_TYPES; colorCount++) {
		sColorAttributes[colorCount] = ::Q3AttributeSet_New();
		theColorValue.r = (float) sColors[colorCount].red / 0xFFFF;
		theColorValue.g = (float) sColors[colorCount].green / 0xFFFF;
		theColorValue.b = (float) sColors[colorCount].blue / 0xFFFF;
		::Q3AttributeSet_Add(sColorAttributes[colorCount], kQ3AttributeTypeDiffuseColor, &theColorValue);
	}	
}

// ---------------------------------------------------------------------------
//	 MakeObject(mesh_ObjectInstance, LArray*)						
// ---------------------------------------------------------------------------

CMapObject *
CMapObject::MakeObject(mesh_ObjectInstance inObject, LArray* inSourceFiles)
{
	CMapObject* theObject = nil;
	
	switch(inObject.typeFlag) {
		case _marker_model:
			theObject = new CMapModel(inObject, inSourceFiles);
			break;
			
		default:
			theObject = new CMapObject(inObject);
	}
	
	return theObject;
}

// ---------------------------------------------------------------------------
//	 SetObjectTypes(LArray**)						
// ---------------------------------------------------------------------------

void
CMapObject::SetObjectTypes(LArray** inTypeArrays)
{
	for (Int8 arrayCount = 0; arrayCount < NUMBER_OF_MARKER_TYPES; arrayCount++) {
		sObjectTypes[arrayCount] = inTypeArrays[arrayCount];
	}
}

// ---------------------------------------------------------------------------
//	 SetTypeVisible(Int8, Boolean)						
// ---------------------------------------------------------------------------

void
CMapObject::SetTypeVisible(Int8 inType, Boolean inVisible)
{
	if (inType < NUMBER_OF_MARKER_TYPES) {
		sTypeVisible[inType] = inVisible;
	}
}

// ---------------------------------------------------------------------------
//	 SetMapDimensions(Int8, Int8)						
// ---------------------------------------------------------------------------

void
CMapObject::SetMapDimensions(Int8 inWorldHeigth, Int8 inWorldWidth)
{
	sMapDimensions.height = inWorldHeigth * 16 * 1024;
	sMapDimensions.width = inWorldWidth * 16 * 1024;
}

// ---------------------------------------------------------------------------
//	 CMapObject						
// ---------------------------------------------------------------------------

CMapObject::CMapObject(mesh_ObjectInstance inObject)
{
	mObject = inObject;
	
	mWindow = nil;
}

// ---------------------------------------------------------------------------
//	 ~CMapObject						
// ---------------------------------------------------------------------------

CMapObject::~CMapObject()
{
	if (mWindow)
		delete mWindow;
}

// ---------------------------------------------------------------------------
//	 MoveBy(Int32, Int32, Int32)						
// ---------------------------------------------------------------------------

void
CMapObject::MoveBy(Int32 inDeltaX, Int32 inDeltaY, Int32 inDeltaZ)
{
}

// ---------------------------------------------------------------------------
//	 MoveTo(Int32, Int32, Int32, Boolean)						
// ---------------------------------------------------------------------------

void
CMapObject::MoveTo(Int32 inX, Int32 inY, Int32 inZ, Boolean inGrounded)
{
}

// ---------------------------------------------------------------------------
//	 RotateBy(float)						
// ---------------------------------------------------------------------------

void
CMapObject::RotateBy(float inDeltaAngle)
{
}

// ---------------------------------------------------------------------------
//	 RotateTo(float)						
// ---------------------------------------------------------------------------

void
CMapObject::RotateTo(float inAngle)
{
}


// ---------------------------------------------------------------------------
//	 IsVisible()						
// ---------------------------------------------------------------------------

Boolean
CMapObject::IsVisible()
{
	if (sTypeVisible[mObject.typeFlag] && (!(mObject.visibilityFlags) || sShowInvisible))
		return true;
	else
		return false;
}

	
// ---------------------------------------------------------------------------
//	 IsVisibleHV(SRect32&)						
// ---------------------------------------------------------------------------

Boolean
CMapObject::IsVisibleHV(SRect32& inVisRect)
{
	SPoint32 position;
	position.h = sMapDimensions.width - mObject.locationX;
	position.v = mObject.locationY;
	
	if ((position.h >= inVisRect.left) &&
		(position.h <= inVisRect.right) &&
		(position.v >= inVisRect.top) &&
		(position.v <= inVisRect.bottom) &&
		(sTypeVisible[mObject.typeFlag] && (!(mObject.visibilityFlags) || sShowInvisible))
		)
		return true;
	else
		return false;
}

// ---------------------------------------------------------------------------
//	 GetPositionHV()						
// ---------------------------------------------------------------------------

SPoint32
CMapObject::GetPositionHV()
{
	SPoint32 position;
	position.h = sMapDimensions.width - mObject.locationX;
	position.v = mObject.locationY;
	
	return position;
}


// ---------------------------------------------------------------------------
//	 GetPositionXYZ()						
// ---------------------------------------------------------------------------

SPosition32
CMapObject::GetPositionXYZ()
{
//	Hack the data type is currently mislabeled

	SPosition32 position;
	position.x = sMapDimensions.width - mObject.locationX;
	position.z = mObject.locationY;
	position.y = mObject.locationZ;
	
	return position;
}

// ---------------------------------------------------------------------------
//	 GetDisplayGroup()						
// ---------------------------------------------------------------------------

TQ3GroupObject
CMapObject::GetDisplayGroup()
{
	return nil;
}


// ---------------------------------------------------------------------------
//	 GetObjectInfo()						
// ---------------------------------------------------------------------------

void
CMapObject::GetObjectInfo(FourCharCode& outGroup, FourCharCode& outSubGroup, Int16& outTeam)
{
	// Get the associated object type, remembering LArray INDICES are 1 based (sheesh)
	mesh_ObjectType theObjectType;
	::BlockMoveData(sObjectTypes[mObject.typeFlag]->GetItemPtr(mObject.typeIndex + 1), &theObjectType, sizeof(mesh_ObjectType));
	
	// Get the appropriate group tag
	FourCharCode objectTypeGroup = '????';
	switch (theObjectType.typeFlag) {
		case _marker_scenery:
			objectTypeGroup = 'scen';
			break;
		case _marker_monster:
			objectTypeGroup = 'unit';
			break;
		case _marker_sound_source:
			objectTypeGroup = 'amso';
			break;
		case _marker_model:
			objectTypeGroup = 'mode';
			break;
		case _marker_projectile:
			objectTypeGroup = 'prgr';
			break;
		case _marker_local_projectile_group:
			objectTypeGroup = 'lpgr';
			break;
		default:
			SignalPStr_("\pUnused marker type located.");
	}

	outGroup = 	objectTypeGroup;
	outSubGroup = theObjectType.subGroupTag;
	outTeam = theObjectType.teamGroupNumber;
}


// ---------------------------------------------------------------------------
//	 DisplayObjectWindow()						
// ---------------------------------------------------------------------------

void
CMapObject::DisplayObjectWindow()
{
	FourCharCode objGroup, objSubGroup;
	Int16 objTeam;
	GetObjectInfo(objGroup, objSubGroup, objTeam);
	
	// try to get the name from our source files
/*	dir_Header theTestHeader;
	LArrayIterator theFiles(*mGlobalSourceFiles);
	CGorFile* currentFile;
	Boolean foundMatch = false;
	
	while(!foundMatch && (theFiles.Next(&currentFile))) {
		foundMatch = currentFile->GetSegmentHeader(objectTypeGroup, theObjectType.subGroupTag, &theTestHeader);
	}
	
	if (foundMatch)
		mTypeCaption->SetDescriptor(LStr255(theTestHeader.name));
	else
*/
	// Make the display window
	mWindow = LWindow::CreateWindow( rPPob_ObjectWindow, LCommander::GetTarget() );
	Assert_(mWindow);
	
	((LCaption*) mWindow->FindPaneByID(kObjectType))->SetDescriptor(LStr255(objSubGroup));
		
	long tempLong = objTeam;
	((LCaption*) mWindow->FindPaneByID(kObjectTeam))->SetDescriptor(LStr255(tempLong));

	SPoint32 posXY = GetPositionHV();
	LStr255 tempString;
	float tempValue = posXY.h; tempValue /= 1024;
	tempString.Assign(tempValue, FIXEDDECIMAL, 3);
	((LCaption*) mWindow->FindPaneByID(kObjectXPos))->SetDescriptor(tempString);
	
	tempValue = posXY.v; tempValue /= 1024;
	tempString.Assign(tempValue, FIXEDDECIMAL, 3);
	((LCaption*) mWindow->FindPaneByID(kObjectYPos))->SetDescriptor(tempString);
	
	tempValue = mObject.locationZ; tempValue /= 1024;
	tempString.Assign(tempValue, FIXEDDECIMAL, 3);
	((LCaption*) mWindow->FindPaneByID(kObjectZPos))->SetDescriptor(tempString);
	
	tempValue = mObject.facingAngle; tempValue /= 0xFFFF; tempValue *= 360;
	tempString.Assign(tempValue, FIXEDDECIMAL, 3);
	((LCaption*) mWindow->FindPaneByID(kObjectFacing))->SetDescriptor(tempString);

	tempLong = mObject.objectID;
	((LCaption*) mWindow->FindPaneByID(kObjectID))->SetDescriptor(LStr255(tempLong));

	((LStdControl*) mWindow->FindPaneByID(kObjectHidden))->SetValue(mObject.visibilityFlags);
}
