// =================================================================================
//	CMapObject.h				1997 eli curtz. All rights reserved.
// =================================================================================

#pragma once

#include <LPane.h>
#include <LCommander.h>

#include <QD3D.h>

#include "gorStructs.h"

// From markers.h
enum // marker types
{
	_marker_team, // no tag
	_marker_scenery,
	_marker_item,
	_marker_monster, // refers to unit tag
	_marker_projectile_generator,
	_marker_sound_source,
	_marker_model,
	_marker_media_current,
	_marker_script,
	_marker_projectile,
	_marker_local_projectile_group,
	NUMBER_OF_MARKER_TYPES
};

struct	SPosition32	{
	Int32	x;
	Int32	y;
	Int32	z;
};


class CMapObject{
public:
					CMapObject(mesh_ObjectInstance inObject);
					~CMapObject();
	
	static CMapObject*	MakeObject(mesh_ObjectInstance inObject, LArray* inSourceFiles);
	
	void			MoveBy(Int32 inDeltaX, Int32 inDeltaY, Int32 inDeltaZ = 0);
	void			MoveTo(Int32 inX, Int32 inY, Int32 inZ = 0, Boolean inGrounded = false);
	
	void			RotateBy(float inDeltaAngle);
	void			RotateTo(float inAngle);
	
	Boolean			IsVisible();
	Boolean			IsVisibleHV(SRect32& inVisRect);
	SPoint32		GetPositionHV();
	SPosition32		GetPositionXYZ();

	Int16			GetTypeIndex() { return mObject.typeFlag; };
	void			GetObjectInfo(FourCharCode& outGroup, FourCharCode& outSubGroup, Int16& outTeam);
	virtual TQ3GroupObject	GetDisplayGroup();
	
	virtual void	DisplayObjectWindow();
	
	static void		InitializeColors();
	static void		SetObjectTypes(LArray** inTypeArrays);

	static void		SetTypeVisible(Int8 inType, Boolean inVisible);
	static void		SetShowInvisible(Boolean inShow) { sShowInvisible = inShow; };
	
	static void		SetMapDimensions(Int8 inWorldHeigth, Int8 inWorldWidth);

	static			RGBColor sColors[NUMBER_OF_MARKER_TYPES];
	static			TQ3AttributeSet sColorAttributes[NUMBER_OF_MARKER_TYPES];

protected:
	mesh_ObjectInstance mObject;
	Boolean			mKeepGrounded;
	
	LWindow*		mWindow;

private:
	static LArray*	sObjectTypes[NUMBER_OF_MARKER_TYPES];

	static Boolean	sTypeVisible[NUMBER_OF_MARKER_TYPES];
	static Boolean	sShowInvisible;
	
	static SDimension32	sMapDimensions;
};