// ===========================================================================
//	CMapView.cp				   1997 eli curtz. All rights reserved.
// ===========================================================================
//

#ifdef PowerPlant_PCH
#include PowerPlant_PCH
#endif

#include <LPicture.h>
#include <LStream.h>
#include <UDrawingState.h>
#include <UGWorld.h>

#include "Constants.h"
#include "CMapObject.h"

#include "CMapView.h"

// ---------------------------------------------------------------------------
//	 CMapView						Default Constructor		  [public]
// ---------------------------------------------------------------------------

CMapView::CMapView()
	: LView(), LBroadcaster()
{
	mGWorld = nil;
	mScale = 1;
}


// ---------------------------------------------------------------------------
//	 CMapView(LStream*)			Stream Constructor	  [public]
// ---------------------------------------------------------------------------

CMapView::CMapView(
	LStream	*inStream)
		: LView(inStream), LBroadcaster()
{
	mGWorld = nil;
	mScale = 1;
}


// ---------------------------------------------------------------------------
//	 SetScale													  [public]
// ---------------------------------------------------------------------------

void
CMapView::SetScale(UInt8 inScale)
{
	if ((inScale != mScale) && mGWorld) {
		mScale = inScale;
		Rect gWorldBounds;
		mGWorld->GetBounds(gWorldBounds);
		ResizeImageTo((gWorldBounds.right - gWorldBounds.left) / mScale, (gWorldBounds.bottom - gWorldBounds.top) / mScale, false);

		Refresh();
	}
}


// ---------------------------------------------------------------------------
//	 GetGWorld													  [public]
// ---------------------------------------------------------------------------

LGWorld*
CMapView::GetGWorld()
{
	return mGWorld;
}


// ---------------------------------------------------------------------------
//	 SetGWorld													  [public]
// ---------------------------------------------------------------------------

void
CMapView::SetGWorld(
	LGWorld*	inGWorld)
{
	if (mGWorld)
		delete mGWorld;
		
	mGWorld = inGWorld;
	Rect gWorldBounds;
	mGWorld->GetBounds(gWorldBounds);
	ResizeImageTo((gWorldBounds.right - gWorldBounds.left) / mScale, (gWorldBounds.bottom - gWorldBounds.top) / mScale, false);

	Refresh();
}


// ---------------------------------------------------------------------------
//	 DrawSelf													   [protected]
// ---------------------------------------------------------------------------

void
CMapView::DrawSelf()
{		
	if (mGWorld != nil) {
		Rect viewBounds, viewFrame;
		SDimension32 viewSize;
		GetImageSize(viewSize);
		::SetRect(&viewBounds, 0, 0, viewSize.width, viewSize.height);
		GetRevealedRect(viewFrame);
		PortToLocalPoint(topLeft(viewFrame));
		PortToLocalPoint(botRight(viewFrame));
		::SectRect(&viewFrame, &viewBounds, &viewFrame);
		
		Rect gWorldBounds, gWorldFrame;
		mGWorld->GetBounds(gWorldBounds);
		
		gWorldFrame = viewFrame;
		::MapRect(&gWorldFrame, &viewBounds, &gWorldBounds);
		::SectRect(&gWorldFrame, &gWorldBounds, &gWorldFrame);
		
		// LGWorld is missing this so we do it ourselves
		// mGWorld->CopyImage(this->GetMacPort(), frame);
		
		::CopyBits(&((GrafPtr) mGWorld->GetMacGWorld())->portBits,
					&(this->GetMacPort())->portBits,
					&gWorldFrame, &viewFrame, srcCopy, nil);
		
		// Display the visible marker objects
		SRect32 mapRect;
		mapRect.top = gWorldFrame.top * 64;
		mapRect.left = gWorldFrame.left * 64;
		mapRect.bottom = gWorldFrame.bottom * 64;
		mapRect.right = gWorldFrame.right * 64;
		
		CMapObject* currentObject = nil;
		LArrayIterator objectIter(*mObjects);
		while (objectIter.Next(&currentObject)) {
			if (currentObject->IsVisibleHV(mapRect)) {
				SPoint32 objectLoc = currentObject->GetPositionHV();
				Rect displayRect;
				objectLoc.h /= 64; objectLoc.h /= mScale;
				objectLoc.v /= 64; objectLoc.v /= mScale;
				displayRect.left = displayRect.right = objectLoc.h;
				displayRect.top = displayRect.bottom = objectLoc.v;
				if (mScale >= 4) {
					displayRect.right++;
					displayRect.bottom++;
				}
				else
					::InsetRect(&displayRect, (0 - (2 / mScale)), (0 - (2 / mScale)));
				::RGBForeColor(&CMapObject::sColors[currentObject->GetTypeIndex()]);
				::PaintRect(&displayRect);
			}
		}
	}
	else {
		Rect viewFrame;
		GetRevealedRect(viewFrame);

		::PenNormal();
		::FillRect(&viewFrame, &UQDGlobals::GetQDGlobals()->ltGray);
		::FrameRect(&viewFrame);
	}
}
	
// ---------------------------------------------------------------------------
//	 ClickSelf(const SMouseDownEvent&)								 [protected]
// ---------------------------------------------------------------------------

void
CMapView::ClickSelf(
	const SMouseDownEvent& inMouseDown)
{
	if (GetClickCount() > 1) {
		Point clickPoint = inMouseDown.wherePort;
		PortToLocalPoint(clickPoint);
		SRect32 testRect;
		testRect.top = (clickPoint.v  - (4 / mScale)) * 64 * mScale;
		testRect.left = (clickPoint.h - (4 / mScale)) * 64 * mScale;
		testRect.bottom = (clickPoint.v  + (4 / mScale)) * 64 * mScale;
		testRect.right = (clickPoint.h + (4 / mScale)) * 64 * mScale;

		CMapObject* currentObject = nil;
		LArrayIterator objectIter(*mObjects);
		while (objectIter.Next(&currentObject)) {
			if (currentObject->IsVisibleHV(testRect))
				currentObject->DisplayObjectWindow();
		}
	}
	else {
	}
}

// ---------------------------------------------------------------------------
//	 SetObjects(LArray*)								
// ---------------------------------------------------------------------------

void
CMapView::SetObjects(LArray* inObjects)
{
	mObjects = inObjects;
}