// =================================================================================
//	CMythographyApp.cp					1997 eli curtz. All rights reserved.
// =================================================================================

#include <LGrowZone.h>
#include <LMenu.h>
#include <LMenuBar.h>
#include <LString.h>
#include <LWindow.h>
#include <LPreferencesFile.h>
#include <PP_Messages.h>
#include <PPobClasses.h>
#include <UDesktop.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <UReanimator.h>
#include <URegistrar.h>
#include <UTextTraits.h>

#include "MoreFiles.h"
#include "MoreFilesExtras.h"
#include "MoreDesktopMgr.h"
#include "IterateDirectory.h"
#include "Constants.h"
#include "CMapDocument.h"
#include "CMapView.h"
#include "C3DMapView.h"
#include "CMapObject.h"
#include "CMiniIconTable.h"
#include "CGorFile.h"
#include "UQuickDraw3D.h"

#include "CMythographyApp.h"

// =================================================================================
//		 Main Program
// =================================================================================

void
main( void )
{
	// Setup the throw and signal actions.
	SetDebugThrow_( debugAction_Alert );
	SetDebugSignal_( debugAction_Alert );

	// Initialize the heap. Parameter is number
	// of master handle blocks to allocate.
	InitializeHeap( 4 );
	
	// Initialize the MacOS toolbox.
	UQDGlobals::InitializeToolbox( &qd );
	
	// Initialize QuickDraw3D
	UQuickDraw3D::Initialize();

	// Install a GrowZone function to catch  low memory situations.
	// Parameter is the size of the memory reserve in bytes.
	new LGrowZone( 20000 );

	// Create the application object and run it.
	CMythographyApp	theApp;
	theApp.Run();
}


// =================================================================================
//		 Utility routine for MoreFiles to use in folder iteration
// =================================================================================

pascal	void
AddAliasFilter(
	const CInfoPBRec * const cpbPtr,
	Boolean *quitFlag,
	void *yourDataPtr)
{
	*quitFlag = false;

	if (cpbPtr->hFileInfo.ioFlFndrInfo.fdType == kMythGorFileType) { 	// is it a '.gor' file?
		Try_ {
			Int16 fileID = cpbPtr->hFileInfo.ioDirID;
			FSSpec theFileSpec;
			theFileSpec.vRefNum = cpbPtr->hFileInfo.ioVRefNum;
			theFileSpec.parID = cpbPtr->hFileInfo.ioFlParID;
			::BlockMoveData(cpbPtr->hFileInfo.ioNamePtr, &theFileSpec.name, sizeof(Str63));

			LArray* theAliasArray = (LArray*) yourDataPtr;
			CGorFile* theNewFile = new CGorFile(theFileSpec);
			theAliasArray->InsertItemsAt(1, LArray::index_Last, &theNewFile);
			
		} Catch_(inErr) {
			*quitFlag = true;	// bail if we get into trouble
		} EndCatch_
	}
}


// ---------------------------------------------------------------------------------
//		 CMythographyApp
// ---------------------------------------------------------------------------------

CMythographyApp::CMythographyApp()
{
	// Register PowerPlant class creator functions.
	RegisterAllPPClasses();

	// Register custom classes.
	RegisterClass_(CMiniIconTable);
	RegisterClass_(CMapView);
	RegisterClass_(C3DMapView);

//hack	
	CMapObject::InitializeColors();
}


// ---------------------------------------------------------------------------------
//		 ~CMythographyApp
// ---------------------------------------------------------------------------------

CMythographyApp::~CMythographyApp()
{
	delete mPrefFile;
}


// ---------------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------------

void
CMythographyApp::StartUp()
{
	// Get a file spec for the application to use with aliases
	Str255				appName;
	FSSpec				appSpec;
	ProcessInfoRec		info;
	ProcessSerialNumber	curPSN;

	ThrowIfOSErr_(::GetCurrentProcess(&curPSN));
	
	info.processInfoLength = sizeof(ProcessInfoRec);
	info.processName = appName;
	info.processAppSpec = &appSpec;
	ThrowIfOSErr_(::GetProcessInformation(&curPSN, &info));
	
	mAppFile = new LFile(appSpec);

	// Open or create the preferences file
	mPrefFile = new LPreferencesFile(kPrefFileName);
	Int16 prefFileRefNum = mPrefFile->OpenOrCreateResourceFork(fsRdWrPerm, kCreatorType, kPrefFileType, smSystemScript);
	::UseResFile(prefFileRefNum);
	
	// Build the list of source files
	mSourceFiles = new LArray(sizeof(CGorFile*));
	UInt16 theAliasResCount = ::Count1Resources(kSourceFileAliasType);
	if (theAliasResCount > 0) {		// Read in the aliases
		Handle theAliasRes = nil;
		for (UInt8 resIndex = 1; resIndex <= theAliasResCount; resIndex++) {
			theAliasRes = ::Get1IndResource(kSourceFileAliasType, resIndex);
			Assert_(theAliasRes);
			::RemoveResource(theAliasRes);
			
			Boolean aliasChanged = false;
			CGorFile* theNewFile = new CGorFile((AliasHandle) theAliasRes, aliasChanged);
			mSourceFiles->InsertItemsAt(1, LArray::index_Last, &theNewFile);
			::DisposeHandle(theAliasRes); theAliasRes = nil;
		}

	}
	else { 							// No files to use, go add the defaults
		FSSpec	volSpec;
		Int16	volCountReturned = 0;
		Int16	volIndex = 1;
		FSSpec	mythAppSpec;
		OSErr	resultErr = noErr;
		
		do {
			resultErr =	::OnLine(&volSpec, 1, &volCountReturned, &volIndex);
			if ((resultErr == noErr) || (resultErr == nsvErr))
				resultErr = ::FSpDTGetAPPL(nil, volSpec.vRefNum, kMythApplicationCreator, &mythAppSpec);
		} while (resultErr == afpItemNotFound);
		
		FSSpec	tagFolderSpec;
		resultErr = ::FSMakeFSSpec(mythAppSpec.vRefNum, mythAppSpec.parID, kTagsFolderName, &tagFolderSpec);
		if (resultErr == fnfErr)	// maybe we found the version on the CD
			resultErr = ::FSMakeFSSpec(mythAppSpec.vRefNum, mythAppSpec.parID, kTagsCDFolderName, &tagFolderSpec);

		resultErr = ::FSpIterateDirectory(&tagFolderSpec, 1, &AddAliasFilter, mSourceFiles);
	}
	
	mPrefFile->CloseResourceFork();

//	ObeyCommand( cmd_New, nil );
	ChooseDocument();
}


// ---------------------------------------------------------------------------------
//		 OpenDocument
// ---------------------------------------------------------------------------------

void
CMythographyApp::OpenDocument(
	FSSpec	*inMacFSSpec )
{
	// Create a new document using the file spec.
	CMapDocument* theNewDoc = new CMapDocument( this, inMacFSSpec, mSourceFiles );
}


// ---------------------------------------------------------------------------------
//		 MakeNewDocument
// ---------------------------------------------------------------------------------

LModelObject *
CMythographyApp::MakeNewDocument()
{
	// Don't make empty docs for now
	return nil;

	// Make a new empty document.
//	return new CMapDocument( this, nil );
}


// ---------------------------------------------------------------------------------
//		 ChooseDocument
// ---------------------------------------------------------------------------------

void
CMythographyApp::ChooseDocument()
{
	// Deactivate the desktop.
	::UDesktop::Deactivate();

	// Browse for a document.
	SFTypeList			theTypeList = {'MESH'};
	StandardFileReply	theReply;
	::StandardGetFile( nil, 1, theTypeList, &theReply );

	// Activate the desktop.
	::UDesktop::Activate();
	
	// Send an apple event to open the file.	
	if ( theReply.sfGood ) SendAEOpenDoc( theReply.sfFile );
}


// ---------------------------------------------------------------------------------
//		 PrintDocument
// ---------------------------------------------------------------------------------

void
CMythographyApp::PrintDocument(
	FSSpec	*inMacFSSpec )
{
	// Create a new document using the file spec.
	CMapDocument	*theDocument = new CMapDocument( this, inMacFSSpec );
	Assert_( theDocument != nil );

	// Tell the document to print.
	theDocument->DoPrint();
}
