// =================================================================================
// CModelManager.cp
// =================================================================================


#include "CModelManager.h"

#include "CTagManager.h"
#include "CModelComparator.h"

#include <QD3DGroup.h>

// ---------------------------------------------------------------------------------
// CModelManager
// ---------------------------------------------------------------------------------

CModelManager::CModelManager( )
//	: fModelArray( CModelComparator::GetComparator() )
{
	// 7/18 jk changed this to prevent destruction of shared model comparator
	fModelArray.SetComparator( CModelComparator::GetComparator(), false );
	
	LGrowZone::GetGrowZone()->AddListener( this );
}


// ---------------------------------------------------------------------------------
// ~CModelManager
// ---------------------------------------------------------------------------------

CModelManager::~CModelManager()
{
}

	
// ---------------------------------------------------------------------------------
// Open
// ---------------------------------------------------------------------------------

void CModelManager::Open( FourCharCode inTagID )
{
#pragma unused( inTagID )
	// Pre build models on the mesh
}


// ---------------------------------------------------------------------------------
// ListenToMessage
// ---------------------------------------------------------------------------------

void CModelManager::ListenToMessage( MessageT inMessage, void *ioParam )
{
	if ( inMessage == msg_GrowZone )
	{
		Int32	theFreedSize	= 0;
		
		*(Int32*)ioParam = theFreedSize;
	}
}


// ---------------------------------------------------------------------------------
// GetModel
// ---------------------------------------------------------------------------------

TQ3GroupObject CModelManager::GetModel( FourCharCode inTagID, Int32 inPermutation )
{
	TQ3GroupObject theModel = NULL;
	
	ArrayIndexT theModelIndex = fModelArray.FetchIndexOfKey( &inTagID );
	if ( !fModelArray.ValidIndex( theModelIndex ) ) {
		theModelIndex = BuildModelData(inTagID);
	}
	
	Int16 theAdjustedPermutation = LArray::index_First + ( inPermutation % fModelArray[theModelIndex].fPermutations->GetCount() );
	theModel = (*fModelArray[theModelIndex].fPermutations)[theAdjustedPermutation];
	
	return theModel;
}


// ---------------------------------------------------------------------------------
// BuildModelData
// ---------------------------------------------------------------------------------
typedef TArray<TQ3PolyhedronTriangleData> TQ3TriangleArrayT;

ArrayIndexT CModelManager::BuildModelData( FourCharCode inTagID )
{
	ArrayIndexT theModelIndex = LArray::index_Bad;
	SModelInfo 	theModelInfo;
	theModelInfo.fModelTag = inTagID;
	theModelInfo.fPermutations = new TArray<TQ3GroupObject>();

	Try_
	{
		CTagManager::ReadTagHeader( MODEL_GROUP_TAG, inTagID, (char *) &theModelInfo.fModelHeader );
		CTagManager::ReadTagHeader( GEOMETRY_GROUP_TAG, theModelInfo.fModelHeader.geometry_tag, (char *) &theModelInfo.fGeometryHeader );
		
		Assert_( theModelInfo.fModelHeader.model_cell_count 	<= MAXIMUM_CELLS_PER_MODEL );

		Assert_( theModelInfo.fGeometryHeader.material_count 	<= MAXIMUM_MATERIALS_PER_GEOMETRY );
		Assert_( theModelInfo.fGeometryHeader.surface_count 	<= MAXIMUM_SURFACES_PER_GEOMETRY );
		Assert_( theModelInfo.fGeometryHeader.vertex_count 		<= MAXIMUM_VERTICES_PER_GEOMETRY );

		for ( Int32 permutationIndex = 0; permutationIndex < theModelInfo.fModelHeader.model_permutation_count; permutationIndex++ )
		{
			TQ3GroupObject theGroup = ::Q3Group_New();
			ThrowIfNULL_( theGroup );
			theModelInfo.fPermutations->AddItem( theGroup );
		}
		
		// Read the vertices
		TArray<geometry_vertex> theGeomVertices( CTagManager::ReadTagData(
													GEOMETRY_GROUP_TAG, theModelInfo.fModelHeader.geometry_tag,
													theModelInfo.fGeometryHeader.vertices_offset,
													theModelInfo.fGeometryHeader.vertices_size ) );
		Assert_( theGeomVertices.GetCount() == theModelInfo.fGeometryHeader.vertex_count );

		// Build the QuickDraw3D vertices
		TArray<TQ3Vertex3D> theQD3DVertices( theGeomVertices.GetCount() );
		TQ3Vertex3D theQD3DVertex;
		theQD3DVertex.attributeSet 	= NULL;
		for ( Int32 vertexIndex = LArray::index_First; vertexIndex <= theGeomVertices.GetCount(); vertexIndex++ )
		{
			// As always transforming from Myth to QD3D coordinates
			theQD3DVertex.point.x		= 0.0 - WORLD_TO_FLOAT( theGeomVertices[vertexIndex].p.x );
			theQD3DVertex.point.y 		= WORLD_TO_FLOAT( theGeomVertices[vertexIndex].p.z );
			theQD3DVertex.point.z 		= WORLD_TO_FLOAT( theGeomVertices[vertexIndex].p.y );
			
			theQD3DVertices.AddItem(theQD3DVertex);
		}
		
		// Build the QuickDraw3D polyhedron data sets
		TQ3PolyhedronData theQD3DPolyhedronData;
		theQD3DPolyhedronData.numVertices = theQD3DVertices.GetCount();
		theQD3DPolyhedronData.vertices = NULL;
		theQD3DPolyhedronData.numEdges = 0;
		theQD3DPolyhedronData.edges = NULL;
		theQD3DPolyhedronData.numTriangles = 0;
		theQD3DPolyhedronData.triangles = NULL;
		theQD3DPolyhedronData.polyhedronAttributeSet = NULL;
		TArray<TQ3PolyhedronData> thePolyhedrons( theModelInfo.fGeometryHeader.material_count );
		theQD3DVertices.Lock();
		Handle theQD3DVerticesH = theQD3DVertices.GetItemsHandle();
		for ( Int32 polyhedronIndex = 0; polyhedronIndex < theModelInfo.fGeometryHeader.material_count; polyhedronIndex++ )
		{
			theQD3DPolyhedronData.vertices = new TQ3Vertex3D[theQD3DPolyhedronData.numVertices];
			::BlockMoveData( *theQD3DVerticesH, theQD3DPolyhedronData.vertices, ::GetHandleSize(theQD3DVerticesH) );
			
			thePolyhedrons.AddItem(theQD3DPolyhedronData);
		}
		theQD3DVertices.Unlock();

		// Read the surfaces
		TArray<geometry_surface> theGeomSurfaces( CTagManager::ReadTagData(
													GEOMETRY_GROUP_TAG, theModelInfo.fModelHeader.geometry_tag,
													theModelInfo.fGeometryHeader.surfaces_offset,
													theModelInfo.fGeometryHeader.surfaces_size ) );
		Assert_( theGeomSurfaces.GetCount() == theModelInfo.fGeometryHeader.surface_count );
		
		// Build the QuickDraw3D triangle arrays
		TArray<TQ3TriangleArrayT*> theQD3DTriangles( theModelInfo.fGeometryHeader.material_count );
		for (Int32 triangleArrayIndex = 0; triangleArrayIndex < theModelInfo.fGeometryHeader.material_count; triangleArrayIndex++ )
		{
			theQD3DTriangles.AddItem( new TQ3TriangleArrayT() );
		}

		
		TQ3PolyhedronTriangleData theQD3DTriangleData;
		theQD3DTriangleData.edgeFlag = kQ3PolyhedronEdgeAll;
		theQD3DTriangleData.triangleAttributeSet = NULL;
		for ( Int32 surfaceIndex = LArray::index_First; surfaceIndex <= theGeomSurfaces.GetCount(); surfaceIndex++ )
		{
			geometry_surface &currentSurface = theGeomSurfaces[surfaceIndex];
			TQ3PolyhedronData &currentPolyhedron = thePolyhedrons[LArray::index_First + currentSurface.material_index];
			
			for ( Int32 cornerIndex = 0; cornerIndex < NUMBER_OF_CORNERS_PER_TRIANGLE; cornerIndex++ )
			{
				theQD3DTriangleData.vertexIndices[cornerIndex] = currentSurface.corners[cornerIndex].vertex_index;
				TQ3Vertex3D &currentVertex = currentPolyhedron.vertices[theQD3DTriangleData.vertexIndices[cornerIndex]];
				if (currentVertex.attributeSet == NULL)
				{
					currentVertex.attributeSet = ::Q3AttributeSet_New();
					ThrowIfNULL_( currentVertex.attributeSet );
					
					TQ3Param2D theVertexUV;
					theVertexUV.u = (float) currentSurface.corners[cornerIndex].interpolants[1] / 0xFFFF;
					theVertexUV.v = 1.0 - ( (float) currentSurface.corners[cornerIndex].interpolants[0] / 0xFFFF );
					Assert_((theVertexUV.u >= 0.0) && (theVertexUV.u <= 1.0));
					Assert_((theVertexUV.v >= 0.0) && (theVertexUV.v <= 1.0));
					
					::Q3AttributeSet_Add( currentVertex.attributeSet, kQ3AttributeTypeSurfaceUV, &theVertexUV );
				}
			}
			theQD3DTriangles[LArray::index_First + currentSurface.material_index]->AddItem(theQD3DTriangleData);
		}

		for ( Int32 polyhedronIndex = LArray::index_First; polyhedronIndex <= thePolyhedrons.GetCount() ; polyhedronIndex++ )
		{
			thePolyhedrons[polyhedronIndex].numTriangles = theQD3DTriangles[polyhedronIndex]->GetCount();
			thePolyhedrons[polyhedronIndex].triangles = new TQ3PolyhedronTriangleData[thePolyhedrons[polyhedronIndex].numTriangles];
			theQD3DTriangles[polyhedronIndex]->Lock();
			Handle theQD3DTrianglesH = theQD3DTriangles[polyhedronIndex]->GetItemsHandle();
			::BlockMoveData( *theQD3DTrianglesH, thePolyhedrons[polyhedronIndex].triangles, ::GetHandleSize(theQD3DTrianglesH) );
			theQD3DTriangles[polyhedronIndex]->Unlock();

			TQ3GeometryObject theQD3DPolyhedron = ::Q3Polyhedron_New( &thePolyhedrons[polyhedronIndex] );
			ThrowIfNULL_( theQD3DPolyhedron );

			for ( Int32 permutationIndex = LArray::index_First; permutationIndex <= theModelInfo.fPermutations->GetCount(); permutationIndex++ )
			{
				::Q3Group_AddObject( (*theModelInfo.fPermutations)[permutationIndex], theQD3DPolyhedron );
			}
			
			::Q3Object_Dispose( theQD3DPolyhedron );
		}

		for (Int32 triangleArrayIndex = LArray::index_First; triangleArrayIndex <= theQD3DTriangles.GetCount(); triangleArrayIndex++ )
		{
			delete theQD3DTriangles[triangleArrayIndex];
		}
		
		theModelIndex = fModelArray.AddItem( theModelInfo );
	}
	Catch_ ( inErr )
	{
	}
	EndCatch_
	
	return theModelIndex;
}