// =================================================================================
// CTextureManager.h
// =================================================================================


#pragma once


#include "PiriReisTypes.h"


const MessageT	msg_TextureChanged	=	'CTEX';


struct SBitmapRef
{
	bitmap_reference	fBitmapRef;
	Handle				fImage;
	Int32				fTimestamp;
	Boolean				fIsDirty;
};


// Bungie-supplied structure for blood remapping
struct histogram_entry
{
	word	flags;
	short	index;
	long	count;
};


class StProgressDialog;


class CTextureManager : public LBroadcaster,
						public LListener
{
public:
								CTextureManager();
	virtual						~CTextureManager();

	virtual	void				ListenToMessage( MessageT inMessage, void *ioParam );
	
			void				BringToFront();
	
			void				Open(   FourCharCode	inTextureTagID,
									    Int16			inSubmeshWidth,
									    Int16			inSubmeshHeight );
			void				New(    FourCharCode	inTextureTagID,
									    TString<Str31>	inTagName,
									    Int16			inSubmeshWidth,
									    Int16			inSubmeshHeight );
			void				SaveAs( FourCharCode	inTextureTagID,
										TString<Str31>	inTextureName );
			void				Save();
	
			Boolean				IsDirty()
								{ return fIsDirty; }
	
			Int16				GetPixelWidth()
								{ return fSubmeshWidth * SUBMESH_TEXTURE_WIDTH; };
							
			Int16				GetPixelHeight()
								{ return fSubmeshHeight * SUBMESH_TEXTURE_WIDTH; };
			
			void				ImportTextureMap( FSSpec &inFileSpec );
			void				ExportTextureMap( FSSpec &inFileSpec );

			void				ImportShadowMap( FSSpec &inFileSpec );			
			void				ExportShadowMap( FSSpec &inFileSpec );
	
			CTabHandle			GetColorTable();
			Handle				GetBlendTable();
			Handle				GetRemapTable();
	
			Handle				GetTextureMap( Int32 inSubmeshIndex );
			Handle				GetShadowMap( Int32 inSubmeshIndex );
			Handle				GetCombinedMap( Int32 inSubmeshIndex );
			
			void				SetTextureMap( Int32 inSubmeshIndex, Handle inImage );
			void				SetShadowMap( Int32 inSubmeshIndex, Handle inImage );
			
			void				Changed();
		
protected:
			Boolean				fIsDirty;
			FourCharCode		fTextureTagID;
			Int16				fSubmeshWidth;
			Int16				fSubmeshHeight;
			collection_header	fHeader;
			CTabHandle			fColorTable;
			Handle				fBlendTable;
			Handle				fRemapTable;
			TArray<SBitmapRef>	fBitmapArray;
			
			void				UnloadAllImages();
			void				UnloadColorTable();
			void				UnloadBlendTable();
			void				UnloadRemapTable();
			
			// Read data from tag file
			void				ReadHeader();
			void				ReadBitmapArray();
			void				ReadColorTable();
			void				ReadBlendTable();
			void				ReadRemapTable();
			void				ReadImage( ArrayIndexT inBitmapIndex );
			
			// Write data to tag file
			void				WriteHeader();
			void				WriteBitmapArray();
			void				WriteColorTable( CTabHandle inColorTable );
			void				WriteBlendTable( Handle inBlendTable );
			void				WriteRemapTable( Handle inRemapTable );
			void				WriteImage( ArrayIndexT inBitmapIndex );
			void				WriteImage( UInt32 inOffset, Handle inImage );
			
			Handle				BuildColorTable( CTabHandle &inMacColorTable );
			
			Handle				BuildImage();
			
			void				ImportImages(		FSSpec				&inFileSpec,
													Boolean				inIsLandscape,
													StProgressDialog	*inProgress );
			void				FlipImage( char* inSource, char* inDest,
										   UInt16 inHeight, UInt16 inWidth );
										 
			void				ExportToPhotoshopFile( const FSSpec &inFileSpec,
													   Boolean		inIsLandscape );

			// Blend Table and Blood Remapping Table 
			void				BuildBlendTable(	 CTabHandle			inMacColorTable,
													 StProgressDialog	*inProgress );
			void				BuildRemappingTable( CTabHandle			inMacColorTable,
													 StProgressDialog	*inProgress );

			Int16				FindClosestColorIndex( CTabHandle inColorTable, RGBColor &inColor );
			void				GenerateBloodColor( RGBColor &inOriginalColor,
												    RGBColor &ioBloodyColor );
};


class CHistogramComparator : public LComparator
{
public:
						CHistogramComparator() {};
	virtual				~CHistogramComparator() {};

	virtual Int32		Compare( const void*	inItemOne,
								 const void* 	inItemTwo,
								 Uint32			inSizeOne,
								 Uint32			inSizeTwo) const;
};
