// ===========================================================================
// C3DMapView.h
// ===========================================================================


#pragma once


#include "PiriReisTypes.h"

#include <QD3D.h>
#include <QD3DGeometry.h>


class CTextureManager;
class CMeshEditor;
class CMeshCellManager;
class CMeshMarkerManager;
class CModelManager;


enum ERenderType
{
	kWireframeRender	= 1,
	kTextureRender		= 3,
	kMediaRender		= 4,
	kTerrainTypeRender	= 5
};

enum EDetailLevel
{
	kNoTextureResolution	= 0,
	kQuarterResolution		= 4,
	kHalfResolution			= 8,
	kFullResolution			= 16
};

enum ECameraKeys
{
	kCamera_MoveForward	= 0,
	kCamera_MoveBack,
	kCamera_MoveLeft,
	kCamera_MoveRight,
	kCamera_PanLeft,
	kCamera_PanRight,
	kCamera_OrbitLeft,
	kCamera_OrbitRight,
	kCamera_OrbitUp,
	kCamera_OrbitDown,
	kCamera_ZoomIn,
	kCamera_ZoomOut,
	NUMBER_OF_CAMERA_KEYS
};

// ---------------------------------------------------------------------------
// C3DMapView
// ---------------------------------------------------------------------------

class C3DMapView : public LView,
				   public LListener,
				   public LCommander
{
public:
	enum { class_ID = '3DMV' };

							C3DMapView( LStream *inStream );
	virtual					~C3DMapView();
	
			void			SetMeshEditor( CMeshEditor *inMeshEditor );
			
			void			SetRenderType( ERenderType inRenderType );
			
			void			SetShowMarkers( Boolean inShowIt );
			void			SetShowModels( Boolean inShowIt );

	virtual void			ListenToMessage( MessageT inMessage, void *ioParam );

	virtual void			ResizeFrameBy( Int16 inWidthDelta, Int16 inHeightDelta, Boolean inRefresh);
	virtual Boolean			FocusDraw( LPane *inSubPane = nil );

	virtual Boolean			HandleKeyPress( const EventRecord &inKeyEvent );
			
			void			CameraOrbit( float inDegrees, float inVerticalDegrees = 0.0 );
			void			CameraPan( float inDegrees );
			void			CameraZoom( float inDistance );
			void			CameraMove( float inDistance, float inOrthogonalDistance );
								
protected:
			CMeshEditor			*fMeshEditor;
			CTextureManager		*fTextureManager;
			CMeshCellManager	*fMeshCellManager;
			CMeshMarkerManager	*fMeshMarkerManager;
			CModelManager		*fModelManager;

			ERenderType		fRenderType;
			
			Boolean			fShowMarkers;
			Boolean			fShowModels;
			
			UInt16			fCameraKeys[NUMBER_OF_CAMERA_KEYS];
			
			TQ3ViewObject		fQD3DView;
			TQ3GroupObject		fQD3DMeshGroup;
			TQ3PickObject		fQD3DPointPicker;
			TQ3PickObject		fQD3DRectPicker;
			TQ3GeometryObject	fQD3DTriGrid;
			
			LGWorld*		fTextureWorld;
			EDetailLevel	fTextureDetail;
			TQ3AttributeSet fTextureAttribute;
			Int32			fTextureBufferSize;
			
			RgnHandle		fMeshSelection;
			
	virtual	void			FinishCreateSelf();
			
	virtual	void			DrawSelf();
	
	virtual void			InitializeQD3DView();

	virtual void			BuildTexture( EDetailLevel inDetail = kFullResolution );
	
	virtual	void			ClickSelf( const SMouseDownEvent &inMouseDown );
//	virtual	Boolean			PointsAreClose( Point inFirstPt, Point inSecondPt ) const;
	virtual ArrayIndexT		FindMarkerHitBy( Point &inPortPoint ) const;
	
	virtual void			SelectMarkersInRect( const Rect &inPortRect );
	virtual void			SelectVerticesInRect( const Rect &inPortRect );

			void			CalcModelTranslation(  long inMarkerX, long inMarkerY, long inMarkerZ, TQ3Matrix4x4 &outMatrix ) const;
			void			CalcModelRotation(  long inMarkerAngle, TQ3Matrix4x4 &outMatrix ) const;
	virtual	void			CalcMarkerBox( long inMarkerX, long inMarkerY, long inMarkerZ, TQ3BoxData &outBox ) const;
			TQ3AttributeSet	GetColorAttribute( Boolean inIsSelected, Boolean inIsVisible, Int16 inTeamIndex );
		
			Boolean			LocalToMythPoint( const Point &inPortPoint, world_point3d &ioMythPoint );
			
			void			HandleSelectMeshToolClick( const SMouseDownEvent &inMouseDown );
			void			HandleSelectMarkerToolClick( const SMouseDownEvent &inMouseDown );
			void			HandlePlaceMarkerToolClick( const SMouseDownEvent &inMouseDown );
			void			HandleTerrainTypeToolClick( const SMouseDownEvent &inMouseDown );
};