// ===========================================================================
// C3DMapWindow.cp
// ===========================================================================


#include "C3DMapWindow.h"
#include "C3DMapView.h"
#include "CMeshEditor.h"
#include "CMeshCellManager.h"

#include "PiriReisCommands.h"

#include <LStaticText.h>
#include <LBevelButton.h>


const ResIDT	kNoOverlayIcon		= 270;
const ResIDT	kShadowMapIcon		= 272;
const ResIDT	kMediaMapIcon		= 273;


// ---------------------------------------------------------------------------------
// C3DMapWindow
// ---------------------------------------------------------------------------------

C3DMapWindow::C3DMapWindow( LStream *inStream )
			: LWindow( inStream )
{
	fMapView		= NULL;
	fOverlayButton	= NULL;
}


// ---------------------------------------------------------------------------------
// ~C3DMapWindow
// ---------------------------------------------------------------------------------

C3DMapWindow::~C3DMapWindow()
{
}


// ---------------------------------------------------------------------------------
// FinishCreateSelf
// ---------------------------------------------------------------------------------

void C3DMapWindow::FinishCreateSelf()
{
	fMapView		= dynamic_cast<C3DMapView*>   ( this->FindPaneByID( '3DMV' ) );
	fOverlayButton	= dynamic_cast<LBevelButton*> ( this->FindPaneByID( 'OVER' ) );

	ThrowIfNULL_( fMapView		 );
	ThrowIfNULL_( fOverlayButton );

	fMapView	  	->SetMeshEditor( (CMeshEditor*) this->GetSuperCommander() );
	fOverlayButton	->AddListener( this );
	
	fMapView->SetSuperCommander( this );
	this	->SetLatentSub( fMapView );
	
	this->DoOverlayButton( 1 );
}

	
// ---------------------------------------------------------------------------------
// SetDescriptor
// ---------------------------------------------------------------------------------

void C3DMapWindow::SetDescriptor( ConstStringPtr inDescriptor )
{
	LStr255	theWindowTitle = "\p3D Map: " + LStr255( inDescriptor );
	
	LWindow::SetDescriptor( theWindowTitle );
}


// ---------------------------------------------------------------------------------
// ListenToMessage
// ---------------------------------------------------------------------------------

void C3DMapWindow::ListenToMessage( MessageT inMessage, void *ioParam )
{
	switch ( inMessage )
	{			
		case 'OVER':
			this->DoOverlayButton( *((Int32*) ioParam) );
			break;
	}
}


// ---------------------------------------------------------------------------------
// DoOverlayButton
// ---------------------------------------------------------------------------------

void C3DMapWindow::DoOverlayButton( Int32 inItem )
{
	// Sanity Check
	SignalIf_( fMapView == NULL );
	SignalIf_( fOverlayButton == NULL );

	ControlButtonContentInfo	theInfo;
	
	theInfo.contentType	= kControlContentCIconRes;
	theInfo.u.resID		= kNoOverlayIcon + inItem - 1;

	fOverlayButton->SetCurrentMenuItem( inItem );
	fOverlayButton->SetContentInfo( theInfo );

	switch ( inItem )
	{
		case 1:
			// Hide Overlays
			fMapView->SetRenderType( kWireframeRender );
			break;
			
		case 3:
			// Show Texture Map
			fMapView->SetRenderType( kTextureRender );
			break;
			
		case 4:
			// Show Media Map
			fMapView->SetRenderType( kMediaRender );
			break;
			
		case 6:
			// Show Terrain Type
			fMapView->SetRenderType( kTerrainTypeRender );
			break;
			
		default:
			SignalPStr_( "\pUnknown Overlay type." );
	}
}