// =================================================================================
//	CfontDocument.cp				1997 eli curtz. All rights reserved.
// =================================================================================

#include <LFile.h>
#include <LPlaceHolder.h>
#include <LPrintout.h>
#include <LString.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <UMemoryMgr.h>
#include <UWindows.h>

#include "Constants.h"
#include "CfontView.h"
#include "fontStructs.h"

#include "CfontDocument.h"

// ---------------------------------------------------------------------------------
//		 CfontDocument(LCommander*, FSSpec*)
// ---------------------------------------------------------------------------------


CfontDocument::CfontDocument(
	LCommander	*inSuper,
	FSSpec		*inFileSpec )
		: LSingleDoc( inSuper ), LListener()
{
	// Create window for our document.
	mWindow = LWindow::CreateWindow( rPPob_Window, this );
	Assert_( mWindow != nil );
	
	// Find the active panes
	mfontView = (CfontView *) mWindow->FindPaneByID( kMainView );
	Assert_( mfontView != nil );
	
	if ( inFileSpec == nil ) {

		// Set the name of new window.
		NameNewDoc();
		
	} else {
	
		// Display contents of file in window.
		OpenFile( *inFileSpec );
	
	}
	
	// Make the window visible.
	mWindow->Show();
}


// ---------------------------------------------------------------------------------
//		 NameNewDoc
// ---------------------------------------------------------------------------------

void
CfontDocument::NameNewDoc()
{
	// Setup the window title. Start with the default title.
	LStr255	theTitle( "\pUntitled" );

	// Find the first available title. We could also check the window
	// pane id if we wanted to make sure we didn't collide with other
	// window types.
	Int32	theNumber = 0;
	while ( UWindows::FindNamedWindow( theTitle ) != nil ) {

		// An existing window has the current name
		// Increment counter and try again.
		++theNumber;
		theTitle = "\pUntitled ";
		theTitle += (LStr255) theNumber;

	}		
	
	// Finally, set window title.
	mWindow->SetDescriptor( theTitle );
}


// ---------------------------------------------------------------------------------
//		 OpenFile
// ---------------------------------------------------------------------------------

void
CfontDocument::OpenFile(
	FSSpec	&inFileSpec )
{
	Try_ {

		// Create a new file object.
		mFileStream = new LFileStream( inFileSpec );
		mFile = mFileStream;	// In case the superclass is using it
		
		// Open the data fork.
		mFileStream->OpenDataFork( fsRdPerm );
		
		// Set the window title to the name of the file and
		// flag that the document has an associated file.
		mWindow->SetDescriptor( inFileSpec.name );
		mIsSpecified = true;

		// Skip the file header and read the font header
		dir_Header theFileHeader;
		Int32 byteCount = sizeof(dir_Header);
		mFileStream->GetBytes(&theFileHeader, byteCount);
		Assert_(theFileHeader.group == 'font');
		
		byteCount = sizeof(font_Header);
		mFileStream->GetBytes(&mfontHeader, byteCount);
		mHeaderOffset = sizeof(dir_Header) + mfontHeader.headerSize;

	} Catch_( inErr ) {

		// Cleanup and rethrow the error.
		delete this;
		Throw_( inErr );
	
	} EndCatch_
}


// ---------------------------------------------------------------------------------
//		 IsModified
// ---------------------------------------------------------------------------------

Boolean
CfontDocument::IsModified()
{
	return false;
}


// ---------------------------------------------------------------------------------
//		 DoAESave
// ---------------------------------------------------------------------------------

void
CfontDocument::DoAESave(
	FSSpec	&inFileSpec,
	OSType	inFileType )
{
/*
	// Delete the existing file object.
	// Note: this does nothing to the actual file on disk.
	delete mFileStream;
	
	// Make a new file object.
	mFileStream = new LFile( inFileSpec );
	
	// Get the proper file type.
	OSType	theFileType = 'TEXT';
	if ( inFileType != fileType_Default ) theFileType = inFileType;

	// Make new file on disk (we'll use
	// SimpleText's creator for this example).
	mFileStream->CreateNewDataFile( 'ttxt', theFileType );
	
	// Write out the data.
	DoSave();

	// Change window title to reflect the new name.
	mWindow->SetDescriptor( inFileSpec.name );

	// Document now has a specified file.
	mIsSpecified = true;
*/
}


// ---------------------------------------------------------------------------------
//		 DoSave
// ---------------------------------------------------------------------------------

void
CfontDocument::DoSave()
{
/*
	// Open the data fork.
	mFileStream->OpenDataFork( fsRdWrPerm );

	// Get the text from the text view.
	Handle	theTextH = mTextView->GetTextHandle();
	
	// Lock the text handle.
	StHandleLocker	theLock( theTextH );
	
	// Write the text to the file.
	mFileStream->WriteDataFork( *theTextH, ::GetHandleSize( theTextH ) );

	// Close the data fork.
	mFileStream->CloseDataFork();

	// Saving makes doc un-dirty.
	mTextView->SetDirty( false );
*/
}


// ---------------------------------------------------------------------------------
//		 DoRevert
// ---------------------------------------------------------------------------------

void
CfontDocument::DoRevert()
{
/*
	// Open the data fork.
	mFileStream->OpenDataFork( fsRdWrPerm );

	// Read the entire file contents and close the file.
	Handle	theTextH = mFileStream->ReadDataFork();
	mFileStream->CloseDataFork();
	
	// Put the contents in the text view
	// and clear the dirty flag.
	mTextView->SetTextHandle( theTextH );
	mTextView->SetDirty( false );
	
	// Dispose of the text.
	::DisposeHandle( theTextH );

	// Refresh the text view.
	mTextView->Refresh();
*/
}


// ---------------------------------------------------------------------------------
//		 DoPrint
// ---------------------------------------------------------------------------------

void
CfontDocument::DoPrint()
{
/*
	// Create the printout.
	LPrintout	*thePrintout = LPrintout::CreatePrintout( rPPob_TextPrintout );
	Assert_( thePrintout != nil );
	
	// Set the print record.
	
	if (mPrintRecordH)
	{
		thePrintout->SetPrintRecord( mPrintRecordH );
	}
	
	// Get the text placeholder.
	LPlaceHolder	*thePlaceholder;
	thePlaceholder = (LPlaceHolder*) thePrintout->FindPaneByID( kTextPlaceholder );
	Assert_( thePlaceholder != nil );
	
	// Install the text view in the placeholder.
	thePlaceholder->InstallOccupant( mTextView, atNone );
	
	// Set the frame size.
	SetPrintFrameSize();
	
	// Print.
	thePrintout->DoPrintJob();
	
	// Delete the printout. The text view is returned
	// to the window when the placeholder is destroyed.
	delete thePrintout;
*/
}


// ---------------------------------------------------------------------------------
//		 SetPrintFrameSize
// ---------------------------------------------------------------------------------

void
CfontDocument::SetPrintFrameSize()
{
/*
	// Get the frame size.
	SDimension16	theFrameSize;
	mTextView->GetFrameSize( theFrameSize );
	
	// Get the text edit record handle.
	TEHandle	theTextEditH = mTextView->GetMacTEH();
	
	// Calculate the number of lines per page.
	Int16	theLinesPerPage;
	theLinesPerPage = theFrameSize.height / (**theTextEditH).lineHeight;

	// Resize the frame to an integral number of lines.
	mTextView->ResizeFrameTo( theFrameSize.width,
		(**theTextEditH).lineHeight * theLinesPerPage, false );

*/
}


void
CfontDocument::ListenToMessage(
	MessageT inMessage,
	void * /* ioParam */)
{
}
