// =================================================================================
//	CmeshApp.cp					1997 eli curtz. All rights reserved.
// =================================================================================

#include <LGrowZone.h>
#include <LMenu.h>
#include <LMenuBar.h>
#include <LString.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <PPobClasses.h>
#include <UDesktop.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <UReanimator.h>
#include <URegistrar.h>
#include <UTextTraits.h>

#include "UQuickDraw3D.h"
#include "Constants.h"
#include "CmeshDocument.h"
#include "CmeshView.h"

#include "CmeshApp.h"


// =================================================================================
//		 Main Program
// =================================================================================

void
main( void )
{
	// Setup the throw and signal actions.
	SetDebugThrow_( debugAction_Alert );
	SetDebugSignal_( debugAction_Alert );

	// Initialize the heap. Parameter is number
	// of master handle blocks to allocate.
	InitializeHeap( 4 );
	
	// Initialize the MacOS toolbox.
	UQDGlobals::InitializeToolbox( &qd );

	// Install a GrowZone function to catch  low memory situations.
	// Parameter is the size of the memory reserve in bytes.
	new LGrowZone( 20000 );

	// Create the application object and run it.
	CmeshApp	theApp;
	theApp.Run();
}


// ---------------------------------------------------------------------------------
//		 CmeshApp
// ---------------------------------------------------------------------------------

CmeshApp::CmeshApp()
{
	// Initialize QD3D
	UQuickDraw3D::Initialize();
	Assert_(UQuickDraw3D::QuickDraw3DIsPresent());
	
	// Register PowerPlant class creator functions.
	RegisterAllPPClasses();

	// Register custom classes.
	RegisterClass_(LQD3DViewer);
	RegisterClass_(CmeshView);
}


// ---------------------------------------------------------------------------------
//		 ~CmeshApp
// ---------------------------------------------------------------------------------

CmeshApp::~CmeshApp()
{
}


// ---------------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------------

void
CmeshApp::StartUp()
{
//	ObeyCommand( cmd_New, nil );
	ChooseDocument();
}


// ---------------------------------------------------------------------------------
//		 OpenDocument
// ---------------------------------------------------------------------------------

void
CmeshApp::OpenDocument(
	FSSpec	*inMacFSSpec )
{
	// Create a new document using the file spec.
	new CmeshDocument( this, inMacFSSpec );
}


// ---------------------------------------------------------------------------------
//		 MakeNewDocument
// ---------------------------------------------------------------------------------

LModelObject *
CmeshApp::MakeNewDocument()
{
	// Don't make empty docs for now
	return nil;

	// Make a new empty document.
//	return new CmeshDocument( this, nil );
}


// ---------------------------------------------------------------------------------
//		 ChooseDocument
// ---------------------------------------------------------------------------------

void
CmeshApp::ChooseDocument()
{
	// Deactivate the desktop.
	::UDesktop::Deactivate();

	// Browse for a document.
	SFTypeList			theTypeList = {'MESH'};
	StandardFileReply	theReply;
	::StandardGetFile( nil, 1, theTypeList, &theReply );

	// Activate the desktop.
	::UDesktop::Activate();
	
	// Send an apple event to open the file.	
	if ( theReply.sfGood ) SendAEOpenDoc( theReply.sfFile );
}


// ---------------------------------------------------------------------------------
//		 PrintDocument
// ---------------------------------------------------------------------------------

void
CmeshApp::PrintDocument(
	FSSpec	*inMacFSSpec )
{
	// Create a new document using the file spec.
	CmeshDocument	*theDocument = new CmeshDocument( this, inMacFSSpec );
	Assert_( theDocument != nil );

	// Tell the document to print.
	theDocument->DoPrint();
}
