// ===========================================================================
//	CobjectView.cp				   1993-1997 eli curtz. All rights reserved.
// ===========================================================================
//

#ifdef PowerPlant_PCH
#include PowerPlant_PCH
#endif

#include <LPicture.h>
#include <LStream.h>
#include <UDrawingState.h>

#include <UGWorld.h>

#include "CobjectView.h"

// ---------------------------------------------------------------------------
//	 CobjectView						Default Constructor		  [public]
// ---------------------------------------------------------------------------

CobjectView::CobjectView()
{
	mGWorld = nil;
}


// ---------------------------------------------------------------------------
//	 CobjectView(LStream*)			Stream Constructor	  [public]
// ---------------------------------------------------------------------------

CobjectView::CobjectView(
	LStream	*inStream)
		: LView(inStream)
{
	mGWorld = nil;
}


// ---------------------------------------------------------------------------
//	 GetGWorld													  [public]
// ---------------------------------------------------------------------------

LGWorld*
CobjectView::GetGWorld()
{
	return mGWorld;
}


// ---------------------------------------------------------------------------
//	 SetGWorld													  [public]
// ---------------------------------------------------------------------------

void
CobjectView::SetGWorld(
	LGWorld*	inGWorld)
{
	if (mGWorld)
		delete mGWorld;
		
	mGWorld = inGWorld;
	Rect gWorldBounds;
	mGWorld->GetBounds(gWorldBounds);
	ResizeImageTo((gWorldBounds.right - gWorldBounds.left), (gWorldBounds.bottom - gWorldBounds.top), false);

	Refresh();
}


// ---------------------------------------------------------------------------
//	 DrawSelf													   [protected]
// ---------------------------------------------------------------------------

void
CobjectView::DrawSelf()
{		
	if (mGWorld != nil) {
		Rect	frame;
		GetRevealedRect(frame);
		PortToLocalPoint(topLeft(frame));
		PortToLocalPoint(botRight(frame));
		
		Rect gWorldFrame;
		mGWorld->GetBounds(gWorldFrame);
		
		::SectRect(&frame, &gWorldFrame, &frame);
		
		// LGWorld is missing this so we do it ourselves
		// mGWorld->CopyImage(this->GetMacPort(), frame);
		
		::CopyBits(&((GrafPtr) mGWorld->GetMacGWorld())->portBits,
					&(this->GetMacPort())->portBits,
					&frame, &frame, srcCopy, nil);
	}
	else {
		Rect	frame;
		GetRevealedRect(frame);

		::PenNormal();
		::FillRect(&frame, &UQDGlobals::GetQDGlobals()->ltGray);
		::FrameRect(&frame);
	}
}
	
